@@ -47,7 +47,10 @@ void TunnelManager::addServer(const char *ip, int port) {
 
 void TunnelManager::cleanServers() {
 	mServerAddrs.clear();
-
+	if (mLongRunningTaskId > 0) {
+		sal_end_background_task(mLongRunningTaskId);
+		mLongRunningTaskId = 0;
+	}
 	UdpMirrorClientList::iterator it;
 	for (it = mUdpMirrorClients.begin(); it != mUdpMirrorClients.end();) {
 		UdpMirrorClient& s=*it++;
@@ -62,8 +65,8 @@ void TunnelManager::reconnect(){
 		mTunnelClient->reconnect();
 }
 
-static void sCloseRtpTransport(RtpTransport *t, void *userData){
-	TunnelSocket *s=(TunnelSocket*)userData;
+static void sCloseRtpTransport(RtpTransport *t){
+	TunnelSocket *s=(TunnelSocket*)t->data;
 	TunnelManager *manager=(TunnelManager*)s->getUserPointer();
 	manager->closeRtpTransport(t, s);
 }
@@ -89,6 +92,7 @@ RtpTransport *TunnelManager::createRtpTransport(int port){
 	t->t_close=sCloseRtpTransport;
 	t->t_destroy=sDestroyRtpTransport;
 	t->data=socket;
+	ms_message("Creating tunnel RTP transport for local virtual port %i", port);
 	return t;
 }
 
@@ -96,6 +100,15 @@ void TunnelManager::startClient() {
 	ms_message("TunnelManager: Starting tunnel client");
 	mTunnelClient = new TunnelClient(TRUE);
 	mTunnelClient->setCallback((TunnelClientController::StateCallback)tunnelCallback,this);
+	if (mVerifyServerCertificate) {
+		const char *rootCertificatePath = linphone_core_get_root_ca(mCore);
+		if (rootCertificatePath != NULL) {
+			ms_message("TunnelManager: Load root certificate from %s", rootCertificatePath);
+			mTunnelClient->setRootCertificate(rootCertificatePath); /* give the path to root certificate to the tunnel client in order to be able to verify the server certificate */
+		} else {
+			ms_warning("TunnelManager is set to verify server certificate but no root certificate is available in linphoneCore");
+		}
+	}
 	list<ServerAddr>::iterator it;
 	for(it=mServerAddrs.begin();it!=mServerAddrs.end();++it){
 		const ServerAddr &addr=*it;
@@ -123,6 +136,8 @@ int TunnelManager::customRecvfrom(struct _RtpTransport *t, mblk_t *msg, int flag
 	int err=((TunnelSocket*)t->data)->recvfrom(msg->b_wptr,msg->b_datap->db_lim-msg->b_datap->db_base,from,*fromlen);
 	//to make ice happy
 	inet_aton(((TunnelManager*)((TunnelSocket*)t->data)->getUserPointer())->mLocalAddr,&msg->recv_addr.addr.ipi_addr);
+	msg->recv_addr.family = AF_INET;
+	msg->recv_addr.port = htons((unsigned short)((TunnelSocket*)t->data)->getPort());
 	if (err>0) return err;
 	return 0;
 }
@@ -134,7 +149,9 @@ TunnelManager::TunnelManager(LinphoneCore* lc) :
 	mTunnelizeSipPackets(true),
 	mTunnelClient(NULL),
 	mHttpProxyPort(0),
-	mVTable(NULL)
+	mVTable(NULL),
+	mLongRunningTaskId(0),
+	mSimulateUdpLoss(false)
 {
 	linphone_core_add_iterate_hook(mCore,(LinphoneCoreIterateHook)sOnIterate,this);
 	mTransportFactories.audio_rtcp_func=sCreateRtpTransport;
@@ -152,17 +169,22 @@ TunnelManager::TunnelManager(LinphoneCore* lc) :
 }
 
 TunnelManager::~TunnelManager(){
+	if (mLongRunningTaskId > 0) {
+		sal_end_background_task(mLongRunningTaskId);
+		mLongRunningTaskId = 0;
+	}
 	for(UdpMirrorClientList::iterator udpMirror = mUdpMirrorClients.begin(); udpMirror != mUdpMirrorClients.end(); udpMirror++) {
 		udpMirror->stop();
 	}
 	if(mTunnelClient) delete mTunnelClient;
+	sal_set_tunnel(mCore->sal,NULL);
 	linphone_core_remove_listener(mCore, mVTable);
 	linphone_core_v_table_destroy(mVTable);
 }
 
 void TunnelManager::doRegistration(){
 	LinphoneProxyConfig* lProxy;
-	linphone_core_get_default_proxy(mCore, &lProxy);
+	lProxy = linphone_core_get_default_proxy_config(mCore);
 	if (lProxy) {
 		ms_message("TunnelManager: New registration");
 		lProxy->commit = TRUE;
@@ -171,7 +193,7 @@ void TunnelManager::doRegistration(){
 
 void TunnelManager::doUnregistration() {
 	LinphoneProxyConfig *lProxy;
-	linphone_core_get_default_proxy(mCore, &lProxy);
+	lProxy = linphone_core_get_default_proxy_config(mCore);
 	if(lProxy) {
 		_linphone_proxy_config_unregister(lProxy);
 	}
@@ -188,7 +210,7 @@ void TunnelManager::processTunnelEvent(const Event &ev){
 				_linphone_core_apply_transports(mCore);
 				doRegistration();
 			}
-			
+
 		}
 	} else {
 		ms_error("TunnelManager: tunnel has been disconnected");
@@ -247,6 +269,10 @@ void TunnelManager::setMode(LinphoneTunnelMode mode) {
 
 void TunnelManager::tunnelCallback(bool connected, TunnelManager *zis){
 	Event ev;
+	if (zis->mLongRunningTaskId > 0) {
+		sal_end_background_task(zis->mLongRunningTaskId);
+		zis->mLongRunningTaskId = 0;
+	}
 	ev.mType=TunnelEvent;
 	ev.mData.mConnected=connected;
 	zis->postEvent(ev);
@@ -319,20 +345,17 @@ LinphoneTunnelMode TunnelManager::getMode() const {
 
 void TunnelManager::processUdpMirrorEvent(const Event &ev){
 	if(mState != autodetecting) return;
-	if (ev.mData.mHaveUdp) {
-		ms_message("TunnelManager: UDP mirror test succeed");
-		if(mTunnelClient) {
-			if(mTunnelizeSipPackets) doUnregistration();
-			delete mTunnelClient;
-			mTunnelClient = NULL;
-			if(mTunnelizeSipPackets) doRegistration();
+	if (mSimulateUdpLoss || !ev.mData.mHaveUdp) {
+		if (mSimulateUdpLoss) {
+			ms_message("TunnelManager: simulate UDP lost on %s:%d", mCurrentUdpMirrorClient->getServerAddress().mAddr.c_str(), mCurrentUdpMirrorClient->getServerAddress().mPort);
+		} else {
+			ms_message("TunnelManager: UDP mirror test failed on %s:%d", mCurrentUdpMirrorClient->getServerAddress().mAddr.c_str(), mCurrentUdpMirrorClient->getServerAddress().mPort);
 		}
-		mState = disabled;
-	} else {
-		ms_message("TunnelManager: UDP mirror test failed");
 		mCurrentUdpMirrorClient++;
 		if (mCurrentUdpMirrorClient !=mUdpMirrorClients.end()) {
-			ms_message("TunnelManager: trying another UDP mirror");
+			ms_message("TunnelManager: trying another UDP mirror on %s:%d", mCurrentUdpMirrorClient->getServerAddress().mAddr.c_str(), mCurrentUdpMirrorClient->getServerAddress().mPort);
+			if (mLongRunningTaskId == 0)
+				mLongRunningTaskId = sal_begin_background_task("Tunnel auto detect", NULL, NULL);
 			UdpMirrorClient &lUdpMirrorClient=*mCurrentUdpMirrorClient;
 			lUdpMirrorClient.start(TunnelManager::sUdpMirrorClientCallback,(void*)this);
 		} else {
@@ -344,6 +367,16 @@ void TunnelManager::processUdpMirrorEvent(const Event &ev){
 				mState = ready;
 			}
 		}
+	} else {
+		ms_message("TunnelManager: UDP mirror test succeed on %s:%d", mCurrentUdpMirrorClient->getServerAddress().mAddr.c_str(), mCurrentUdpMirrorClient->getServerAddress().mPort);
+		if(mTunnelClient) {
+			if(mTunnelizeSipPackets) doUnregistration();
+			sal_set_tunnel(mCore->sal,NULL);
+			delete mTunnelClient;
+			mTunnelClient = NULL;
+			if(mTunnelizeSipPackets) doRegistration();
+		}
+		mState = disabled;
 	}
 }
 
@@ -364,8 +397,15 @@ void TunnelManager::sUdpMirrorClientCallback(bool isUdpAvailable, void* data) {
 void TunnelManager::networkReachableCb(LinphoneCore *lc, bool_t reachable) {
 	TunnelManager *tunnel = bcTunnel(linphone_core_get_tunnel(lc));
 	if(reachable && tunnel->getMode() == LinphoneTunnelModeAuto && tunnel->mState != connecting && tunnel->mState != autodetecting) {
+		LOGI("TunnelManager: Network is now reachable, starting auto detection");
 		tunnel->startAutoDetection();
 		tunnel->mState = autodetecting;
+	} else if (!reachable && tunnel->mState == autodetecting) {
+		// if network is no more reachable, cancel autodetection
+		for(UdpMirrorClientList::iterator udpMirror = tunnel->mUdpMirrorClients.begin(); udpMirror != tunnel->mUdpMirrorClients.end(); udpMirror++) {
+			udpMirror->stop();
+		}
+		tunnel->mState = disabled;
 	}
 	linphone_core_get_local_ip_for(AF_INET, NULL,tunnel->mLocalAddr);
 }
@@ -377,6 +417,8 @@ bool TunnelManager::startAutoDetection() {
 	}
 	ms_message("TunnelManager: Starting auto-detection");
 	mCurrentUdpMirrorClient = mUdpMirrorClients.begin();
+	if (mLongRunningTaskId == 0)
+		 mLongRunningTaskId = sal_begin_background_task("Tunnel auto detect", NULL, NULL);
 	UdpMirrorClient &lUdpMirrorClient=*mCurrentUdpMirrorClient;
 	lUdpMirrorClient.start(TunnelManager::sUdpMirrorClientCallback,(void*)this);
 	return true;
@@ -408,6 +450,14 @@ bool TunnelManager::tunnelizeSipPacketsEnabled() const {
 	return mTunnelizeSipPackets;
 }
 
+void TunnelManager::verifyServerCertificate(bool enable){
+	mVerifyServerCertificate = enable;
+}
+
+bool TunnelManager::verifyServerCertificateEnabled() const {
+	return mVerifyServerCertificate;
+}
+
 void TunnelManager::setHttpProxy(const char *host,int port, const char *username, const char *passwd){
 	mHttpUserName=username?username:"";
 	mHttpPasswd=passwd?passwd:"";
@@ -419,3 +469,7 @@ void TunnelManager::setHttpProxy(const char *host,int port, const char *username
 LinphoneCore *TunnelManager::getLinphoneCore() const{
 	return mCore;
 }
+
+void TunnelManager::simulateUdpLoss(bool enabled) {
+	mSimulateUdpLoss = enabled;
+}
