@@ -130,14 +130,14 @@ void linphone_account_creator_set_user_data(LinphoneAccountCreator *creator, voi
 }
 
 static LinphoneAccountCreatorStatus validate_uri(const char* username, const char* domain, const char* route, const char* display_name) {
-	LinphoneProxyConfig* proxy = linphone_proxy_config_new();
 	LinphoneAddress* addr;
-
-	linphone_proxy_config_set_identity(proxy, "sip:user@domain.com");
+	LinphoneAccountCreatorStatus status = LinphoneAccountCreatorOK;
+	LinphoneProxyConfig* proxy = linphone_proxy_config_new();
+	linphone_proxy_config_set_identity(proxy, "sip:userame@domain.com");
 
 	if (route && linphone_proxy_config_set_route(proxy, route) != 0) {
-		linphone_proxy_config_destroy(proxy);
-		return LinphoneAccountCreatorRouteInvalid;
+		status = LinphoneAccountCreatorRouteInvalid;
+		goto end;
 	}
 
 	if (username) {
@@ -145,24 +145,43 @@ static LinphoneAccountCreatorStatus validate_uri(const char* username, const cha
 	} else {
 		addr = linphone_address_clone(linphone_proxy_config_get_identity_address(proxy));
 	}
-	linphone_proxy_config_destroy(proxy);
 
 	if (addr == NULL) {
-		return LinphoneAccountCreatorUsernameInvalid;
+		status = LinphoneAccountCreatorUsernameInvalid;
+		goto end;
 	}
 
-	if (domain) {
-		ms_error("TODO: detect invalid domain");
-		linphone_address_set_domain(addr, domain);
+	if (domain && linphone_address_set_domain(addr, domain) != 0) {
+		status = LinphoneAccountCreatorDomainInvalid;
 	}
 
-	if (display_name) {
-		ms_error("TODO: detect invalid display name");
-		linphone_address_set_display_name(addr, display_name);
+	if (display_name && (!strlen(display_name) || linphone_address_set_display_name(addr, display_name) != 0)) {
+		status = LinphoneAccountCreatorDisplayNameInvalid;
 	}
-
 	linphone_address_unref(addr);
-	return LinphoneAccountCreatorOk;
+end:
+	linphone_proxy_config_destroy(proxy);
+	return status;
+}
+
+static char* _get_identity(const LinphoneAccountCreator *creator) {
+	char *identity = NULL;
+	if (creator->username && creator->domain) {
+		//we must escape username
+		LinphoneProxyConfig* proxy = linphone_proxy_config_new();
+		LinphoneAddress* addr;
+		// creator->domain may contain some port or some transport (eg. toto.org:443;transport=tcp),
+		// we will accept that
+		char * tmpidentity = ms_strdup_printf("sip:username@%s", creator->domain);
+		linphone_proxy_config_set_identity(proxy, tmpidentity);
+		ms_free(tmpidentity);
+		addr = linphone_proxy_config_normalize_sip_uri(proxy, creator->username);
+
+		identity = linphone_address_as_string(addr);
+		linphone_address_destroy(addr);
+		linphone_proxy_config_destroy(proxy);
+	}
+	return identity;
 }
 
 static bool_t is_matching_regex(const char *entry, const char* regex) {
@@ -185,24 +204,29 @@ static bool_t is_matching_regex(const char *entry, const char* regex) {
 }
 
 LinphoneAccountCreatorStatus linphone_account_creator_set_username(LinphoneAccountCreator *creator, const char *username) {
-	int min_length = lp_config_get_int(creator->core->config, "assistant", "username_min_length", 0);
-	int fixed_length = lp_config_get_int(creator->core->config, "assistant", "username_length", 0);
+	int min_length = lp_config_get_int(creator->core->config, "assistant", "username_min_length", -1);
+	int fixed_length = lp_config_get_int(creator->core->config, "assistant", "username_length", -1);
+	int max_length = lp_config_get_int(creator->core->config, "assistant", "username_max_length", -1);
 	bool_t use_phone_number = lp_config_get_int(creator->core->config, "assistant", "use_phone_number", 0);
 	const char* regex = lp_config_get_string(creator->core->config, "assistant", "username_regex", 0);
 	LinphoneAccountCreatorStatus status;
-	if (min_length > 0 && strlen(username) < min_length) {
+	if (min_length > 0 && strlen(username) < (size_t)min_length) {
 		return LinphoneAccountCreatorUsernameTooShort;
-	} else if (fixed_length > 0 && strlen(username) != fixed_length) {
+	} else if (max_length > 0 && strlen(username) > (size_t)max_length) {
+		return LinphoneAccountCreatorUsernameTooLong;
+	} else if (fixed_length > 0 && strlen(username) != (size_t)fixed_length) {
 		return LinphoneAccountCreatorUsernameInvalidSize;
 	} else if (use_phone_number && !linphone_proxy_config_is_phone_number(NULL, username)) {
 		return LinphoneAccountCreatorUsernameInvalid;
 	} else if (regex && !is_matching_regex(username, regex)) {
 		return LinphoneAccountCreatorUsernameInvalid;
-	} else if ((status = validate_uri(username, NULL, NULL, NULL)) != LinphoneAccountCreatorOk) {
+	} else if ((status = validate_uri(username, NULL, NULL, NULL)) != LinphoneAccountCreatorOK) {
 		return status;
 	}
+
 	set_string(&creator->username, username, TRUE);
-	return LinphoneAccountCreatorOk;
+
+	return LinphoneAccountCreatorOK;
 }
 
 const char * linphone_account_creator_get_username(const LinphoneAccountCreator *creator) {
@@ -210,12 +234,15 @@ const char * linphone_account_creator_get_username(const LinphoneAccountCreator
 }
 
 LinphoneAccountCreatorStatus linphone_account_creator_set_password(LinphoneAccountCreator *creator, const char *password){
-	int min_length = lp_config_get_int(creator->core->config, "assistant", "password_min_length", 0);
-	if (min_length > 0 && strlen(password) < min_length) {
+	int min_length = lp_config_get_int(creator->core->config, "assistant", "password_min_length", -1);
+	int max_length = lp_config_get_int(creator->core->config, "assistant", "password_max_length", -1);
+	if (min_length > 0 && strlen(password) < (size_t)min_length) {
 		return LinphoneAccountCreatorPasswordTooShort;
+	} else if (max_length > 0 && strlen(password) > (size_t)max_length) {
+		return LinphoneAccountCreatorPasswordTooLong;
 	}
 	set_string(&creator->password, password, FALSE);
-	return LinphoneAccountCreatorOk;
+	return LinphoneAccountCreatorOK;
 }
 
 const char * linphone_account_creator_get_password(const LinphoneAccountCreator *creator) {
@@ -227,7 +254,7 @@ LinphoneAccountCreatorStatus linphone_account_creator_set_transport(LinphoneAcco
 		return LinphoneAccountCreatorTransportNotSupported;
 	}
 	creator->transport = transport;
-	return LinphoneAccountCreatorOk;
+	return LinphoneAccountCreatorOK;
 }
 
 LinphoneTransportType linphone_account_creator_get_transport(const LinphoneAccountCreator *creator) {
@@ -239,7 +266,7 @@ LinphoneAccountCreatorStatus linphone_account_creator_set_domain(LinphoneAccount
 		return LinphoneAccountCreatorDomainInvalid;
 	}
 	set_string(&creator->domain, domain, TRUE);
-	return LinphoneAccountCreatorOk;
+	return LinphoneAccountCreatorOK;
 }
 
 const char * linphone_account_creator_get_domain(const LinphoneAccountCreator *creator) {
@@ -251,7 +278,7 @@ LinphoneAccountCreatorStatus linphone_account_creator_set_route(LinphoneAccountC
 		return LinphoneAccountCreatorRouteInvalid;
 	}
 	set_string(&creator->route, route, TRUE);
-	return LinphoneAccountCreatorOk;
+	return LinphoneAccountCreatorOK;
 }
 
 const char * linphone_account_creator_get_route(const LinphoneAccountCreator *creator) {
@@ -263,7 +290,7 @@ LinphoneAccountCreatorStatus linphone_account_creator_set_display_name(LinphoneA
 		return LinphoneAccountCreatorDisplayNameInvalid;
 	}
 	set_string(&creator->display_name, display_name, FALSE);
-	return LinphoneAccountCreatorOk;
+	return LinphoneAccountCreatorOK;
 }
 
 const char * linphone_account_creator_get_display_name(const LinphoneAccountCreator *creator) {
@@ -275,7 +302,7 @@ LinphoneAccountCreatorStatus linphone_account_creator_set_email(LinphoneAccountC
 		return LinphoneAccountCreatorEmailInvalid;
 	}
 	set_string(&creator->email, email, TRUE);
-	return LinphoneAccountCreatorOk;
+	return LinphoneAccountCreatorOK;
 }
 
 const char * linphone_account_creator_get_email(const LinphoneAccountCreator *creator) {
@@ -297,10 +324,10 @@ LinphoneAccountCreatorCbs * linphone_account_creator_get_callbacks(const Linphon
 static void _test_existence_cb(LinphoneXmlRpcRequest *request) {
 	LinphoneAccountCreator *creator = (LinphoneAccountCreator *)linphone_xml_rpc_request_get_user_data(request);
 	if (creator->callbacks->existence_tested != NULL) {
-		LinphoneAccountCreatorStatus status = LinphoneAccountCreatorFailed;
-		if ((linphone_xml_rpc_request_get_status(request) == LinphoneXmlRpcStatusOk)
-			&& (linphone_xml_rpc_request_get_int_response(request) == 0)) {
-			status = LinphoneAccountCreatorOk;
+		LinphoneAccountCreatorStatus status = LinphoneAccountCreatorReqFailed;
+		if (linphone_xml_rpc_request_get_status(request) == LinphoneXmlRpcStatusOk) {
+			int resp = linphone_xml_rpc_request_get_int_response(request);
+			status = (resp == 0) ? LinphoneAccountCreatorAccountNotExist : LinphoneAccountCreatorAccountExist;
 		}
 		creator->callbacks->existence_tested(creator, status);
 	}
@@ -308,15 +335,14 @@ static void _test_existence_cb(LinphoneXmlRpcRequest *request) {
 
 LinphoneAccountCreatorStatus linphone_account_creator_test_existence(LinphoneAccountCreator *creator) {
 	LinphoneXmlRpcRequest *request;
-	char *identity;
-
-	if (!creator->username || !creator->domain) {
+	char *identity = _get_identity(creator);
+	if (!identity) {
 		if (creator->callbacks->existence_tested != NULL) {
-			creator->callbacks->existence_tested(creator, LinphoneAccountCreatorFailed);
+			creator->callbacks->existence_tested(creator, LinphoneAccountCreatorReqFailed);
 		}
-		return LinphoneAccountCreatorFailed;
+		return LinphoneAccountCreatorReqFailed;
 	}
-	identity = ms_strdup_printf("%s@%s", creator->username, creator->domain);
+
 	request = linphone_xml_rpc_request_new_with_args("check_account", LinphoneXmlRpcArgInt,
 		LinphoneXmlRpcArgString, identity,
 		LinphoneXmlRpcArgNone);
@@ -325,16 +351,16 @@ LinphoneAccountCreatorStatus linphone_account_creator_test_existence(LinphoneAcc
 	linphone_xml_rpc_session_send_request(creator->xmlrpc_session, request);
 	linphone_xml_rpc_request_unref(request);
 	ms_free(identity);
-	return LinphoneAccountCreatorOk;
+	return LinphoneAccountCreatorOK;
 }
 
 static void _test_validation_cb(LinphoneXmlRpcRequest *request) {
 	LinphoneAccountCreator *creator = (LinphoneAccountCreator *)linphone_xml_rpc_request_get_user_data(request);
 	if (creator->callbacks->validation_tested != NULL) {
-		LinphoneAccountCreatorStatus status = LinphoneAccountCreatorFailed;
-		if ((linphone_xml_rpc_request_get_status(request) == LinphoneXmlRpcStatusOk)
-			&& (linphone_xml_rpc_request_get_int_response(request) == 1)) {
-			status = LinphoneAccountCreatorOk;
+		LinphoneAccountCreatorStatus status = LinphoneAccountCreatorReqFailed;
+		if (linphone_xml_rpc_request_get_status(request) == LinphoneXmlRpcStatusOk) {
+			int resp = linphone_xml_rpc_request_get_int_response(request);
+			status = (resp == 0) ? LinphoneAccountCreatorAccountNotValidated : LinphoneAccountCreatorAccountValidated;
 		}
 		creator->callbacks->validation_tested(creator, status);
 	}
@@ -342,16 +368,13 @@ static void _test_validation_cb(LinphoneXmlRpcRequest *request) {
 
 LinphoneAccountCreatorStatus linphone_account_creator_test_validation(LinphoneAccountCreator *creator) {
 	LinphoneXmlRpcRequest *request;
-	char *identity;
-
-	if (!creator->username || !creator->domain) {
+	char *identity = _get_identity(creator);
+	if (!identity) {
 		if (creator->callbacks->validation_tested != NULL) {
-			creator->callbacks->validation_tested(creator, LinphoneAccountCreatorFailed);
+			creator->callbacks->validation_tested(creator, LinphoneAccountCreatorReqFailed);
 		}
-		return LinphoneAccountCreatorFailed;
+		return LinphoneAccountCreatorReqFailed;
 	}
-
-	identity = ms_strdup_printf("%s@%s", creator->username, creator->domain);
 	request = linphone_xml_rpc_request_new_with_args("check_account_validated", LinphoneXmlRpcArgInt,
 		LinphoneXmlRpcArgString, identity,
 		LinphoneXmlRpcArgNone);
@@ -360,16 +383,16 @@ LinphoneAccountCreatorStatus linphone_account_creator_test_validation(LinphoneAc
 	linphone_xml_rpc_session_send_request(creator->xmlrpc_session, request);
 	linphone_xml_rpc_request_unref(request);
 	ms_free(identity);
-	return LinphoneAccountCreatorOk;
+	return LinphoneAccountCreatorOK;
 }
 
 static void _create_account_cb(LinphoneXmlRpcRequest *request) {
 	LinphoneAccountCreator *creator = (LinphoneAccountCreator *)linphone_xml_rpc_request_get_user_data(request);
 	if (creator->callbacks->create_account != NULL) {
-		LinphoneAccountCreatorStatus status = LinphoneAccountCreatorFailed;
-		if ((linphone_xml_rpc_request_get_status(request) == LinphoneXmlRpcStatusOk)
-			&& (linphone_xml_rpc_request_get_int_response(request) == 0)) {
-			status = LinphoneAccountCreatorOk;
+		LinphoneAccountCreatorStatus status = LinphoneAccountCreatorReqFailed;
+		if (linphone_xml_rpc_request_get_status(request) == LinphoneXmlRpcStatusOk) {
+			int resp = linphone_xml_rpc_request_get_int_response(request);
+			status = (resp == 0) ? LinphoneAccountCreatorAccountCreated : LinphoneAccountCreatorAccountNotCreated;
 		}
 		creator->callbacks->create_account(creator, status);
 	}
@@ -377,16 +400,15 @@ static void _create_account_cb(LinphoneXmlRpcRequest *request) {
 
 LinphoneAccountCreatorStatus linphone_account_creator_create_account(LinphoneAccountCreator *creator) {
 	LinphoneXmlRpcRequest *request;
-	char *identity;
-
-	if (!creator->username || !creator->domain) {
+	char *identity = _get_identity(creator);
+	if (!identity || !creator->email) {
 		if (creator->callbacks->create_account != NULL) {
-			creator->callbacks->create_account(creator, LinphoneAccountCreatorFailed);
+			creator->callbacks->create_account(creator, LinphoneAccountCreatorReqFailed);
 		}
-		return LinphoneAccountCreatorFailed;
+		if (identity) ms_free(identity);
+		return LinphoneAccountCreatorReqFailed;
 	}
 
-	identity = ms_strdup_printf("%s@%s", creator->username, creator->domain);
 	request = linphone_xml_rpc_request_new_with_args("create_account", LinphoneXmlRpcArgInt,
 		LinphoneXmlRpcArgString, identity,
 		LinphoneXmlRpcArgString, creator->password,
@@ -398,24 +420,31 @@ LinphoneAccountCreatorStatus linphone_account_creator_create_account(LinphoneAcc
 	linphone_xml_rpc_session_send_request(creator->xmlrpc_session, request);
 	linphone_xml_rpc_request_unref(request);
 	ms_free(identity);
-	return LinphoneAccountCreatorOk;
+	return LinphoneAccountCreatorOK;
 }
 
 LinphoneProxyConfig * linphone_account_creator_configure(const LinphoneAccountCreator *creator) {
 	LinphoneAuthInfo *info;
 	LinphoneProxyConfig *cfg = linphone_core_create_proxy_config(creator->core);
-	char *identity_str = ms_strdup_printf("sip:%s@%s", creator->username, creator->domain);
-	LinphoneAddress *identity = linphone_address_new(identity_str);
+	char *identity_str = _get_identity(creator);
+ 	LinphoneAddress *identity = linphone_address_new(identity_str);
+	char *route = NULL;
+	char *domain = NULL;
+	ms_free(identity_str);
 	if (creator->display_name) {
 		linphone_address_set_display_name(identity, creator->display_name);
 	}
-
-	linphone_proxy_config_set_identity(cfg, linphone_address_as_string(identity));
-	linphone_proxy_config_set_server_addr(cfg, creator->domain);
-	linphone_proxy_config_set_route(cfg, creator->route);
+	if (creator->route) {
+		route = ms_strdup_printf("%s;transport=%s", creator->route, linphone_transport_to_string(creator->transport));
+	}
+	if (creator->domain) {
+		domain = ms_strdup_printf("%s;transport=%s", creator->domain, linphone_transport_to_string(creator->transport));
+	}
+	linphone_proxy_config_set_identity_address(cfg, identity);
+	linphone_proxy_config_set_server_addr(cfg, domain);
+	linphone_proxy_config_set_route(cfg, route);
 	linphone_proxy_config_enable_publish(cfg, FALSE);
 	linphone_proxy_config_enable_register(cfg, TRUE);
-	ms_free(identity_str);
 
 	if (strcmp(creator->domain, "sip.linphone.org") == 0) {
 		linphone_proxy_config_enable_avpf(cfg, TRUE);
@@ -426,7 +455,7 @@ LinphoneProxyConfig * linphone_account_creator_configure(const LinphoneAccountCr
 			linphone_address_set_transport(addr, LinphoneTransportTls);
 			tmp = linphone_address_as_string(addr);
 			linphone_proxy_config_set_server_addr(cfg, tmp);
-			linphone_proxy_config_set_route(cfg, tmp);
+			linphone_proxy_config_set_route(cfg, "");
 			ms_free(tmp);
 			linphone_address_destroy(addr);
 		}
