@@ -89,37 +89,42 @@ const char *linphone_address_get_domain(const LinphoneAddress *u){
 /**
  * Sets the display name.
 **/
-void linphone_address_set_display_name(LinphoneAddress *u, const char *display_name){
+int linphone_address_set_display_name(LinphoneAddress *u, const char *display_name){
 	sal_address_set_display_name(u,display_name);
+	return 0;
 }
 
 /**
  * Sets the username.
 **/
-void linphone_address_set_username(LinphoneAddress *uri, const char *username){
+int linphone_address_set_username(LinphoneAddress *uri, const char *username){
 	sal_address_set_username(uri,username);
+	return 0;
 }
 
 /**
  * Sets the domain.
 **/
-void linphone_address_set_domain(LinphoneAddress *uri, const char *host){
+int linphone_address_set_domain(LinphoneAddress *uri, const char *host){
 	sal_address_set_domain(uri,host);
+	return 0;
 }
 
 
 /**
  * Sets the port number.
 **/
-void linphone_address_set_port(LinphoneAddress *uri, int port){
+int linphone_address_set_port(LinphoneAddress *uri, int port){
 	sal_address_set_port(uri,port);
+	return 0;
 }
 
 /**
  * Set a transport.
 **/
-void linphone_address_set_transport(LinphoneAddress *uri, LinphoneTransportType tp){
+int linphone_address_set_transport(LinphoneAddress *uri, LinphoneTransportType tp){
 	sal_address_set_transport(uri,(SalTransport)tp);
+	return 0;
 }
 
 /**
@@ -129,6 +134,20 @@ LinphoneTransportType linphone_address_get_transport(const LinphoneAddress *uri)
 	return (LinphoneTransportType)sal_address_get_transport(uri);
 }
 
+/**
+ * Set the value of the method parameter
+**/
+void linphone_address_set_method_param(LinphoneAddress *addr, const char *method) {
+	sal_address_set_method_param(addr, method);
+}
+
+/**
+ * Get the value of the method parameter
+**/
+const char *linphone_address_get_method_param(const LinphoneAddress *addr) {
+	return sal_address_get_method_param(addr);
+}
+
 /**
  * Removes address's tags and uri headers so that it is displayable to the user.
 **/
@@ -169,7 +188,8 @@ bool_t linphone_address_get_secure(const LinphoneAddress *uri){
 
 /**
  * Make the address refer to a secure location (sips scheme)
- * @param enabled TRUE if address is requested to be secure.
+ * @param[in] addr A #LinphoneAddress object
+ * @param[in] enabled TRUE if address is requested to be secure.
 **/
 void linphone_address_set_secure(LinphoneAddress *addr, bool_t enabled){
 	sal_address_set_secure(addr, enabled);
