@@ -114,6 +114,15 @@ const char* sal_address_get_transport_name(const SalAddress* addr){
 	return NULL;
 }
 
+const char *sal_address_get_method_param(const SalAddress *addr) {
+	belle_sip_header_address_t* header_addr = BELLE_SIP_HEADER_ADDRESS(addr);
+	belle_sip_uri_t* uri = belle_sip_header_address_get_uri(header_addr);
+	if (uri) {
+		return belle_sip_uri_get_method_param(uri);
+	}
+	return NULL;
+}
+
 void sal_address_set_display_name(SalAddress *addr, const char *display_name){
 	belle_sip_header_address_t* header_addr = BELLE_SIP_HEADER_ADDRESS(addr);
 	belle_sip_header_address_set_displayname(header_addr,display_name);
@@ -199,6 +208,11 @@ void sal_address_set_uri_params(SalAddress *addr, const char *params){
 	belle_sip_parameters_set(parameters,params);
 }
 
+bool_t sal_address_has_uri_param(SalAddress *addr, const char *name){
+	belle_sip_parameters_t* parameters = BELLE_SIP_PARAMETERS(belle_sip_header_address_get_uri(BELLE_SIP_HEADER_ADDRESS(addr)));
+	return belle_sip_parameters_has_parameter(parameters, name);
+}
+
 void sal_address_set_header(SalAddress *addr, const char *header_name, const char *header_value){
 	belle_sip_uri_set_header(belle_sip_header_address_get_uri(BELLE_SIP_HEADER_ADDRESS(addr)),header_name, header_value);
 }
@@ -213,6 +227,10 @@ void sal_address_set_transport_name(SalAddress* addr,const char *transport){
 	SAL_ADDRESS_SET(addr,transport_param,transport);
 }
 
+void sal_address_set_method_param(SalAddress *addr, const char *method) {
+	SAL_ADDRESS_SET(addr, method_param, method);
+}
+
 SalAddress *sal_address_ref(SalAddress *addr){
 	return (SalAddress*)belle_sip_object_ref(BELLE_SIP_HEADER_ADDRESS(addr));
 }
