@@ -32,8 +32,9 @@ static void call_set_released(SalOp* op){
 	}
 }
 
-static void call_set_error(SalOp* op,belle_sip_response_t* response){
+static void call_set_error(SalOp* op,belle_sip_response_t* response, bool_t fatal){
 	sal_op_set_error_info_from_response(op,response);
+	if (fatal) op->state = SalOpStateTerminating;
 	op->base.root->callbacks.call_failure(op);
 }
 static void set_addr_to_0000(char value[]) {
@@ -56,20 +57,21 @@ static void sdp_process(SalOp *h){
 
 	h->result=sal_media_description_new();
 	if (h->sdp_offering){
-		offer_answer_initiate_outgoing(h->base.local_media,h->base.remote_media,h->result);
+		offer_answer_initiate_outgoing(h->base.root->factory, h->base.local_media,h->base.remote_media,h->result);
 	}else{
 		int i;
 		if (h->sdp_answer){
 			belle_sip_object_unref(h->sdp_answer);
 		}
-		offer_answer_initiate_incoming(h->base.local_media,h->base.remote_media,h->result,h->base.root->one_matching_codec);
+		offer_answer_initiate_incoming(h->base.root->factory, h->base.local_media,h->base.remote_media,h->result,h->base.root->one_matching_codec);
 		/*for backward compatibility purpose*/
 		if(h->cnx_ip_to_0000_if_sendonly_enabled && sal_media_description_has_dir(h->result,SalStreamSendOnly)) {
 			set_addr_to_0000(h->result->addr);
 			for(i=0;i<SAL_MEDIA_DESCRIPTION_MAX_STREAMS;++i){
-				if (h->result->streams[i].dir == SalStreamSendOnly)
-						set_addr_to_0000(h->result->streams[i].rtp_addr);
-						set_addr_to_0000(h->result->streams[i].rtcp_addr);
+				if (h->result->streams[i].dir == SalStreamSendOnly) {
+					set_addr_to_0000(h->result->streams[i].rtp_addr);
+					set_addr_to_0000(h->result->streams[i].rtcp_addr);
+				}
 			}
 		}
 		h->sdp_answer=(belle_sdp_session_description_t *)belle_sip_object_ref(media_description_to_sdp(h->result));
@@ -149,6 +151,7 @@ static void call_process_io_error(void *user_ctx, const belle_sip_io_error_event
 		/*call terminated very early*/
 		sal_error_info_set(&op->error_info,SalReasonIOError,503,"IO error",NULL);
 		op->base.root->callbacks.call_failure(op);
+		op->state = SalOpStateTerminating;
 		call_set_released(op);
 	} else {
 		/*dialog will terminated shortly, nothing to do*/
@@ -162,6 +165,14 @@ static void process_dialog_terminated(void *ctx, const belle_sip_dialog_terminat
 		ms_message("Dialog [%p] terminated for op [%p]",belle_sip_dialog_terminated_event_get_dialog(event),op);
 
 		switch(belle_sip_dialog_get_previous_state(op->dialog)) {
+			case BELLE_SIP_DIALOG_EARLY:
+			case BELLE_SIP_DIALOG_NULL:
+				if (op->state!=SalOpStateTerminated && op->state!=SalOpStateTerminating) {
+					/*this is an early termination due to incorrect response received*/
+					op->base.root->callbacks.call_failure(op);
+					op->state=SalOpStateTerminating;
+				}
+			break;
 			case BELLE_SIP_DIALOG_CONFIRMED:
 				if (op->state!=SalOpStateTerminated && op->state!=SalOpStateTerminating) {
 					/*this is probably a normal termination from a BYE*/
@@ -201,7 +212,23 @@ static void cancelling_invite(SalOp* op ){
 	belle_sip_request_t* cancel;
 	ms_message("Cancelling INVITE request from [%s] to [%s] ",sal_op_get_from(op), sal_op_get_to(op));
 	cancel = belle_sip_client_transaction_create_cancel(op->pending_client_trans);
-	sal_op_send_request(op,cancel);
+	if (cancel){
+		sal_op_send_request(op,cancel);
+	}else if (op->dialog){
+		belle_sip_dialog_state_t state = belle_sip_dialog_get_state(op->dialog);;
+		/*case where the response received is invalid (could not establish a dialog), but the transaction is not cancellable 
+		 * because already terminated*/
+		switch(state){
+			case BELLE_SIP_DIALOG_EARLY:
+			case BELLE_SIP_DIALOG_NULL:
+				/*force kill the dialog*/
+				ms_warning("op [%p]: force kill of dialog [%p]", op, op->dialog);
+				belle_sip_dialog_delete(op->dialog);
+			break;
+			default:
+			break;
+		}
+	}
 	op->state=SalOpStateTerminating;
 }
 
@@ -263,7 +290,7 @@ static void call_process_response(void *op_base, const belle_sip_response_event_
 					}
 					belle_sip_object_data_set(BELLE_SIP_OBJECT(dialog),"early_response",belle_sip_object_ref(response),belle_sip_object_unref);
 				} else if (code>=300){
-					call_set_error(op,response);
+					call_set_error(op, response, TRUE);
 					if (op->dialog==NULL) call_set_released(op);
 				}
 			} else if (code >=200
@@ -282,7 +309,7 @@ static void call_process_response(void *op_base, const belle_sip_response_event_
 						if (code >=200 && code<300) {
 							handle_sdp_from_response(op,response);
 							ack=belle_sip_dialog_create_ack(op->dialog,belle_sip_dialog_get_local_seq_number(op->dialog));
-							if (ack==NULL) {
+							if (ack == NULL) {
 								ms_error("This call has been already terminated.");
 								return ;
 							}
@@ -291,11 +318,12 @@ static void call_process_response(void *op_base, const belle_sip_response_event_
 								belle_sip_object_unref(op->sdp_answer);
 								op->sdp_answer=NULL;
 							}
+							belle_sip_message_add_header(BELLE_SIP_MESSAGE(ack),BELLE_SIP_HEADER(op->base.root->user_agent));
 							belle_sip_dialog_send_ack(op->dialog,ack);
 							op->base.root->callbacks.call_accepted(op); /*INVITE*/
 							op->state=SalOpStateActive;
 						}else if (code >= 300){
-							call_set_error(op,response);
+							call_set_error(op,response, FALSE);
 						}
 					}else if (strcmp("INFO",method)==0){
 						if (code == 491
@@ -324,7 +352,7 @@ static void call_process_response(void *op_base, const belle_sip_response_event_
 		break;
 		case BELLE_SIP_DIALOG_TERMINATED: {
 			if (strcmp("INVITE",method)==0 && code >= 300){
-				call_set_error(op,response);
+				call_set_error(op,response, TRUE);
 			}
 		}
 		break;
@@ -345,6 +373,7 @@ static void call_process_timeout(void *user_ctx, const belle_sip_timeout_event_t
 		/*call terminated very early*/
 		sal_error_info_set(&op->error_info,SalReasonRequestTimeout,408,"Request timeout",NULL);
 		op->base.root->callbacks.call_failure(op);
+		op->state = SalOpStateTerminating;
 		call_set_released(op);
 	} else {
 		/*dialog will terminated shortly, nothing to do*/
@@ -378,6 +407,7 @@ static void call_process_transaction_terminated(void *user_ctx, const belle_sip_
 	}else if (op->state == SalOpStateEarly && code < 200){
 		/*call terminated early*/
 		sal_error_info_set(&op->error_info,SalReasonIOError,503,"I/O error",NULL);
+		op->state = SalOpStateTerminating;
 		op->base.root->callbacks.call_failure(op);
 		release_call=TRUE;
 	}
@@ -396,6 +426,7 @@ static void call_process_transaction_terminated(void *user_ctx, const belle_sip_
 
 static void call_terminated(SalOp* op,belle_sip_server_transaction_t* server_transaction, belle_sip_request_t* request,int status_code) {
 	belle_sip_response_t* resp;
+	op->state = SalOpStateTerminating;
 	op->base.root->callbacks.call_terminated(op,op->dir==SalOpDirIncoming?sal_op_get_from(op):sal_op_get_to(op));
 	resp=sal_op_create_response_from_request(op,request,status_code);
 	belle_sip_server_transaction_send_response(server_transaction,resp);
@@ -640,15 +671,19 @@ static void process_request_event(void *op_base, const belle_sip_request_event_t
 
 				}
 			}else{
-				SalBody salbody;
-				if (sal_op_get_body(op,(belle_sip_message_t*)req,&salbody)) {
-					if (sal_body_has_type(&salbody,"application","dtmf-relay")){
+				belle_sip_message_t *msg = BELLE_SIP_MESSAGE(req);
+				belle_sip_body_handler_t *body_handler = BELLE_SIP_BODY_HANDLER(sal_op_get_body_handler(op, msg));
+				if (body_handler) {
+					belle_sip_header_content_type_t *content_type = belle_sip_message_get_header_by_type(msg, belle_sip_header_content_type_t);
+					if (content_type
+						&& (strcmp(belle_sip_header_content_type_get_type(content_type), "application") == 0)
+						&& (strcmp(belle_sip_header_content_type_get_subtype(content_type), "dtmf-relay") == 0)) {
 						char tmp[10];
-						if (sal_lines_get_value(salbody.data, "Signal",tmp, sizeof(tmp))){
+						if (sal_lines_get_value(belle_sip_message_get_body(msg), "Signal",tmp, sizeof(tmp))){
 							op->base.root->callbacks.dtmf_received(op,tmp[0]);
 						}
 					}else
-						op->base.root->callbacks.info_received(op,&salbody);
+						op->base.root->callbacks.info_received(op, (SalBodyHandler *)body_handler);
 				} else {
 					op->base.root->callbacks.info_received(op,NULL);
 				}
