@@ -1,3 +1,4 @@
+
 /*
 linphone
 Copyright (C) 2012  Belledonne Communications, Grenoble, France
@@ -69,7 +70,7 @@ static void add_ice_remote_candidates(belle_sdp_media_description_t *md, const S
 }
 
 static bool_t is_rtcp_fb_trr_int_the_same_for_all_payloads(const SalStreamDescription *stream, uint16_t *trr_int) {
-	MSList *pt_it;
+	bctbx_list_t *pt_it;
 	bool_t first = TRUE;
 	for (pt_it = stream->payloads; pt_it != NULL; pt_it = pt_it->next) {
 		PayloadType *pt = (PayloadType *)pt_it->data;
@@ -118,14 +119,14 @@ static void add_rtcp_fb_ccm_attribute(belle_sdp_media_description_t *media_desc,
 }
 
 static void add_rtcp_fb_attributes(belle_sdp_media_description_t *media_desc, const SalMediaDescription *md, const SalStreamDescription *stream) {
-	MSList *pt_it;
+	bctbx_list_t *pt_it;
 	PayloadType *pt;
 	PayloadTypeAvpfParams avpf_params;
 	bool_t general_trr_int;
 	uint16_t trr_int = 0;
 
 	general_trr_int = is_rtcp_fb_trr_int_the_same_for_all_payloads(stream, &trr_int);
-	if (general_trr_int == TRUE) {
+	if (general_trr_int == TRUE && trr_int != 0) {
 		add_rtcp_fb_trr_int_attribute(media_desc, -1, trr_int);
 	}
 	if (stream->rtcp_fb.generic_nack_enabled == TRUE) {
@@ -143,7 +144,7 @@ static void add_rtcp_fb_attributes(belle_sdp_media_description_t *media_desc, co
 		avpf_params = payload_type_get_avpf_params(pt);
 
 		/* Add trr-int if not set generally. */
-		if (general_trr_int != TRUE) {
+		if (general_trr_int != TRUE && trr_int != 0) {
 			add_rtcp_fb_trr_int_attribute(media_desc, payload_type_get_number(pt), avpf_params.trr_interval);
 		}
 
@@ -190,7 +191,7 @@ static void stream_description_to_sdp ( belle_sdp_session_description_t *session
 	belle_sdp_mime_parameter_t* mime_param;
 	belle_sdp_media_description_t* media_desc;
 	int j;
-	MSList* pt_it;
+	bctbx_list_t* pt_it;
 	PayloadType* pt;
 	char buffer[1024];
 	char* dir=NULL;
@@ -287,6 +288,11 @@ static void stream_description_to_sdp ( belle_sdp_session_description_t *session
 		ms_free(ssrc_attribute);
 	}
 
+	/* insert zrtp-hash attribute if needed */
+	if (stream->haveZrtpHash == 1) {
+		belle_sdp_media_description_add_attribute(media_desc, belle_sdp_attribute_create("zrtp-hash", (const char *)(stream->zrtphash)));
+	}
+
 	switch ( stream->dir ) {
 		case SalStreamSendRecv:
 			/*dir="sendrecv";*/
@@ -304,6 +310,10 @@ static void stream_description_to_sdp ( belle_sdp_session_description_t *session
 	}
 	if ( dir ) belle_sdp_media_description_add_attribute ( media_desc,belle_sdp_attribute_create ( dir,NULL ) );
 	
+	if (stream->rtcp_mux){
+		belle_sdp_media_description_add_attribute(media_desc, belle_sdp_attribute_create ("rtcp-mux",NULL ) );
+	}
+	
 	if (rtp_port != 0) {
 		different_rtp_and_rtcp_addr = (rtcp_addr[0] != '\0') && (strcmp(rtp_addr, rtcp_addr) != 0);
 		if ((rtcp_port != (rtp_port + 1)) || (different_rtp_and_rtcp_addr == TRUE)) {
@@ -315,7 +325,7 @@ static void stream_description_to_sdp ( belle_sdp_session_description_t *session
 			belle_sdp_media_description_add_attribute(media_desc,belle_sdp_attribute_create ("rtcp",buffer));
 		}
 	}
-	if (stream->ice_completed == TRUE) {
+	if (stream->set_nortpproxy == TRUE) {
 		belle_sdp_media_description_add_attribute(media_desc,belle_sdp_attribute_create ("nortpproxy","yes"));
 	}
 	if (stream->ice_mismatch == TRUE) {
@@ -331,7 +341,7 @@ static void stream_description_to_sdp ( belle_sdp_session_description_t *session
 		}
 	}
 
-	if ((rtp_port != 0) && sal_stream_description_has_avpf(stream)) {
+	if ((rtp_port != 0) && (sal_stream_description_has_avpf(stream) || sal_stream_description_has_implicit_avpf(stream))) {
 		add_rtcp_fb_attributes(media_desc, md, stream);
 	}
 
@@ -418,7 +428,7 @@ belle_sdp_session_description_t * media_description_to_sdp ( const SalMediaDescr
 		belle_sdp_session_description_set_bandwidth ( session_desc,"AS",desc->bandwidth );
 	}
 	
-	if (desc->ice_completed == TRUE) belle_sdp_session_description_add_attribute(session_desc, belle_sdp_attribute_create("nortpproxy","yes"));
+	if (desc->set_nortpproxy == TRUE) belle_sdp_session_description_add_attribute(session_desc, belle_sdp_attribute_create("nortpproxy","yes"));
 	if (desc->ice_pwd[0] != '\0') belle_sdp_session_description_add_attribute(session_desc, belle_sdp_attribute_create("ice-pwd",desc->ice_pwd));
 	if (desc->ice_ufrag[0] != '\0') belle_sdp_session_description_add_attribute(session_desc, belle_sdp_attribute_create("ice-ufrag",desc->ice_ufrag));
 
@@ -460,7 +470,7 @@ static void sdp_parse_payload_types(belle_sdp_media_description_t *media_desc, S
 		pt->channels=belle_sdp_mime_parameter_get_channel_count ( mime_param );
 		payload_type_set_send_fmtp ( pt,belle_sdp_mime_parameter_get_parameters ( mime_param ) );
 		payload_type_set_avpf_params(pt, avpf_params);
-		stream->payloads=ms_list_append ( stream->payloads,pt );
+		stream->payloads=bctbx_list_append ( stream->payloads,pt );
 		stream->ptime=belle_sdp_mime_parameter_get_ptime ( mime_param );
 		ms_message ( "Found payload %s/%i fmtp=%s",pt->mime_type,pt->clock_rate,
 						pt->send_fmtp ? pt->send_fmtp : "" );
@@ -567,7 +577,7 @@ static void sdp_parse_media_ice_parameters(belle_sdp_media_description_t *media_
 }
 
 static void enable_avpf_for_stream(SalStreamDescription *stream) {
-	MSList *pt_it;
+	bctbx_list_t *pt_it;
 	for (pt_it = stream->payloads; pt_it != NULL; pt_it = pt_it->next) {
 		PayloadType *pt = (PayloadType *)pt_it->data;
 		payload_type_set_flag(pt, PAYLOAD_TYPE_RTCP_FEEDBACK_ENABLED);
@@ -625,14 +635,15 @@ static void apply_rtcp_fb_attribute_to_payload(belle_sdp_rtcp_fb_attribute_t *fb
 	payload_type_set_avpf_params(pt, avpf_params);
 }
 
-static void sdp_parse_rtcp_fb_parameters(belle_sdp_media_description_t *media_desc, SalStreamDescription *stream) {
+static bool_t sdp_parse_rtcp_fb_parameters(belle_sdp_media_description_t *media_desc, SalStreamDescription *stream) {
 	belle_sip_list_t *it;
 	belle_sdp_attribute_t *attribute;
 	belle_sdp_rtcp_fb_attribute_t *fb_attribute;
-	MSList *pt_it;
+	bctbx_list_t *pt_it;
 	PayloadType *pt;
 	int8_t pt_num;
-
+	bool_t retval = FALSE;
+    
 	/* Handle rtcp-fb attributes that concern all payload types. */
 	for (it = belle_sdp_media_description_get_attributes(media_desc); it != NULL; it = it->next) {
 		attribute = BELLE_SDP_ATTRIBUTE(it->data);
@@ -642,6 +653,7 @@ static void sdp_parse_rtcp_fb_parameters(belle_sdp_media_description_t *media_de
 				for (pt_it = stream->payloads; pt_it != NULL; pt_it = pt_it->next) {
 					pt = (PayloadType *)pt_it->data;
 					apply_rtcp_fb_attribute_to_payload(fb_attribute, stream, pt);
+					retval = TRUE;
 				}
 			}
 		}
@@ -655,12 +667,14 @@ static void sdp_parse_rtcp_fb_parameters(belle_sdp_media_description_t *media_de
 			pt_num = belle_sdp_rtcp_fb_attribute_get_id(fb_attribute);
 			for (pt_it = stream->payloads; pt_it != NULL; pt_it = pt_it->next) {
 				pt = (PayloadType *)pt_it->data;
+				retval = TRUE;
 				if (payload_type_get_number(pt) == (int)pt_num) {
 					apply_rtcp_fb_attribute_to_payload(fb_attribute, stream, pt);
 				}
 			}
 		}
 	}
+	return retval;
 }
 
 static void sal_init_rtcp_xr_description(OrtpRtcpXrConfiguration *config) {
@@ -723,6 +737,7 @@ static SalStreamDescription * sdp_to_stream_description(SalMediaDescription *md,
 	belle_sip_list_t *custom_attribute_it;
 	const char* value;
 	const char *mtype,*proto;
+	bool_t has_avpf_attributes;
 
 	stream=&md->streams[md->nb_streams];
 	media=belle_sdp_media_description_get_media ( media_desc );
@@ -781,6 +796,8 @@ static SalStreamDescription * sdp_to_stream_description(SalMediaDescription *md,
 		stream->dir=md->dir; /*takes default value if not present*/
 	}
 
+	stream->rtcp_mux = belle_sdp_media_description_get_attribute(media_desc, "rtcp-mux") != NULL;
+	
 	/* Get media payload types */
 	sdp_parse_payload_types(media_desc, stream);
 
@@ -822,13 +839,25 @@ static SalStreamDescription * sdp_to_stream_description(SalMediaDescription *md,
 		sdp_parse_media_crypto_parameters(media_desc, stream);
 	}
 
+	/* Read zrtp-hash attribute */
+	if ((attribute=belle_sdp_media_description_get_attribute(media_desc,"zrtp-hash"))!=NULL) {
+		if ((value=belle_sdp_attribute_get_value(attribute))!=NULL) {
+			strncpy((char *)(stream->zrtphash), belle_sdp_attribute_get_value(attribute),sizeof(stream->zrtphash));
+			stream->haveZrtpHash = 1;
+		}
+	}
+
 	/* Get ICE candidate attributes if any */
 	sdp_parse_media_ice_parameters(media_desc, stream);
+    
+	has_avpf_attributes = sdp_parse_rtcp_fb_parameters(media_desc, stream);
 
 	/* Get RTCP-FB attributes if any */
 	if (sal_stream_description_has_avpf(stream)) {
 		enable_avpf_for_stream(stream);
-		sdp_parse_rtcp_fb_parameters(media_desc, stream);
+	}else if (has_avpf_attributes ){
+		enable_avpf_for_stream(stream);
+		stream->implicit_rtcp_fb = TRUE;
 	}
 
 	/* Get RTCP-XR attributes if any */
