@@ -22,7 +22,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include <time.h>
 #include "private.h"
 
-#ifdef CALL_LOGS_STORAGE_ENABLED
+#ifdef SQLITE_STORAGE_ENABLED
 #ifndef _WIN32
 #if !defined(ANDROID) && !defined(__QNXNTO__)
 #	include <langinfo.h>
@@ -74,9 +74,9 @@ static void set_call_log_date(LinphoneCallLog *cl, time_t start_time){
  ******************************************************************************/
 
 void call_logs_write_to_config_file(LinphoneCore *lc){
-	MSList *elem;
+	bctbx_list_t *elem;
 	char logsection[32];
-	int i;
+	unsigned int i;
 	char *tmp;
 	LpConfig *cfg=lc->config;
 
@@ -147,7 +147,7 @@ void call_logs_read_from_config_file(LinphoneCore *lc){
 			cl->video_enabled=lp_config_get_int(cfg,logsection,"video_enabled",0);
 			tmp=lp_config_get_string(cfg,logsection,"call_id",NULL);
 			if (tmp) cl->call_id=ms_strdup(tmp);
-			lc->call_logs=ms_list_append(lc->call_logs,cl);
+			lc->call_logs=bctbx_list_append(lc->call_logs,cl);
 		}else break;
 	}
 }
@@ -248,6 +248,10 @@ bool_t linphone_call_log_video_enabled(LinphoneCallLog *cl) {
 	return cl->video_enabled;
 }
 
+bool_t linphone_call_log_was_conference(LinphoneCallLog *cl) {
+	return cl->was_conference;
+}
+
 
 /*******************************************************************************
  * Reference and user data handling functions                                  *
@@ -321,7 +325,7 @@ BELLE_SIP_INSTANCIATE_VPTR(LinphoneCallLog, belle_sip_object_t,
  * SQL storage related functions                                               *
  ******************************************************************************/
 
-#ifdef CALL_LOGS_STORAGE_ENABLED
+#ifdef SQLITE_STORAGE_ENABLED
 
 static void linphone_create_table(sqlite3* db) {
 	char* errmsg=NULL;
@@ -345,7 +349,7 @@ static void linphone_create_table(sqlite3* db) {
 	}
 }
 
-void linphone_update_call_log_table(sqlite3* db) {
+static void linphone_update_call_log_table(sqlite3* db) {
 	char* errmsg=NULL;
 	int ret;
 
@@ -365,34 +369,6 @@ void linphone_update_call_log_table(sqlite3* db) {
 	}
 }
 
-static int _linphone_sqlite3_open(const char *db_file, sqlite3 **db) {
-#if defined(ANDROID) || defined(__QNXNTO__)
-	return sqlite3_open(db_file, db);
-#elif defined(_WIN32)
-	int ret;
-	wchar_t db_file_utf16[MAX_PATH_SIZE];
-	ret = MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, db_file, -1, db_file_utf16, MAX_PATH_SIZE);
-	if(ret == 0) db_file_utf16[0] = '\0';
-	return sqlite3_open16(db_file_utf16, db);
-#else
-	char db_file_locale[MAX_PATH_SIZE] = {'\0'};
-	char db_file_utf8[MAX_PATH_SIZE] = "";
-	char *inbuf=db_file_locale, *outbuf=db_file_utf8;
-	size_t inbyteleft = MAX_PATH_SIZE, outbyteleft = MAX_PATH_SIZE;
-	iconv_t cb;
-
-	strncpy(db_file_locale, db_file, MAX_PATH_SIZE-1);
-	cb = iconv_open("UTF-8", nl_langinfo(CODESET));
-	if(cb != (iconv_t)-1) {
-		int ret;
-		ret = iconv(cb, &inbuf, &inbyteleft, &outbuf, &outbyteleft);
-		if(ret == -1) db_file_utf8[0] = '\0';
-		iconv_close(cb);
-	}
-	return sqlite3_open(db_file_utf8, db);
-#endif
-}
-
 void linphone_core_call_log_storage_init(LinphoneCore *lc) {
 	int ret;
 	const char *errmsg;
@@ -411,7 +387,7 @@ void linphone_core_call_log_storage_init(LinphoneCore *lc) {
 	linphone_create_table(db);
 	linphone_update_call_log_table(db);
 	lc->logs_db = db;
-	
+
 	// Load the existing call logs
 	linphone_core_get_call_history(lc);
 }
@@ -438,26 +414,30 @@ void linphone_core_call_log_storage_close(LinphoneCore *lc) {
  * | 11 | refkey
  */
 static int create_call_log(void *data, int argc, char **argv, char **colName) {
-	MSList **list = (MSList **)data;
+	bctbx_list_t **list = (bctbx_list_t **)data;
 	LinphoneAddress *from;
 	LinphoneAddress *to;
 	LinphoneCallDir dir;
 	LinphoneCallLog *log;
-	
-	unsigned int storage_id = atoi(argv[0]);
+
+	unsigned int storage_id = (unsigned int)atoi(argv[0]);
 	from = linphone_address_new(argv[1]);
 	to = linphone_address_new(argv[2]);
+	
+	if (from == NULL || to == NULL) goto error;
+	
 	dir = (LinphoneCallDir) atoi(argv[3]);
 	log = linphone_call_log_new(dir, from, to);
-	
+
 	log->storage_id = storage_id;
 	log->duration = atoi(argv[4]);
 	log->start_date_time = (time_t)atol(argv[5]);
+	set_call_log_date(log,log->start_date_time);
 	log->connected_date_time = (time_t)atol(argv[6]);
 	log->status = (LinphoneCallStatus) atoi(argv[7]);
 	log->video_enabled = atoi(argv[8]) == 1;
-	log->quality = atof(argv[9]);
-	
+	log->quality = (float)atof(argv[9]);
+
 	if (argc > 10) {
 		if (argv[10] != NULL) {
 			log->call_id = ms_strdup(argv[10]);
@@ -466,13 +446,22 @@ static int create_call_log(void *data, int argc, char **argv, char **colName) {
 			log->refkey = ms_strdup(argv[11]);
 		}
 	}
+
+	*list = bctbx_list_append(*list, log);
+	return 0;
 	
-	*list = ms_list_append(*list, log);
-	
+error:
+	if (from){
+		linphone_address_destroy(from);
+	}
+	if (to){
+		linphone_address_destroy(to);
+	}
+	ms_error("Bad call log at storage_id %u", storage_id);
 	return 0;
 }
 
-void linphone_sql_request_call_log(sqlite3 *db, const char *stmt, MSList **list) {
+static void linphone_sql_request_call_log(sqlite3 *db, const char *stmt, bctbx_list_t **list) {
 	char* errmsg = NULL;
 	int ret;
 	ret = sqlite3_exec(db, stmt, create_call_log, list, &errmsg);
@@ -482,7 +471,7 @@ void linphone_sql_request_call_log(sqlite3 *db, const char *stmt, MSList **list)
 	}
 }
 
-int linphone_sql_request_generic(sqlite3* db, const char *stmt) {
+static int linphone_sql_request_generic(sqlite3* db, const char *stmt) {
 	char* errmsg = NULL;
 	int ret;
 	ret = sqlite3_exec(db, stmt, NULL, NULL, &errmsg);
@@ -497,7 +486,7 @@ void linphone_core_store_call_log(LinphoneCore *lc, LinphoneCallLog *log) {
 	if (lc && lc->logs_db){
 		char *from, *to;
 		char *buf;
-		
+
 		from = linphone_address_as_string(log->from);
 		to = linphone_address_as_string(log->to);
 		buf = sqlite3_mprintf("INSERT INTO call_history VALUES(NULL,%Q,%Q,%i,%i,%lld,%lld,%i,%i,%f,%Q,%Q);",
@@ -517,56 +506,56 @@ void linphone_core_store_call_log(LinphoneCore *lc, LinphoneCallLog *log) {
 		sqlite3_free(buf);
 		ms_free(from);
 		ms_free(to);
-		
-		log->storage_id = sqlite3_last_insert_rowid(lc->logs_db);
+
+		log->storage_id = (unsigned int)sqlite3_last_insert_rowid(lc->logs_db);
 	}
-	
+
 	if (lc) {
-		lc->call_logs = ms_list_prepend(lc->call_logs, linphone_call_log_ref(log));
+		lc->call_logs = bctbx_list_prepend(lc->call_logs, linphone_call_log_ref(log));
 	}
 }
 
-static void copy_user_data_from_existing_log(MSList *existing_logs, LinphoneCallLog *log) {
+static void copy_user_data_from_existing_log(bctbx_list_t *existing_logs, LinphoneCallLog *log) {
 	while (existing_logs) {
 		LinphoneCallLog *existing_log = (LinphoneCallLog *)existing_logs->data;
 		if (existing_log->storage_id == log->storage_id) {
 			log->user_data = existing_log->user_data;
 			break;
 		}
-		existing_logs = ms_list_next(existing_logs);
+		existing_logs = bctbx_list_next(existing_logs);
 	}
 }
 
-static void copy_user_data_from_existing_logs(MSList *existing_logs, MSList *new_logs) {
+static void copy_user_data_from_existing_logs(bctbx_list_t *existing_logs, bctbx_list_t *new_logs) {
 	while (new_logs) {
 		LinphoneCallLog *new_log = (LinphoneCallLog *)new_logs->data;
 		copy_user_data_from_existing_log(existing_logs, new_log);
-		new_logs = ms_list_next(new_logs);
+		new_logs = bctbx_list_next(new_logs);
 	}
 }
 
-const MSList *linphone_core_get_call_history(LinphoneCore *lc) {
+const bctbx_list_t *linphone_core_get_call_history(LinphoneCore *lc) {
 	char *buf;
 	uint64_t begin,end;
-	MSList *result = NULL;
+	bctbx_list_t *result = NULL;
 
 	if (!lc || lc->logs_db == NULL) return NULL;
 
-	buf = sqlite3_mprintf("SELECT * FROM call_history ORDER BY id DESC LIMIT %i", lc->max_call_logs);
+	buf = sqlite3_mprintf("SELECT * FROM call_history ORDER BY id DESC LIMIT %u", lc->max_call_logs);
 
 	begin = ortp_get_cur_time_ms();
 	linphone_sql_request_call_log(lc->logs_db, buf, &result);
 	end = ortp_get_cur_time_ms();
 	ms_message("%s(): completed in %i ms",__FUNCTION__, (int)(end-begin));
 	sqlite3_free(buf);
-	
+
 	if (lc->call_logs) {
 		copy_user_data_from_existing_logs(lc->call_logs, result);
 	}
-	
-	lc->call_logs = ms_list_free_with_data(lc->call_logs, (void (*)(void*))linphone_call_log_unref);
+
+	lc->call_logs = bctbx_list_free_with_data(lc->call_logs, (void (*)(void*))linphone_call_log_unref);
 	lc->call_logs = result;
-	
+
 	return lc->call_logs;
 }
 
@@ -585,7 +574,7 @@ void linphone_core_delete_call_log(LinphoneCore *lc, LinphoneCallLog *log) {
 
 	if (!lc || lc->logs_db == NULL) return ;
 
-	buf = sqlite3_mprintf("DELETE FROM call_history WHERE id = %i", log->storage_id);
+	buf = sqlite3_mprintf("DELETE FROM call_history WHERE id = %u", log->storage_id);
 	linphone_sql_request_generic(lc->logs_db, buf);
 	sqlite3_free(buf);
 }
@@ -611,14 +600,14 @@ int linphone_core_get_call_history_size(LinphoneCore *lc) {
 	return numrows;
 }
 
-MSList * linphone_core_get_call_history_for_address(LinphoneCore *lc, const LinphoneAddress *addr) {
+bctbx_list_t * linphone_core_get_call_history_for_address(LinphoneCore *lc, const LinphoneAddress *addr) {
 	char *buf;
 	char *sipAddress;
 	uint64_t begin,end;
-	MSList *result = NULL;
+	bctbx_list_t *result = NULL;
 
 	if (!lc || lc->logs_db == NULL || addr == NULL) return NULL;
-	
+
 	/*since we want to append query parameters depending on arguments given, we use malloc instead of sqlite3_mprintf*/
 	sipAddress = linphone_address_as_string_uri_only(addr);
 	buf = sqlite3_mprintf("SELECT * FROM call_history WHERE caller LIKE '%%%q%%' OR callee LIKE '%%%q%%' ORDER BY id DESC", sipAddress, sipAddress); // The '%%%q%%' takes care of the eventual presence of a display name
@@ -629,22 +618,22 @@ MSList * linphone_core_get_call_history_for_address(LinphoneCore *lc, const Linp
 	ms_message("%s(): completed in %i ms",__FUNCTION__, (int)(end-begin));
 	sqlite3_free(buf);
 	ms_free(sipAddress);
-	
+
 	if (lc->call_logs) {
 		copy_user_data_from_existing_logs(lc->call_logs, result);
 	}
-	
+
 	return result;
 }
 
 LinphoneCallLog * linphone_core_get_last_outgoing_call_log(LinphoneCore *lc) {
 	char *buf;
 	uint64_t begin,end;
-	MSList *list = NULL;
+	bctbx_list_t *list = NULL;
 	LinphoneCallLog* result = NULL;
 
 	if (!lc || lc->logs_db == NULL) return NULL;
-	
+
 	/*since we want to append query parameters depending on arguments given, we use malloc instead of sqlite3_mprintf*/
 	buf = sqlite3_mprintf("SELECT * FROM call_history WHERE direction = 0 ORDER BY id DESC LIMIT 1");
 
@@ -653,15 +642,43 @@ LinphoneCallLog * linphone_core_get_last_outgoing_call_log(LinphoneCore *lc) {
 	end = ortp_get_cur_time_ms();
 	ms_message("%s(): completed in %i ms",__FUNCTION__, (int)(end-begin));
 	sqlite3_free(buf);
-	
+
 	if (list) {
 		result = (LinphoneCallLog*)list->data;
 	}
-	
+
 	if (lc->call_logs && result) {
 		copy_user_data_from_existing_log(lc->call_logs, result);
 	}
-	
+
+	return result;
+}
+
+LinphoneCallLog * linphone_core_find_call_log_from_call_id(LinphoneCore *lc, const char *call_id) {
+	char *buf;
+	uint64_t begin,end;
+	bctbx_list_t *list = NULL;
+	LinphoneCallLog* result = NULL;
+
+	if (!lc || lc->logs_db == NULL) return NULL;
+
+	/*since we want to append query parameters depending on arguments given, we use malloc instead of sqlite3_mprintf*/
+	buf = sqlite3_mprintf("SELECT * FROM call_history WHERE call_id = '%q' ORDER BY id DESC LIMIT 1", call_id);
+
+	begin = ortp_get_cur_time_ms();
+	linphone_sql_request_call_log(lc->logs_db, buf, &list);
+	end = ortp_get_cur_time_ms();
+	ms_message("%s(): completed in %i ms",__FUNCTION__, (int)(end-begin));
+	sqlite3_free(buf);
+
+	if (list) {
+		result = (LinphoneCallLog*)list->data;
+	}
+
+	if (lc->call_logs && result) {
+		copy_user_data_from_existing_log(lc->call_logs, result);
+	}
+
 	return result;
 }
 
@@ -676,7 +693,7 @@ void linphone_core_call_log_storage_close(LinphoneCore *lc) {
 void linphone_core_store_call_log(LinphoneCore *lc, LinphoneCallLog *log) {
 }
 
-const MSList *linphone_core_get_call_history(LinphoneCore *lc) {
+const bctbx_list_t *linphone_core_get_call_history(LinphoneCore *lc) {
 	return NULL;
 }
 
@@ -690,7 +707,7 @@ int linphone_core_get_call_history_size(LinphoneCore *lc) {
 	return 0;
 }
 
-MSList * linphone_core_get_call_history_for_address(LinphoneCore *lc, const LinphoneAddress *addr) {
+bctbx_list_t * linphone_core_get_call_history_for_address(LinphoneCore *lc, const LinphoneAddress *addr) {
 	return NULL;
 }
 
@@ -698,4 +715,8 @@ LinphoneCallLog * linphone_core_get_last_outgoing_call_log(LinphoneCore *lc) {
 	return NULL;
 }
 
-#endif
\ No newline at end of file
+LinphoneCallLog * linphone_core_find_call_log_from_call_id(LinphoneCore *lc, const char *call_id) {
+	return NULL;
+}
+
+#endif
