@@ -111,12 +111,12 @@ void linphone_call_update_frozen_payloads(LinphoneCall *call, SalMediaDescriptio
 	SalMediaDescription *local=call->localdesc;
 	int i;
 	for(i=0;i<result_desc->nb_streams;++i){
-		MSList *elem;
+		bctbx_list_t *elem;
 		for (elem=result_desc->streams[i].payloads;elem!=NULL;elem=elem->next){
 			PayloadType *pt=(PayloadType*)elem->data;
 			if (is_payload_type_number_available(local->streams[i].already_assigned_payloads, payload_type_get_number(pt), NULL)){
 				/*new codec, needs to be added to the list*/
-				local->streams[i].already_assigned_payloads=ms_list_append(local->streams[i].already_assigned_payloads, payload_type_clone(pt));
+				local->streams[i].already_assigned_payloads=bctbx_list_append(local->streams[i].already_assigned_payloads, payload_type_clone(pt));
 				ms_message("LinphoneCall[%p] : payload type %i %s/%i fmtp=%s added to frozen list.",
 					   call, payload_type_get_number(pt), pt->mime_type, pt->clock_rate, pt->recv_fmtp ? pt->recv_fmtp : "");
 			}
@@ -158,12 +158,12 @@ void linphone_core_update_streams(LinphoneCore *lc, LinphoneCall *call, SalMedia
 					if (call->all_muted){
 						ms_message("Early media finished, unmuting inputs...");
 						/*we were in early media, now we want to enable real media */
-						linphone_call_enable_camera (call,linphone_call_camera_enabled (call));
+						call->all_muted = FALSE;
 						if (call->audiostream)
 							linphone_core_enable_mic(lc, linphone_core_mic_enabled(lc));
 #ifdef VIDEO_ENABLED
 						if (call->videostream && call->camera_enabled)
-							video_stream_change_camera(call->videostream,linphone_call_get_video_device(call));
+							linphone_call_enable_camera(call, linphone_call_camera_enabled(call));
 #endif
 					}
 					/*FIXME ZRTP, might be restarted in any cases ? */
@@ -187,7 +187,7 @@ void linphone_core_update_streams(LinphoneCore *lc, LinphoneCall *call, SalMedia
 			ms_message("Media ip type has changed, destroying sessions context on call [%p]",call);
 			ms_media_stream_sessions_uninit(&call->sessions[call->main_audio_stream_index]);
 			ms_media_stream_sessions_uninit(&call->sessions[call->main_video_stream_index]);
-			if (call->params->realtimetext_enabled) ms_media_stream_sessions_uninit(&call->sessions[call->main_text_stream_index]);
+			ms_media_stream_sessions_uninit(&call->sessions[call->main_text_stream_index]);
 		}
 		linphone_call_init_media_streams (call);
 	}
@@ -201,7 +201,7 @@ void linphone_core_update_streams(LinphoneCore *lc, LinphoneCall *call, SalMedia
 		prepare_early_media_forking(call);
 	}
 	linphone_call_start_media_streams(call, target_state);
-	if (call->state==LinphoneCallPausing && call->paused_by_app && ms_list_size(lc->calls)==1){
+	if (call->state==LinphoneCallPausing && call->paused_by_app && bctbx_list_size(lc->calls)==1){
 		linphone_core_play_named_tone(lc,LinphoneToneCallOnHold);
 	}
 	linphone_call_update_frozen_payloads(call, new_md);
@@ -212,7 +212,7 @@ void linphone_core_update_streams(LinphoneCore *lc, LinphoneCall *call, SalMedia
 }
 #if 0
 static bool_t is_duplicate_call(LinphoneCore *lc, const LinphoneAddress *from, const LinphoneAddress *to){
-	MSList *elem;
+	bctbx_list_t *elem;
 	for(elem=lc->calls;elem!=NULL;elem=elem->next){
 		LinphoneCall *call=(LinphoneCall*)elem->data;
 		if (linphone_address_weak_equal(call->log->from,from) &&
@@ -225,7 +225,7 @@ static bool_t is_duplicate_call(LinphoneCore *lc, const LinphoneAddress *from, c
 #endif
 
 static bool_t already_a_call_with_remote_address(const LinphoneCore *lc, const LinphoneAddress *remote) {
-	MSList *elem;
+	bctbx_list_t *elem;
 	ms_message("Searching for already_a_call_with_remote_address.");
 
 	for(elem=lc->calls;elem!=NULL;elem=elem->next){
@@ -254,27 +254,19 @@ static void call_received(SalOp *h){
 	if (linphone_presence_model_get_basic_status(lc->presence_model) == LinphonePresenceBasicStatusClosed) {
 		LinphonePresenceActivity *activity = linphone_presence_model_get_activity(lc->presence_model);
 		switch (linphone_presence_activity_get_type(activity)) {
-			case LinphonePresenceActivityBusy:
-				sal_call_decline(h,SalReasonBusy,NULL);
-				break;
-			case LinphonePresenceActivityAppointment:
-			case LinphonePresenceActivityMeeting:
-			case LinphonePresenceActivityOffline:
-			case LinphonePresenceActivityWorship:
-				sal_call_decline(h,SalReasonTemporarilyUnavailable,NULL);
-				break;
 			case LinphonePresenceActivityPermanentAbsence:
 				alt_contact = linphone_presence_model_get_contact(lc->presence_model);
 				if (alt_contact != NULL) {
 					sal_call_decline(h,SalReasonRedirect,alt_contact);
 					ms_free(alt_contact);
+					sal_op_release(h);
+					return;
 				}
 				break;
 			default:
+				/*nothing special to be done*/
 				break;
 		}
-		sal_op_release(h);
-		return;
 	}
 
 	if (!linphone_core_can_we_add_call(lc)){/*busy*/
@@ -344,7 +336,7 @@ static void call_received(SalOp *h){
 
 	call->bg_task_id=sal_begin_background_task("liblinphone call notification", NULL, NULL);
 
-	if ((linphone_core_get_firewall_policy(lc) == LinphonePolicyUseIce) && (call->ice_session != NULL)) {
+	if (call->defer_notify_incoming) {
 		/* Defer ringing until the end of the ICE candidates gathering process. */
 		ms_message("Defer ringing to gather ICE candidates");
 		return;
@@ -405,7 +397,7 @@ static void start_remote_ring(LinphoneCore *lc, LinphoneCall *call) {
 		if (call->audiostream)
 			audio_stream_unprepare_sound(call->audiostream);
 		if( lc->sound_conf.remote_ring ){
-			lc->ringstream=ring_start(lc->sound_conf.remote_ring,2000,ringcard);
+			lc->ringstream=ring_start(lc->factory, lc->sound_conf.remote_ring,2000,ringcard);
 		}
 	}
 }
@@ -503,10 +495,7 @@ static void process_call_accepted(LinphoneCore *lc, LinphoneCall *call, SalOp *o
 	if (call->params->internal_call_update)
 		call->params->internal_call_update = FALSE;
 
-	/* Handle remote ICE attributes if any. */
-	if (call->ice_session != NULL && rmd) {
-		linphone_call_update_ice_from_remote_media_description(call, rmd);
-	}
+
 #ifdef BUILD_UPNP
 	if (call->upnp_session != NULL && rmd) {
 		linphone_core_update_upnp_from_remote_media_description(call, rmd);
@@ -522,6 +511,12 @@ static void process_call_accepted(LinphoneCore *lc, LinphoneCall *call, SalOp *o
 		md = NULL;
 	}
 	if (md){ /*there is a valid SDP in the response, either offer or answer, and we're able to start/update the streams*/
+
+		/* Handle remote ICE attributes if any. */
+		if (call->ice_session != NULL && rmd) {
+			linphone_call_update_ice_from_remote_media_description(call, rmd, FALSE);
+		}
+
 		switch (call->state){
 			case LinphoneCallResuming:
 				linphone_core_notify_display_status(lc,_("Call resumed."));
@@ -709,7 +704,7 @@ static void call_updating(SalOp *op, bool_t is_update){
 	if (call->state!=LinphoneCallPaused){
 		/*Refresh the local description, but in paused state, we don't change anything.*/
 		if (rmd == NULL && lp_config_get_int(call->core->config,"sip","sdp_200_ack_follow_video_policy",0)) {
-			LinphoneCallParams *p=linphone_core_create_default_call_parameters (lc);
+			LinphoneCallParams *p=linphone_core_create_call_params(lc, NULL);
 			ms_message("Applying default policy for offering SDP on call [%p]",call);
 			linphone_call_set_new_params(call, p);
 			linphone_call_params_destroy(p);
@@ -791,7 +786,7 @@ static void call_terminated(SalOp *op, const char *from){
 		linphone_core_start_refered_call(lc,call,NULL);
 	}
 	//we stop the call only if we have this current call or if we are in call
-	if (lc->ringstream!=NULL && ( (ms_list_size(lc->calls)  == 1) || linphone_core_in_call(lc) )) {
+	if ((bctbx_list_size(lc->calls)  == 1) || linphone_core_in_call(lc)) {
 		linphone_core_stop_ringing(lc);
 	}
 	linphone_call_stop_media_streams(call);
@@ -1035,10 +1030,11 @@ static void register_failure(SalOp *op){
 	} else {
 		linphone_proxy_config_set_state(cfg,LinphoneRegistrationFailed,details);
 	}
-	if (cfg->publish_op){
+	if (cfg->long_term_event){
 		/*prevent publish to be sent now until registration gets successful*/
-		sal_op_release(cfg->publish_op);
-		cfg->publish_op=NULL;
+		linphone_event_terminate(cfg->long_term_event);
+		linphone_event_unref(cfg->long_term_event);
+		cfg->long_term_event=NULL;
 		cfg->send_publish=cfg->publish;
 	}
 }
@@ -1065,7 +1061,15 @@ static void dtmf_received(SalOp *op, char dtmf){
 static void refer_received(Sal *sal, SalOp *op, const char *referto){
 	LinphoneCore *lc=(LinphoneCore *)sal_get_user_pointer(sal);
 	LinphoneCall *call=(LinphoneCall*)sal_op_get_user_pointer(op);
-	if (call){
+	LinphoneAddress *refer_to_addr = linphone_address_new(referto);
+	char method[20] = "";
+
+	if(refer_to_addr) {
+		const char *tmp = linphone_address_get_method_param(refer_to_addr);
+		if(tmp) strncpy(method, tmp, sizeof(method));
+		linphone_address_destroy(refer_to_addr);
+	}
+	if (call && (strlen(method) == 0 || strcmp(method, "INVITE") == 0)) {
 		if (call->refer_to!=NULL){
 			ms_free(call->refer_to);
 		}
@@ -1084,8 +1088,8 @@ static void refer_received(Sal *sal, SalOp *op, const char *referto){
 }
 
 static bool_t is_duplicate_msg(LinphoneCore *lc, const char *msg_id){
-	MSList *elem=lc->last_recv_msg_ids;
-	MSList *tail=NULL;
+	bctbx_list_t *elem=lc->last_recv_msg_ids;
+	bctbx_list_t *tail=NULL;
 	int i;
 	bool_t is_duplicate=FALSE;
 	for(i=0;elem!=NULL;elem=elem->next,i++){
@@ -1095,11 +1099,11 @@ static bool_t is_duplicate_msg(LinphoneCore *lc, const char *msg_id){
 		tail=elem;
 	}
 	if (!is_duplicate){
-		lc->last_recv_msg_ids=ms_list_prepend(lc->last_recv_msg_ids,ms_strdup(msg_id));
+		lc->last_recv_msg_ids=bctbx_list_prepend(lc->last_recv_msg_ids,ms_strdup(msg_id));
 	}
 	if (i>=10){
 		ms_free(tail->data);
-		lc->last_recv_msg_ids=ms_list_remove_link(lc->last_recv_msg_ids,tail);
+		lc->last_recv_msg_ids=bctbx_list_remove_link(lc->last_recv_msg_ids,tail);
 	}
 	return is_duplicate;
 }
@@ -1122,11 +1126,18 @@ static void is_composing_received(SalOp *op, const SalIsComposing *is_composing)
 }
 
 static void parse_presence_requested(SalOp *op, const char *content_type, const char *content_subtype, const char *body, SalPresenceModel **result) {
-	linphone_notify_parse_presence(op, content_type, content_subtype, body, result);
+	linphone_notify_parse_presence(content_type, content_subtype, body, result);
 }
 
 static void convert_presence_to_xml_requested(SalOp *op, SalPresenceModel *presence, const char *contact, char **content) {
-	linphone_notify_convert_presence_to_xml(op, presence, contact, content);
+	/*for backward compatibility because still used by notify. No loguer used for publish*/
+
+	if(linphone_presence_model_get_presentity((LinphonePresenceModel*)presence) == NULL) {
+		LinphoneAddress * presentity = linphone_address_new(contact);
+		linphone_presence_model_set_presentity((LinphonePresenceModel*)presence, presentity);
+		linphone_address_unref(presentity);
+	}
+	*content = linphone_presence_model_to_xml((LinphonePresenceModel*)presence);
 }
 
 static void notify_presence(SalOp *op, SalSubscribeStatus ss, SalPresenceModel *model, const char *msg){
@@ -1276,9 +1287,9 @@ static void text_delivery_update(SalOp *op, SalTextDeliveryStatus status){
 	}
 }
 
-static void info_received(SalOp *op, const SalBody *body){
+static void info_received(SalOp *op, SalBodyHandler *body_handler){
 	LinphoneCore *lc=(LinphoneCore *)sal_get_user_pointer(sal_op_get_sal(op));
-	linphone_core_notify_info_message(lc,op,body);
+	linphone_core_notify_info_message(lc,op,body_handler);
 }
 
 static void subscribe_response(SalOp *op, SalSubscribeStatus status){
@@ -1292,31 +1303,37 @@ static void subscribe_response(SalOp *op, SalSubscribeStatus status){
 	}else if (status==SalSubscribePending){
 		linphone_event_set_state(lev,LinphoneSubscriptionPending);
 	}else{
-		if (lev->subscription_state==LinphoneSubscriptionActive && ei->reason==SalReasonIOError){
+		if (lev->subscription_state==LinphoneSubscriptionActive && (ei->reason==SalReasonIOError || ei->reason == SalReasonNoMatch)){
 			linphone_event_set_state(lev,LinphoneSubscriptionOutgoingProgress);
 		}
 		else linphone_event_set_state(lev,LinphoneSubscriptionError);
 	}
 }
 
-static void notify(SalOp *op, SalSubscribeStatus st, const char *eventname, const SalBody *body){
+static void notify(SalOp *op, SalSubscribeStatus st, const char *eventname, SalBodyHandler *body_handler){
 	LinphoneEvent *lev=(LinphoneEvent*)sal_op_get_user_pointer(op);
 	LinphoneCore *lc=(LinphoneCore *)sal_get_user_pointer(sal_op_get_sal(op));
-
-	if (lev==NULL) {
-		/*out of subscribe notify */
-		lev=linphone_event_new_with_out_of_dialog_op(lc,op,LinphoneSubscriptionOutgoing,eventname);
+	bool_t out_of_dialog = (lev==NULL);
+	if (out_of_dialog) {
+		/*out of dialog notify */
+		lev = linphone_event_new_with_out_of_dialog_op(lc,op,LinphoneSubscriptionOutgoing,eventname);
 	}
 	{
-		LinphoneContent *ct=linphone_content_from_sal_body(body);
-		if (ct) linphone_core_notify_notify_received(lc,lev,eventname,ct);
+		LinphoneContent *ct=linphone_content_from_sal_body_handler(body_handler);
+		if (ct) {
+			linphone_core_notify_notify_received(lc,lev,eventname,ct);
+			linphone_content_unref(ct);
+		}
 	}
-	if (st!=SalSubscribeNone){
+	if (out_of_dialog){
+		/*out of dialog NOTIFY do not create an implicit subscription*/
+		linphone_event_set_state(lev, LinphoneSubscriptionTerminated);
+	}else if (st!=SalSubscribeNone){ 
 		linphone_event_set_state(lev,linphone_subscription_state_from_sal(st));
 	}
 }
 
-static void subscribe_received(SalOp *op, const char *eventname, const SalBody *body){
+static void subscribe_received(SalOp *op, const char *eventname, const SalBodyHandler *body_handler){
 	LinphoneEvent *lev=(LinphoneEvent*)sal_op_get_user_pointer(op);
 	LinphoneCore *lc=(LinphoneCore *)sal_get_user_pointer(sal_op_get_sal(op));
 
@@ -1329,7 +1346,7 @@ static void subscribe_received(SalOp *op, const char *eventname, const SalBody *
 
 }
 
-static void subscribe_closed(SalOp *op){
+static void incoming_subscribe_closed(SalOp *op){
 	LinphoneEvent *lev=(LinphoneEvent*)sal_op_get_user_pointer(op);
 
 	linphone_event_set_state(lev,LinphoneSubscriptionTerminated);
@@ -1354,6 +1371,7 @@ static void on_publish_response(SalOp* op){
 	}
 }
 
+
 static void on_expire(SalOp *op){
 	LinphoneEvent *lev=(LinphoneEvent*)sal_op_get_user_pointer(op);
 
@@ -1366,6 +1384,25 @@ static void on_expire(SalOp *op){
 	}
 }
 
+static void on_notify_response(SalOp *op){
+	LinphoneEvent *lev=(LinphoneEvent*)sal_op_get_user_pointer(op);
+
+	if (lev==NULL) return;
+	/*this is actually handling out of dialogs notify - for the moment*/
+	if (!lev->is_out_of_dialog_op) return;
+	switch (linphone_event_get_subscription_state(lev)){
+		case LinphoneSubscriptionIncomingReceived:
+			if (sal_op_get_error_info(op)->reason == SalReasonNone){
+				linphone_event_set_state(lev, LinphoneSubscriptionTerminated);
+			}else{
+				linphone_event_set_state(lev, LinphoneSubscriptionError);
+			}
+		break;
+		default:
+			ms_warning("Unhandled on_notify_response() case %s", linphone_subscription_state_to_string(linphone_event_get_subscription_state(lev)));
+	}
+}
+
 SalCallbacks linphone_sal_callbacks={
 	call_received,
 	call_ringing,
@@ -1386,7 +1423,7 @@ SalCallbacks linphone_sal_callbacks={
 	is_composing_received,
 	notify_refer,
 	subscribe_received,
-	subscribe_closed,
+	incoming_subscribe_closed,
 	subscribe_response,
 	notify,
 	subscribe_presence_received,
@@ -1398,7 +1435,8 @@ SalCallbacks linphone_sal_callbacks={
 	auth_requested,
 	info_received,
 	on_publish_response,
-	on_expire
+	on_expire,
+	on_notify_response
 };
 
 
