@@ -27,6 +27,7 @@
 #include "lpconfig.h"
 #include "belle-sip/belle-sip.h"
 #include "ortp/b64.h"
+#include "lime.h"
 
 #include <libxml/parser.h>
 #include <libxml/tree.h>
@@ -113,18 +114,18 @@ void linphone_chat_message_cbs_set_file_transfer_progress_indication(
 BELLE_SIP_DECLARE_NO_IMPLEMENTED_INTERFACES(LinphoneChatMessage);
 
 static void _linphone_chat_room_destroy(LinphoneChatRoom *cr) {
-	ms_list_free_with_data(cr->transient_messages, (void (*)(void *))linphone_chat_message_release);
+	bctbx_list_free_with_data(cr->transient_messages, (void (*)(void *))linphone_chat_message_release);
 	linphone_chat_room_delete_composing_idle_timer(cr);
 	linphone_chat_room_delete_composing_refresh_timer(cr);
 	linphone_chat_room_delete_remote_composing_refresh_timer(cr);
 	if (cr->lc != NULL) {
-		if (ms_list_find(cr->lc->chatrooms, cr)) {
+		if (bctbx_list_find(cr->lc->chatrooms, cr)) {
 			ms_error("LinphoneChatRoom[%p] is destroyed while still being used by the LinphoneCore. This is abnormal."
 					 " linphone_core_get_chat_room() doesn't give a reference, there is no need to call "
 					 "linphone_chat_room_unref(). "
 					 "In order to remove a chat room from the core, use linphone_core_delete_chat_room().",
 					 cr);
-			cr->lc->chatrooms = ms_list_remove(cr->lc->chatrooms, cr);
+			cr->lc->chatrooms = bctbx_list_remove(cr->lc->chatrooms, cr);
 		}
 	}
 	linphone_address_destroy(cr->peer_url);
@@ -166,7 +167,7 @@ bool_t linphone_core_chat_enabled(const LinphoneCore *lc) {
 	return lc->chat_deny_code != LinphoneReasonNone;
 }
 
-const MSList *linphone_core_get_chat_rooms(LinphoneCore *lc) {
+const bctbx_list_t *linphone_core_get_chat_rooms(LinphoneCore *lc) {
 	return lc->chatrooms;
 }
 
@@ -188,12 +189,13 @@ static LinphoneChatRoom *_linphone_core_create_chat_room_base(LinphoneCore *lc,
 	cr->peer = linphone_address_as_string(addr);
 	cr->peer_url = addr;
 	cr->unread_count = -1;
+	cr->received_rtt_characters = NULL;
 	return cr;
 }
 
 static LinphoneChatRoom *_linphone_core_create_chat_room(LinphoneCore *lc, LinphoneAddress *addr) {
 	LinphoneChatRoom *cr = _linphone_core_create_chat_room_base(lc, addr);
-	lc->chatrooms = ms_list_append(lc->chatrooms, (void *)cr);
+	lc->chatrooms = bctbx_list_append(lc->chatrooms, (void *)cr);
 	return cr;
 }
 
@@ -214,8 +216,8 @@ static LinphoneChatRoom *_linphone_core_create_chat_room_from_url(LinphoneCore *
 
 LinphoneChatRoom *_linphone_core_get_chat_room(LinphoneCore *lc, const LinphoneAddress *addr) {
 	LinphoneChatRoom *cr = NULL;
-	MSList *elem;
-	for (elem = lc->chatrooms; elem != NULL; elem = ms_list_next(elem)) {
+	bctbx_list_t *elem;
+	for (elem = lc->chatrooms; elem != NULL; elem = bctbx_list_next(elem)) {
 		cr = (LinphoneChatRoom *)elem->data;
 		if (linphone_chat_room_matches(cr, addr)) {
 			break;
@@ -250,8 +252,8 @@ LinphoneChatRoom *linphone_core_get_chat_room(LinphoneCore *lc, const LinphoneAd
 }
 
 void linphone_core_delete_chat_room(LinphoneCore *lc, LinphoneChatRoom *cr) {
-	if (ms_list_find(lc->chatrooms, cr)) {
-		lc->chatrooms = ms_list_remove(cr->lc->chatrooms, cr);
+	if (bctbx_list_find(lc->chatrooms, cr)) {
+		lc->chatrooms = bctbx_list_remove(cr->lc->chatrooms, cr);
 		linphone_chat_room_delete_history(cr);
 		linphone_chat_room_unref(cr);
 	} else {
@@ -263,6 +265,50 @@ LinphoneChatRoom *linphone_core_get_chat_room_from_uri(LinphoneCore *lc, const c
 	return _linphone_core_get_or_create_chat_room(lc, to);
 }
 
+bool_t linphone_chat_room_lime_available(LinphoneChatRoom *cr) {
+	if (cr) {
+		switch (linphone_core_lime_enabled(cr->lc)) {
+			case LinphoneLimeDisabled: return FALSE;
+			case LinphoneLimeMandatory: return TRUE;
+			case LinphoneLimePreferred: {
+				FILE *CACHEFD = NULL;
+				if (cr->lc->zrtp_secrets_cache != NULL) {
+					CACHEFD = fopen(cr->lc->zrtp_secrets_cache, "rb+");
+					if (CACHEFD) {
+						size_t cacheSize;
+						xmlDocPtr cacheXml;
+						char *cacheString=ms_load_file_content(CACHEFD, &cacheSize);
+						if (!cacheString){
+							ms_warning("Unable to load content of ZRTP ZID cache to decrypt message");
+							return FALSE;
+						}
+						cacheString[cacheSize] = '\0';
+						cacheSize += 1;
+						fclose(CACHEFD);
+						cacheXml = xmlParseDoc((xmlChar*)cacheString);
+						ms_free(cacheString);
+						if (cacheXml) {
+							bool_t res;
+							limeURIKeys_t associatedKeys;
+							/* retrieve keys associated to the peer URI */
+							associatedKeys.peerURI = (uint8_t *)malloc(strlen(cr->peer)+1);
+							strcpy((char *)(associatedKeys.peerURI), cr->peer);
+							associatedKeys.associatedZIDNumber  = 0;
+							associatedKeys.peerKeys = NULL;
+
+							res = (lime_getCachedSndKeysByURI(cacheXml, &associatedKeys) == 0 && associatedKeys.associatedZIDNumber != 0);
+							lime_freeKeys(&associatedKeys);
+							xmlFreeDoc(cacheXml);
+							return res;
+						}
+					}
+				}
+			}
+		}
+	}
+	return FALSE;
+}
+
 static void linphone_chat_room_delete_composing_idle_timer(LinphoneChatRoom *cr) {
 	if (cr->composing_idle_timer) {
 		if (cr->lc && cr->lc->sal)
@@ -291,6 +337,9 @@ static void linphone_chat_room_delete_remote_composing_refresh_timer(LinphoneCha
 }
 
 void linphone_chat_room_destroy(LinphoneChatRoom *cr) {
+	if (cr->received_rtt_characters) {
+		cr->received_rtt_characters = bctbx_list_free(cr->received_rtt_characters);
+	}
 	linphone_chat_room_unref(cr);
 }
 
@@ -321,25 +370,30 @@ void _linphone_chat_room_send_message(LinphoneChatRoom *cr, LinphoneChatMessage
 	if (cr->call && linphone_call_params_realtime_text_enabled(linphone_call_get_current_params(cr->call))) {
 		uint32_t new_line = 0x2028;
 		linphone_chat_message_put_char(msg, new_line); // New Line
-		linphone_chat_message_set_state(msg, LinphoneChatMessageStateDelivered);
 		linphone_chat_message_unref(msg);
 		return;
 	}
 
 	msg->dir = LinphoneChatMessageOutgoing;
 
-	// add to transient list
-	cr->transient_messages = ms_list_append(cr->transient_messages, linphone_chat_message_ref(msg));
 
 	/* Check if we shall upload a file to a server */
 	if (msg->file_transfer_information != NULL && msg->content_type == NULL) {
 		/* open a transaction with the server and send an empty request(RCS5.1 section 3.5.4.8.3.1) */
-		linphone_chat_room_upload_file(msg);
+		if (linphone_chat_room_upload_file(msg) == 0) {
+			// add to transient list only if message is going out
+			cr->transient_messages = bctbx_list_append(cr->transient_messages, linphone_chat_message_ref(msg));
+		} else {
+			linphone_chat_message_unref(msg);
+			return;
+		}
 	} else {
 		SalOp *op = NULL;
-		LinphoneCall *call;
+		LinphoneCall *call=NULL;
 		char *content_type;
 		const char *identity = NULL;
+		// add to transient list
+		cr->transient_messages = bctbx_list_append(cr->transient_messages, linphone_chat_message_ref(msg));
 		msg->time = ms_time(0);
 		if (lp_config_get_int(cr->lc->config, "sip", "chat_use_call_dialogs", 0) != 0) {
 			if ((call = linphone_core_get_call_by_remote_address(cr->lc, cr->peer)) != NULL) {
@@ -373,7 +427,7 @@ void _linphone_chat_room_send_message(LinphoneChatRoom *cr, LinphoneChatMessage
 			char *peer_uri = linphone_address_as_string_uri_only(linphone_chat_room_get_peer_address(cr));
 			const char *content_type;
 
-			if (linphone_core_lime_enabled(cr->lc)) {
+			if (linphone_chat_room_lime_available(cr)) {
 				/* ref the msg or it may be destroyed by callback if the encryption failed */
 				if (msg->content_type && strcmp(msg->content_type, "application/vnd.gsma.rcs-ft-http+xml") == 0) {
 					/* it's a file transfer, content type shall be set to
@@ -412,9 +466,22 @@ void _linphone_chat_room_send_message(LinphoneChatRoom *cr, LinphoneChatMessage
 		}
 		linphone_chat_room_delete_composing_idle_timer(cr);
 		linphone_chat_room_delete_composing_refresh_timer(cr);
+		
+		if (call && call->op == op) {
+			/*In this case, chat delivery status is not notified, so unrefing chat message right now*/
+			/*Might be better fixed by delivering status, but too costly for now*/
+			msg->chat_room->transient_messages = bctbx_list_remove(msg->chat_room->transient_messages, msg);
+			linphone_chat_message_unref(msg);
+			linphone_chat_message_unref(msg);
+			return;
+		}
+		
 
 	}
-	linphone_chat_message_set_state(msg, LinphoneChatMessageStateInProgress);
+	// if operation failed, we should not change message state
+	if (msg->dir == LinphoneChatMessageOutgoing) {
+		linphone_chat_message_set_state(msg, LinphoneChatMessageStateInProgress);
+	}
 }
 
 void linphone_chat_message_update_state(LinphoneChatMessage *msg, LinphoneChatMessageState new_state) {
@@ -423,7 +490,7 @@ void linphone_chat_message_update_state(LinphoneChatMessage *msg, LinphoneChatMe
 
 	if (msg->state == LinphoneChatMessageStateDelivered || msg->state == LinphoneChatMessageStateNotDelivered) {
 		// msg is not transient anymore, we can remove it from our transient list and unref it
-		msg->chat_room->transient_messages = ms_list_remove(msg->chat_room->transient_messages, msg);
+		msg->chat_room->transient_messages = bctbx_list_remove(msg->chat_room->transient_messages, msg);
 		linphone_chat_message_unref(msg);
 	}
 }
@@ -671,7 +738,7 @@ bool_t linphone_chat_room_is_remote_composing(const LinphoneChatRoom *cr) {
 }
 
 LinphoneCore *linphone_chat_room_get_lc(LinphoneChatRoom *cr) {
-	return cr->lc;
+	return linphone_chat_room_get_core(cr);
 }
 
 LinphoneCore *linphone_chat_room_get_core(LinphoneChatRoom *cr) {
@@ -700,7 +767,7 @@ LinphoneChatMessage *linphone_chat_room_create_message_2(LinphoneChatRoom *cr, c
 														 const char *external_body_url, LinphoneChatMessageState state,
 														 time_t time, bool_t is_read, bool_t is_incoming) {
 	LinphoneChatMessage *msg = linphone_chat_room_create_message(cr, message);
-	LinphoneCore *lc = linphone_chat_room_get_lc(cr);
+	LinphoneCore *lc = linphone_chat_room_get_core(cr);
 	msg->external_body_url = external_body_url ? ms_strdup(external_body_url) : NULL;
 	msg->time = time;
 	msg->is_read = is_read;
@@ -808,17 +875,55 @@ static void linphone_chat_room_send_is_composing_notification(LinphoneChatRoom *
 	}
 }
 
+static char* utf8_to_char(uint32_t ic) {
+	char *result = ms_malloc(sizeof(char) * 5);
+	int size = 0;
+	if (ic < 0x80) {
+		result[0] = ic;
+		size = 1;
+	} else if (ic < 0x800) {
+		result[1] = 0x80 + ((ic & 0x3F));
+		result[0] = 0xC0 + ((ic >> 6) & 0x1F);
+		size = 2;
+	} else if (ic < 0x100000) {
+		result[2] = 0x80 + (ic & 0x3F);
+		result[1] = 0x80 + ((ic >> 6) & 0x3F);
+		result[0] = 0xE0 + ((ic >> 12) & 0xF);
+		size = 3;
+	} else if (ic < 0x110000) {
+		result[3] = 0x80 + (ic & 0x3F);
+		result[2] = 0x80 + ((ic >> 6) & 0x3F);
+		result[1] = 0x80 + ((ic >> 12) & 0x3F);
+		result[0] = 0xF0 + ((ic >> 18) & 0x7);
+		size = 4;
+	}
+	result[size] = '\0';
+	return result;
+}
+
 void linphone_core_real_time_text_received(LinphoneCore *lc, LinphoneChatRoom *cr, uint32_t character, LinphoneCall *call) {
 	uint32_t new_line = 0x2028;
+	uint32_t crlf = 0x0D0A;
+	uint32_t lf = 0x0A;
+
 	if (call && linphone_call_params_realtime_text_enabled(linphone_call_get_current_params(call))) {
+		LinphoneChatMessageCharacter *cmc = ms_new0(LinphoneChatMessageCharacter, 1);
+
 		if (cr->pending_message == NULL) {
 			cr->pending_message = linphone_chat_room_create_message(cr, "");
 		}
 
-		if (character == new_line) {
+		cmc->value = character;
+		cmc->has_been_read = FALSE;
+		cr->received_rtt_characters = bctbx_list_append(cr->received_rtt_characters, (void *)cmc);
+
+		cr->remote_is_composing = LinphoneIsComposingActive;
+		linphone_core_notify_is_composing_received(cr->lc, cr);
+
+		if (character == new_line || character == crlf || character == lf) {
 			// End of message
 			LinphoneChatMessage *msg = cr->pending_message;
-			ms_message("New line received, forge a message with content %s", cr->pending_message->message);
+			ms_debug("New line received, forge a message with content %s", cr->pending_message->message);
 
 			linphone_chat_message_set_from(msg, cr->peer_url);
 			if (msg->to)
@@ -829,7 +934,10 @@ void linphone_core_real_time_text_received(LinphoneCore *lc, LinphoneChatRoom *c
 			msg->state = LinphoneChatMessageStateDelivered;
 			msg->is_read = FALSE;
 			msg->dir = LinphoneChatMessageIncoming;
-			msg->storage_id = linphone_chat_message_store(msg);
+
+			if (lp_config_get_int(lc->config, "misc", "store_rtt_messages", 1) == 1) {
+				msg->storage_id = linphone_chat_message_store(msg);
+			}
 
 			if (cr->unread_count < 0) cr->unread_count = 1;
 			else cr->unread_count++;
@@ -837,34 +945,64 @@ void linphone_core_real_time_text_received(LinphoneCore *lc, LinphoneChatRoom *c
 			linphone_chat_room_message_received(cr, lc, msg);
 			linphone_chat_message_unref(msg);
 			cr->pending_message = NULL;
+			cr->received_rtt_characters = bctbx_list_free(cr->received_rtt_characters);
 		} else {
-			char *value = ms_strdup_printf("%c%c%c%c",((char*)&character)[0],((char*)&character)[1],((char*)&character)[2],((char*)&character)[3]);
+			char *value = utf8_to_char(character);
 			cr->pending_message->message = ms_strcat_printf(cr->pending_message->message, value);
-			ms_message("Received RTT character: %s (%lu), pending text is %s", value, (unsigned long)character, cr->pending_message->message);
+			ms_debug("Received RTT character: %s (%lu), pending text is %s", value, (unsigned long)character, cr->pending_message->message);
 			ms_free(value);
-
-			cr->remote_is_composing = LinphoneIsComposingActive;
-			linphone_core_notify_is_composing_received(cr->lc, cr);
 		}
 	}
 }
 
 uint32_t linphone_chat_room_get_char(const LinphoneChatRoom *cr) {
-	if (cr->pending_message && strlen(cr->pending_message->message) > 0) {
-		return cr->pending_message->message[strlen(cr->pending_message->message)-1];
+	if (cr && cr->received_rtt_characters) {
+		bctbx_list_t *characters = cr->received_rtt_characters;
+		while (characters != NULL) {
+			LinphoneChatMessageCharacter *cmc = (LinphoneChatMessageCharacter *)characters->data;
+			if (!cmc->has_been_read) {
+				cmc->has_been_read = TRUE;
+				return cmc->value;
+			}
+			characters = bctbx_list_next(characters);
+		}
 	}
 	return 0;
 }
 
-int linphone_chat_message_put_char(LinphoneChatMessage *msg, uint32_t charater) {
+int linphone_chat_message_put_char(LinphoneChatMessage *msg, uint32_t character) {
 	LinphoneChatRoom *cr = linphone_chat_message_get_chat_room(msg);
 	LinphoneCall *call = cr->call;
+	LinphoneCore *lc = cr->lc;
+	uint32_t new_line = 0x2028;
+	uint32_t crlf = 0x0D0A;
+	uint32_t lf = 0x0A;
 
 	if (!call || !call->textstream) {
 		return -1;
 	}
 
-	text_stream_putchar32(call->textstream, charater);
+	if (character == new_line || character == crlf || character == lf) {
+		if (lc && lp_config_get_int(lc->config, "misc", "store_rtt_messages", 1) == 1) {
+			ms_debug("New line sent, forge a message with content %s", msg->message);
+			msg->time = ms_time(0);
+			msg->state = LinphoneChatMessageStateDelivered;
+			msg->is_read = TRUE;
+			msg->dir = LinphoneChatMessageOutgoing;
+			if (msg->from) linphone_address_destroy(msg->from);
+			msg->from = linphone_address_new(linphone_core_get_identity(lc));
+			msg->storage_id = linphone_chat_message_store(msg);
+			ms_free(msg->message);
+			msg->message = NULL;
+		}
+	} else {
+		char *value = utf8_to_char(character);
+		msg->message = ms_strcat_printf(msg->message, value);
+		ms_debug("Sent RTT character: %s (%lu), pending text is %s", value, (unsigned long)character, msg->message);
+		ms_free(value);
+	}
+
+	text_stream_putchar32(call->textstream, character);
 	return 0;
 }
 
