@@ -30,7 +30,7 @@
 #define FILE_TRANSFER_KEY_SIZE 32
 
 static bool_t file_transfer_in_progress_and_valid(LinphoneChatMessage* msg) {
-	return (msg->chat_room && msg->http_request && !belle_http_request_is_cancelled(msg->http_request));
+	return (msg->chat_room && msg->chat_room->lc && msg->http_request && !belle_http_request_is_cancelled(msg->http_request));
 }
 
 static void _release_http_request(LinphoneChatMessage* msg) {
@@ -172,7 +172,8 @@ static void linphone_chat_message_process_response_from_post_file(void *data,
 			belle_sip_body_handler_t *first_part_bh;
 
 			/* shall we encrypt the file */
-			if (linphone_core_lime_for_file_sharing_enabled(msg->chat_room->lc)) {
+			if (linphone_chat_room_lime_available(msg->chat_room) &&
+			 linphone_core_lime_for_file_sharing_enabled(msg->chat_room->lc)) {
 				char keyBuffer
 					[FILE_TRANSFER_KEY_SIZE]; /* temporary storage of generated key: 192 bits of key + 64 bits of
 												 initial vector */
@@ -215,8 +216,7 @@ static void linphone_chat_message_process_response_from_post_file(void *data,
 												  linphone_content_get_subtype(msg->file_transfer_information)));
 
 			/* insert it in a multipart body handler which will manage the boundaries of multipart msg */
-			bh = belle_sip_multipart_body_handler_new(linphone_chat_message_file_transfer_on_progress, msg,
-													  first_part_bh);
+			bh = belle_sip_multipart_body_handler_new(linphone_chat_message_file_transfer_on_progress, msg, first_part_bh, NULL);
 
 			linphone_chat_message_ref(msg);
 			_release_http_request(msg);
@@ -291,20 +291,22 @@ static void linphone_chat_message_process_response_from_post_file(void *data,
 					msg->message = ms_strdup(body);
 				}
 				msg->content_type = ms_strdup("application/vnd.gsma.rcs-ft-http+xml");
-				linphone_chat_message_set_state(msg, LinphoneChatMessageStateFileTransferDone);
 				linphone_chat_message_ref(msg);
+				linphone_chat_message_set_state(msg, LinphoneChatMessageStateFileTransferDone);
 				_release_http_request(msg);
 				_linphone_chat_room_send_message(msg->chat_room, msg);
 				linphone_chat_message_unref(msg);
 			} else {
 				ms_warning("Received empty response from server, file transfer failed");
-				linphone_chat_message_set_state(msg, LinphoneChatMessageStateNotDelivered);
+				linphone_chat_message_update_state(msg, LinphoneChatMessageStateNotDelivered);
 				_release_http_request(msg);
+				linphone_chat_message_unref(msg);
 			}
 		} else {
 			ms_warning("Unhandled HTTP code response %d for file transfer", code);
-			linphone_chat_message_set_state(msg, LinphoneChatMessageStateNotDelivered);
+			linphone_chat_message_update_state(msg, LinphoneChatMessageStateNotDelivered);
 			_release_http_request(msg);
+			linphone_chat_message_unref(msg);
 		}
 	}
 }
@@ -316,18 +318,23 @@ const LinphoneContent *linphone_chat_message_get_file_transfer_information(const
 static void on_recv_body(belle_sip_user_body_handler_t *bh, belle_sip_message_t *m, void *data, size_t offset,
 						 const uint8_t *buffer, size_t size) {
 	LinphoneChatMessage *msg = (LinphoneChatMessage *)data;
-	LinphoneCore *lc = msg->chat_room->lc;
+	LinphoneCore *lc;
 
+	if (!msg->chat_room) {
+		linphone_chat_message_cancel_file_transfer(msg);
+		return;
+	}
+	lc = msg->chat_room->lc;
+
+	if (lc == NULL){
+		return; /*might happen during linphone_core_destroy()*/
+	}
 
 	if (!msg->http_request || belle_http_request_is_cancelled(msg->http_request)) {
 		ms_warning("Cancelled request for msg [%p], ignoring %s", msg, __FUNCTION__);
 		return;
 	}
 
-	if (!msg->chat_room) {
-		linphone_chat_message_cancel_file_transfer(msg);
-	}
-
 	/* first call may be with a zero size, ignore it */
 	if (size == 0) {
 		return;
@@ -449,6 +456,9 @@ static void linphone_chat_process_response_from_get_file(void *data, const belle
 				linphone_core_notify_file_transfer_recv(lc, msg, msg->file_transfer_information, NULL, 0);
 			}
 			linphone_chat_message_set_state(msg, LinphoneChatMessageStateFileTransferDone);
+		} else if (code >= 400 && code < 500) {
+			ms_warning("File transfer failed with code %d", code);
+			linphone_chat_message_set_state(msg, LinphoneChatMessageStateFileTransferError);
 		} else {
 			ms_warning("Unhandled HTTP code response %d for file transfer", code);
 		}
@@ -458,7 +468,7 @@ static void linphone_chat_process_response_from_get_file(void *data, const belle
 
 int _linphone_chat_room_start_http_transfer(LinphoneChatMessage *msg, const char* url, const char* action, const belle_http_request_listener_callbacks_t *cbs) {
 	belle_generic_uri_t *uri = NULL;
-	char* ua;
+	const char* ua = linphone_core_get_user_agent(msg->chat_room->lc);
 
 	if (url == NULL) {
 		ms_warning("Cannot process file transfer msg: no file remote URI configured.");
@@ -470,9 +480,7 @@ int _linphone_chat_room_start_http_transfer(LinphoneChatMessage *msg, const char
 		goto error;
 	}
 
-	ua = ms_strdup_printf("%s/%s", linphone_core_get_user_agent_name(), linphone_core_get_user_agent_version());
 	msg->http_request = belle_http_request_create(action, uri, belle_sip_header_create("User-Agent", ua), NULL);
-	ms_free(ua);
 
 	if (msg->http_request == NULL) {
 		ms_warning("Could not create http request for uri %s", url);
@@ -539,7 +547,9 @@ void linphone_chat_message_start_file_download(LinphoneChatMessage *msg,
 
 void linphone_chat_message_cancel_file_transfer(LinphoneChatMessage *msg) {
 	if (msg->http_request) {
-		linphone_chat_message_set_state(msg, LinphoneChatMessageStateNotDelivered);
+		if (msg->state == LinphoneChatMessageStateInProgress) {
+			linphone_chat_message_set_state(msg, LinphoneChatMessageStateNotDelivered);
+		}
 		if (!belle_http_request_is_cancelled(msg->http_request)) {
 			if (msg->chat_room) {
 				ms_message("Canceling file transfer %s - msg [%p] chat room[%p]"
