@@ -44,7 +44,7 @@ bool_t lsd_player_loop_enabled(const LsdPlayer *p);
 void lsd_player_set_gain(LsdPlayer *p, float gain);
 LinphoneSoundDaemon *lsd_player_get_daemon(const LsdPlayer *p);
 
-LinphoneSoundDaemon * linphone_sound_daemon_new(const char *cardname, int rate, int nchannels);
+LinphoneSoundDaemon * linphone_sound_daemon_new(MSFactory* factory, const char *cardname, int rate, int nchannels);
 LsdPlayer * linphone_sound_daemon_get_player(LinphoneSoundDaemon *lsd);
 void linphone_sound_daemon_release_player(LinphoneSoundDaemon *lsd, LsdPlayer *lsdplayer);
 void linphone_sound_daemon_stop_all_players(LinphoneSoundDaemon *obj);
@@ -71,7 +71,7 @@ typedef void (*LinphoneEcCalibrationAudioUninit)(void *data);
  *
  * Start an echo calibration of the sound devices, in order to find adequate settings for the echo canceller automatically.
 **/
-int linphone_core_start_echo_calibration(LinphoneCore *lc, LinphoneEcCalibrationCallback cb,
+LINPHONE_PUBLIC int linphone_core_start_echo_calibration(LinphoneCore *lc, LinphoneEcCalibrationCallback cb,
 					 LinphoneEcCalibrationAudioInit audio_init_cb, LinphoneEcCalibrationAudioUninit audio_uninit_cb, void *cb_data);
 /**
  * @ingroup IOS
@@ -96,14 +96,14 @@ void linphone_core_remove_iterate_hook(LinphoneCore *lc, LinphoneCoreIterateHook
  *@param iso country code alpha2
  *@return call country code or -1 if not found
  */
-int linphone_dial_plan_lookup_ccc_from_iso(const char* iso); 
+LINPHONE_PUBLIC	int linphone_dial_plan_lookup_ccc_from_iso(const char* iso); 
 /**
  * @ingroup misc
  *Function to get  call country code from  an e164 number, ex: +33952650121 will return 33
  *@param e164 phone number
  *@return call country code or -1 if not found
  */
-int linphone_dial_plan_lookup_ccc_from_e164(const char* e164);
+LINPHONE_PUBLIC	int linphone_dial_plan_lookup_ccc_from_e164(const char* e164);
 
 #ifdef __cplusplus
 }
