@@ -24,18 +24,21 @@
 
 #ifndef _PRIVATE_H
 #define _PRIVATE_H
-#ifdef __cplusplus
 
-extern "C" {
-#endif
 #include "linphonecore.h"
 #include "linphonefriend.h"
+#include "friendlist.h"
 #include "linphone_tunnel.h"
 #include "linphonecore_utils.h"
+#include "conference.h"
 #include "sal/sal.h"
 #include "sipsetup.h"
 #include "quality_reporting.h"
+#include "ringtoneplayer.h"
+#include "vcard.h"
 
+#include <bctoolbox/port.h>
+#include <bctoolbox/vfs.h>
 #include <belle-sip/object.h>
 #include <belle-sip/dict.h>
 
@@ -51,9 +54,6 @@ extern "C" {
 #include "upnp.h"
 #endif //BUILD_UPNP
 
-#ifdef MSG_STORAGE_ENABLED
-#include "sqlite3.h"
-#endif
 
 #ifndef LIBLINPHONE_VERSION
 #define LIBLINPHONE_VERSION LINPHONE_VERSION
@@ -110,6 +110,25 @@ extern "C" {
 #endif
 #endif
 #endif
+#ifdef _MSC_VER
+#if (_MSC_VER >= 1900)
+#define LINPHONE_MSC_VER_GREATER_19
+#endif
+#endif
+
+#include <libxml/xmlreader.h>
+#include <libxml/xmlwriter.h>
+#include <libxml/xpath.h>
+#include <libxml/xpathInternals.h>
+
+
+#ifdef SQLITE_STORAGE_ENABLED
+#include <sqlite3.h>
+#endif
+
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 struct _LinphoneCallParams{
 	belle_sip_object_t base;
@@ -138,17 +157,21 @@ struct _LinphoneCallParams{
 	bool_t has_audio;
 	bool_t has_video;
 	bool_t avpf_enabled; /* RTCP feedback messages are enabled */
+	bool_t implicit_rtcp_fb;
+
 	bool_t real_early_media; /*send real media even during early media (for outgoing calls)*/
 	bool_t in_conference; /*in conference mode */
-
 	bool_t low_bandwidth;
 	bool_t no_user_consent;/*when set to TRUE an UPDATE request will be used instead of reINVITE*/
-	uint16_t avpf_rr_interval; /*in milliseconds*/
 
+	uint16_t avpf_rr_interval; /*in milliseconds*/
 	bool_t internal_call_update; /*use mark that call update was requested internally (might be by ice) - unused for the moment*/
 	bool_t video_multicast_enabled;
+
 	bool_t audio_multicast_enabled;
 	bool_t realtimetext_enabled;
+	bool_t update_call_when_ice_completed;
+	bool_t encryption_mandatory;
 };
 
 BELLE_SIP_DECLARE_VPTR(LinphoneCallParams);
@@ -179,6 +202,7 @@ struct _LinphoneCallLog{
 	char* call_id; /**unique id of a call*/
 	struct _LinphoneQualityReporting reporting;
 	bool_t video_enabled;
+	bool_t was_conference; /**<That call was a call with a conference server */
 	unsigned int storage_id;
 };
 
@@ -213,7 +237,6 @@ struct _LinphoneChatMessage {
 	LinphoneChatMessageCbs *callbacks;
 	LinphoneChatMessageDir dir;
 	char* message;
-	LinphoneChatMessageStateChangedCb message_state_changed_cb;
 	void* message_state_changed_user_data;
 	void* message_userdata;
 	char* appdata;
@@ -231,6 +254,20 @@ struct _LinphoneChatMessage {
 	belle_http_request_t *http_request; /**< keep a reference to the http_request in case of file transfer in order to be able to cancel the transfer */
 	belle_http_request_listener_t *http_listener; /* our listener, only owned by us*/
 	char *file_transfer_filepath;
+
+#if defined(__clang__) || ((__GNUC__ == 4 && __GNUC_MINOR__ >= 6) || __GNUC__ > 4)
+#pragma GCC diagnostic push
+#endif
+#if defined(__clang__) || defined(__GNUC__)
+#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#endif
+#ifdef _MSC_VER
+#pragma deprecated(message_state_changed_cb)
+#endif
+	LinphoneChatMessageStateChangedCb message_state_changed_cb;
+#if defined(__clang__) || ((__GNUC__ == 4 && __GNUC_MINOR__ >= 6) || __GNUC__ > 4)
+#pragma GCC diagnostic pop
+#endif
 };
 
 BELLE_SIP_DECLARE_VPTR(LinphoneChatMessage);
@@ -310,7 +347,9 @@ struct _LinphoneCall{
 	belle_sip_source_t *dtmfs_timer; /*DTMF timer needed to send a DTMF sequence*/
 
 	char *dtls_certificate_fingerprint; /**> This fingerprint is computed during stream init and is stored in call to be used when making local media description */
+	char *onhold_file; /*set if a on-hold file is to be played*/
 	LinphoneChatRoom *chat_room;
+	LinphoneConference *conf_ref; /**> Point on the associated conference if this call is part of a conference. NULL instead. */
 	bool_t refer_pending;
 	bool_t expect_media_in_ack;
 	bool_t audio_muted;
@@ -328,6 +367,7 @@ struct _LinphoneCall{
 
 	bool_t paused_by_app;
 	bool_t broken; /*set to TRUE when the call is in broken state due to network disconnection or transport */
+	bool_t defer_notify_incoming;
 };
 
 BELLE_SIP_DECLARE_VPTR(LinphoneCall);
@@ -360,7 +400,6 @@ void linphone_core_write_auth_info(LinphoneCore *lc, LinphoneAuthInfo *ai);
 const LinphoneAuthInfo *_linphone_core_find_auth_info(LinphoneCore *lc, const char *realm, const char *username, const char *domain, bool_t ignore_realm);
 
 void linphone_core_update_proxy_register(LinphoneCore *lc);
-void linphone_core_refresh_subscribes(LinphoneCore *lc);
 int linphone_core_abort_call(LinphoneCore *lc, LinphoneCall *call, const char *error);
 const char *linphone_core_get_nat_address_resolved(LinphoneCore *lc);
 
@@ -369,26 +408,47 @@ void linphone_proxy_config_set_state(LinphoneProxyConfig *cfg, LinphoneRegistrat
 void linphone_proxy_config_stop_refreshing(LinphoneProxyConfig *obj);
 void linphone_proxy_config_write_all_to_config_file(LinphoneCore *lc);
 void _linphone_proxy_config_release(LinphoneProxyConfig *cfg);
+void _linphone_proxy_config_unpublish(LinphoneProxyConfig *obj);
+
 /*
  * returns service route as defined in as defined by rfc3608, might be a list instead of just one.
  * Can be NULL
  * */
 const LinphoneAddress* linphone_proxy_config_get_service_route(const LinphoneProxyConfig* cfg);
 
+void linphone_friend_list_invalidate_subscriptions(LinphoneFriendList *list);
+void linphone_friend_list_notify_presence_received(LinphoneFriendList *list, LinphoneEvent *lev, const LinphoneContent *body);
+void linphone_friend_list_subscription_state_changed(LinphoneCore *lc, LinphoneEvent *lev, LinphoneSubscriptionState state);
+void _linphone_friend_list_release(LinphoneFriendList *list);
+void linphone_friend_invalidate_subscription(LinphoneFriend *lf);
 void linphone_friend_close_subscriptions(LinphoneFriend *lf);
+void _linphone_friend_release(LinphoneFriend *lf);
 void linphone_friend_update_subscribes(LinphoneFriend *fr, LinphoneProxyConfig *cfg, bool_t only_when_registered);
 void linphone_friend_notify(LinphoneFriend *lf, LinphonePresenceModel *presence);
+void linphone_friend_apply(LinphoneFriend *fr, LinphoneCore *lc);
 void linphone_friend_add_incoming_subscription(LinphoneFriend *lf, SalOp *op);
 void linphone_friend_remove_incoming_subscription(LinphoneFriend *lf, SalOp *op);
-LinphoneFriend *linphone_find_friend_by_inc_subscribe(MSList *l, SalOp *op);
-LinphoneFriend *linphone_find_friend_by_out_subscribe(MSList *l, SalOp *op);
+LinphoneFriend *linphone_friend_list_find_friend_by_inc_subscribe(const LinphoneFriendList *list, SalOp *op);
+LinphoneFriend *linphone_friend_list_find_friend_by_out_subscribe(const LinphoneFriendList *list, SalOp *op);
+LinphoneFriend *linphone_core_find_friend_by_out_subscribe(const LinphoneCore *lc, SalOp *op);
+LinphoneFriend *linphone_core_find_friend_by_inc_subscribe(const LinphoneCore *lc, SalOp *op);
 MSList *linphone_find_friend_by_address(MSList *fl, const LinphoneAddress *addr, LinphoneFriend **lf);
 bool_t linphone_core_should_subscribe_friends_only_when_registered(const LinphoneCore *lc);
 void linphone_core_update_friends_subscriptions(LinphoneCore *lc, LinphoneProxyConfig *cfg, bool_t only_when_registered);
+void linphone_core_friends_storage_init(LinphoneCore *lc);
+void linphone_core_friends_storage_close(LinphoneCore *lc);
+void linphone_core_store_friend_in_db(LinphoneCore *lc, LinphoneFriend *lf);
+void linphone_core_remove_friend_from_db(LinphoneCore *lc, LinphoneFriend *lf);
+void linphone_core_store_friends_list_in_db(LinphoneCore *lc, LinphoneFriendList *list);
+void linphone_core_remove_friends_list_from_db(LinphoneCore *lc, LinphoneFriendList *list);
+LINPHONE_PUBLIC MSList* linphone_core_fetch_friends_from_db(LinphoneCore *lc, LinphoneFriendList *list);
+LINPHONE_PUBLIC MSList* linphone_core_fetch_friends_lists_from_db(LinphoneCore *lc);
+LINPHONE_PUBLIC LinphoneFriendListStatus linphone_friend_list_import_friend(LinphoneFriendList *list, LinphoneFriend *lf, bool_t synchronize);
 
+int linphone_parse_host_port(const char *input, char *host, size_t hostlen, int *port);
 int parse_hostname_to_addr(const char *server, struct sockaddr_storage *ss, socklen_t *socklen, int default_port);
 
-bool_t host_has_ipv6_network();
+bool_t host_has_ipv6_network(void);
 bool_t lp_spawn_command_line_sync(const char *command, char **result,int *command_ret);
 
 static MS2_INLINE int get_min_bandwidth(int dbw, int ubw){
@@ -431,7 +491,7 @@ void linphone_process_authentication(LinphoneCore* lc, SalOp *op);
 void linphone_authentication_ok(LinphoneCore *lc, SalOp *op);
 void linphone_subscription_new(LinphoneCore *lc, SalOp *op, const char *from);
 void linphone_core_send_presence(LinphoneCore *lc, LinphonePresenceModel *presence);
-void linphone_notify_parse_presence(SalOp *op, const char *content_type, const char *content_subtype, const char *body, SalPresenceModel **result);
+void linphone_notify_parse_presence(const char *content_type, const char *content_subtype, const char *body, SalPresenceModel **result);
 void linphone_notify_convert_presence_to_xml(SalOp *op, SalPresenceModel *presence, const char *contact, char **content);
 void linphone_notify_recv(LinphoneCore *lc, SalOp *op, SalSubscribeStatus ss, SalPresenceModel *model);
 void linphone_proxy_config_process_authentication_failure(LinphoneCore *lc, SalOp *op);
@@ -449,12 +509,15 @@ void linphone_core_resolve_stun_server(LinphoneCore *lc);
 LINPHONE_PUBLIC const struct addrinfo *linphone_core_get_stun_server_addrinfo(LinphoneCore *lc);
 void linphone_core_adapt_to_network(LinphoneCore *lc, int ping_time_ms, LinphoneCallParams *params);
 int linphone_core_gather_ice_candidates(LinphoneCore *lc, LinphoneCall *call);
+LINPHONE_PUBLIC void linphone_core_enable_forced_ice_relay(LinphoneCore *lc, bool_t enable);
+LINPHONE_PUBLIC void linphone_core_enable_short_turn_refresh(LinphoneCore *lc, bool_t enable);
 void linphone_core_update_ice_state_in_call_stats(LinphoneCall *call);
 void linphone_call_stats_fill(LinphoneCallStats *stats, MediaStream *ms, OrtpEvent *ev);
 void linphone_call_stop_ice_for_inactive_streams(LinphoneCall *call, SalMediaDescription *result);
-void _update_local_media_description_from_ice(SalMediaDescription *desc, IceSession *session);
+void _update_local_media_description_from_ice(SalMediaDescription *desc, IceSession *session, bool_t use_nortpproxy);
 void linphone_call_update_local_media_description_from_ice_or_upnp(LinphoneCall *call);
-void linphone_call_update_ice_from_remote_media_description(LinphoneCall *call, const SalMediaDescription *md);
+void linphone_call_update_ice_from_remote_media_description(LinphoneCall *call, const SalMediaDescription *md, bool_t is_offer);
+void linphone_call_clear_unused_ice_candidates(LinphoneCall *call, const SalMediaDescription *md);
 bool_t linphone_core_media_description_contains_video_stream(const SalMediaDescription *md);
 
 void linphone_core_send_initial_subscribes(LinphoneCore *lc);
@@ -466,7 +529,7 @@ void linphone_proxy_config_update(LinphoneProxyConfig *cfg);
 LinphoneProxyConfig * linphone_core_lookup_known_proxy(LinphoneCore *lc, const LinphoneAddress *uri);
 const char *linphone_core_find_best_identity(LinphoneCore *lc, const LinphoneAddress *to);
 int linphone_core_get_local_ip_for(int type, const char *dest, char *result);
-void linphone_core_get_local_ip(LinphoneCore *lc, int af, const char *dest, char *result);
+LINPHONE_PUBLIC void linphone_core_get_local_ip(LinphoneCore *lc, int af, const char *dest, char *result);
 
 LinphoneProxyConfig *linphone_proxy_config_new_from_config_file(LinphoneCore *lc, int index);
 void linphone_proxy_config_write_to_config_file(struct _LpConfig* config,LinphoneProxyConfig *obj, int index);
@@ -507,7 +570,7 @@ int linphone_core_start_accept_call_update(LinphoneCore *lc, LinphoneCall *call,
 void linphone_core_notify_incoming_call(LinphoneCore *lc, LinphoneCall *call);
 bool_t linphone_core_incompatible_security(LinphoneCore *lc, SalMediaDescription *md);
 extern SalCallbacks linphone_sal_callbacks;
-bool_t linphone_core_rtcp_enabled(const LinphoneCore *lc);
+LINPHONE_PUBLIC bool_t linphone_core_rtcp_enabled(const LinphoneCore *lc);
 bool_t linphone_core_symmetric_rtp_enabled(LinphoneCore*lc);
 
 void linphone_core_queue_task(LinphoneCore *lc, belle_sip_source_func_t task_fun, void *data, const char *task_description);
@@ -559,8 +622,8 @@ struct _LinphoneProxyConfig
 	int auth_failures;
 	char *dial_prefix;
 	LinphoneRegistrationState state;
-	SalOp *publish_op;
 	LinphoneAVPFMode avpf_mode;
+	LinphoneNatPolicy *nat_policy;
 
 	bool_t commit;
 	bool_t reg_sendregister;
@@ -579,7 +642,11 @@ struct _LinphoneProxyConfig
 	LinphoneAddress *saved_identity;
 	/*---*/
 	LinphoneAddress *pending_contact; /*use to store previous contact in case of network failure*/
+	LinphoneEvent *long_term_event;
+	unsigned long long previous_publish_config_hash[2];
 
+	char *refkey;
+	char *sip_etag; /*publish context*/
 };
 
 BELLE_SIP_DECLARE_VPTR(LinphoneProxyConfig);
@@ -615,8 +682,14 @@ struct _LinphoneChatRoom{
 	belle_sip_source_t *composing_refresh_timer;
 	LinphoneCall *call;
 	LinphoneChatMessage *pending_message;
+	MSList *received_rtt_characters;
 };
 
+typedef struct _LinphoneChatMessageCharacter {
+	uint32_t value;
+	bool_t has_been_read;
+} LinphoneChatMessageCharacter;
+
 BELLE_SIP_DECLARE_VPTR(LinphoneChatRoom);
 
 
@@ -636,10 +709,47 @@ struct _LinphoneFriend{
 	bool_t inc_subscribe_pending;
 	bool_t commit;
 	bool_t initial_subscribes_sent; /*used to know if initial subscribe message was sent or not*/
+	bool_t presence_received;
+	LinphoneVcard *vcard;
+	unsigned int storage_id;
+	LinphoneFriendList *friend_list;
+	LinphoneSubscriptionState out_sub_state;
 };
 
 BELLE_SIP_DECLARE_VPTR(LinphoneFriend);
 
+struct _LinphoneFriendListCbs {
+	belle_sip_object_t base;
+	void *user_data;
+	LinphoneFriendListCbsContactCreatedCb contact_created_cb;
+	LinphoneFriendListCbsContactDeletedCb contact_deleted_cb;
+	LinphoneFriendListCbsContactUpdatedCb contact_updated_cb;
+	LinphoneFriendListCbsSyncStateChangedCb sync_state_changed_cb;
+};
+
+BELLE_SIP_DECLARE_VPTR(LinphoneFriendListCbs);
+
+struct _LinphoneFriendList {
+	belle_sip_object_t base;
+	void *user_data;
+	LinphoneCore *lc;
+	LinphoneEvent *event;
+	char *display_name;
+	char *rls_uri;
+	MSList *friends;
+	unsigned char *content_digest;
+	int expected_notification_version;
+	unsigned int storage_id;
+	char *uri;
+	MSList *dirty_friends_to_update;
+	int revision;
+	LinphoneFriendListCbs *cbs;
+	bool_t enable_subscriptions;
+};
+
+BELLE_SIP_DECLARE_VPTR(LinphoneFriendList);
+
+
 
 typedef struct sip_config
 {
@@ -698,7 +808,6 @@ typedef struct net_config
 {
 	char *nat_address; /* may be IP or host name */
 	char *nat_address_ip; /* ip translated from nat_address */
-	char *stun_server;
 	struct addrinfo *stun_addrinfo;
 	SalResolverContext * stun_res;
 	int download_bw;
@@ -776,16 +885,6 @@ typedef struct autoreplier_config
 	const char *message;		/* the path of the file to be played */
 }autoreplier_config_t;
 
-struct _LinphoneConference{
-	MSAudioConference *conf;
-	AudioStream *local_participant;
-	MSAudioEndpoint *local_endpoint;
-	MSAudioEndpoint *record_endpoint;
-	RtpProfile *local_dummy_profile;
-	bool_t local_muted;
-	bool_t terminated;
-};
-
 
 typedef struct _LinphoneToneDescription{
 	LinphoneReason reason;
@@ -800,7 +899,6 @@ void linphone_core_play_call_error_tone(LinphoneCore *lc, LinphoneReason reason)
 void _linphone_core_set_tone(LinphoneCore *lc, LinphoneReason reason, LinphoneToneID id, const char *audiofile);
 const char *linphone_core_get_tone_file(const LinphoneCore *lc, LinphoneToneID id);
 int _linphone_core_accept_call_update(LinphoneCore *lc, LinphoneCall *call, const LinphoneCallParams *params, LinphoneCallState next_state, const char *state_info);
-typedef struct _LinphoneConference LinphoneConference;
 
 typedef struct _LinphoneTaskList{
 	MSList *hooks;
@@ -814,6 +912,7 @@ void linphone_task_list_free(LinphoneTaskList *t);
 
 struct _LinphoneCore
 {
+	MSFactory* factory;
 	MSList* vtable_refs;
 	Sal *sal;
 	LinphoneGlobalState state;
@@ -831,7 +930,7 @@ struct _LinphoneCore
 	ui_config_t ui_conf;
 	autoreplier_config_t autoreplier_conf;
 	LinphoneProxyConfig *default_proxy;
-	MSList *friends;
+	MSList *friends_lists;
 	MSList *auth_info;
 	struct _RingStream *ringstream;
 	time_t dmfs_playing_start_time;
@@ -841,7 +940,7 @@ struct _LinphoneCore
 	MSList *queued_calls;	/* used by the autoreplier */
 	MSList *call_logs;
 	MSList *chatrooms;
-	int max_call_logs;
+	unsigned int max_call_logs;
 	int missed_calls;
 	VideoPreview *previewstream;
 	struct _MSEventQueue *msevq;
@@ -862,11 +961,12 @@ struct _LinphoneCore
 	time_t netup_time; /*time when network went reachable */
 	struct _EcCalibrator *ecc;
 	LinphoneTaskList hooks; /*tasks periodically executed in linphone_core_iterate()*/
-	LinphoneConference conf_ctx;
+	LinphoneConference *conf_ctx;
 	char* zrtp_secrets_cache;
 	char* user_certificates_path;
 	LinphoneVideoPolicy video_policy;
 	time_t network_last_check;
+	LinphoneNatPolicy *nat_policy;
 
 	bool_t use_files;
 	bool_t apply_nat_settings;
@@ -875,13 +975,18 @@ struct _LinphoneCore
 
 	bool_t preview_finished;
 	bool_t auto_net_state_mon;
-	bool_t network_reachable;
-	bool_t network_reachable_to_be_notified; /*set to true when state must be notified in next iterate*/
+	bool_t sip_network_reachable;
+	bool_t media_network_reachable;
 
+	bool_t network_reachable_to_be_notified; /*set to true when state must be notified in next iterate*/
 	bool_t use_preview_window;
 	bool_t network_last_status;
 	bool_t ringstream_autorelease;
+
 	bool_t vtables_running;
+	bool_t send_call_stats_periodical_updates;
+	bool_t forced_ice_relay;
+	bool_t short_turn_refresh;
 	char localip[LINPHONE_IPADDR_SIZE];
 	int device_rotation;
 	int max_calls;
@@ -889,19 +994,19 @@ struct _LinphoneCore
 	char* device_id;
 	MSList *last_recv_msg_ids;
 	char *chat_db_file;
-#ifdef MSG_STORAGE_ENABLED
-	sqlite3 *db;
-	bool_t debug_storage;
-#endif
 	char *logs_db_file;
-#ifdef CALL_LOGS_STORAGE_ENABLED
+	char *friends_db_file;
+#ifdef SQLITE_STORAGE_ENABLED
+	sqlite3 *db;
 	sqlite3 *logs_db;
+	sqlite3 *friends_db;
+	bool_t debug_storage;
 #endif
 #ifdef BUILD_UPNP
 	UpnpContext *upnp;
 #endif //BUILD_UPNP
 	belle_http_provider_t *http_provider;
-	belle_tls_verify_policy_t *http_verify_policy;
+	belle_tls_crypto_config_t *http_crypto_config;
 	belle_http_request_listener_t *provisioning_http_listener;
 	MSList *tones;
 	LinphoneReason chat_deny_code;
@@ -909,6 +1014,7 @@ struct _LinphoneCore
 	const char **supported_formats;
 	LinphoneContent *log_collection_upload_information;
 	LinphoneCoreVTable *current_vtable; // the latest vtable to call a callback, see linphone_core_get_current_vtable
+	LinphoneRingtonePlayer *ringtoneplayer;
 #ifdef ANDROID
 	jobject wifi_lock;
 	jclass wifi_lock_class;
@@ -919,10 +1025,12 @@ struct _LinphoneCore
 	jmethodID multicast_lock_acquire_id;
 	jmethodID multicast_lock_release_id;
 #endif
+	LinphoneVcardContext *vcard_context;
 };
 
 
 struct _LinphoneEvent{
+	belle_sip_object_t base;
 	LinphoneSubscriptionDir dir;
 	LinphoneCore *lc;
 	SalOp *op;
@@ -930,13 +1038,14 @@ struct _LinphoneEvent{
 	LinphoneSubscriptionState subscription_state;
 	LinphonePublishState publish_state;
 	void *userdata;
-	int refcnt;
 	char *name;
 	int expires;
 	bool_t terminating;
 	bool_t is_out_of_dialog_op; /*used for out of dialog notify*/
+	bool_t internal;
 };
 
+BELLE_SIP_DECLARE_VPTR(LinphoneEvent);
 
 LinphoneTunnel *linphone_core_tunnel_new(LinphoneCore *lc);
 void linphone_tunnel_destroy(LinphoneTunnel *tunnel);
@@ -959,9 +1068,10 @@ void linphone_core_update_streams(LinphoneCore *lc, LinphoneCall *call, SalMedia
 bool_t linphone_core_is_payload_type_usable_for_bandwidth(LinphoneCore *lc, const PayloadType *pt,  int bandwidth_limit);
 
 #define linphone_core_ready(lc) ((lc)->state==LinphoneGlobalOn || (lc)->state==LinphoneGlobalShutdown)
-void _linphone_core_configure_resolver();
+void _linphone_core_configure_resolver(void);
 
 struct _EcCalibrator{
+	MSFactory *factory;
 	ms_thread_t thread;
 	MSSndCard *play_card,*capt_card;
 	MSFilter *sndread,*det,*rec;
@@ -977,6 +1087,7 @@ struct _EcCalibrator{
 	unsigned int rate;
 	LinphoneEcCalibratorStatus status;
 	bool_t freq1,freq2,freq3;
+	bool_t play_cool_tones;
 };
 
 typedef struct _EcCalibrator EcCalibrator;
@@ -988,18 +1099,12 @@ void ec_calibrator_destroy(EcCalibrator *ecc);
 void linphone_call_background_tasks(LinphoneCall *call, bool_t one_second_elapsed);
 void linphone_call_set_broken(LinphoneCall *call);
 void linphone_call_repair_if_broken(LinphoneCall *call);
-void linphone_core_preempt_sound_resources(LinphoneCore *lc);
+void linphone_core_repair_calls(LinphoneCore *lc);
+int linphone_core_preempt_sound_resources(LinphoneCore *lc);
 int _linphone_core_pause_call(LinphoneCore *lc, LinphoneCall *call);
 
 /*conferencing subsystem*/
 void _post_configure_audio_stream(AudioStream *st, LinphoneCore *lc, bool_t muted);
-/* When a conference participant pause the conference he may send a music.
- * We don't want to hear that music or to send it to the other participants.
- * Use muted=yes to handle this case.
- */
-void linphone_call_add_to_conf(LinphoneCall *call, bool_t muted);
-void linphone_call_remove_from_conf(LinphoneCall *call);
-void linphone_core_conference_check_uninit(LinphoneCore *lc);
 bool_t linphone_core_sound_resources_available(LinphoneCore *lc);
 void linphone_core_notify_refer_state(LinphoneCore *lc, LinphoneCall *referer, LinphoneCall *newcall);
 unsigned int linphone_core_get_audio_features(LinphoneCore *lc);
@@ -1029,8 +1134,9 @@ int linphone_core_get_edge_ptime(LinphoneCore *lc);
 int linphone_upnp_init(LinphoneCore *lc);
 void linphone_upnp_destroy(LinphoneCore *lc);
 
-#ifdef MSG_STORAGE_ENABLED
-sqlite3 * linphone_message_storage_init();
+#ifdef SQLITE_STORAGE_ENABLED
+int _linphone_sqlite3_open(const char *db_file, sqlite3 **db);
+sqlite3 * linphone_message_storage_init(void);
 void linphone_message_storage_init_chat_rooms(LinphoneCore *lc);
 #endif
 void linphone_chat_message_store_state(LinphoneChatMessage *msg);
@@ -1045,29 +1151,37 @@ const char *linphone_core_create_uuid(LinphoneCore *lc);
 void linphone_configure_op(LinphoneCore *lc, SalOp *op, const LinphoneAddress *dest, SalCustomHeader *headers, bool_t with_contact);
 void linphone_call_create_op(LinphoneCall *call);
 int linphone_call_prepare_ice(LinphoneCall *call, bool_t incoming_offer);
-void linphone_core_notify_info_message(LinphoneCore* lc,SalOp *op, const SalBody *body);
+void linphone_core_notify_info_message(LinphoneCore* lc,SalOp *op, SalBodyHandler *body);
 LinphoneContent * linphone_content_new(void);
 LinphoneContent * linphone_content_copy(const LinphoneContent *ref);
-SalBody *sal_body_from_content(SalBody *body, const LinphoneContent *content);
+SalBodyHandler *sal_body_handler_from_content(const LinphoneContent *content);
 SalReason linphone_reason_to_sal(LinphoneReason reason);
 LinphoneReason linphone_reason_from_sal(SalReason reason);
 LinphoneEvent *linphone_event_new(LinphoneCore *lc, LinphoneSubscriptionDir dir, const char *name, int expires);
 LinphoneEvent *linphone_event_new_with_op(LinphoneCore *lc, SalOp *op, LinphoneSubscriptionDir dir, const char *name);
+void linphone_event_unpublish(LinphoneEvent *lev);
 /**
  * Useful for out of dialog notify
  * */
 LinphoneEvent *linphone_event_new_with_out_of_dialog_op(LinphoneCore *lc, SalOp *op, LinphoneSubscriptionDir dir, const char *name);
+void linphone_event_set_internal(LinphoneEvent *lev, bool_t internal);
+bool_t linphone_event_is_internal(LinphoneEvent *lev);
 void linphone_event_set_state(LinphoneEvent *lev, LinphoneSubscriptionState state);
 void linphone_event_set_publish_state(LinphoneEvent *lev, LinphonePublishState state);
 LinphoneSubscriptionState linphone_subscription_state_from_sal(SalSubscribeStatus ss);
-LinphoneContent *linphone_content_from_sal_body(const SalBody *ref);
+LinphoneContent *linphone_content_from_sal_body_handler(SalBodyHandler *ref);
 void linphone_core_invalidate_friend_subscriptions(LinphoneCore *lc);
+void linphone_core_register_offer_answer_providers(LinphoneCore *lc);
 
 
 struct _LinphoneContent {
 	belle_sip_object_t base;
 	void *user_data;
-	struct _LinphoneContentPrivate lcp;
+	SalBodyHandler *body_handler;
+	char *name; /**< used by RCS File transfer messages to store the original filename of the file to be downloaded from server */
+	char *key; /**< used by RCS File transfer messages to store the key to encrypt file if needed */
+	size_t keyLength; /**< Length of key in bytes */
+	void *cryptoContext; /**< crypto context used to encrypt file for RCS file transfer */
 	bool_t owned_fields;
 };
 
@@ -1082,6 +1196,25 @@ struct _LinphoneBuffer {
 
 BELLE_SIP_DECLARE_VPTR(LinphoneBuffer);
 
+struct _LinphoneNatPolicy {
+	belle_sip_object_t base;
+	void *user_data;
+	LinphoneCore *lc;
+	SalResolverContext *stun_resolver_context;
+	struct addrinfo *stun_addrinfo;
+	char *stun_server;
+	char *stun_server_username;
+	char *ref;
+	bool_t stun_enabled;
+	bool_t turn_enabled;
+	bool_t ice_enabled;
+	bool_t upnp_enabled;
+};
+
+BELLE_SIP_DECLARE_VPTR(LinphoneNatPolicy);
+
+void linphone_nat_policy_save_to_config(const LinphoneNatPolicy *policy);
+
 
 /*****************************************************************************
  * XML-RPC interface                                                         *
@@ -1159,9 +1292,41 @@ struct _LinphoneAccountCreator {
 
 BELLE_SIP_DECLARE_VPTR(LinphoneAccountCreator);
 
+/*****************************************************************************
+ * CardDAV interface                                                         *
+ ****************************************************************************/
+
+struct _LinphoneCardDavContext {
+	LinphoneFriendList *friend_list;
+	int ctag;
+	void *user_data;
+	LinphoneCardDavContactCreatedCb contact_created_cb;
+	LinphoneCardDavContactUpdatedCb contact_updated_cb;
+	LinphoneCardDavContactRemovedCb contact_removed_cb;
+	LinphoneCardDavSynchronizationDoneCb sync_done_cb;
+	LinphoneAuthInfo *auth_info;
+};
+
+struct _LinphoneCardDavQuery {
+	LinphoneCardDavContext *context;
+	char *url;
+	const char *method;
+	char *body;
+	const char *depth;
+	const char *ifmatch;
+	belle_http_request_listener_t *http_request_listener;
+	void *user_data;
+	LinphoneCardDavQueryType type;
+};
+
+struct _LinphoneCardDavResponse {
+	char *etag;
+	char *url;
+	char *vcard;
+};
 
 /*****************************************************************************
- * REMOTE PROVISIONING FUNCTIONS                                                     *
+ * REMOTE PROVISIONING FUNCTIONS                                             *
  ****************************************************************************/
 
 void linphone_configuring_terminated(LinphoneCore *lc, LinphoneConfiguringState state, const char *message);
@@ -1169,7 +1334,7 @@ int linphone_remote_provisioning_download_and_apply(LinphoneCore *lc, const char
 LINPHONE_PUBLIC int linphone_remote_provisioning_load_file( LinphoneCore* lc, const char* file_path);
 
 /*****************************************************************************
- * Player interface
+ * Player interface                                                          *
  ****************************************************************************/
 
 struct _LinphonePlayer{
@@ -1194,11 +1359,6 @@ void _linphone_player_destroy(LinphonePlayer *player);
  * XML UTILITY FUNCTIONS                                                     *
  ****************************************************************************/
 
-#include <libxml/xmlreader.h>
-#include <libxml/xmlwriter.h>
-#include <libxml/xpath.h>
-#include <libxml/xpathInternals.h>
-
 #define XMLPARSING_BUFFER_LEN 2048
 #define MAX_XPATH_LENGTH 256
 
@@ -1214,8 +1374,10 @@ void linphone_xmlparsing_context_destroy(xmlparsing_context_t *ctx);
 void linphone_xmlparsing_genericxml_error(void *ctx, const char *fmt, ...);
 int linphone_create_xml_xpath_context(xmlparsing_context_t *xml_ctx);
 char * linphone_get_xml_text_content(xmlparsing_context_t *xml_ctx, const char *xpath_expression);
+const char * linphone_get_xml_attribute_text_content(xmlparsing_context_t *xml_ctx, const char *xpath_expression, const char *attribute_name);
 void linphone_free_xml_text_content(const char *text);
 xmlXPathObjectPtr linphone_get_xml_xpath_object_for_node_list(xmlparsing_context_t *xml_ctx, const char *xpath_expression);
+void linphone_xml_xpath_context_init_carddav_ns(xmlparsing_context_t *xml_ctx);
 
 /*****************************************************************************
  * OTHER UTILITY FUNCTIONS                                                     *
@@ -1267,9 +1429,14 @@ BELLE_SIP_TYPE_ID(LinphoneLDAPContactProvider),
 BELLE_SIP_TYPE_ID(LinphoneLDAPContactSearch),
 BELLE_SIP_TYPE_ID(LinphoneProxyConfig),
 BELLE_SIP_TYPE_ID(LinphoneFriend),
+BELLE_SIP_TYPE_ID(LinphoneFriendList),
 BELLE_SIP_TYPE_ID(LinphoneXmlRpcRequest),
 BELLE_SIP_TYPE_ID(LinphoneXmlRpcRequestCbs),
-BELLE_SIP_TYPE_ID(LinphoneXmlRpcSession)
+BELLE_SIP_TYPE_ID(LinphoneXmlRpcSession),
+BELLE_SIP_TYPE_ID(LinphoneTunnelConfig),
+BELLE_SIP_TYPE_ID(LinphoneFriendListCbs),
+BELLE_SIP_TYPE_ID(LinphoneEvent),
+BELLE_SIP_TYPE_ID(LinphoneNatPolicy)
 BELLE_SIP_DECLARE_TYPES_END
 
 
@@ -1310,6 +1477,8 @@ void linphone_core_notify_subscription_state_changed(LinphoneCore *lc, LinphoneE
 void linphone_core_notify_publish_state_changed(LinphoneCore *lc, LinphoneEvent *lev, LinphonePublishState state);
 void linphone_core_notify_log_collection_upload_state_changed(LinphoneCore *lc, LinphoneCoreLogCollectionUploadState state, const char *info);
 void linphone_core_notify_log_collection_upload_progress_indication(LinphoneCore *lc, size_t offset, size_t total);
+void linphone_core_notify_friend_list_created(LinphoneCore *lc, LinphoneFriendList *list);
+void linphone_core_notify_friend_list_removed(LinphoneCore *lc, LinphoneFriendList *list);
 
 void set_mic_gain_db(AudioStream *st, float gain);
 void set_playback_gain_db(AudioStream *st, float gain);
@@ -1325,7 +1494,7 @@ SalStreamDir sal_dir_from_call_params_dir(LinphoneMediaDirection cpdir);
  * @param[in] content LinphoneContent object.
  * @return The key to encrypt/decrypt the file associated to this content.
  */
-const char *linphone_content_get_key(const LinphoneContent *content);
+LINPHONE_PUBLIC const char *linphone_content_get_key(const LinphoneContent *content);
 
 /**
  * Get the size of key associated with a RCS file transfer message if encrypted
@@ -1333,10 +1502,12 @@ const char *linphone_content_get_key(const LinphoneContent *content);
  * @return The key size in bytes
  */
 size_t linphone_content_get_key_size(const LinphoneContent *content);
+
 /**
  * Set the key associated with a RCS file transfer message if encrypted
  * @param[in] content LinphoneContent object.
  * @param[in] key The key to be used to encrypt/decrypt file associated to this content.
+ * @param[in] keyLength The lengh of the key.
  */
 void linphone_content_set_key(LinphoneContent *content, const char *key, const size_t keyLength);
 
@@ -1359,18 +1530,30 @@ struct _VTableReference{
 	LinphoneCoreVTable *vtable;
 	bool_t valid;
 	bool_t autorelease;
+	bool_t internal;
 };
 
 typedef struct _VTableReference  VTableReference;
 
 void v_table_reference_destroy(VTableReference *ref);
 
-void _linphone_core_add_listener(LinphoneCore *lc, LinphoneCoreVTable *vtable, bool_t autorelease);
+LINPHONE_PUBLIC void _linphone_core_add_listener(LinphoneCore *lc, LinphoneCoreVTable *vtable, bool_t autorelease, bool_t internal);
+
 #ifdef VIDEO_ENABLED
 LINPHONE_PUBLIC MSWebCam *linphone_call_get_video_device(const LinphoneCall *call);
-MSWebCam *get_nowebcam_device();
+MSWebCam *get_nowebcam_device(MSFactory *f);
 #endif
-bool_t linphone_core_lime_for_file_sharing_enabled(const LinphoneCore *lc);
+LinphoneLimeState linphone_core_lime_for_file_sharing_enabled(const LinphoneCore *lc);
+
+BELLE_SIP_DECLARE_VPTR(LinphoneTunnelConfig);
+
+int linphone_core_get_default_proxy_config_index(LinphoneCore *lc);
+
+char *linphone_presence_model_to_xml(LinphonePresenceModel *model) ;
+
+#define LINPHONE_SQLITE3_VFS "sqlite3bctbx_vfs"
+
+void linphone_call_check_ice_session(LinphoneCall *call, IceRole role, bool_t is_reinvite);
 
 #ifdef __cplusplus
 }
