@@ -61,6 +61,7 @@ SalMediaDescription *sal_media_description_new(){
 		md->streams[i].dir=SalStreamInactive;
 		md->streams[i].rtp_port = 0;
 		md->streams[i].rtcp_port = 0;
+		md->streams[i].haveZrtpHash = 0;
 	}
 	return md;
 }
@@ -68,8 +69,8 @@ SalMediaDescription *sal_media_description_new(){
 static void sal_media_description_destroy(SalMediaDescription *md){
 	int i;
 	for(i=0;i<SAL_MEDIA_DESCRIPTION_MAX_STREAMS;i++){
-		ms_list_free_with_data(md->streams[i].payloads,(void (*)(void *))payload_type_destroy);
-		ms_list_free_with_data(md->streams[i].already_assigned_payloads,(void (*)(void *))payload_type_destroy);
+		bctbx_list_free_with_data(md->streams[i].payloads,(void (*)(void *))payload_type_destroy);
+		bctbx_list_free_with_data(md->streams[i].already_assigned_payloads,(void (*)(void *))payload_type_destroy);
 		md->streams[i].payloads=NULL;
 		md->streams[i].already_assigned_payloads=NULL;
 		sal_custom_sdp_attribute_free(md->streams[i].custom_sdp_attributes);
@@ -167,7 +168,7 @@ static bool_t is_null_address(const char *addr){
 /*check for the presence of at least one stream with requested direction */
 static bool_t has_dir(const SalMediaDescription *md, SalStreamDir stream_dir){
 	int i;
-	
+
 	/* we are looking for at least one stream with requested direction, inactive streams are ignored*/
 	for(i=0;i<SAL_MEDIA_DESCRIPTION_MAX_STREAMS;++i){
 		const SalStreamDescription *ss=&md->streams[i];
@@ -219,6 +220,13 @@ bool_t sal_stream_description_has_avpf(const SalStreamDescription *sd) {
 	return FALSE;
 }
 
+bool_t sal_stream_description_has_ipv6(const SalStreamDescription *sd){
+	return strchr(sd->rtp_addr,':') != NULL;
+}
+
+bool_t sal_stream_description_has_implicit_avpf(const SalStreamDescription *sd){
+	return sd->implicit_rtcp_fb;
+}
 /*these are switch case, so that when a new proto is added we can't forget to modify this function*/
 bool_t sal_stream_description_has_srtp(const SalStreamDescription *sd) {
 	switch (sd->proto){
@@ -250,6 +258,11 @@ bool_t sal_stream_description_has_dtls(const SalStreamDescription *sd) {
 	return FALSE;
 }
 
+bool_t sal_stream_description_has_zrtp(const SalStreamDescription *sd) {
+	if (sd->haveZrtpHash==1) return TRUE;
+	return FALSE;
+}
+
 bool_t sal_media_description_has_avpf(const SalMediaDescription *md) {
 	int i;
 	if (md->nb_streams == 0) return FALSE;
@@ -260,6 +273,16 @@ bool_t sal_media_description_has_avpf(const SalMediaDescription *md) {
 	return TRUE;
 }
 
+bool_t sal_media_description_has_implicit_avpf(const SalMediaDescription *md) {
+    int i;
+    if (md->nb_streams == 0) return FALSE;
+    for (i = 0; i < SAL_MEDIA_DESCRIPTION_MAX_STREAMS; i++) {
+        if (!sal_stream_description_active(&md->streams[i])) continue;
+        if (sal_stream_description_has_implicit_avpf(&md->streams[i]) != TRUE) return FALSE;
+    }
+    return TRUE;
+}
+
 bool_t sal_media_description_has_srtp(const SalMediaDescription *md) {
 	int i;
 	if (md->nb_streams == 0) return FALSE;
@@ -280,6 +303,30 @@ bool_t sal_media_description_has_dtls(const SalMediaDescription *md) {
 	return TRUE;
 }
 
+bool_t sal_media_description_has_zrtp(const SalMediaDescription *md) {
+	int i;
+	if (md->nb_streams == 0) return FALSE;
+	for (i = 0; i < SAL_MEDIA_DESCRIPTION_MAX_STREAMS; i++) {
+		if (!sal_stream_description_active(&md->streams[i])) continue;
+		if (sal_stream_description_has_zrtp(&md->streams[i]) != TRUE) return FALSE;
+	}
+	return TRUE;
+}
+
+bool_t sal_media_description_has_ipv6(const SalMediaDescription *md){
+	int i;
+	if (md->nb_streams == 0) return FALSE;
+	for (i = 0; i < SAL_MEDIA_DESCRIPTION_MAX_STREAMS; i++) {
+		if (!sal_stream_description_active(&md->streams[i])) continue;
+		if (md->streams[i].rtp_addr[0] != '\0'){
+			if (!sal_stream_description_has_ipv6(&md->streams[i])) return FALSE;
+		}else{
+			if (strchr(md->addr,':') == NULL) return FALSE;
+		}
+	}
+	return TRUE;
+}
+
 /*
 static bool_t fmtp_equals(const char *p1, const char *p2){
 	if (p1 && p2 && strcmp(p1,p2)==0) return TRUE;
@@ -309,8 +356,8 @@ static bool_t is_recv_only(PayloadType *p){
 	return (p->flags & PAYLOAD_TYPE_FLAG_CAN_RECV) && ! (p->flags & PAYLOAD_TYPE_FLAG_CAN_SEND);
 }
 
-static bool_t payload_list_equals(const MSList *l1, const MSList *l2){
-	const MSList *e1,*e2;
+static bool_t payload_list_equals(const bctbx_list_t *l1, const bctbx_list_t *l2){
+	const bctbx_list_t *e1,*e2;
 	for(e1=l1,e2=l2;e1!=NULL && e2!=NULL; e1=e1->next,e2=e2->next){
 		PayloadType *p1=(PayloadType*)e1->data;
 		PayloadType *p2=(PayloadType*)e2->data;
@@ -480,18 +527,18 @@ void sal_op_set_route(SalOp *op, const char *route){
 	char* route_string=(void *)0;
 	SalOpBase* op_base = (SalOpBase*)op;
 	if (op_base->route_addresses) {
-		ms_list_for_each(op_base->route_addresses,(void (*)(void *))sal_address_destroy);
-		op_base->route_addresses=ms_list_free(op_base->route_addresses);
+		bctbx_list_for_each(op_base->route_addresses,(void (*)(void *))sal_address_destroy);
+		op_base->route_addresses=bctbx_list_free(op_base->route_addresses);
 	}
 	if (route) {
-		op_base->route_addresses=ms_list_append(NULL,NULL);
+		op_base->route_addresses=bctbx_list_append(NULL,NULL);
 		assign_address((SalAddress**)&(op_base->route_addresses->data),route);
 		route_string=sal_address_as_string((SalAddress*)op_base->route_addresses->data); \
 	}
 	assign_string(&op_base->route,route_string); \
 	if(route_string) ms_free(route_string);
 }
-const MSList* sal_op_get_route_addresses(const SalOp *op) {
+const bctbx_list_t* sal_op_get_route_addresses(const SalOp *op) {
 	return ((SalOpBase*)op)->route_addresses;
 }
 void sal_op_set_route_address(SalOp *op, const SalAddress *address){
@@ -502,7 +549,7 @@ void sal_op_set_route_address(SalOp *op, const SalAddress *address){
 void sal_op_add_route_address(SalOp *op, const SalAddress *address){
 	SalOpBase* op_base = (SalOpBase*)op;
 	if (op_base->route_addresses) {
-		op_base->route_addresses=ms_list_append(op_base->route_addresses,(void*)sal_address_clone(address));
+		op_base->route_addresses=bctbx_list_append(op_base->route_addresses,(void*)sal_address_clone(address));
 	} else {
 		sal_op_set_route_address(op,address);
 	}
@@ -554,13 +601,6 @@ const SalAddress *sal_op_get_to_address(const SalOp *op){
 	return ((SalOpBase*)op)->to_address;
 }
 
-const char *sal_op_get_route(const SalOp *op){
-#ifdef BELLE_SIP
-ms_fatal("sal_op_get_route not supported, use sal_op_get_route_addresses instead");
-#endif
-	return ((SalOpBase*)op)->route;
-}
-
 const char *sal_op_get_remote_ua(const SalOp *op){
 	return ((SalOpBase*)op)->remote_ua;
 }
@@ -597,7 +637,9 @@ void __sal_op_set_network_origin(SalOp *op, const char *origin){
 }
 
 void __sal_op_set_remote_contact(SalOp *op, const char* remote_contact){
-	SET_PARAM(op,remote_contact);
+	assign_address(&((SalOpBase*)op)->remote_contact_address,remote_contact);\
+	/*to preserve header params*/
+	assign_string(&((SalOpBase*)op)->remote_contact,remote_contact); \
 }
 
 void __sal_op_set_network_origin_address(SalOp *op, SalAddress *origin){
@@ -671,13 +713,18 @@ void __sal_op_free(SalOp *op){
 		sal_address_destroy(b->service_route);
 	}
 	if (b->route_addresses){
-		ms_list_for_each(b->route_addresses,(void (*)(void*)) sal_address_destroy);
-		b->route_addresses=ms_list_free(b->route_addresses);
+		bctbx_list_for_each(b->route_addresses,(void (*)(void*)) sal_address_destroy);
+		b->route_addresses=bctbx_list_free(b->route_addresses);
 	}
 	if (b->recv_custom_headers)
 		sal_custom_header_free(b->recv_custom_headers);
 	if (b->sent_custom_headers)
 		sal_custom_header_free(b->sent_custom_headers);
+	
+	if (b->entity_tag != NULL){
+		ms_free(b->entity_tag);
+		b->entity_tag = NULL;
+	}
 	ms_free(op);
 }
 
@@ -807,7 +854,7 @@ const char* sal_privacy_to_string(SalPrivacy privacy) {
 }
 
 static void remove_trailing_spaces(char *line){
-	size_t i;
+	int i;
 	for(i=strlen(line)-1;i>=0;--i){
 		if (isspace(line[i])) line[i]='\0';
 		else break;
@@ -849,10 +896,134 @@ int sal_lines_get_value(const char *data, const char *key, char *value, size_t v
 	return FALSE;
 }
 
-int sal_body_has_type(const SalBody *body, const char *type, const char *subtype){
-	return body->type && body->subtype
-		&& strcmp(body->type,type)==0
-		&& strcmp(body->subtype,subtype)==0;
+static belle_sip_header_t * sal_body_handler_find_header(const SalBodyHandler *body_handler, const char *header_name) {
+	belle_sip_body_handler_t *bsbh = BELLE_SIP_BODY_HANDLER(body_handler);
+	const belle_sip_list_t *l = belle_sip_body_handler_get_headers(bsbh);
+	for (; l != NULL; l = l->next) {
+		belle_sip_header_t *header = BELLE_SIP_HEADER(l->data);
+		if (strcmp(belle_sip_header_get_name(header), header_name) == 0) {
+			return header;
+		}
+	}
+	return NULL;
+}
+
+SalBodyHandler * sal_body_handler_new(void) {
+	belle_sip_memory_body_handler_t *body_handler = belle_sip_memory_body_handler_new(NULL, NULL);
+	return (SalBodyHandler *)BELLE_SIP_BODY_HANDLER(body_handler);
+}
+
+SalBodyHandler * sal_body_handler_ref(SalBodyHandler *body_handler) {
+	return (SalBodyHandler *)belle_sip_object_ref(BELLE_SIP_OBJECT(body_handler));
+}
+
+void sal_body_handler_unref(SalBodyHandler *body_handler) {
+	belle_sip_object_unref(BELLE_SIP_OBJECT(body_handler));
+}
+
+const char * sal_body_handler_get_type(const SalBodyHandler *body_handler) {
+	belle_sip_header_content_type_t *content_type = BELLE_SIP_HEADER_CONTENT_TYPE(sal_body_handler_find_header(body_handler, "Content-Type"));
+	if (content_type != NULL) {
+		return belle_sip_header_content_type_get_type(content_type);
+	}
+	return NULL;
+}
+
+void sal_body_handler_set_type(SalBodyHandler *body_handler, const char *type) {
+	belle_sip_header_content_type_t *content_type = BELLE_SIP_HEADER_CONTENT_TYPE(sal_body_handler_find_header(body_handler, "Content-Type"));
+	if (content_type == NULL) {
+		content_type = belle_sip_header_content_type_new();
+		belle_sip_body_handler_add_header(BELLE_SIP_BODY_HANDLER(body_handler), BELLE_SIP_HEADER(content_type));
+	}
+	belle_sip_header_content_type_set_type(content_type, type);
+}
+
+const char * sal_body_handler_get_subtype(const SalBodyHandler *body_handler) {
+	belle_sip_header_content_type_t *content_type = BELLE_SIP_HEADER_CONTENT_TYPE(sal_body_handler_find_header(body_handler, "Content-Type"));
+	if (content_type != NULL) {
+		return belle_sip_header_content_type_get_subtype(content_type);
+	}
+	return NULL;
+}
+
+void sal_body_handler_set_subtype(SalBodyHandler *body_handler, const char *subtype) {
+	belle_sip_header_content_type_t *content_type = BELLE_SIP_HEADER_CONTENT_TYPE(sal_body_handler_find_header(body_handler, "Content-Type"));
+	if (content_type == NULL) {
+		content_type = belle_sip_header_content_type_new();
+		belle_sip_body_handler_add_header(BELLE_SIP_BODY_HANDLER(body_handler), BELLE_SIP_HEADER(content_type));
+	}
+	belle_sip_header_content_type_set_subtype(content_type, subtype);
+}
+
+const char * sal_body_handler_get_encoding(const SalBodyHandler *body_handler) {
+	belle_sip_header_t *content_encoding = sal_body_handler_find_header(body_handler, "Content-Encoding");
+	if (content_encoding != NULL) {
+		return belle_sip_header_get_unparsed_value(content_encoding);
+	}
+	return NULL;
+}
+
+void sal_body_handler_set_encoding(SalBodyHandler *body_handler, const char *encoding) {
+	belle_sip_header_t *content_encoding = sal_body_handler_find_header(body_handler, "Content-Encoding");
+	if (content_encoding != NULL) {
+		belle_sip_body_handler_remove_header_from_ptr(BELLE_SIP_BODY_HANDLER(body_handler), content_encoding);
+	}
+	belle_sip_body_handler_add_header(BELLE_SIP_BODY_HANDLER(body_handler), belle_sip_header_create("Content-Encoding", encoding));
+}
+
+void * sal_body_handler_get_data(const SalBodyHandler *body_handler) {
+	return belle_sip_memory_body_handler_get_buffer(BELLE_SIP_MEMORY_BODY_HANDLER(body_handler));
+}
+
+void sal_body_handler_set_data(SalBodyHandler *body_handler, void *data) {
+	belle_sip_memory_body_handler_set_buffer(BELLE_SIP_MEMORY_BODY_HANDLER(body_handler), data);
+}
+
+size_t sal_body_handler_get_size(const SalBodyHandler *body_handler) {
+	return belle_sip_body_handler_get_size(BELLE_SIP_BODY_HANDLER(body_handler));
+}
+
+void sal_body_handler_set_size(SalBodyHandler *body_handler, size_t size) {
+	belle_sip_header_content_length_t *content_length = BELLE_SIP_HEADER_CONTENT_LENGTH(sal_body_handler_find_header(body_handler, "Content-Length"));
+	if (content_length == NULL) {
+		content_length = belle_sip_header_content_length_new();
+		belle_sip_body_handler_add_header(BELLE_SIP_BODY_HANDLER(body_handler), BELLE_SIP_HEADER(content_length));
+	}
+	belle_sip_header_content_length_set_content_length(content_length, size);
+	belle_sip_body_handler_set_size(BELLE_SIP_BODY_HANDLER(body_handler), size);
+}
+
+bool_t sal_body_handler_is_multipart(const SalBodyHandler *body_handler) {
+	if (BELLE_SIP_IS_INSTANCE_OF(body_handler, belle_sip_multipart_body_handler_t)) return TRUE;
+	return FALSE;
+}
+
+SalBodyHandler * sal_body_handler_get_part(const SalBodyHandler *body_handler, int idx) {
+	const belle_sip_list_t *l = belle_sip_multipart_body_handler_get_parts(BELLE_SIP_MULTIPART_BODY_HANDLER(body_handler));
+	return (SalBodyHandler *)belle_sip_list_nth_data(l, idx);
+}
+
+SalBodyHandler * sal_body_handler_find_part_by_header(const SalBodyHandler *body_handler, const char *header_name, const char *header_value) {
+	const belle_sip_list_t *l = belle_sip_multipart_body_handler_get_parts(BELLE_SIP_MULTIPART_BODY_HANDLER(body_handler));
+	for (; l != NULL; l = l->next) {
+		belle_sip_body_handler_t *bsbh = BELLE_SIP_BODY_HANDLER(l->data);
+		const belle_sip_list_t *headers = belle_sip_body_handler_get_headers(bsbh);
+		for (; headers != NULL; headers = headers->next) {
+			belle_sip_header_t *header = BELLE_SIP_HEADER(headers->data);
+			if ((strcmp(belle_sip_header_get_name(header), header_name) == 0) && (strcmp(belle_sip_header_get_unparsed_value(header), header_value) == 0)) {
+				return (SalBodyHandler *)bsbh;
+			}
+		}
+	}
+	return NULL;
+}
+
+const char * sal_body_handler_get_header(const SalBodyHandler *body_handler, const char *header_name) {
+	belle_sip_header_t *header = sal_body_handler_find_header(body_handler, header_name);
+	if (header != NULL) {
+		return belle_sip_header_get_unparsed_value(header);
+	}
+	return NULL;
 }
 
 belle_sip_stack_t *sal_get_belle_sip_stack(Sal *sal) {
@@ -865,3 +1036,17 @@ char* sal_op_get_public_uri(SalOp *op) {
 	}
 	return NULL;
 }
+const char *sal_op_get_entity_tag(const SalOp* op) {
+	SalOpBase* op_base = (SalOpBase*)op;
+	return op_base->entity_tag;
+}
+void sal_op_set_entity_tag(SalOp *op, const char* entity_tag) {
+	SalOpBase* op_base = (SalOpBase*)op;
+	if (op_base->entity_tag != NULL){
+		ms_free(op_base->entity_tag);
+	}
+	if (entity_tag)
+		op_base->entity_tag = ms_strdup(entity_tag);
+	else
+		op_base->entity_tag = NULL;
+}
