@@ -29,27 +29,28 @@ static SipSetup *all_sip_setups[]={
 	NULL
 };
 
-static MSList *registered_sip_setups=NULL;
+static bctbx_list_t *registered_sip_setups=NULL;
 
 void sip_setup_register(SipSetup *ss){
-	registered_sip_setups=ms_list_append(registered_sip_setups,ss);
+	registered_sip_setups=bctbx_list_append(registered_sip_setups,ss);
 }
 
-void sip_setup_register_all(void){
+void sip_setup_register_all(MSFactory *factory){
 	SipSetup **p=all_sip_setups;
-	ms_load_plugins(LINPHONE_PLUGINS_DIR);
+	ms_factory_load_plugins(factory, LINPHONE_PLUGINS_DIR);
+	//ms_load_plugins(LINPHONE_PLUGINS_DIR);
 	while(*p!=NULL){
 		sip_setup_register(*p);
 		p++;
 	}
 }
 
-const MSList * linphone_core_get_sip_setups(LinphoneCore *lc){
+const bctbx_list_t * linphone_core_get_sip_setups(LinphoneCore *lc){
 	return registered_sip_setups;
 }
 
 SipSetup *sip_setup_lookup(const char *type_name){
-	MSList *elem;
+	bctbx_list_t *elem;
 	for(elem=registered_sip_setups;elem!=NULL;elem=elem->next){
 		SipSetup *ss=(SipSetup*)elem->data;
 		if ( strcasecmp(ss->name,type_name)==0){
@@ -68,7 +69,7 @@ SipSetup *sip_setup_lookup(const char *type_name){
 }
 
 void sip_setup_unregister_all(void){
-	MSList *elem;
+	bctbx_list_t *elem;
 	for(elem=registered_sip_setups;elem!=NULL;elem=elem->next){
 		SipSetup *ss=(SipSetup*)elem->data;
 		if (ss->initialized){
@@ -76,7 +77,7 @@ void sip_setup_unregister_all(void){
 			ss->initialized=FALSE;
 		}
 	}
-	registered_sip_setups = ms_list_free(registered_sip_setups);
+	registered_sip_setups = bctbx_list_free(registered_sip_setups);
 }
 
 void buddy_lookup_request_set_key(BuddyLookupRequest *req, const char *key){
@@ -94,8 +95,8 @@ void buddy_lookup_request_set_max_results(BuddyLookupRequest *req, int ncount){
 void buddy_lookup_request_free(BuddyLookupRequest *req){
 	if (req->key) ms_free(req->key);
 	if (req->results){
-		ms_list_for_each(req->results,(void (*)(void*))buddy_info_free);
-		ms_list_free(req->results);
+		bctbx_list_for_each(req->results,(void (*)(void*))buddy_info_free);
+		bctbx_list_free(req->results);
 	}
 	ms_free(req);
 }
