@@ -42,13 +42,13 @@ LinphoneCoreVTable *linphone_core_get_current_vtable(LinphoneCore *lc) {
 }
 
 static void cleanup_dead_vtable_refs(LinphoneCore *lc){
-	MSList *it,*next_it;
+	bctbx_list_t *it,*next_it;
 	for(it=lc->vtable_refs; it!=NULL; ){
 		VTableReference *ref=(VTableReference*)it->data;
 		next_it=it->next;
 		if (ref->valid==0){
 			ref->valid=0;
-			lc->vtable_refs=ms_list_remove_link(lc->vtable_refs, it);
+			lc->vtable_refs=bctbx_list_remove_link(lc->vtable_refs, it);
 			ms_free(ref);
 		}
 		it=next_it;
@@ -56,7 +56,7 @@ static void cleanup_dead_vtable_refs(LinphoneCore *lc){
 }
 
 #define NOTIFY_IF_EXIST(function_name, ...) \
-	MSList* iterator; \
+	bctbx_list_t* iterator; \
 	VTableReference *ref; \
 	bool_t has_cb = FALSE; \
 	for (iterator=lc->vtable_refs; iterator!=NULL; iterator=iterator->next)\
@@ -66,6 +66,17 @@ static void cleanup_dead_vtable_refs(LinphoneCore *lc){
 		}\
 	if (has_cb) ms_message("Linphone core [%p] notifying [%s]",lc,#function_name)
 
+#define NOTIFY_IF_EXIST_INTERNAL(function_name, internal_val, ...) \
+	bctbx_list_t* iterator; \
+	VTableReference *ref; \
+	bool_t has_cb = FALSE; \
+	for (iterator=lc->vtable_refs; iterator!=NULL; iterator=iterator->next)\
+		if ((ref=(VTableReference*)iterator->data)->valid && (lc->current_vtable=ref->vtable)->function_name && (ref->internal == internal_val)) {\
+			lc->current_vtable->function_name(__VA_ARGS__);\
+			has_cb = TRUE;\
+		}\
+	if (has_cb) ms_message("Linphone core [%p] notifying [%s]",lc,#function_name)
+
 void linphone_core_notify_global_state_changed(LinphoneCore *lc, LinphoneGlobalState gstate, const char *message) {
 	NOTIFY_IF_EXIST(global_state_changed,lc,gstate,message);
 	cleanup_dead_vtable_refs(lc);
@@ -85,7 +96,14 @@ void linphone_core_notify_registration_state_changed(LinphoneCore *lc, LinphoneP
 	NOTIFY_IF_EXIST(registration_state_changed, lc,cfg,cstate,message);
 	cleanup_dead_vtable_refs(lc);
 }
-
+#if __clang__ || ((__GNUC__ == 4 && __GNUC_MINOR__ >= 6) || __GNUC__ > 4)
+#pragma GCC diagnostic push
+#endif
+#ifdef _MSC_VER
+#pragma warning(disable : 4996)
+#else
+#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#endif
 void linphone_core_notify_show_interface(LinphoneCore *lc){
 	NOTIFY_IF_EXIST(show, lc);
 	cleanup_dead_vtable_refs(lc);
@@ -110,7 +128,9 @@ void linphone_core_notify_display_url(LinphoneCore *lc, const char *message, con
 	NOTIFY_IF_EXIST(display_url, lc,message,url);
 	cleanup_dead_vtable_refs(lc);
 }
-
+#if __clang__ || ((__GNUC__ == 4 && __GNUC_MINOR__ >= 6) || __GNUC__ > 4)
+#pragma GCC diagnostic pop
+#endif
 void linphone_core_notify_notify_presence_received(LinphoneCore *lc, LinphoneFriend * lf){
 	NOTIFY_IF_EXIST(notify_presence_received, lc,lf);
 	cleanup_dead_vtable_refs(lc);
@@ -130,17 +150,35 @@ void linphone_core_notify_call_log_updated(LinphoneCore *lc, LinphoneCallLog *ne
 	NOTIFY_IF_EXIST(call_log_updated, lc,newcl);
 	cleanup_dead_vtable_refs(lc);
 }
+#if __clang__ || ((__GNUC__ == 4 && __GNUC_MINOR__ >= 6) || __GNUC__ > 4)
+#pragma GCC diagnostic push
+#endif
+#ifdef _MSC_VER
+#pragma warning(disable : 4996)
+#else
+#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#endif
 
 void linphone_core_notify_text_message_received(LinphoneCore *lc, LinphoneChatRoom *room, const LinphoneAddress *from, const char *message){
 	NOTIFY_IF_EXIST(text_received, lc,room,from,message);
 	cleanup_dead_vtable_refs(lc);
 }
+#if __clang__ || ((__GNUC__ == 4 && __GNUC_MINOR__ >= 6) || __GNUC__ > 4)
+#pragma GCC diagnostic pop
+#endif
 
 void linphone_core_notify_message_received(LinphoneCore *lc, LinphoneChatRoom *room, LinphoneChatMessage *message){
 	NOTIFY_IF_EXIST(message_received, lc,room,message);
 	cleanup_dead_vtable_refs(lc);
 }
-
+#if __clang__ || ((__GNUC__ == 4 && __GNUC_MINOR__ >= 6) || __GNUC__ > 4)
+#pragma GCC diagnostic push
+#endif
+#ifdef _MSC_VER
+#pragma warning(disable : 4996)
+#else
+#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#endif
 void linphone_core_notify_file_transfer_recv(LinphoneCore *lc, LinphoneChatMessage *message, const LinphoneContent* content, const char* buff, size_t size) {
 	NOTIFY_IF_EXIST(file_transfer_recv, lc,message,content,buff,size);
 	cleanup_dead_vtable_refs(lc);
@@ -155,7 +193,9 @@ void linphone_core_notify_file_transfer_progress_indication(LinphoneCore *lc, Li
 	NOTIFY_IF_EXIST(file_transfer_progress_indication, lc,message,content,offset,total);
 	cleanup_dead_vtable_refs(lc);
 }
-
+#if __clang__ || ((__GNUC__ == 4 && __GNUC_MINOR__ >= 6) || __GNUC__ > 4)
+#pragma GCC diagnostic pop
+#endif
 void linphone_core_notify_is_composing_received(LinphoneCore *lc, LinphoneChatRoom *room) {
 	NOTIFY_IF_EXIST(is_composing_received, lc,room);
 	cleanup_dead_vtable_refs(lc);
@@ -167,7 +207,7 @@ void linphone_core_notify_dtmf_received(LinphoneCore* lc, LinphoneCall *call, in
 }
 
 bool_t linphone_core_dtmf_received_has_listener(const LinphoneCore* lc) {
-	MSList* iterator;
+	bctbx_list_t* iterator;
 	for (iterator=lc->vtable_refs; iterator!=NULL; iterator=iterator->next){
 		VTableReference *ref=(VTableReference*)iterator->data;
 		if (ref->valid && ref->vtable->dtmf_received)
@@ -212,17 +252,17 @@ void linphone_core_notify_network_reachable(LinphoneCore *lc, bool_t reachable)
 }
 
 void linphone_core_notify_notify_received(LinphoneCore *lc, LinphoneEvent *lev, const char *notified_event, const LinphoneContent *body) {
-	NOTIFY_IF_EXIST(notify_received, lc,lev,notified_event,body);
+	NOTIFY_IF_EXIST_INTERNAL(notify_received, linphone_event_is_internal(lev), lc, lev, notified_event, body);
 	cleanup_dead_vtable_refs(lc);
 }
 
 void linphone_core_notify_subscription_state_changed(LinphoneCore *lc, LinphoneEvent *lev, LinphoneSubscriptionState state) {
-	NOTIFY_IF_EXIST(subscription_state_changed, lc,lev,state);
+	NOTIFY_IF_EXIST_INTERNAL(subscription_state_changed,linphone_event_is_internal(lev), lc,lev,state);
 	cleanup_dead_vtable_refs(lc);
 }
 
 void linphone_core_notify_publish_state_changed(LinphoneCore *lc, LinphoneEvent *lev, LinphonePublishState state) {
-	NOTIFY_IF_EXIST(publish_state_changed, lc,lev,state);
+	NOTIFY_IF_EXIST_INTERNAL(publish_state_changed, linphone_event_is_internal(lev), lc, lev, state);
 	cleanup_dead_vtable_refs(lc);
 }
 
@@ -236,10 +276,21 @@ void linphone_core_notify_log_collection_upload_progress_indication(LinphoneCore
 	cleanup_dead_vtable_refs(lc);
 }
 
-static VTableReference * v_table_reference_new(LinphoneCoreVTable *vtable, bool_t autorelease){
+void linphone_core_notify_friend_list_created(LinphoneCore *lc, LinphoneFriendList *list) {
+	NOTIFY_IF_EXIST(friend_list_created, lc, list);
+	cleanup_dead_vtable_refs(lc);
+}
+
+void linphone_core_notify_friend_list_removed(LinphoneCore *lc, LinphoneFriendList *list) {
+	NOTIFY_IF_EXIST(friend_list_removed, lc, list);
+	cleanup_dead_vtable_refs(lc);
+}
+
+static VTableReference * v_table_reference_new(LinphoneCoreVTable *vtable, bool_t autorelease, bool_t internal){
 	VTableReference *ref=ms_new0(VTableReference,1);
 	ref->valid=1;
 	ref->autorelease=autorelease;
+	ref->internal = internal;
 	ref->vtable=vtable;
 	return ref;
 }
@@ -249,17 +300,17 @@ void v_table_reference_destroy(VTableReference *ref){
 	ms_free(ref);
 }
 
-void _linphone_core_add_listener(LinphoneCore *lc, LinphoneCoreVTable *vtable, bool_t autorelease) {
-	ms_message("Vtable [%p] registered on core [%p]",lc,vtable);
-	lc->vtable_refs=ms_list_append(lc->vtable_refs,v_table_reference_new(vtable, autorelease));
+void _linphone_core_add_listener(LinphoneCore *lc, LinphoneCoreVTable *vtable, bool_t autorelease, bool_t internal) {
+	ms_message("Vtable [%p] registered on core [%p]",vtable, lc);
+	lc->vtable_refs=bctbx_list_append(lc->vtable_refs,v_table_reference_new(vtable, autorelease, internal));
 }
 
 void linphone_core_add_listener(LinphoneCore *lc, LinphoneCoreVTable *vtable){
-	_linphone_core_add_listener(lc, vtable, FALSE);
+	_linphone_core_add_listener(lc, vtable, FALSE, FALSE);
 }
 
 void linphone_core_remove_listener(LinphoneCore *lc, const LinphoneCoreVTable *vtable) {
-	MSList *it;
+	bctbx_list_t *it;
 	ms_message("Vtable [%p] unregistered on core [%p]",lc,vtable);
 	for(it=lc->vtable_refs; it!=NULL; it=it->next){
 		VTableReference *ref=(VTableReference*)it->data;
