@@ -18,16 +18,17 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
 #include "linphone.h"
+#include <bctoolbox/vfs.h>
 
 #ifdef HAVE_GTK_OSX
 #include <gtkosxapplication.h>
 #endif
 
-#if defined(WIN32) && !defined(F_OK)
+#if defined(_WIN32) && !defined(F_OK)
 #define F_OK 00 /*visual studio does not define F_OK*/
 #endif
 
-#define NB_MSG_HIST 250
+#define NB_MSG_HIST 250000
 
 #define CONFIG_FILE ".linphone-history.db"
 
@@ -62,10 +63,10 @@ char *linphone_gtk_message_storage_get_db_file(const char *filename){
 	db_file=(char *)g_malloc(path_max*sizeof(char));
 	if (filename==NULL) filename=CONFIG_FILE;
 	/*try accessing a local file first if exists*/
-	if (access(CONFIG_FILE,F_OK)==0){
+	if (bctbx_file_exist(CONFIG_FILE)==0){
 		snprintf(db_file,path_max,"%s",filename);
 	}else{
-#ifdef WIN32
+#ifdef _WIN32
 		const char *appdata=getenv("APPDATA");
 		if (appdata){
 			snprintf(db_file,path_max,"%s\\%s",appdata,LINPHONE_CONFIG_DIR);
@@ -81,6 +82,12 @@ char *linphone_gtk_message_storage_get_db_file(const char *filename){
 	return db_file;
 }
 
+void linphone_gtk_mark_chat_read(LinphoneChatRoom *cr) {
+	linphone_chat_room_mark_as_read(cr);
+#ifdef __APPLE__
+	linphone_gtk_update_badge_count();
+#endif
+}
 
 void linphone_gtk_quit_chatroom(LinphoneChatRoom *cr) {
 	GtkWidget *main_window=linphone_gtk_get_main_window ();
@@ -91,7 +98,7 @@ void linphone_gtk_quit_chatroom(LinphoneChatRoom *cr) {
 
 	g_return_if_fail(w!=NULL);
 	gtk_notebook_remove_page(GTK_NOTEBOOK(nb),gtk_notebook_page_num(GTK_NOTEBOOK(nb),w));
-	linphone_chat_room_mark_as_read(cr);
+	linphone_gtk_mark_chat_read(cr);
 	g_object_set_data(G_OBJECT(friendlist),"chatview",NULL);
 	from=g_object_get_data(G_OBJECT(w),"from_message");
 	if (from){
@@ -126,11 +133,9 @@ void update_chat_header(GtkNotebook *notebook, GtkWidget *chat_view, LinphoneCha
 
 static gboolean scroll_to_end(GtkTextView *w){
 	GtkTextBuffer *buffer=gtk_text_view_get_buffer(w);
-	GtkTextMark *mark;
 	GtkTextIter iter;
 	gtk_text_buffer_get_end_iter(buffer,&iter);
-	mark=gtk_text_buffer_create_mark(buffer,NULL,&iter,FALSE);
-	gtk_text_view_scroll_mark_onscreen(w,mark);
+	gtk_text_view_scroll_to_iter (GTK_TEXT_VIEW (w), &iter, 0.0, FALSE, 0, 0);
 	return FALSE;
 }
 
@@ -175,6 +180,11 @@ void linphone_gtk_push_text(GtkWidget *w, const LinphoneAddress *from,
 	}
 	ms_free(from_str);
 
+	if (!message) {
+		const char *external_body_url = linphone_chat_message_get_external_body_url(msg);
+		if (external_body_url) message = external_body_url;
+	}
+
 	// Inserts message body and tags URIs as hypertext links
 	if(message) {
 		g_regex_match(uri_regex, message, 0, &match_info);
@@ -185,7 +195,7 @@ void linphone_gtk_push_text(GtkWidget *w, const LinphoneAddress *from,
 			pos = end;
 			g_match_info_next(match_info, NULL);
 		}
-		if(pos < strlen(message)) write_body(buffer, &iter, &message[pos], -1, me, FALSE);
+		if((size_t)pos < strlen(message)) write_body(buffer, &iter, &message[pos], -1, me, FALSE);
 		gtk_text_buffer_insert(buffer,&iter,"\n",-1);
 		g_match_info_free(match_info);
 	}
@@ -205,7 +215,7 @@ void linphone_gtk_push_text(GtkWidget *w, const LinphoneAddress *from,
 			if(tnow_day != tm->tm_yday || (tnow_day == tm->tm_yday && tnow_year != tm->tm_year)) {
 				strftime(buf,80,"%a %x, %H:%M",tm);
 			} else {
-				strftime(buf,80,"%H:%M",tm);
+				strftime(buf,80,"%H:%M:%S",tm);
 			}
 			gtk_text_buffer_insert_with_tags_by_name(buffer,&iter,buf,-1,"status", me ? "me" : NULL, NULL);
 			break;
@@ -256,7 +266,7 @@ void update_chat_state_message(LinphoneChatMessageState state,LinphoneChatMessag
 				case LinphoneChatMessageStateDelivered:
 					t=time(NULL);
 					tm=localtime(&t);
-					strftime(buf,80,"%H:%M",tm);
+					strftime(buf,80,"%H:%M:%S",tm);
 					gtk_text_buffer_insert_with_tags_by_name(b,&iter,(gchar*)buf,-1,"status", "me", NULL);
 					break;
 				case LinphoneChatMessageStateNotDelivered:
@@ -280,12 +290,12 @@ void linphone_gtk_compose_text(void) {
 	LinphoneChatRoom *cr=g_object_get_data(G_OBJECT(w),"cr");
 	if (cr) {
 		linphone_chat_room_compose(cr);
-		linphone_chat_room_mark_as_read(cr);
+		linphone_gtk_mark_chat_read(cr);
 		linphone_gtk_friend_list_update_button_display(GTK_TREE_VIEW(friendlist));
 	}
 }
 
-void linphone_gtk_send_text(){
+void linphone_gtk_send_text(void){
 	GtkWidget *main_window=linphone_gtk_get_main_window();
 	GtkWidget *friendlist=linphone_gtk_get_widget(main_window,"contact_list");
 	GtkWidget *w=(GtkWidget*)g_object_get_data(G_OBJECT(friendlist),"chatview");
@@ -296,7 +306,7 @@ void linphone_gtk_send_text(){
 	if (strlen(entered)>0) {
 		LinphoneChatMessage *msg;
 		LinphoneChatMessageCbs *cbs;
-		msg=linphone_chat_room_create_message(cr,entered);
+		msg=linphone_chat_message_ref(linphone_chat_room_create_message(cr,entered));
 		cbs=linphone_chat_message_get_callbacks(msg);
 		linphone_chat_message_cbs_set_msg_state_changed(cbs,on_chat_state_changed);
 		linphone_chat_room_send_chat_message(cr,msg);
@@ -307,6 +317,8 @@ void linphone_gtk_send_text(){
 		g_signal_handlers_disconnect_by_func(G_OBJECT(entry),(GCallback)linphone_gtk_compose_text,NULL);
 		gtk_entry_set_text(GTK_ENTRY(entry),"");
 		g_signal_connect_swapped(G_OBJECT(entry),"changed",(GCallback)linphone_gtk_compose_text,NULL);
+
+		linphone_chat_message_unref(msg);
 	}
 }
 
@@ -314,14 +326,14 @@ static void linphone_gtk_chat_message_destroy(LinphoneChatMessage *msg){
 	linphone_chat_message_destroy(msg);
 }
 
-void linphone_gtk_free_list(MSList *messages){
-	ms_list_for_each(messages,(void (*)(void*))linphone_gtk_chat_message_destroy);
-	ms_list_free(messages);
+void linphone_gtk_free_list(bctbx_list_t *messages){
+	bctbx_list_for_each(messages,(void (*)(void*))linphone_gtk_chat_message_destroy);
+	bctbx_list_free(messages);
 }
 
-void display_history_message(GtkWidget *chat_view,MSList *messages,const LinphoneAddress *with){
+void display_history_message(GtkWidget *chat_view,bctbx_list_t *messages,const LinphoneAddress *with){
 	if (messages != NULL){
-		MSList *it;
+		bctbx_list_t *it;
 		char *from_str;
 		char *with_str;
 		gchar *tmp;
@@ -341,24 +353,25 @@ void display_history_message(GtkWidget *chat_view,MSList *messages,const Linphon
 			g_object_set_data(G_OBJECT(chat_view),"from_message",NULL);
 			g_free(tmp);
 		}
-		
+
 		linphone_gtk_free_list(messages);
 	}
 }
 
 static void linphone_gtk_chat_add_contact(const LinphoneAddress *addr){
 	LinphoneFriend *lf=NULL;
+	LinphoneCore *lc = linphone_gtk_get_core();
 	gboolean show_presence=FALSE;
 	char *uri=linphone_address_as_string(addr);
 
-	lf=linphone_friend_new_with_address(uri);
+	lf=linphone_core_create_friend_with_address(lc, uri);
 	ms_free(uri);
 
 	linphone_friend_set_inc_subscribe_policy(lf,LinphoneSPWait);
 	linphone_friend_send_subscribe(lf,show_presence);
 
 	linphone_friend_set_address(lf,addr);
-	linphone_core_add_friend(linphone_gtk_get_core(),lf);
+	linphone_core_add_friend(lc, lf);
 	linphone_gtk_show_friends();
 }
 
@@ -381,7 +394,7 @@ static gboolean link_event_handler(GtkTextTag *tag, GObject *text_view,GdkEvent
 		LinphoneChatRoom *chat_room = (LinphoneChatRoom *)g_object_get_data(G_OBJECT(chat_view), "cr");
 		GtkWidget *main_window = linphone_gtk_get_main_window();
 		GtkWidget *friendlist = linphone_gtk_get_widget(main_window, "contact_list");
-		
+
 		gtk_text_iter_backward_to_tag_toggle(&uri_begin, tag);
 		gtk_text_iter_forward_to_tag_toggle(&uri_end, tag);
 		uri = gtk_text_iter_get_slice(&uri_begin, &uri_end);
@@ -393,10 +406,10 @@ static gboolean link_event_handler(GtkTextTag *tag, GObject *text_view,GdkEvent
 			gtk_menu_popup(menu, NULL, NULL, NULL, NULL, 3, gdk_event_get_time(event));
 		}
 		g_free(uri);
-		
-		linphone_chat_room_mark_as_read(chat_room);
+
+		linphone_gtk_mark_chat_read(chat_room);
 		linphone_gtk_friend_list_update_button_display(GTK_TREE_VIEW(friendlist));
-		
+
 		return TRUE;
 	}
 	return FALSE;
@@ -427,8 +440,8 @@ static gboolean chatroom_event(GtkWidget *widget, GdkEvent *event, gpointer user
 	GtkTextIter iter;
 	if(event->type == GDK_MOTION_NOTIFY) {
 		GdkEventMotion *motion_ev = (GdkEventMotion *)event;
-		wx = motion_ev->x;
-		wy = motion_ev->y;
+		wx = (gint)motion_ev->x;
+		wy = (gint)motion_ev->y;
 		gtk_text_view_window_to_buffer_coords(chatroom, GTK_TEXT_WINDOW_TEXT, wx, wy, &bx, &by);
 		gtk_text_view_get_iter_at_location(chatroom, &iter, bx, by);
 		if(gtk_text_iter_has_tag(&iter, link_tag)) {
@@ -454,6 +467,28 @@ static gboolean copy_uri_into_clipboard_handler(GtkMenuItem *menuitem, gpointer
 	return FALSE;
 }
 
+static void refresh_lime_icon(GtkWidget* chat_view, LinphoneChatRoom*cr) {
+	GtkWidget *lime_icon = linphone_gtk_get_widget(chat_view, "lime_icon");
+	if (linphone_chat_room_lime_available(cr)) {
+		gtk_widget_show(lime_icon);
+	} else {
+		gtk_widget_hide(lime_icon);
+	}
+}
+
+static gint linphone_gtk_window_focused(GtkWidget* widget, GdkEvent *event, gpointer user_data) {
+	// if we are in a chat, mark it as read
+	GtkWidget *main_window=linphone_gtk_get_main_window();
+	GtkWidget *friendlist=linphone_gtk_get_widget(main_window,"contact_list");
+	GtkWidget *w=(GtkWidget*)g_object_get_data(G_OBJECT(friendlist),"chatview");
+	LinphoneChatRoom *cr=w?g_object_get_data(G_OBJECT(w),"cr"):NULL;
+	if (cr) {
+		linphone_gtk_mark_chat_read(cr);
+	}
+	refresh_lime_icon(w, cr);
+	return FALSE;
+}
+
 GtkWidget* linphone_gtk_init_chatroom(LinphoneChatRoom *cr, const LinphoneAddress *with){
 	GtkWidget *chat_view=linphone_gtk_create_widget("chatroom_frame");
 	GtkWidget *main_window=linphone_gtk_get_main_window();
@@ -465,7 +500,7 @@ GtkWidget* linphone_gtk_init_chatroom(LinphoneChatRoom *cr, const LinphoneAddres
 	int idx;
 	GtkWidget *button;
 	GtkWidget *entry = linphone_gtk_get_widget(chat_view,"text_entry");
-	MSList *messages;
+	bctbx_list_t *messages;
 	GHashTable *table;
 	GtkTextTag *tmp_tag;
 	GtkWidget *link_ctx_menu = gtk_menu_new();
@@ -530,6 +565,11 @@ GtkWidget* linphone_gtk_init_chatroom(LinphoneChatRoom *cr, const LinphoneAddres
 	g_signal_connect_swapped(G_OBJECT(entry),"activate",(GCallback)linphone_gtk_send_text,NULL);
 	g_signal_connect_swapped(G_OBJECT(entry),"changed",(GCallback)linphone_gtk_compose_text,NULL);
 	g_signal_connect(G_OBJECT(notebook),"switch_page",(GCallback)linphone_gtk_notebook_tab_select,NULL);
+
+	gtk_signal_connect(GTK_OBJECT(main_window), "focus-in-event", GTK_SIGNAL_FUNC(linphone_gtk_window_focused), NULL);
+
+	refresh_lime_icon(chat_view, cr);
+
 	return chat_view;
 }
 
@@ -546,7 +586,7 @@ void linphone_gtk_load_chatroom(LinphoneChatRoom *cr,const LinphoneAddress *uri,
 	char *from_str=linphone_address_as_string_uri_only(from);
 	char *uri_str=linphone_address_as_string(uri);
 	char *uri_only=linphone_address_as_string_uri_only(uri);
-	MSList *messages=NULL;
+	bctbx_list_t *messages=NULL;
 
 	if(g_strcmp0(from_str,uri_only)!=0){
 		GtkTextView *text_view=GTK_TEXT_VIEW(linphone_gtk_get_widget(chat_view,"textview"));
@@ -566,6 +606,8 @@ void linphone_gtk_load_chatroom(LinphoneChatRoom *cr,const LinphoneAddress *uri,
 		gtk_text_buffer_get_end_iter(text_buffer,&end);
 		gtk_text_view_scroll_to_iter(text_view,&end,0,FALSE,1.0,0);
 	}
+	refresh_lime_icon(chat_view, cr);
+
 	ms_free(from_str);
 	ms_free(uri_str);
 	ms_free(uri_only);
@@ -601,7 +643,7 @@ void linphone_gtk_text_received ( LinphoneCore *lc, LinphoneChatRoom *room,
 		}
 	} else {
 	/* Chat window closed */
-#ifdef MSG_STORAGE_ENABLED
+#ifdef SQLITE_STORAGE_ENABLED
 		send=FALSE;
 #else
 		send=TRUE;
