@@ -21,8 +21,20 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "config.h"
 #endif
 
+#if __clang__ || ((__GNUC__ == 4 && __GNUC_MINOR__ >= 6) || __GNUC__ > 4)
+#pragma GCC diagnostic push
+#endif
+#ifndef _MSC_VER
+#pragma GCC diagnostic ignored "-Wstrict-prototypes"
+#endif
+
 #include <gtk/gtk.h>
-#ifdef WIN32
+
+#if __clang__ || ((__GNUC__ == 4 && __GNUC_MINOR__ >= 6) || __GNUC__ > 4)
+#pragma GCC diagnostic pop
+#endif
+
+#ifdef _WIN32
 // alloca is already defined by gtk
 #undef alloca
 #endif
@@ -106,12 +118,13 @@ LINPHONE_PUBLIC GtkWidget *linphone_gtk_make_tab_header(const gchar *label, cons
 
 char *linphone_gtk_message_storage_get_db_file(const char *filename);
 char *linphone_gtk_call_logs_storage_get_db_file(const char *filename);
+char *linphone_gtk_friends_storage_get_db_file(const char* filename);
 LINPHONE_PUBLIC void linphone_gtk_close_assistant(void);
 
 LINPHONE_PUBLIC LinphoneCore *linphone_gtk_get_core(void);
-LINPHONE_PUBLIC GtkWidget *linphone_gtk_get_main_window();
+LINPHONE_PUBLIC GtkWidget *linphone_gtk_get_main_window(void);
 LINPHONE_PUBLIC void linphone_gtk_display_something(GtkMessageType type, const gchar *message);
-LINPHONE_PUBLIC void linphone_gtk_call_terminated();
+LINPHONE_PUBLIC void linphone_gtk_call_terminated(LinphoneCall *call, const char *error);
 LINPHONE_PUBLIC void linphone_gtk_set_my_presence(LinphoneOnlineStatus ss);
 LINPHONE_PUBLIC void linphone_gtk_show_parameters(void);
 LINPHONE_PUBLIC void linphone_gtk_fill_soundcards(GtkWidget *pb);
@@ -124,7 +137,7 @@ LINPHONE_PUBLIC void linphone_gtk_show_main_window(void);
 LINPHONE_PUBLIC void linphone_gtk_log_push(OrtpLogLevel lev, const char *fmt, va_list args);
 LINPHONE_PUBLIC void linphone_gtk_destroy_log_window(void);
 LINPHONE_PUBLIC void linphone_gtk_refer_received(LinphoneCore *lc, const char *refer_to);
-LINPHONE_PUBLIC gboolean linphone_gtk_check_logs();
+LINPHONE_PUBLIC gboolean linphone_gtk_check_logs(void);
 LINPHONE_PUBLIC const gchar *linphone_gtk_get_ui_config(const char *key, const char *def);
 LINPHONE_PUBLIC int linphone_gtk_get_ui_config_int(const char *key, int def);
 LINPHONE_PUBLIC void linphone_gtk_set_ui_config_int(const char *key, int val);
@@ -149,7 +162,7 @@ LINPHONE_PUBLIC void linphone_gtk_status_icon_set_blinking(gboolean val);
 LINPHONE_PUBLIC void linphone_gtk_notify(LinphoneCall *call, LinphoneChatMessage *chat_message, const char *msg);
 
 LINPHONE_PUBLIC void linphone_gtk_load_chatroom(LinphoneChatRoom *cr, const LinphoneAddress *uri, GtkWidget *chat_view);
-LINPHONE_PUBLIC void linphone_gtk_send_text();
+LINPHONE_PUBLIC void linphone_gtk_send_text(void);
 LINPHONE_PUBLIC GtkWidget * linphone_gtk_init_chatroom(LinphoneChatRoom *cr, const LinphoneAddress *with);
 LINPHONE_PUBLIC LinphoneChatRoom * linphone_gtk_create_chatroom(const LinphoneAddress *with);
 LINPHONE_PUBLIC void linphone_gtk_text_received(LinphoneCore *lc, LinphoneChatRoom *room, LinphoneChatMessage *msg);
@@ -170,7 +183,7 @@ LINPHONE_PUBLIC void linphone_gtk_show_contact(LinphoneFriend *lf, GtkWidget *pa
 LINPHONE_PUBLIC void linphone_gtk_buddy_info_updated(LinphoneCore *lc, LinphoneFriend *lf);
 
 /*functions controlling the different views*/
-LINPHONE_PUBLIC gboolean linphone_gtk_use_in_call_view();
+LINPHONE_PUBLIC gboolean linphone_gtk_use_in_call_view(void);
 LINPHONE_PUBLIC LinphoneCall *linphone_gtk_get_currently_displayed_call(gboolean *is_conf);
 LINPHONE_PUBLIC void linphone_gtk_create_in_call_view(LinphoneCall *call);
 LINPHONE_PUBLIC void linphone_gtk_in_call_view_set_calling(LinphoneCall *call);
@@ -188,6 +201,7 @@ LINPHONE_PUBLIC void linphone_gtk_enable_transfer_button(LinphoneCore *lc, gbool
 LINPHONE_PUBLIC void linphone_gtk_enable_conference_button(LinphoneCore *lc, gboolean value);
 LINPHONE_PUBLIC void linphone_gtk_set_in_conference(LinphoneCall *call);
 LINPHONE_PUBLIC void linphone_gtk_unset_from_conference(LinphoneCall *call);
+LINPHONE_PUBLIC bool_t linphone_gtk_call_is_in_conference_view(LinphoneCall *call);
 LINPHONE_PUBLIC void linphone_gtk_terminate_conference_participant(LinphoneCall *call);
 LINPHONE_PUBLIC void linphone_gtk_in_call_view_show_encryption(LinphoneCall *call);
 LINPHONE_PUBLIC void linphone_gtk_in_call_view_hide_encryption(LinphoneCall *call);
@@ -199,7 +213,7 @@ LINPHONE_PUBLIC void linphone_gtk_show_login_frame(LinphoneProxyConfig *cfg, gbo
 LINPHONE_PUBLIC void linphone_gtk_exit_login_frame(void);
 LINPHONE_PUBLIC void linphone_gtk_set_ui_config(const char *key, const char *value);
 
-LINPHONE_PUBLIC void linphone_gtk_log_uninit();
+LINPHONE_PUBLIC void linphone_gtk_log_uninit(void);
 
 LINPHONE_PUBLIC bool_t linphone_gtk_init_instance(const char *app_name, int option, const char *addr_to_call);
 LINPHONE_PUBLIC void linphone_gtk_uninit_instance(void);
@@ -208,6 +222,7 @@ LINPHONE_PUBLIC void linphone_gtk_unmonitor_usb(void);
 
 LINPHONE_PUBLIC void linphone_gtk_fill_combo_box(GtkWidget *combo, const char **devices, const char *selected, DeviceCap cap);
 LINPHONE_PUBLIC gchar *linphone_gtk_get_record_path(const LinphoneAddress *address, gboolean is_conference);
+LINPHONE_PUBLIC gchar *linphone_gtk_get_snapshot_path(void);
 LINPHONE_PUBLIC void linphone_gtk_schedule_restart(void);
 
 LINPHONE_PUBLIC void linphone_gtk_show_audio_assistant(void);
@@ -281,6 +296,7 @@ LINPHONE_PUBLIC void linphone_gtk_mtu_set(GtkWidget *w);
 LINPHONE_PUBLIC void linphone_gtk_mtu_changed(GtkWidget *w);
 LINPHONE_PUBLIC void linphone_gtk_use_sip_info_dtmf_toggled(GtkWidget *w);
 LINPHONE_PUBLIC void linphone_gtk_ipv6_toggled(GtkWidget *w);
+LINPHONE_PUBLIC void linphone_gtk_lime_changed(GtkComboBoxText *comboext);
 LINPHONE_PUBLIC void linphone_gtk_disabled_udp_port_toggle(GtkCheckButton *button);
 LINPHONE_PUBLIC void linphone_gtk_random_udp_port_toggle(GtkCheckButton *button);
 LINPHONE_PUBLIC void linphone_gtk_udp_port_value_changed(GtkSpinButton *button);
@@ -348,5 +364,16 @@ LINPHONE_PUBLIC void linphone_gtk_reload_sound_devices(void);
 LINPHONE_PUBLIC void linphone_gtk_reload_video_devices(void);
 LINPHONE_PUBLIC bool_t linphone_gtk_is_friend(LinphoneCore *lc, const char *contact);
 LINPHONE_PUBLIC gboolean linphone_gtk_auto_answer_enabled(void);
+LINPHONE_PUBLIC void linphone_gtk_auto_answer_delay_changed(GtkSpinButton *spinbutton, gpointer user_data);
 LINPHONE_PUBLIC void linphone_gtk_update_status_bar_icons(void);
 LINPHONE_PUBLIC void linphone_gtk_enable_auto_answer(GtkToggleButton *checkbox, gpointer user_data);
+
+LINPHONE_PUBLIC void linphone_gtk_import_contacts(void);
+LINPHONE_PUBLIC void linphone_gtk_export_contacts(void);
+
+LINPHONE_PUBLIC void linphone_gtk_mark_chat_read(LinphoneChatRoom *cr);
+#ifdef __APPLE__
+LINPHONE_PUBLIC void linphone_gtk_update_badge_count();
+#endif
+
+LINPHONE_PUBLIC gboolean linphone_gtk_on_key_press(GtkWidget *widget, GdkEvent *event, gpointer user_data);
\ No newline at end of file
