@@ -23,11 +23,11 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "linphone.h"
 #include "lpconfig.h"
 #include "liblinphone_gitversion.h"
-
+#include <bctoolbox/vfs.h>
 
 #include <sys/types.h>
 #include <sys/stat.h>
-#ifndef WIN32
+#ifndef _WIN32
 #include <unistd.h>
 #endif
 
@@ -35,9 +35,9 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include <gtkosxapplication.h>
 #endif
 
-#ifdef WIN32
-#define chdir _chdir
+#ifdef _WIN32
 #include "direct.h"
+#define chdir _chdir
 #ifndef F_OK
 #define F_OK 00 /*visual studio does not define F_OK*/
 #endif
@@ -69,11 +69,7 @@ static void linphone_gtk_registration_state_changed(LinphoneCore *lc, LinphonePr
 static void linphone_gtk_notify_recv(LinphoneCore *lc, LinphoneFriend * fid);
 static void linphone_gtk_new_unknown_subscriber(LinphoneCore *lc, LinphoneFriend *lf, const char *url);
 static void linphone_gtk_auth_info_requested(LinphoneCore *lc, const char *realm, const char *username, const char *domain);
-static void linphone_gtk_display_status(LinphoneCore *lc, const char *status);
 static void linphone_gtk_configuring_status(LinphoneCore *lc, LinphoneConfiguringState status, const char *message);
-static void linphone_gtk_display_message(LinphoneCore *lc, const char *msg);
-static void linphone_gtk_display_warning(LinphoneCore *lc, const char *warning);
-static void linphone_gtk_display_url(LinphoneCore *lc, const char *msg, const char *url);
 static void linphone_gtk_call_log_updated(LinphoneCore *lc, LinphoneCallLog *cl);
 static void linphone_gtk_call_state_changed(LinphoneCore *lc, LinphoneCall *call, LinphoneCallState cs, const char *msg);
 static void linphone_gtk_call_encryption_changed(LinphoneCore *lc, LinphoneCall *call, bool_t enabled, const char *token);
@@ -152,7 +148,7 @@ static GOptionEntry linphone_options[]={
 #define RELATIVE_XML_DIR
 #define BUILD_TREE_XML_DIR "gtk"
 
-#ifndef WIN32
+#ifndef _WIN32
 #define CONFIG_FILE ".linphonerc"
 #define SECRETS_FILE ".linphone-zidcache"
 #define CERTIFICATES_PATH ".linphone-usr-crt"
@@ -169,7 +165,7 @@ char *linphone_gtk_get_config_file(const char *filename){
 	if (g_path_is_absolute(filename)) {
 		snprintf(config_file,path_max,"%s",filename);
 	} else{
-#ifdef WIN32
+#ifdef _WIN32
 		const char *appdata=getenv("APPDATA");
 		if (appdata){
 			snprintf(config_file,path_max,"%s\\%s",appdata,LINPHONE_CONFIG_DIR);
@@ -187,10 +183,10 @@ char *linphone_gtk_get_config_file(const char *filename){
 
 #define FACTORY_CONFIG_FILE "linphonerc.factory"
 static char _factory_config_file[1024];
-static const char *linphone_gtk_get_factory_config_file(){
+static const char *linphone_gtk_get_factory_config_file(void){
 	char* path = NULL;
 	/*try accessing a local file first if exists*/
-	if (access(FACTORY_CONFIG_FILE,F_OK)==0){
+	if (bctbx_file_exist(FACTORY_CONFIG_FILE)==0){
 		path = ms_strdup(FACTORY_CONFIG_FILE);
 	} else {
 		char *progdir;
@@ -198,7 +194,7 @@ static const char *linphone_gtk_get_factory_config_file(){
 		if (progpath != NULL) {
 			char *basename;
 			progdir = strdup(progpath);
-#ifdef WIN32
+#ifdef _WIN32
 			basename = strrchr(progdir, '\\');
 			if (basename != NULL) {
 				basename ++;
@@ -220,7 +216,7 @@ static const char *linphone_gtk_get_factory_config_file(){
 	}
 	if (path) {
 		//use factory file only if it exists
-		if (access(path,F_OK)==0){
+		if (bctbx_file_exist(path)==0){
 			snprintf(_factory_config_file, sizeof(_factory_config_file), "%s", path);
 			ms_free(path);
 			return _factory_config_file;
@@ -256,7 +252,8 @@ gboolean linphone_gtk_get_audio_assistant_option(void){
 }
 
 static void linphone_gtk_init_liblinphone(const char *config_file,
-		const char *factory_config_file, const char *chat_messages_db_file, const char *call_logs_db_file) {
+		const char *factory_config_file, const char *chat_messages_db_file,
+		const char *call_logs_db_file, const char *friends_db_file) {
 	LinphoneCoreVTable vtable={0};
 	gchar *secrets_file=linphone_gtk_get_config_file(SECRETS_FILE);
 	gchar *user_certificates_dir=linphone_gtk_get_config_file(CERTIFICATES_PATH);
@@ -267,12 +264,7 @@ static void linphone_gtk_init_liblinphone(const char *config_file,
 	vtable.notify_presence_received=linphone_gtk_notify_recv;
 	vtable.new_subscription_requested=linphone_gtk_new_unknown_subscriber;
 	vtable.auth_info_requested=linphone_gtk_auth_info_requested;
-	vtable.display_status=linphone_gtk_display_status;
-	vtable.display_message=linphone_gtk_display_message;
-	vtable.display_warning=linphone_gtk_display_warning;
-	vtable.display_url=linphone_gtk_display_url;
 	vtable.call_log_updated=linphone_gtk_call_log_updated;
-	//vtable.text_received=linphone_gtk_text_received;
 	vtable.message_received=linphone_gtk_text_received;
 	vtable.is_composing_received=linphone_gtk_is_composing_received;
 	vtable.refer_received=linphone_gtk_refer_received;
@@ -308,17 +300,18 @@ static void linphone_gtk_init_liblinphone(const char *config_file,
 	}
 	if (chat_messages_db_file) linphone_core_set_chat_database_path(the_core,chat_messages_db_file);
 	if (call_logs_db_file) linphone_core_set_call_logs_database_path(the_core, call_logs_db_file);
+	if (friends_db_file) linphone_core_set_friends_database_path(the_core, friends_db_file);
 }
 
 LinphoneCore *linphone_gtk_get_core(void){
 	return the_core;
 }
 
-GtkWidget *linphone_gtk_get_main_window(){
+GtkWidget *linphone_gtk_get_main_window(void){
 	return the_ui;
 }
 
-void linphone_gtk_destroy_main_window() {
+void linphone_gtk_destroy_main_window(void) {
 	linphone_gtk_destroy_window(the_ui);
 	the_ui = NULL;
 }
@@ -339,9 +332,9 @@ static void linphone_gtk_configure_window(GtkWidget *w, const char *window_name)
 
 static int get_ui_file(const char *name, char *path, int pathsize){
 	snprintf(path,pathsize,"%s/%s.ui",BUILD_TREE_XML_DIR,name);
-	if (access(path,F_OK)!=0){
+	if (bctbx_file_exist(path)!=0){
 		snprintf(path,pathsize,"%s/%s.ui",INSTALLED_XML_DIR,name);
-		if (access(path,F_OK)!=0){
+		if (bctbx_file_exist(path)!=0){
 			g_error("Could not locate neither %s/%s.ui nor %s/%s.ui",BUILD_TREE_XML_DIR,name,
 				INSTALLED_XML_DIR,name);
 			return -1;
@@ -576,7 +569,7 @@ static gboolean uribar_completion_matchfunc(GtkEntryCompletion *completion, cons
 	return ret;
 }
 
-static void load_uri_history(){
+static void load_uri_history(void){
 	GtkEntry *uribar=GTK_ENTRY(linphone_gtk_get_widget(linphone_gtk_get_main_window(),"uribar"));
 	char key[20];
 	int i;
@@ -603,7 +596,7 @@ static void load_uri_history(){
 	g_signal_connect (G_OBJECT (uribar), "changed", G_CALLBACK(linphone_gtk_on_uribar_changed), NULL);
 }
 
-static void save_uri_history(){
+static void save_uri_history(void){
 	LinphoneCore *lc=linphone_gtk_get_core();
 	LpConfig *cfg=linphone_core_get_config(lc);
 	GtkEntry *uribar=GTK_ENTRY(linphone_gtk_get_widget(linphone_gtk_get_main_window(),"uribar"));
@@ -652,7 +645,7 @@ static void completion_add_text(GtkEntry *entry, const char *text){
 	save_uri_history();
 }
 
-void on_contact_provider_search_results( LinphoneContactSearch* req, MSList* friends, void* data )
+void on_contact_provider_search_results( LinphoneContactSearch* req, bctbx_list_t* friends, void* data )
 {
 	GtkTreeIter    iter;
 	GtkEntry*    uribar = GTK_ENTRY(data);
@@ -800,10 +793,10 @@ void linphone_gtk_call_terminated(LinphoneCall *call, const char *error){
 static void linphone_gtk_update_call_buttons(LinphoneCall *call){
 	LinphoneCore *lc=linphone_gtk_get_core();
 	GtkWidget *mw=linphone_gtk_get_main_window();
-	const MSList *calls=linphone_core_get_calls(lc);
+	const bctbx_list_t *calls=linphone_core_get_calls(lc);
 	GtkWidget *button;
 	bool_t add_call=(calls!=NULL);
-	int call_list_size=ms_list_size(calls);
+	int call_list_size=bctbx_list_size(calls);
 	GtkWidget *conf_frame;
 
 	button=linphone_gtk_get_widget(mw,"start_call");
@@ -844,7 +837,7 @@ gchar *linphone_gtk_get_record_path(const LinphoneAddress *address, gboolean is_
 	int i;
 	const char *ext="wav";
 
-#ifdef WIN32
+#ifdef _WIN32
 	loctime=*localtime(&curtime);
 #else
 	localtime_r(&curtime,&loctime);
@@ -878,13 +871,36 @@ gchar *linphone_gtk_get_record_path(const LinphoneAddress *address, gboolean is_
 	return g_build_filename(dir,filename,NULL);
 }
 
+gchar *linphone_gtk_get_snapshot_path(void) {
+	const char *dir=g_get_user_special_dir(G_USER_DIRECTORY_PICTURES);
+	char filename[256]={0};
+	char date[64]={0};
+	time_t curtime=time(NULL);
+	struct tm loctime;
+	const char *ext="jpg";
+
+#ifdef _WIN32
+	loctime=*localtime(&curtime);
+#else
+	localtime_r(&curtime,&loctime);
+#endif
+	snprintf(date,sizeof(date)-1,"%i%02i%02i-%02i%02i%02i",loctime.tm_year+1900,loctime.tm_mon+1,loctime.tm_mday, loctime.tm_hour, loctime.tm_min, loctime.tm_sec);
+	snprintf(filename,sizeof(filename)-1,"%s-snapshot-%s.%s",
+			linphone_gtk_get_ui_config("title","Linphone"),
+			date, ext);
+	if (!dir) {
+		ms_message ("No directory for pictures, using [%s] instead",dir=getenv("HOME"));
+	}
+	return g_build_filename(dir,filename,NULL);
+}
+
 static gboolean linphone_gtk_start_call_do(GtkWidget *uri_bar){
 	const char *entered=gtk_entry_get_text(GTK_ENTRY(uri_bar));
 	LinphoneCore *lc=linphone_gtk_get_core();
 	LinphoneAddress *addr=linphone_core_interpret_url(lc,entered);
 
 	if (addr!=NULL){
-		LinphoneCallParams *params=linphone_core_create_default_call_parameters(lc);
+		LinphoneCallParams *params=linphone_core_create_call_params(lc, NULL);
 		gchar *record_file=linphone_gtk_get_record_path(addr,FALSE);
 		linphone_call_params_set_record_file(params,record_file);
 		linphone_core_invite_address_with_params(lc,addr,params);
@@ -898,7 +914,6 @@ static gboolean linphone_gtk_start_call_do(GtkWidget *uri_bar){
 	return FALSE;
 }
 
-
 static void accept_incoming_call(LinphoneCall *call){
 	LinphoneCore *lc=linphone_gtk_get_core();
 	LinphoneCallParams *params = linphone_core_create_call_params(lc, call);
@@ -1007,7 +1022,7 @@ void linphone_gtk_used_identity_changed(GtkWidget *w){
 
 void on_proxy_refresh_button_clicked(GtkWidget *w){
 	LinphoneCore *lc=linphone_gtk_get_core();
-	MSList const *item=linphone_core_get_proxy_config_list(lc);
+	bctbx_list_t const *item=linphone_core_get_proxy_config_list(lc);
 	while (item != NULL) {
 		LinphoneProxyConfig *lpc=(LinphoneProxyConfig*)item->data;
 		linphone_proxy_config_edit(lpc);
@@ -1016,6 +1031,21 @@ void on_proxy_refresh_button_clicked(GtkWidget *w){
 	}
 }
 
+static gboolean grab_focus(GtkWidget *w){
+	gtk_widget_grab_focus(w);
+	return FALSE;
+}
+
+void linphone_gtk_viewswitch_changed(GtkNotebook *notebook, GtkWidget *page, guint page_num, gpointer user_data){
+	GtkWidget *main_window = linphone_gtk_get_main_window();
+	GtkWidget *friendlist = linphone_gtk_get_widget(main_window,"contact_list");
+	GtkWidget *w = (GtkWidget*)g_object_get_data(G_OBJECT(friendlist),"chatview");
+
+	if (page_num == gtk_notebook_page_num(GTK_NOTEBOOK(notebook),w)) {
+		g_idle_add((GSourceFunc)grab_focus,linphone_gtk_get_widget(page,"text_entry"));
+	}
+}
+
 static void linphone_gtk_notify_recv(LinphoneCore *lc, LinphoneFriend * fid){
 	linphone_gtk_show_friends();
 }
@@ -1133,34 +1163,12 @@ static void linphone_gtk_dtmf_received(LinphoneCore *lc, LinphoneCall *call, int
 	ms_message("Dtmf %c received.",dtmf);
 }
 
-static void linphone_gtk_display_status(LinphoneCore *lc, const char *status){
-	GtkWidget *w=linphone_gtk_get_main_window();
-	GtkWidget *status_bar=linphone_gtk_get_widget(w,"status_bar");
-
-	gtk_statusbar_push(GTK_STATUSBAR(status_bar),
-			gtk_statusbar_get_context_id(GTK_STATUSBAR(status_bar),""),
-			status);
-}
 
 static void linphone_gtk_configuring_status(LinphoneCore *lc, LinphoneConfiguringState status, const char *message) {
 	if (config_fetching_dialog) linphone_gtk_close_config_fetching(config_fetching_dialog, status);
 	config_fetching_dialog=NULL;
 }
 
-static void linphone_gtk_display_message(LinphoneCore *lc, const char *msg){
-	linphone_gtk_display_something(GTK_MESSAGE_INFO,msg);
-}
-
-static void linphone_gtk_display_warning(LinphoneCore *lc, const char *warning){
-	linphone_gtk_display_something(GTK_MESSAGE_WARNING,warning);
-}
-
-static void linphone_gtk_display_url(LinphoneCore *lc, const char *msg, const char *url){
-	char richtext[4096];
-	snprintf(richtext,sizeof(richtext),"%s %s",msg,url);
-	linphone_gtk_display_something(GTK_MESSAGE_INFO,richtext);
-}
-
 static void linphone_gtk_call_log_updated(LinphoneCore *lc, LinphoneCallLog *cl){
 	GtkWidget *w=(GtkWidget*)g_object_get_data(G_OBJECT(linphone_gtk_get_main_window()),"call_logs");
 	if (w) linphone_gtk_call_log_update(w);
@@ -1168,7 +1176,7 @@ static void linphone_gtk_call_log_updated(LinphoneCore *lc, LinphoneCallLog *cl)
 }
 
 #ifdef HAVE_NOTIFY
-static bool_t notify_actions_supported() {
+static bool_t notify_actions_supported(void) {
 	bool_t accepts_actions = FALSE;
 	GList *capabilities = notify_get_server_caps();
 	GList *c;
@@ -1241,6 +1249,7 @@ void linphone_gtk_notify(LinphoneCall *call, LinphoneChatMessage *chat_message,
 		linphone_gtk_show_main_window();
 #endif
 	} else if (!gtk_window_is_active((GtkWindow*)linphone_gtk_get_main_window())) {
+		gboolean show_main_window = FALSE;
 #ifdef HAVE_NOTIFY
 		char *body=NULL;
 		char *remote=call!=NULL ? linphone_call_get_remote_address_as_string(call) : NULL;
@@ -1254,13 +1263,15 @@ void linphone_gtk_notify(LinphoneCall *call, LinphoneChatMessage *chat_message,
 			break;
 			case LinphoneCallIncomingReceived:
 				n=build_notification(_("Incoming call"),body=g_markup_printf_escaped("<b>%s</b>",remote));
-				if (notify_actions_supported()) {
-					notify_notification_add_action (n,"answer", _("Answer"),
-						NOTIFY_ACTION_CALLBACK(linphone_gtk_answer_clicked),NULL,NULL);
-					notify_notification_add_action (n,"decline",_("Decline"),
-						NOTIFY_ACTION_CALLBACK(linphone_gtk_decline_clicked),NULL,NULL);
-				}
-				show_notification(n);
+				if (n){
+					if (notify_actions_supported()) {
+						notify_notification_add_action (n,"answer", _("Answer"),
+							NOTIFY_ACTION_CALLBACK(linphone_gtk_answer_clicked),NULL,NULL);
+						notify_notification_add_action (n,"decline",_("Decline"),
+							NOTIFY_ACTION_CALLBACK(linphone_gtk_decline_clicked),NULL,NULL);
+					}
+					show_notification(n);
+				}else show_main_window = TRUE;
 			break;
 			case LinphoneCallPausedByRemote:
 				make_notification(_("Call paused"),body=g_markup_printf_escaped(_("<b>by %s</b>"),remote));
@@ -1270,7 +1281,11 @@ void linphone_gtk_notify(LinphoneCall *call, LinphoneChatMessage *chat_message,
 		}
 		if (body) g_free(body);
 		if (remote) g_free(remote);
+#else
+		if (linphone_call_get_state(call) == LinphoneCallIncomingReceived)
+			show_main_window = TRUE;
 #endif
+		if (show_main_window) linphone_gtk_show_main_window();
 	}
 }
 
@@ -1299,7 +1314,7 @@ static void on_call_updated_response(GtkWidget *dialog, gint responseid, gpointe
 	LinphoneCall *call = (LinphoneCall *)g_object_get_data(G_OBJECT(dialog), "call");
 	if (linphone_call_get_state(call)==LinphoneCallUpdatedByRemote){
 		LinphoneCore *lc=linphone_call_get_core(call);
-		LinphoneCallParams *params=linphone_call_params_copy(linphone_call_get_current_params(call));
+		LinphoneCallParams *params = linphone_core_create_call_params(lc, call);
 		linphone_call_params_enable_video(params,responseid==GTK_RESPONSE_YES);
 		linphone_core_accept_call_update(lc,call,params);
 		linphone_call_params_destroy(params);
@@ -1471,7 +1486,7 @@ void linphone_gtk_open_browser(const char *uri) {
 		g_warning("Could not open %s: %s", uri, error->message);
 		g_error_free(error);
 	}
-#elif defined(WIN32)
+#elif defined(_WIN32)
 	HINSTANCE instance = ShellExecute(NULL, "open", uri, NULL, NULL, SW_SHOWNORMAL);
 	if ((int)instance <= 32) {
 		g_warning("Could not open %s (error #%i)", uri, (int)instance);
@@ -1491,7 +1506,7 @@ void linphone_gtk_link_to_website(GtkWidget *item){
 	linphone_gtk_open_browser(home);
 }
 
-static GtkWidget *create_icon_menu(){
+static GtkWidget *create_icon_menu(void){
 	GtkWidget *menu=gtk_menu_new();
 	GtkWidget *menu_item;
 	GtkWidget *image;
@@ -1579,6 +1594,9 @@ void linphone_gtk_status_icon_set_blinking(gboolean val) {
 	if(icon) {
 		linphone_status_icon_enable_blinking(icon, val);
 	}
+#ifdef __APPLE__
+	linphone_gtk_update_badge_count();
+#endif
 }
 
 void linphone_gtk_options_activate(GtkWidget *item){
@@ -1602,7 +1620,7 @@ static void init_identity_combo(GtkComboBox *box){
 }
 
 void linphone_gtk_load_identities(void){
-	const MSList *elem;
+	const bctbx_list_t *elem;
 	GtkComboBox *box=GTK_COMBO_BOX(linphone_gtk_get_widget(linphone_gtk_get_main_window(),"identities"));
 	char *def_identity;
 	LinphoneProxyConfig *def=NULL;
@@ -1617,14 +1635,14 @@ void linphone_gtk_load_identities(void){
 		store=GTK_LIST_STORE(gtk_combo_box_get_model(box));
 	}
 	gtk_list_store_clear(store);
-	linphone_core_get_default_proxy(linphone_gtk_get_core(),&def);
+	def = linphone_core_get_default_proxy_config(linphone_gtk_get_core());
 	def_identity=g_strdup_printf(_("%s (Default)"),linphone_core_get_primary_contact(linphone_gtk_get_core()));
 	gtk_list_store_append(store,&iter);
 	gtk_list_store_set(store,&iter,0,def_identity,1,NULL,2,NULL,-1);
 	g_free(def_identity);
 	for(i=1,elem=linphone_core_get_proxy_config_list(linphone_gtk_get_core());
 			elem!=NULL;
-			elem=ms_list_next(elem),i++){
+			elem=bctbx_list_next(elem),i++){
 		LinphoneProxyConfig *cfg=(LinphoneProxyConfig*)elem->data;
 		gtk_list_store_append(store,&iter);
 		gtk_list_store_set(store,&iter,0,linphone_proxy_config_get_identity(cfg),1,
@@ -1644,7 +1662,7 @@ static void linphone_gtk_dtmf_pressed(GtkButton *button){
 	gtk_editable_insert_text(GTK_EDITABLE(uri_bar),label,1,&pos);
 	linphone_core_play_dtmf (linphone_gtk_get_core(),label[0],-1);
 	if (linphone_core_in_call(linphone_gtk_get_core())){
-		linphone_core_send_dtmf(linphone_gtk_get_core(),label[0]);
+		linphone_call_send_dtmf(linphone_core_get_current_call(linphone_gtk_get_core()),label[0]);
 	}
 }
 
@@ -1674,9 +1692,9 @@ static void linphone_gtk_check_menu_items(void){
 	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(selfview_item),selfview);
 }
 
-static gboolean linphone_gtk_can_manage_accounts(){
+static gboolean linphone_gtk_can_manage_accounts(void){
 	LinphoneCore *lc=linphone_gtk_get_core();
-	const MSList *elem;
+	const bctbx_list_t *elem;
 	for(elem=linphone_core_get_sip_setups(lc);elem!=NULL;elem=elem->next){
 		SipSetup *ss=(SipSetup*)elem->data;
 		if (sip_setup_get_capabilities(ss) & SIP_SETUP_CAP_ACCOUNT_MANAGER){
@@ -1686,7 +1704,7 @@ static gboolean linphone_gtk_can_manage_accounts(){
 	return FALSE;
 }
 
-static void linphone_gtk_configure_main_window(){
+static void linphone_gtk_configure_main_window(void){
 	static gboolean config_loaded=FALSE;
 	static const char *title;
 	static const char *home;
@@ -1731,8 +1749,7 @@ static void linphone_gtk_configure_main_window(){
 
 void linphone_gtk_manage_login(void){
 	LinphoneCore *lc=linphone_gtk_get_core();
-	LinphoneProxyConfig *cfg=NULL;
-	linphone_core_get_default_proxy(lc,&cfg);
+	LinphoneProxyConfig *cfg=linphone_core_get_default_proxy_config(lc);
 	if (cfg){
 		SipSetup *ss=linphone_proxy_config_get_sip_setup(cfg);
 		if (ss && (sip_setup_get_capabilities(ss) & SIP_SETUP_CAP_LOGIN)){
@@ -1857,6 +1874,36 @@ void linphone_gtk_show_keypad_checked(GtkCheckMenuItem *check_menu_item) {
 	}
 }
 
+void linphone_gtk_import_contacts(void) {
+	GtkWidget *mw = linphone_gtk_get_main_window();
+	GtkWidget *dialog = gtk_file_chooser_dialog_new("Open vCard file", (GtkWindow *)mw, GTK_FILE_CHOOSER_ACTION_OPEN, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, NULL);
+
+	if (gtk_dialog_run(GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT) {
+		LinphoneCore *lc = linphone_gtk_get_core();
+		char *filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
+		LinphoneFriendList *list = linphone_core_get_default_friend_list(lc);
+		linphone_friend_list_import_friends_from_vcard4_file(list, filename);
+		g_free(filename);
+		linphone_gtk_show_friends();
+	}
+	gtk_widget_destroy(dialog);
+}
+
+void linphone_gtk_export_contacts(void) {
+	GtkWidget *mw = linphone_gtk_get_main_window();
+	GtkWidget *dialog = gtk_file_chooser_dialog_new("Save vCards as", (GtkWindow *)mw, GTK_FILE_CHOOSER_ACTION_SAVE, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT, NULL);
+	gtk_file_chooser_set_do_overwrite_confirmation(GTK_FILE_CHOOSER(dialog), TRUE);
+
+	if (gtk_dialog_run(GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT) {
+		LinphoneCore *lc = linphone_gtk_get_core();
+		char *filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
+		LinphoneFriendList *list = linphone_core_get_default_friend_list(lc);
+		linphone_friend_list_export_friends_as_vcard4_file(list, filename);
+		g_free(filename);
+	}
+	gtk_widget_destroy(dialog);
+}
+
 gboolean linphone_gtk_keypad_destroyed_handler(void) {
 	GtkWidget *mw = linphone_gtk_get_main_window();
 	GtkWidget *show_keypad_item = linphone_gtk_get_widget(mw, "show_keypad_menu_item");
@@ -1881,9 +1928,7 @@ static void linphone_gtk_init_main_window(void){
 	load_uri_history();
 	linphone_core_reset_missed_calls_count(linphone_gtk_get_core());
 	main_window=linphone_gtk_get_main_window();
-#ifndef CALL_LOGS_STORAGE_ENABLED
 	linphone_gtk_call_log_update(main_window);
-#endif
 
 	linphone_gtk_update_call_buttons (NULL);
 	g_object_set_data(G_OBJECT(main_window),"keypad",NULL);
@@ -1893,6 +1938,7 @@ static void linphone_gtk_init_main_window(void){
 		G_CALLBACK (linphone_gtk_close), main_window);
 #ifdef HAVE_GTK_OSX
 	{
+		gtk_widget_show(main_window);
 		GtkWidget *menubar=linphone_gtk_get_widget(main_window,"menubar1");
 		GtkosxApplication *theMacApp = gtkosx_application_get();
 		gtkosx_application_set_menu_bar(theMacApp,GTK_MENU_SHELL(menubar));
@@ -1910,7 +1956,7 @@ static void linphone_gtk_init_main_window(void){
 #endif
 }
 
-void linphone_gtk_log_handler(OrtpLogLevel lev, const char *fmt, va_list args){
+void linphone_gtk_log_handler(const char*domain, OrtpLogLevel lev, const char *fmt, va_list args){
 	if (verbose){
 		const char *lname="undef";
 		char *msg;
@@ -1951,16 +1997,25 @@ void linphone_gtk_log_handler(OrtpLogLevel lev, const char *fmt, va_list args){
 
 
 void linphone_gtk_refer_received(LinphoneCore *lc, const char *refer_to){
-	GtkEntry * uri_bar =GTK_ENTRY(linphone_gtk_get_widget(
-		linphone_gtk_get_main_window(), "uribar"));
-	char *text;
-	linphone_gtk_notify(NULL,NULL,(text=ms_strdup_printf(_("We are transferred to %s"),refer_to)));
-	g_free(text);
-	gtk_entry_set_text(uri_bar, refer_to);
-	linphone_gtk_start_call(linphone_gtk_get_main_window());
+	char method[20] = "";
+	LinphoneAddress *addr = linphone_address_new(refer_to);
+	if(addr) {
+		const char *tmp = linphone_address_get_method_param(addr);
+		strncpy(method, tmp, sizeof(20));
+		linphone_address_destroy(addr);
+	}
+	if(strlen(method) == 0 || strcmp(method, "INVITE") == 0) {
+		GtkEntry * uri_bar =GTK_ENTRY(linphone_gtk_get_widget(
+			linphone_gtk_get_main_window(), "uribar"));
+		char *text;
+		linphone_gtk_notify(NULL,NULL,(text=ms_strdup_printf(_("We are transferred to %s"),refer_to)));
+		g_free(text);
+		gtk_entry_set_text(uri_bar, refer_to);
+		linphone_gtk_start_call(linphone_gtk_get_main_window());
+	}
 }
 
-static void linphone_gtk_check_soundcards(){
+static void linphone_gtk_check_soundcards(void){
 	const char **devices=linphone_core_get_sound_devices(linphone_gtk_get_core());
 	if (devices==NULL || devices[0]==NULL){
 		linphone_gtk_display_something(GTK_MESSAGE_WARNING,
@@ -1970,6 +2025,12 @@ static void linphone_gtk_check_soundcards(){
 }
 
 static void linphone_gtk_quit_core(void){
+#ifdef HAVE_GTK_OSX
+	{
+		GtkosxApplication *theMacApp = gtkosx_application_get();
+		gtkosx_application_set_menu_bar(theMacApp,NULL);
+	}
+#endif
 	linphone_gtk_unmonitor_usb();
 	g_source_remove_by_user_data(linphone_gtk_get_core());
 #ifdef BUILD_WIZARD
@@ -2043,11 +2104,11 @@ static void sigint_handler(int signum){
 }
 
 static void populate_xdg_data_dirs_envvar(void) {
-#ifndef WIN32
+#ifndef _WIN32
 	int i;
 	gchar *value;
 	gchar **paths;
-	
+
 	if(g_getenv("XDG_DATA_DIRS") == NULL) {
 		value = g_strdup("/usr/share:/usr/local/share:/opt/local/share");
 	} else {
@@ -2072,7 +2133,7 @@ int main(int argc, char *argv[]){
 	const char *icon_name=LINPHONE_ICON_NAME;
 	const char *app_name="Linphone";
 	LpConfig *factory;
-	char *chat_messages_db_file, *call_logs_db_file;
+	char *chat_messages_db_file, *call_logs_db_file, *friends_db_file;
 	GError *error=NULL;
 	const char *tmp;
 
@@ -2091,7 +2152,7 @@ int main(int argc, char *argv[]){
 	/*for pulseaudio:*/
 	g_setenv("PULSE_PROP_media.role", "phone", TRUE);
 #endif
-	
+
 	populate_xdg_data_dirs_envvar();
 
 	lang=linphone_gtk_get_lang(config_file);
@@ -2100,7 +2161,7 @@ int main(int argc, char *argv[]){
 		if (!lang) lang = g_getenv("LANG");
 	}
 	if (lang && lang[0]!='\0'){
-#ifdef WIN32
+#ifdef _WIN32
 		if (strncmp(lang,"zh",2)==0){
 			workaround_gtk_entry_chinese_bug=TRUE;
 		}
@@ -2118,7 +2179,7 @@ int main(int argc, char *argv[]){
 #else
 	g_message("NLS disabled.\n");
 #endif
-#ifdef WIN32
+#ifdef _WIN32
 	gtk_rc_add_default_file("./gtkrc");
 #endif
 	gdk_threads_enter();
@@ -2195,35 +2256,38 @@ int main(int argc, char *argv[]){
 
 core_start:
 	if (linphone_gtk_init_instance(app_name, start_option, addr_to_call) == FALSE){
-		g_warning("Another running instance of linphone has been detected. It has been woken-up.");
+		g_warning("Another running instance of Linphone has been detected. It has been woken-up.");
 		g_warning("This instance is going to exit now.");
 		gdk_threads_leave();
 		return 0;
 	}
-
 	the_ui=linphone_gtk_create_window("main", NULL);
 
 	g_object_set_data(G_OBJECT(the_ui),"is_created",GINT_TO_POINTER(FALSE));
+	g_signal_connect(G_OBJECT (the_ui), "key_press_event", G_CALLBACK (linphone_gtk_on_key_press), NULL);
 
 	linphone_gtk_create_log_window();
 	linphone_core_enable_logs_with_cb(linphone_gtk_log_handler);
+	/*it is possible to filter in or out some logs by configuring per log domain:*/
+	/*ortp_set_log_level_mask("belle-sip", ORTP_ERROR);*/
 
 	chat_messages_db_file=linphone_gtk_message_storage_get_db_file(NULL);
 	call_logs_db_file = linphone_gtk_call_logs_storage_get_db_file(NULL);
-	linphone_gtk_init_liblinphone(config_file, factory_config_file, chat_messages_db_file, call_logs_db_file);
+	friends_db_file = linphone_gtk_friends_storage_get_db_file(NULL);
+	linphone_gtk_init_liblinphone(config_file, factory_config_file, chat_messages_db_file, call_logs_db_file, friends_db_file);
 	g_free(chat_messages_db_file);
 	g_free(call_logs_db_file);
+	g_free(friends_db_file);
 
-#ifdef CALL_LOGS_STORAGE_ENABLED
 	linphone_gtk_call_log_update(the_ui);
-#endif
+	linphone_gtk_show_friends();
 
 	/* do not lower timeouts under 30 ms because it exhibits a bug on gtk+/win32, with cpu running 20% all the time...*/
 	gtk_timeout_add(30,(GtkFunction)linphone_gtk_iterate,(gpointer)linphone_gtk_get_core());
 	gtk_timeout_add(30,(GtkFunction)linphone_gtk_check_logs,(gpointer)linphone_gtk_get_core());
 
 	signal(SIGINT, sigint_handler);
-	
+
 	gtk_main();
 	linphone_gtk_quit();
 
@@ -2250,7 +2314,7 @@ int CALLBACK WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLi
 GtkWidget *linphone_gtk_make_tab_header(const gchar *label, const gchar *icon_name, gboolean show_quit_button, GCallback cb, gpointer user_data) {
 	GtkWidget *tab_header=gtk_hbox_new (FALSE,0);
 	GtkWidget *label_widget = gtk_label_new (label);
-	
+
 	if(icon_name) {
 		GtkWidget *icon=gtk_image_new_from_icon_name(icon_name, GTK_ICON_SIZE_MENU);
 #ifdef HAVE_GTK_OSX
