@@ -33,7 +33,7 @@ static LinphoneAccountCreator * linphone_gtk_assistant_get_creator(GtkWidget *w)
 
 static void linphone_gtk_create_account_cb(LinphoneAccountCreator *creator, LinphoneAccountCreatorStatus status) {
 	GtkWidget *assistant = (GtkWidget *)linphone_account_creator_get_user_data(creator);
-	if (status == LinphoneAccountCreatorOk) {
+	if (status == LinphoneAccountCreatorAccountCreated) {
 		// Go to page_6_linphone_account_validation_wait
 		gtk_assistant_set_current_page(GTK_ASSISTANT(assistant), 6);
 	} else { // Error when attempting to create the account
@@ -50,7 +50,7 @@ static void create_account(GtkWidget *assistant) {
 
 static void linphone_gtk_test_account_validation_cb(LinphoneAccountCreator *creator, LinphoneAccountCreatorStatus status) {
 	GtkWidget *assistant = (GtkWidget *)linphone_account_creator_get_user_data(creator);
-	if (status == LinphoneAccountCreatorOk) {
+	if (status == LinphoneAccountCreatorAccountValidated) {
 		// Go to page_9_finish
 		gtk_assistant_set_current_page(GTK_ASSISTANT(assistant), 9);
 	} else {
@@ -209,14 +209,18 @@ static gboolean update_interface_with_username_availability(GtkWidget *page) {
 	GtkWidget *assistant = gtk_widget_get_toplevel(page);
 	GtkImage* isUsernameOk = GTK_IMAGE(linphone_gtk_get_widget(assistant, "p4_image_username_ok"));
 	GtkLabel* usernameError = GTK_LABEL(linphone_gtk_get_widget(assistant, "p4_label_error"));
-	int account_existing = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(page), "is_username_used"));
+	int account_status = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(page), "is_username_used"));
 
-	if (account_existing == 0) {
+	if (account_status == LinphoneAccountCreatorAccountNotExist) {
 		g_object_set_data(G_OBJECT(page), "is_username_available", GINT_TO_POINTER(1));
 		gtk_image_set_from_stock(isUsernameOk, GTK_STOCK_OK, GTK_ICON_SIZE_LARGE_TOOLBAR);
 		gtk_label_set_text(usernameError, "");
+	} else if (account_status == LinphoneAccountCreatorAccountExist) {
+		gtk_label_set_text(usernameError, _("Username is already in use!"));
+		g_object_set_data(G_OBJECT(page), "is_username_available", GINT_TO_POINTER(0));
+		gtk_image_set_from_stock(isUsernameOk, GTK_STOCK_NO, GTK_ICON_SIZE_LARGE_TOOLBAR);
 	} else {
-		gtk_label_set_text(usernameError, "Username is already in use!");
+		gtk_label_set_text(usernameError, _("Failed to check username availability. Please try again later."));
 		g_object_set_data(G_OBJECT(page), "is_username_available", GINT_TO_POINTER(0));
 		gtk_image_set_from_stock(isUsernameOk, GTK_STOCK_NO, GTK_ICON_SIZE_LARGE_TOOLBAR);
 	}
@@ -227,8 +231,7 @@ static gboolean update_interface_with_username_availability(GtkWidget *page) {
 static void linphone_gtk_test_account_existence_cb(LinphoneAccountCreator *creator, LinphoneAccountCreatorStatus status) {
 	GtkWidget *assistant = (GtkWidget *)linphone_account_creator_get_user_data(creator);
 	GtkWidget *page = gtk_assistant_get_nth_page(GTK_ASSISTANT(assistant), gtk_assistant_get_current_page(GTK_ASSISTANT(assistant)));
-	gboolean account_existing = (status != LinphoneAccountCreatorOk);
-	g_object_set_data(G_OBJECT(page), "is_username_used", GINT_TO_POINTER(account_existing));
+	g_object_set_data(G_OBJECT(page), "is_username_used", GINT_TO_POINTER(status));
 	gdk_threads_add_idle((GSourceFunc)update_interface_with_username_availability, page);
 }
 
@@ -306,7 +309,7 @@ void linphone_gtk_account_creation_password_changed(GtkEntry *entry) {
 	} else {
 		if (gtk_entry_get_text_length(password) < PASSWORD_MIN_SIZE) {
 			gtk_label_set_text(passwordError, "Password is too short !");
-		} else if (!g_ascii_strcasecmp(gtk_entry_get_text(password), gtk_entry_get_text(password_confirm)) == 0) {
+		} else if (g_ascii_strcasecmp(gtk_entry_get_text(password), gtk_entry_get_text(password_confirm)) != 0) {
 			gtk_label_set_text(passwordError, "Passwords don't match !");
 		}
 		g_object_set_data(G_OBJECT(page), "is_password_correct", GINT_TO_POINTER(0));
@@ -323,7 +326,7 @@ static void linphone_gtk_assistant_init(GtkWidget *w) {
 	linphone_account_creator_cbs_set_validation_tested(cbs, linphone_gtk_test_account_validation_cb);
 	linphone_account_creator_cbs_set_create_account(cbs, linphone_gtk_create_account_cb);
 	g_object_set_data(G_OBJECT(w), "creator", creator);
-	
+
 	gtk_assistant_set_forward_page_func(GTK_ASSISTANT(w), linphone_gtk_assistant_forward, w, NULL);
 }
 
