@@ -59,12 +59,14 @@ struct AndroidReaderContext {
 		androidCamera = 0;
 		previewWindow = 0;
 		rotation = rotationSavedDuringVSize = UNDEFINED_ROTATION;
+		allocator = ms_yuv_buf_allocator_new();
 	};
 
 	~AndroidReaderContext(){
 		if (frame != 0) {
 			freeb(frame);
 		}
+		ms_yuv_buf_allocator_free(allocator);
 		ms_mutex_destroy(&mutex);
 	};
 
@@ -81,6 +83,7 @@ struct AndroidReaderContext {
 	int rotation, rotationSavedDuringVSize;
 	int useDownscaling;
 	char fps_context[64];
+	MSYuvBufAllocator *allocator;
 
 	jobject androidCamera;
 	jobject previewWindow;
@@ -288,7 +291,7 @@ void video_capture_preprocess(MSFilter *f){
 
 	jmethodID method = env->GetStaticMethodID(d->helperClass,"startRecording", "(IIIIIJ)Ljava/lang/Object;");
 
-	ms_message("Starting Android camera '%d' (rotation:%d)\n", ((AndroidWebcamConfig*)d->webcam->data)->id, d->rotation);
+	ms_message("Starting Android camera '%d' (rotation:%d)", ((AndroidWebcamConfig*)d->webcam->data)->id, d->rotation);
 	jobject cam = env->CallStaticObjectMethod(d->helperClass, method,
 			((AndroidWebcamConfig*)d->webcam->data)->id,
 			d->hwCapableSize.width,
@@ -402,7 +405,7 @@ static void video_capture_cam_init(MSWebCam *cam){
 static MSFilter *video_capture_create_reader(MSWebCam *obj){
 	ms_message("Instanciating Android VIDEO capture MS filter");
 
-	MSFilter* lFilter = ms_filter_new_from_desc(&ms_video_capture_desc);
+	MSFilter* lFilter = ms_factory_create_filter_from_desc(ms_web_cam_get_factory(obj), &ms_video_capture_desc);
 	getContext(lFilter)->webcam = obj;
 	
 	return lFilter;
@@ -441,7 +444,7 @@ static void video_capture_detect(MSWebCamManager *obj){
 		env->GetIntArrayRegion(orientation, i, 1, &c->orientation);
 		cam->data = c;
 		cam->name = ms_strdup("Android video name");
-		char* idstring = (char*) malloc(15);
+		char* idstring = (char*) ms_malloc(15);
 		snprintf(idstring, 15, "Android%d", c->id);
 		cam->id = idstring;
 		ms_web_cam_manager_add_cam(obj,cam);
@@ -511,7 +514,7 @@ JNIEXPORT void JNICALL Java_org_linphone_mediastream_video_capture_AndroidVideoA
 	   It only implies one thing: image needs to rotated by that amount to be correctly
 	   displayed.
 	*/
- 	mblk_t* yuv_block = copy_ycbcrbiplanar_to_true_yuv_with_rotation_and_down_scale_by_2(y_src
+ 	mblk_t* yuv_block = copy_ycbcrbiplanar_to_true_yuv_with_rotation_and_down_scale_by_2(d->allocator, y_src
 														, cbcr_src
 														, image_rotation_correction
 														, d->usedSize.width
