@@ -26,7 +26,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "ortp/rtp.h"
 
 #include <stdint.h>
-#if defined(ANDROID) || (defined(WIN32) && !WINAPI_FAMILY_PARTITION(WINAPI_PARTITION_DESKTOP))
+#if defined(MS2_WINDOWS_PHONE)
 #include <opus.h>
 #else
 #include <opus/opus.h>
@@ -35,9 +35,8 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #define SIGNAL_SAMPLE_SIZE  2 // 2 bytes per sample
 
 /* Define codec specific settings */
-#define FRAME_LENGTH			20 // ptime may be 20, 40, 60, 80, 100 or 120, packets composed of multiples 20ms frames
-#define MAX_BYTES_PER_FRAME     500 // Equals peak bitrate of 200 kbps
-#define MAX_INPUT_FRAMES        6
+#define MAX_BYTES_PER_MS	25  // Equals peak bitrate of 200 kbps
+#define MAX_INPUT_FRAMES	5   // The maximum amount of Opus frames in a packet we are using
 
 
 /**
@@ -47,6 +46,8 @@ typedef struct _OpusEncData {
 	OpusEncoder *state;
 	MSBufferizer *bufferizer;
 	uint32_t ts;
+	uint8_t *pcmbuffer;
+	int pcmbufsize;
 	int samplerate;
 	int channels;
 	int application;
@@ -93,7 +94,7 @@ static void ms_opus_enc_init(MSFilter *f) {
 	d->ptime = 20;
 	d->minptime = 20; // defaut shall be 3, but we never use less than 20.
 	d->maxaveragebitrate = -1;
-	d->stereo = 1;
+	d->stereo = 0; /*stereo=0 is the default mode in opus rtp draft*/
 	d->channels = 1;
 	d->vbr = 1;
 	d->useinbandfec = 0;
@@ -104,6 +105,8 @@ static void ms_opus_enc_init(MSFilter *f) {
 
 static void ms_opus_enc_preprocess(MSFilter *f) {
 	int error;
+	int opusComplexity = -1;
+	const char *env = NULL;
 
 	OpusEncData *d = (OpusEncData *)f->data;
 	/* create the encoder */
@@ -113,14 +116,32 @@ static void ms_opus_enc_preprocess(MSFilter *f) {
 		return;
 	}
 
-	/* set complexity to 0 for single processor arm devices */
-#if defined(__arm__) || defined(_M_ARM)
-	if (ms_factory_get_cpu_count(f->factory)==1){
-		opus_encoder_ctl(d->state, OPUS_SET_COMPLEXITY(0));
-	}else{
-		opus_encoder_ctl(d->state, OPUS_SET_COMPLEXITY(5));
-	}
+	
+#ifndef MS2_WINDOWS_UNIVERSAL
+	env = getenv("MS2_OPUS_COMPLEXITY");
 #endif
+	if (env != NULL) {
+		opusComplexity = atoi(env);
+		if (opusComplexity < -1)
+			opusComplexity = -1; /*our default value*/
+		if (opusComplexity > 10)
+			opusComplexity = 10;
+	}
+	if (opusComplexity == -1){
+#if defined(__arm__) || defined(_M_ARM)
+		int cpucount = ms_factory_get_cpu_count(f->factory);
+		if (cpucount == 1){
+			opusComplexity = 0; /* set complexity to 0 for single processor arm devices */ 
+		}else if (cpucount == 2) {
+			opusComplexity = 5; 
+		}
+#endif
+	}
+	if (opusComplexity != -1){
+		ms_message("Set Opus complexity to %d", opusComplexity);
+		opus_encoder_ctl(d->state, OPUS_SET_COMPLEXITY(opusComplexity));
+	} /*otherwise we let opus with its default value, which is 9*/
+	
 	error = opus_encoder_ctl(d->state, OPUS_SET_PACKET_LOSS_PERC(10));
 	if (error != OPUS_OK) {
 		ms_error("Could not set default loss percentage to opus encoder: %s", opus_strerror(error));
@@ -136,6 +157,9 @@ static void ms_opus_enc_preprocess(MSFilter *f) {
 		if (error != OPUS_OK) {
 			ms_error("could not force mono channel to opus encoder: %s", opus_strerror(error));
 		}
+		if (d->channels == 2) ms_message("Opus encoder configured to encode mono despite it is feed with stereo.");
+	}else if (d->channels == 2){
+		ms_message("Opus encoder configured to encode stereo.");
 	}
 
 	ms_filter_lock(f);
@@ -149,77 +173,141 @@ static void ms_opus_enc_preprocess(MSFilter *f) {
 
 static void ms_opus_enc_process(MSFilter *f) {
 	OpusEncData *d = (OpusEncData *)f->data;
-	mblk_t *im;
+	OpusRepacketizer *repacketizer = NULL;
 	mblk_t *om = NULL;
+	int packet_size, pcm_buffer_size;
+	int max_frame_byte_size, ptime = 20;
+	int frame_count = 0, frame_size = 0;
+	opus_int32 total_length = 0;
+	uint8_t *repacketizer_frame_buffer[MAX_INPUT_FRAMES];
 	int i;
-	int frameNumber, packet_size;
-	uint8_t *signalFrameBuffer = NULL;
-	uint8_t *codedFrameBuffer[MAX_INPUT_FRAMES];
-	OpusRepacketizer *rp = opus_repacketizer_create();
-	opus_int32 ret = 0;
-	opus_int32 totalLength = 0;
-	int frame_size = d->samplerate * FRAME_LENGTH / 1000; /* in samples */
-
-	// lock the access while getting ptime
 	ms_filter_lock(f);
-	frameNumber = d->ptime/FRAME_LENGTH; /* encode 20ms frames, ptime is a multiple of 20ms */
-	packet_size = d->samplerate * d->ptime / 1000; /* in samples */
+	ptime = d->ptime;
+	packet_size = d->samplerate * ptime / 1000; /* in samples */
 	ms_filter_unlock(f);
-
-
-	while ((im = ms_queue_get(f->inputs[0])) != NULL) {
-		ms_bufferizer_put(d->bufferizer, im);
+	
+	switch (ptime) {
+		case 10:
+			frame_size = d->samplerate * 10 / 1000;
+			frame_count = 1;
+			break;
+		case 20:
+			frame_size = d->samplerate * 20 / 1000;
+			frame_count = 1;
+			break;
+		case 40:
+			frame_size = d->samplerate * 40 / 1000;
+			frame_count = 1;
+			break;
+		case 60:
+			frame_size = d->samplerate * 60 / 1000;
+			frame_count = 1;
+			break;
+		case 80:
+			frame_size = d->samplerate * 40 / 1000;
+			frame_count = 2;
+			break;
+		case 100:
+			frame_size = d->samplerate * 20 / 1000;
+			frame_count = 5;
+			break;
+		case 120:
+			frame_size = d->samplerate * 60 / 1000;
+			frame_count = 2;
+			break;
+		default:
+			frame_size = d->samplerate * 20 / 1000;
+			frame_count = 1;
 	}
 
+	max_frame_byte_size = MAX_BYTES_PER_MS * ptime/frame_count;
+
+	pcm_buffer_size = d->channels * frame_size * SIGNAL_SAMPLE_SIZE;
+	if (pcm_buffer_size > d->pcmbufsize){
+		if (d->pcmbuffer) ms_free(d->pcmbuffer);
+		d->pcmbuffer = ms_malloc(pcm_buffer_size);
+		d->pcmbufsize = pcm_buffer_size;
+	}
 	for (i=0; i<MAX_INPUT_FRAMES; i++) {
-		codedFrameBuffer[i]=NULL;
+		repacketizer_frame_buffer[i]=NULL;
 	}
-	while (ms_bufferizer_get_avail(d->bufferizer) >= (d->channels * packet_size * SIGNAL_SAMPLE_SIZE)) {
-		totalLength = 0;
-		opus_repacketizer_init(rp);
-		for (i=0; i<frameNumber; i++) { /* encode 20ms by 20ms and repacketize all of them together */
-			if (!codedFrameBuffer[i]) codedFrameBuffer[i] = ms_malloc(MAX_BYTES_PER_FRAME); /* the repacketizer need the pointer to packet to remain valid, so we shall have a buffer for each coded frame */
-			if (!signalFrameBuffer) signalFrameBuffer = ms_malloc(frame_size * SIGNAL_SAMPLE_SIZE * d->channels);
 
-			ms_bufferizer_read(d->bufferizer, signalFrameBuffer, frame_size * SIGNAL_SAMPLE_SIZE * d->channels);
-			ret = opus_encode(d->state, (opus_int16 *)signalFrameBuffer, frame_size, codedFrameBuffer[i], MAX_BYTES_PER_FRAME);
+	ms_bufferizer_put_from_queue(d->bufferizer, f->inputs[0]);
+	while (ms_bufferizer_get_avail(d->bufferizer) >= (size_t)(d->channels * packet_size * SIGNAL_SAMPLE_SIZE)) {
+		opus_int32 ret = 0;
+
+		if (frame_count == 1) { /* One Opus frame, not using the repacketizer */
+			om = allocb(max_frame_byte_size, 0);
+			ms_bufferizer_read(d->bufferizer, d->pcmbuffer, frame_size * SIGNAL_SAMPLE_SIZE * d->channels);
+			ret = opus_encode(d->state, (opus_int16 *)d->pcmbuffer, frame_size, om->b_wptr, max_frame_byte_size);
 			if (ret < 0) {
+				freemsg(om);
+				om=NULL;
 				ms_error("Opus encoder error: %s", opus_strerror(ret));
 				break;
+			} else {
+				total_length = ret;
+				om->b_wptr += total_length;
 			}
-			if (ret > 0) {
-				int err = opus_repacketizer_cat(rp, codedFrameBuffer[i], ret); /* add the encoded frame into the current packet */
-				if (err != OPUS_OK) {
-					ms_error("Opus repacketizer error: %s", opus_strerror(err));
+		} else if(frame_count > 1) { /* We have multiple Opus frames we will use the opus repacketizer */
+
+			repacketizer = opus_repacketizer_create();
+			opus_repacketizer_init(repacketizer);
+
+			/* Do not include FEC/LBRR in any frame after the first one since it will be sent with the previous one */
+			ret = opus_encoder_ctl(d->state, OPUS_SET_INBAND_FEC(0));
+			if (ret != OPUS_OK) {
+				ms_error("could not set inband FEC to opus encoder: %s", opus_strerror(ret));
+			}
+			for (i=0; i<frame_count; i++) {
+				if(frame_count == i+1){ /* if configured, reactivate FEC on the last frame to tell the encoder he should restart saving LBRR frames */
+					ret = opus_encoder_ctl(d->state, OPUS_SET_INBAND_FEC(d->useinbandfec));
+					if (ret != OPUS_OK) {
+						ms_error("could not set inband FEC to opus encoder: %s", opus_strerror(ret));
+					}
+				}
+				if (!repacketizer_frame_buffer[i]) repacketizer_frame_buffer[i] = ms_malloc(max_frame_byte_size); /* the repacketizer need the pointer to packet to remain valid, so we shall have a buffer for each coded frame */
+				ms_bufferizer_read(d->bufferizer, d->pcmbuffer, frame_size * SIGNAL_SAMPLE_SIZE * d->channels);
+				ret = opus_encode(d->state, (opus_int16 *)d->pcmbuffer, frame_size, repacketizer_frame_buffer[i], max_frame_byte_size);
+				if (ret < 0) {
+					ms_error("Opus encoder error: %s", opus_strerror(ret));
 					break;
+				} else if (ret > 0) {
+					int err = opus_repacketizer_cat(repacketizer, repacketizer_frame_buffer[i], ret); /* add the encoded frame into the current packet */
+					if (err != OPUS_OK) {
+						ms_error("Opus repacketizer error: %s", opus_strerror(err));
+						break;
+					}
+					total_length += ret;
+				}
+			}
+
+			om = allocb(total_length + frame_count + 1, 0); /* opus repacketizer API: allocate at least number of frame + size of all data added before */
+			ret = opus_repacketizer_out(repacketizer, om->b_wptr, total_length+frame_count);
+			if(ret < 0){
+				freemsg(om);
+				om=NULL;
+				ms_error("Opus repacketizer out error: %s", opus_strerror(ret));
+			} else {
+				om->b_wptr += ret;
+			}
+			opus_repacketizer_destroy(repacketizer);
+			for (i=0; i<frame_count; i++) {
+				if (repacketizer_frame_buffer[i] != NULL) {
+					ms_free(repacketizer_frame_buffer[i]);
 				}
-				totalLength += ret;
 			}
 		}
 
-		if (ret > 0) {
-			om = allocb(totalLength+frameNumber + 1, 0); /* opus repacktizer API: allocate at leat number of frame + size of all data added before */
-			ret = opus_repacketizer_out(rp, om->b_wptr, totalLength+frameNumber);
-
-			om->b_wptr += ret;
+		if(om) { /* we have an encoded output message */
 			mblk_set_timestamp_info(om, d->ts);
 			ms_bufferizer_fill_current_metas(d->bufferizer, om);
 			ms_queue_put(f->outputs[0], om);
 			d->ts += packet_size*48000/d->samplerate; /* RFC payload RTP opus 03 - section 4: RTP timestamp multiplier : WARNING works only with sr at 48000 */
-			ret = 0;
+			total_length = 0;
 		}
 	}
 
-	opus_repacketizer_destroy(rp);
-
-	if (signalFrameBuffer != NULL) {
-		ms_free(signalFrameBuffer);
-	}
-	for (i=0; i<frameNumber; i++) {
-		if (codedFrameBuffer[i] != NULL) {
-			ms_free(codedFrameBuffer[i]);
-		}
-	}
 }
 
 static void ms_opus_enc_postprocess(MSFilter *f) {
@@ -237,6 +325,7 @@ static void ms_opus_enc_uninit(MSFilter *f) {
 	}
 	ms_bufferizer_destroy(d->bufferizer);
 	d->bufferizer = NULL;
+	if(d->pcmbuffer) ms_free(d->pcmbuffer);
 	ms_free(d);
 }
 
@@ -280,7 +369,7 @@ static void compute_max_bitrate(OpusEncData *d, int ptimeStep) {
 	if (normalized_cbr<6000) {
 		int initial_value = normalized_cbr;
 		normalized_cbr = 6000;
-		d->max_network_bitrate = (normalized_cbr/(pps*8) + 12 + 8 + 20) *8*pps;
+		d->max_network_bitrate = (int)((normalized_cbr/(pps*8) + 12 + 8 + 20) *8*pps);
 		ms_warning("Opus encoder does not support bitrate [%i]. Instead set to 6kbps, network bitrate [%d]", initial_value, d->max_network_bitrate);
 	}
 	if (d->maxaveragebitrate>0)
@@ -289,7 +378,7 @@ static void compute_max_bitrate(OpusEncData *d, int ptimeStep) {
 	if (normalized_cbr>maxaveragebitrate) {
 		int initial_value = normalized_cbr;
 		normalized_cbr = maxaveragebitrate;
-		d->max_network_bitrate = (normalized_cbr/(pps*8) + 12 + 8 + 20) *8*pps;
+		d->max_network_bitrate = (int)((normalized_cbr/(pps*8) + 12 + 8 + 20) *8*pps);
 		ms_warning("Opus encoder cannot set codec bitrate to [%i] because of maxaveragebitrate constraint or absolute maximum bitrate value. New network bitrate is [%i]", initial_value, d->max_network_bitrate);
 	}
 	ms_message("MSOpusEnc: codec bitrate set to [%i] with ptime [%i]", normalized_cbr, d->ptime);
@@ -671,7 +760,7 @@ static void ms_opus_dec_process(MSFilter *f) {
 	while ((im = ms_queue_get(f->inputs[0])) != NULL) {
 		om = allocb(5760 * d->channels * SIGNAL_SAMPLE_SIZE, 0); /* 5760 is the maximum number of sample in a packet (120ms at 48KHz) */
 
-		frames = opus_decode(d->state, (const unsigned char *)im->b_rptr, im->b_wptr - im->b_rptr, (opus_int16 *)om->b_wptr, 5760, 0);
+		frames = opus_decode(d->state, (const unsigned char *)im->b_rptr, (opus_int32)(im->b_wptr - im->b_rptr), (opus_int16 *)om->b_wptr, 5760, 0);
 
 		if (frames < 0) {
 			ms_warning("Opus decoder error: %s", opus_strerror(frames));
