@@ -37,7 +37,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 #define WINSND_NBUFS 10
 #define WINSND_OUT_DELAY 0.020
-#define WINSND_OUT_NBUFS 40
+#define WINSND_OUT_NBUFS 100
 #define WINSND_NSAMPLES 320
 #define WINSND_MINIMUMBUFFER 5
 
@@ -586,20 +586,20 @@ static void winsnd_write_process(MSFilter *f){
 		WAVEHDR *hdr=&d->hdrs_write[outcurbuf];
 		old=(mblk_t*)hdr->dwUser;
 		if (d->nsamples==0){
-			int tmpsize=WINSND_OUT_DELAY*d->wfx.nAvgBytesPerSec;
+			int tmpsize=(int)WINSND_OUT_DELAY*d->wfx.nAvgBytesPerSec;
 			mblk_t *tmp=allocb(tmpsize,0);
 			memset(tmp->b_wptr,0,tmpsize);
 			tmp->b_wptr+=tmpsize;
 			playout_buf(d,hdr,tmp);
 			d->outcurbuf++;
-			d->nsamples+=WINSND_OUT_DELAY*d->wfx.nSamplesPerSec;
+			d->nsamples+=(int)(WINSND_OUT_DELAY*d->wfx.nSamplesPerSec);
 			continue;
 		}
 		m=ms_queue_get(f->inputs[0]);
 		if (!m) break;
 		d->nsamples+=msgdsize(m)/d->wfx.nBlockAlign;
 		/*if the output buffer has finished to play, unprepare it*/
-		if (hdr->dwFlags & WHDR_DONE){
+		if ((hdr->dwFlags & WHDR_DONE) || (hdr->dwFlags == 0)){
 			mr=waveOutUnprepareHeader(d->outdev,hdr,sizeof(*hdr));
 			if (mr != MMSYSERR_NOERROR){
 				ms_error("waveOutUnprepareHeader error");
@@ -617,7 +617,10 @@ static void winsnd_write_process(MSFilter *f){
 			ms_warning("WINSND overrun, restarting");
 			d->overrun=TRUE;
 			d->nsamples=0;
+			d->outcurbuf=0;
 			waveOutReset(d->outdev);
+			ms_queue_flush(f->inputs[0]);
+			freemsg(m);
 			break;
 		}
 		d->outcurbuf++;
@@ -642,6 +645,12 @@ static int set_nchannels(MSFilter *f, void *arg){
 	return 0;
 }
 
+static int get_nchannels(MSFilter *f, void *arg){
+	WinSnd *d=(WinSnd*)f->data;
+	*((int*)arg)=d->wfx.nChannels;
+	return 0;
+}
+
 static int winsnd_get_stat_input(MSFilter *f, void *arg){
 	WinSnd *d=(WinSnd*)f->data;
 	return d->stat_input;
@@ -663,6 +672,7 @@ static MSFilterMethod winsnd_methods[]={
 	{	MS_FILTER_SET_SAMPLE_RATE	, set_rate	},
 	{	MS_FILTER_GET_SAMPLE_RATE	, get_rate	},
 	{	MS_FILTER_SET_NCHANNELS		, set_nchannels	},
+	{	MS_FILTER_GET_NCHANNELS		, get_nchannels },
 	{	MS_FILTER_GET_STAT_INPUT, winsnd_get_stat_input },
 	{	MS_FILTER_GET_STAT_OUTPUT, winsnd_get_stat_ouptut },
 	{	MS_FILTER_GET_STAT_DISCARDED, winsnd_get_stat_discarded },
@@ -703,7 +713,7 @@ MSFilterDesc winsnd_write_desc={
 };
 
 MSFilter *ms_winsnd_read_new(MSSndCard *card){
-	MSFilter *f=ms_filter_new_from_desc(&winsnd_read_desc);
+	MSFilter *f=ms_factory_create_filter_from_desc(ms_snd_card_get_factory(card), &winsnd_read_desc);
 	WinSndCard *wc=(WinSndCard*)card->data;
 	WinSnd *d=(WinSnd*)f->data;
 	d->dev_id=wc->in_devid;
@@ -712,7 +722,7 @@ MSFilter *ms_winsnd_read_new(MSSndCard *card){
 
 
 MSFilter *ms_winsnd_write_new(MSSndCard *card){
-	MSFilter *f=ms_filter_new_from_desc(&winsnd_write_desc);
+	MSFilter *f=ms_factory_create_filter_from_desc(ms_snd_card_get_factory(card), &winsnd_write_desc);
 	WinSndCard *wc=(WinSndCard*)card->data;
 	WinSnd *d=(WinSnd*)f->data;
 	d->dev_id=wc->out_devid;
