@@ -23,38 +23,37 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 #include "mediastreamer2/mssndcard.h"
 
-static MSSndCardManager *scm=NULL;
 
-static MSSndCardManager * create_manager(){
+MSSndCardManager * ms_snd_card_manager_new(void){
 	MSSndCardManager *obj=(MSSndCardManager *)ms_new0(MSSndCardManager,1);
+	obj->factory = NULL;
 	obj->cards=NULL;
 	obj->descs=NULL;
 	return obj;
 }
 
-void ms_snd_card_manager_destroy(void){
+void ms_snd_card_manager_destroy(MSSndCardManager* scm){
 	if (scm!=NULL){
-		MSList *elem;
+		bctbx_list_t *elem;
 		for(elem=scm->descs;elem!=NULL;elem=elem->next){
 			MSSndCardDesc *desc = (MSSndCardDesc*)elem->data;
 			if (desc->unload!=NULL)
 				desc->unload(scm);
 		}
-		ms_list_for_each(scm->cards,(void (*)(void*))ms_snd_card_destroy);
-		ms_list_free(scm->cards);
-		ms_list_free(scm->descs);
+		bctbx_list_for_each(scm->cards,(void (*)(void*))ms_snd_card_destroy);
+		bctbx_list_free(scm->cards);
+		bctbx_list_free(scm->descs);
 	}
 	ms_free(scm);
 	scm=NULL;
 }
 
-MSSndCardManager * ms_snd_card_manager_get(void){
-	if (scm==NULL) scm=create_manager();
-	return scm;
+MSFactory * ms_snd_card_get_factory(MSSndCard * c){
+	return c->sndcardmanager->factory;
 }
 
 MSSndCard * ms_snd_card_manager_get_card(MSSndCardManager *m, const char *id){
-	MSList *elem;
+	bctbx_list_t *elem;
 	for (elem=m->cards;elem!=NULL;elem=elem->next){
 		MSSndCard *card=(MSSndCard*)elem->data;
 		if (id==NULL) return card;
@@ -64,45 +63,75 @@ MSSndCard * ms_snd_card_manager_get_card(MSSndCardManager *m, const char *id){
 	return NULL;
 }
 
-MSSndCard * ms_snd_card_manager_get_default_card(MSSndCardManager *m){
-	/*return the first card that has the capture+playback capability */
-	MSList *elem;
+static MSSndCard *get_card_with_cap(MSSndCardManager *m, const char *id, unsigned int caps){
+	bctbx_list_t *elem;
 	for (elem=m->cards;elem!=NULL;elem=elem->next){
 		MSSndCard *card=(MSSndCard*)elem->data;
-		if ((card->capabilities & MS_SND_CARD_CAP_CAPTURE )
-			&& (card->capabilities & MS_SND_CARD_CAP_PLAYBACK))
-			return card;
+		if ((id== NULL || strcmp(ms_snd_card_get_string_id(card),id)==0) && (card->capabilities & caps) == caps)	return card;
 	}
 	return NULL;
 }
 
+MSSndCard * ms_snd_card_manager_get_playback_card(MSSndCardManager *m, const char *id){
+	MSSndCard *ret;
+	ret = get_card_with_cap(m, id, MS_SND_CARD_CAP_PLAYBACK);
+	if (ret) return ret;
+	if (id != NULL) ms_warning("No playback card with id %s",id);
+	return NULL;
+}
+
+MSSndCard * ms_snd_card_manager_get_capture_card(MSSndCardManager *m, const char *id){
+	MSSndCard *ret;
+	ret = get_card_with_cap(m, id, MS_SND_CARD_CAP_CAPTURE);
+	if (ret) return ret;
+	if (id != NULL) ms_warning("No capture card with id %s",id);
+	return NULL;
+}
+
+MSSndCard * ms_snd_card_manager_get_default_card(MSSndCardManager *m){
+	/*return the first card that has the capture+playback capability */
+	return get_card_with_cap(m, NULL, MS_SND_CARD_CAP_PLAYBACK | MS_SND_CARD_CAP_CAPTURE);
+}
+
 MSSndCard * ms_snd_card_manager_get_default_capture_card(MSSndCardManager *m){
-	MSList *elem;
-	for (elem=m->cards;elem!=NULL;elem=elem->next){
-		MSSndCard *card=(MSSndCard*)elem->data;
-		if (card->capabilities & MS_SND_CARD_CAP_CAPTURE)
-			return card;
-	}
-	return NULL;
+	return get_card_with_cap(m, NULL, MS_SND_CARD_CAP_CAPTURE);
 }
 
 MSSndCard * ms_snd_card_manager_get_default_playback_card(MSSndCardManager *m){
-	MSList *elem;
-	for (elem=m->cards;elem!=NULL;elem=elem->next){
-		MSSndCard *card=(MSSndCard*)elem->data;
-		if (card->capabilities & MS_SND_CARD_CAP_PLAYBACK)
-			return card;
-	}
-	return NULL;
+	return get_card_with_cap(m, NULL, MS_SND_CARD_CAP_PLAYBACK);
 }
 
-const MSList * ms_snd_card_manager_get_list(MSSndCardManager *m){
+const bctbx_list_t * ms_snd_card_manager_get_list(MSSndCardManager *m){
 	return m->cards;
 }
 
+void ms_snd_card_set_manager(MSSndCardManager*m, MSSndCard *c){
+	if (c->sndcardmanager == NULL) c->sndcardmanager = m;
+}
+
+static const char *cap_to_string(unsigned int cap){
+	if ((cap & MS_SND_CARD_CAP_CAPTURE) && (cap & MS_SND_CARD_CAP_PLAYBACK)) return "capture, playback";
+	if (cap & MS_SND_CARD_CAP_CAPTURE) return "capture";
+	if (cap & MS_SND_CARD_CAP_PLAYBACK) return "playback";
+	return "none";
+}
+
 void ms_snd_card_manager_add_card(MSSndCardManager *m, MSSndCard *c){
-	ms_message("Card '%s' added",ms_snd_card_get_string_id(c));
-	m->cards=ms_list_append(m->cards,c);
+	ms_snd_card_set_manager(m,c);
+	ms_message("Card '%s' added with capabilities [%s]",ms_snd_card_get_string_id(c), cap_to_string(c->capabilities));
+	m->cards=bctbx_list_append(m->cards,c);
+}
+
+void ms_snd_card_manager_prepend_cards(MSSndCardManager *m, bctbx_list_t *l) {
+	bctbx_list_t *elem;
+	bctbx_list_t *lcopy = bctbx_list_copy(l);
+	if (m->cards != NULL) m->cards = bctbx_list_concat(lcopy, m->cards);
+	else m->cards = lcopy;
+	for (elem = l; elem != NULL; elem = elem->next) {
+		MSSndCard *card = (MSSndCard *)elem->data;
+		ms_snd_card_set_manager(m, card);
+		ms_message("Card '%s' added", ms_snd_card_get_string_id(card));
+	}
 }
 
 static void card_detect(MSSndCardManager *m, MSSndCardDesc *desc){
@@ -111,14 +140,16 @@ static void card_detect(MSSndCardManager *m, MSSndCardDesc *desc){
 }
 
 void ms_snd_card_manager_register_desc(MSSndCardManager *m, MSSndCardDesc *desc){
-	m->descs=ms_list_append(m->descs,desc);
-	card_detect(m,desc);
+	if (bctbx_list_find(m->descs, desc) == NULL){
+		m->descs=bctbx_list_append(m->descs,desc);
+		card_detect(m,desc);
+	}
 }
 
 void ms_snd_card_manager_reload(MSSndCardManager *m){
-	MSList *elem;
-	ms_list_for_each(m->cards,(void (*)(void*))ms_snd_card_destroy);
-	ms_list_free(m->cards);
+	bctbx_list_t *elem;
+	bctbx_list_for_each(m->cards,(void (*)(void*))ms_snd_card_destroy);
+	bctbx_list_free(m->cards);
 	m->cards=NULL;
 	for(elem=m->descs;elem!=NULL;elem=elem->next)
 		card_detect(m,(MSSndCardDesc*)elem->data);
@@ -137,6 +168,7 @@ MSSndCard * ms_snd_card_new(MSSndCardDesc *desc){
 
 MSSndCard * ms_snd_card_new_with_name(MSSndCardDesc *desc,const char* name) {
 	MSSndCard *obj=(MSSndCard *)ms_new0(MSSndCard,1);
+	obj->sndcardmanager = NULL;
 	obj->desc=desc;
 	obj->name=name?ms_strdup(name):NULL;
 	obj->data=NULL;
@@ -225,7 +257,7 @@ struct _MSFilter * ms_snd_card_create_writer(MSSndCard *obj){
 
 void ms_snd_card_set_usage_hint(MSSndCard *obj, bool_t is_going_to_be_used){
 	if (obj->desc->usage_hint!=NULL)
-		return obj->desc->usage_hint(obj, is_going_to_be_used);
+		obj->desc->usage_hint(obj, is_going_to_be_used);
 }
 
 void ms_snd_card_destroy(MSSndCard *obj){
@@ -256,3 +288,14 @@ void ms_alsa_card_set_forced_sample_rate(int samplerate){
 }
 #endif
 #endif
+
+#ifdef _MSC_VER
+#pragma warning(disable : 4996)
+#else
+#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#endif
+
+MSSndCardManager* ms_snd_card_manager_get(void) {
+	return ms_factory_get_snd_card_manager(ms_factory_get_fallback());
+}
+
