@@ -23,6 +23,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 #ifdef HAVE_LINUX_VIDEODEV2_H
 
+#define POSSIBLE_FORMATS_COUNT 4
 
 #include <fcntl.h>
 #include <unistd.h>
@@ -77,7 +78,6 @@ typedef struct V4l2State{
 	MSVideoSize vsize;
 	MSVideoSize got_vsize;
 	int pix_fmt;
-	int int_pix_fmt; /*internal pixel format */
 	int picture_size;
 	mblk_t *frames[VIDEO_MAX_FRAME];
 	int frame_ind;
@@ -118,6 +118,10 @@ static bool_t v4lv2_try_format( int fd, struct v4l2_format *fmt, int fmtid){
 		ms_message("VIDIOC_TRY_FMT: %s",strerror(errno));
 		return FALSE;
 	}
+	if(fmt->fmt.pix.pixelformat != fmtid) {
+		ms_message("VIDIOC_TRY_FMT: got different format");
+		return FALSE;
+	}
 	if (v4l2_ioctl (fd, VIDIOC_S_FMT, fmt)<0){
 		ms_message("VIDIOC_S_FMT: %s",strerror(errno));
 		return FALSE;
@@ -142,9 +146,11 @@ static int get_picture_buffer_size(MSPixFmt pix_fmt, int w, int h){
 	return 0;
 }
 
+#ifdef VIDIOC_ENUM_FRAMEINTERVALS
+
 static int query_max_fps_for_format_resolution(int fd, int pixelformat, MSVideoSize vsize) {
 	int fps = -1;
-	struct v4l2_frmivalenum frmival;
+	struct v4l2_frmivalenum frmival = { 0 };
 	frmival.index = 0;
 	frmival.pixel_format = pixelformat;
 	frmival.width = vsize.width;
@@ -163,6 +169,8 @@ static int query_max_fps_for_format_resolution(int fd, int pixelformat, MSVideoS
 	return fps;
 }
 
+#endif
+
 typedef struct _V4L2FormatDescription {
 	/* format */
 	int pixel_format;
@@ -174,7 +182,7 @@ typedef struct _V4L2FormatDescription {
 	bool_t compressed;
 	/*format is supported*/
 	bool_t supported;
-	
+
 } V4L2FormatDescription;
 
 static MSPixFmt v4l2_format_to_ms(int v4l2format) {
@@ -195,16 +203,35 @@ static MSPixFmt v4l2_format_to_ms(int v4l2format) {
 
 static const V4L2FormatDescription* query_format_description_for_size(int fd, MSVideoSize vsize) {
 	/* hardcode supported format in preferred order*/
-	static V4L2FormatDescription formats[4];
+	static V4L2FormatDescription formats[POSSIBLE_FORMATS_COUNT];
+	int i=0;
+
 	memset(formats,0,sizeof(formats));
-	formats[0].pixel_format = V4L2_PIX_FMT_YUV420;
-	formats[0].max_fps = -1;
-	formats[1].pixel_format = V4L2_PIX_FMT_YUYV;
-	formats[1].max_fps = -1;
-	formats[2].pixel_format = V4L2_PIX_FMT_MJPEG;
-	formats[2].max_fps = -1;
-	formats[3].pixel_format = V4L2_PIX_FMT_RGB24;
-	formats[3].max_fps = -1;
+
+	formats[i].pixel_format = V4L2_PIX_FMT_YUV420;
+	formats[i].max_fps = -1;
+	i++;
+	/* we must avoid YUYV (and actually any YUV format different than YUV420P) because the pixel converter/scaler implementation
+	 * of ffmpeg is not optimized for arm. So we need to prefer YUV420P if directly available or MJPEG*/
+#ifndef __arm__
+	formats[i].pixel_format = V4L2_PIX_FMT_YUYV;
+	formats[i].max_fps = -1;
+	i++;
+#endif
+
+	formats[i].pixel_format = V4L2_PIX_FMT_MJPEG;
+	formats[i].max_fps = -1;
+	i++;
+
+#ifdef __arm__
+	formats[i].pixel_format = V4L2_PIX_FMT_YUYV;
+	formats[i].max_fps = -1;
+	i++;
+#endif
+
+	formats[i].pixel_format = V4L2_PIX_FMT_RGB24;
+	formats[i].max_fps = -1;
+	i++;
 
 	{
 		struct v4l2_fmtdesc fmt;
@@ -212,11 +239,14 @@ static const V4L2FormatDescription* query_format_description_for_size(int fd, MS
 		fmt.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
 
 		while (v4l2_ioctl(fd, VIDIOC_ENUM_FMT, &fmt) >= 0) {
-			int i;
-			for (i=0; i<4; i++) {
+			for (i=0; i<POSSIBLE_FORMATS_COUNT; i++) {
 				if (fmt.pixelformat == formats[i].pixel_format) {
+#ifdef VIDIOC_ENUM_FRAMEINTERVALS
 					formats[i].max_fps = query_max_fps_for_format_resolution(fd, fmt.pixelformat, vsize);
+#endif
+#ifdef V4L2_FMT_FLAG_EMULATED
 					formats[i].native = !(fmt.flags & V4L2_FMT_FLAG_EMULATED);
+#endif
 					formats[i].compressed = fmt.flags & V4L2_FMT_FLAG_COMPRESSED;
 					formats[i].supported = TRUE;
 					ms_message("format %s : max_fps=%i, native=%i, compressed=%i",
@@ -233,20 +263,20 @@ static const V4L2FormatDescription* query_format_description_for_size(int fd, MS
 	return formats;
 }
 
-static MSPixFmt pick_best_format(int fd, const V4L2FormatDescription* format_desc, MSVideoSize vsize) {
+MSPixFmt msv4l2_pick_best_format_x86(int fd, const V4L2FormatDescription* format_desc, MSVideoSize vsize, float target_fps) {
 	/* rules for picking a format are:
-	    - only max_fps >= 15 images/sec are considered
+	    - only max_fps >= target_fps images/sec are considered
 	    - native > compressed > emulated
 	*/
 	enum { PREFER_NATIVE = 0, PREFER_COMPRESSED, NO_PREFERENCE} i;
 	int j;
 	for (i=PREFER_NATIVE; i<=NO_PREFERENCE; i++) {
-		for (j=0; j<4; j++) {
+		for (j=0; j<POSSIBLE_FORMATS_COUNT; j++) {
 			int candidate = -1;
 			if (!format_desc[j].supported) continue;
 			switch (i) {
 				case PREFER_NATIVE:
-					if (format_desc[j].max_fps >= 15 && format_desc[j].native)
+					if (format_desc[j].max_fps >= target_fps && format_desc[j].native)
 						candidate = j;
 					break;
 				case PREFER_COMPRESSED:
@@ -261,7 +291,7 @@ static MSPixFmt pick_best_format(int fd, const V4L2FormatDescription* format_des
 			}
 
 			if (candidate != -1) {
-				struct v4l2_format fmt;
+				struct v4l2_format fmt = { 0 };
 				fmt.fmt.pix.width       = vsize.width;
 				fmt.fmt.pix.height      = vsize.height;
 				ms_message("Candidate: %i",candidate);
@@ -279,11 +309,45 @@ static MSPixFmt pick_best_format(int fd, const V4L2FormatDescription* format_des
 	return MS_PIX_FMT_UNKNOWN;
 }
 
+MSPixFmt msv4l2_pick_best_format_basic(int fd, const V4L2FormatDescription* format_desc, MSVideoSize vsize, float target_fps) {
+	int j;
+	/* rules for picking a format are:
+	   available fps > requested fps
+	   no matter whether it is compressed or non-native, since swscale pixel converter is supposed to be much less efficient than libv4l's one
+	   in this case.
+	*/
+	for (j=0; j<POSSIBLE_FORMATS_COUNT; j++) {
+		if (!format_desc[j].supported) continue;
+		if (format_desc[j].max_fps >= target_fps || format_desc[j].max_fps==-1 /*max fps unknown*/){
+			struct v4l2_format fmt;
+			fmt.fmt.pix.width       = vsize.width;
+			fmt.fmt.pix.height      = vsize.height;
+
+			if (v4lv2_try_format(fd, &fmt, format_desc[j].pixel_format)) {
+				MSPixFmt selected=v4l2_format_to_ms(format_desc[j].pixel_format);
+				ms_message("V4L2: selected format is %s", ms_pix_fmt_to_string(selected));
+				return selected;
+			}
+		}
+	}
+
+	ms_error("No compatible format found");
+	return MS_PIX_FMT_UNKNOWN;
+}
+
+#if defined(__i386) || defined(__x86_64__)
+#define pick_best_format msv4l2_pick_best_format_x86
+#else
+#define pick_best_format msv4l2_pick_best_format_basic
+#endif
+
 static int set_camera_feature(V4l2State *s, unsigned int ctl_id, int value, const char *feature_name){
 	struct v4l2_ext_control ctl={0};
-	struct v4l2_ext_controls ctls={0};
+	struct v4l2_ext_controls ctls;
 	struct v4l2_queryctrl queryctrl={0};
-	
+
+	memset(&ctls, 0, sizeof(ctls));
+
 	queryctrl.id = ctl_id;
 	if (ioctl (s->fd, VIDIOC_QUERYCTRL, &queryctrl)!=0) {
 		ms_warning("%s not supported: %s",feature_name,strerror(errno));
@@ -292,6 +356,7 @@ static int set_camera_feature(V4l2State *s, unsigned int ctl_id, int value, cons
 		ms_warning("%s setting disabled.",feature_name);
 		return -1;
 	}else {
+#ifdef V4L2_CTRL_CLASS_CAMERA
 		ctl.id=ctl_id;
 		ctl.value=value;
 		ctl.size=sizeof(int);
@@ -303,6 +368,7 @@ static int set_camera_feature(V4l2State *s, unsigned int ctl_id, int value, cons
 			ms_warning("Could not enable %s: %s", feature_name, strerror(errno));
 			return -1;
 		}
+#endif
 	}
 	return 0;
 }
@@ -342,7 +408,7 @@ static int msv4l2_configure(V4l2State *s){
 
 	do{
 		const V4L2FormatDescription* formats_desc = query_format_description_for_size(s->fd, s->vsize);
-		s->pix_fmt = pick_best_format(s->fd, formats_desc, s->vsize);
+		s->pix_fmt = pick_best_format(s->fd, formats_desc, s->vsize, s->fps);
 
 		if (s->pix_fmt == MS_PIX_FMT_UNKNOWN)
 			s->vsize=ms_video_size_get_just_lower_than(s->vsize);
@@ -351,11 +417,22 @@ static int msv4l2_configure(V4l2State *s){
 	if (s->vsize.width==0){
 		ms_message("Could not find any combination of resolution/pixel-format that works !");
 		s->vsize=vsize;
-		return -1;
+		ms_message("Fallback. Trying to force YUV420 format");
+		memset(&fmt, 0, sizeof(fmt));
+		fmt.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
+		fmt.fmt.pix.pixelformat = V4L2_PIX_FMT_YUV420;
+		fmt.fmt.pix.width = s->vsize.width;
+		fmt.fmt.pix.height = s->vsize.height;
+		fmt.fmt.pix.field = V4L2_FIELD_ANY;
+		if(v4l2_ioctl(s->fd, VIDIOC_S_FMT, &fmt) != 0) {
+			ms_error("VIDIOC_S_FMT failed: %s", strerror(errno));
+			return -1;
+		}
+		s->pix_fmt = v4l2_format_to_ms(fmt.fmt.pix.pixelformat);
 	}
-	memset(&fmt,0,sizeof(fmt));
 
-	fmt.type                = V4L2_BUF_TYPE_VIDEO_CAPTURE;
+	memset(&fmt,0,sizeof(fmt));
+	fmt.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
 
 	if (v4l2_ioctl (s->fd, VIDIOC_G_FMT, &fmt)<0){
 		ms_error("VIDIOC_G_FMT failed: %s",strerror(errno));
@@ -365,22 +442,26 @@ static int msv4l2_configure(V4l2State *s){
 		s->vsize.height=fmt.fmt.pix.height;
 	}
 	s->picture_size=get_picture_buffer_size(s->pix_fmt,s->vsize.width,s->vsize.height);
-	
+
 	focus=getenv("MS2_CAM_FOCUS");
 	if (focus){
 		if (strcasecmp(focus,"auto")==0){
 #ifdef V4L2_CID_AUTO_FOCUS_RANGE
 			set_camera_feature(s,V4L2_CID_AUTO_FOCUS_RANGE,V4L2_AUTO_FOCUS_RANGE_AUTO ,"auto range");
 #endif
+#ifdef V4L2_CID_FOCUS_AUTO
 			set_camera_feature(s,V4L2_CID_FOCUS_AUTO,1,"auto-focus");
+#endif
 		}else if (strcasecmp(focus,"infinity")==0){
 #ifdef V4L2_CID_AUTO_FOCUS_RANGE
 			set_camera_feature(s,V4L2_CID_AUTO_FOCUS_RANGE,V4L2_AUTO_FOCUS_RANGE_INFINITY ,"infinity range");
 #endif
+#ifdef V4L2_CID_FOCUS_AUTO
 			set_camera_feature(s,V4L2_CID_FOCUS_AUTO,1,"auto-focus");
+#endif
 		}
 	}
-	
+
 	s->configured=TRUE;
 	return 0;
 }
@@ -507,7 +588,7 @@ static mblk_t * v4lv2_grab_image(V4l2State *s, int poll_timeout_ms){
 	unsigned int k;
 	bool_t no_slot_available = TRUE;
 	mblk_t *ret=NULL;
-	
+
 	memset(&buf,0,sizeof(buf));
 
 	buf.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
@@ -594,13 +675,13 @@ static void *msv4l2_thread(void *ptr){
 		ms_warning("msv4l2 could not be configured");
 		goto close;
 	}
-	
+
 	if (msv4l2_do_mmap(s)!=0)
 	{
 		ms_warning("msv4l2 do mmap");
 		goto close;
 	}
-	
+
 	ms_message("V4L2 video capture started.");
 	while(s->thread_run)
 	{
@@ -773,7 +854,7 @@ MSFilterDesc ms_v4l2_desc={
 MS_FILTER_DESC_EXPORT(ms_v4l2_desc)
 
 static MSFilter *msv4l2_create_reader(MSWebCam *obj){
-	MSFilter *f=ms_filter_new(MS_V4L2_CAPTURE_ID);
+	MSFilter *f=ms_factory_create_filter(ms_web_cam_get_factory(obj),MS_V4L2_CAPTURE_ID);
 	msv4l2_set_devfile(f,obj->name);
 	return f;
 }
@@ -798,7 +879,9 @@ static void msv4l2_detect(MSWebCamManager *obj){
 
 	for(i=0;i<10;++i){
 		int fd;
+
 		snprintf(devname,sizeof(devname),"/dev/video%i",i);
+
 		fd=open(devname,O_RDWR);
 		if (fd!=-1){
 			if (v4l2_ioctl (fd, VIDIOC_QUERYCAP, &cap)==0) {
@@ -808,6 +891,8 @@ static void msv4l2_detect(MSWebCamManager *obj){
 				if (cap.capabilities & V4L2_CAP_DEVICE_CAPS) {
 					camera_caps = cap.device_caps;
 				}
+#else
+				camera_caps = V4L2_CAP_VIDEO_CAPTURE;
 #endif
 				if (((camera_caps & V4L2_CAP_VIDEO_CAPTURE)
 #ifdef V4L2_CAP_VIDEO_CAPTURE_MPLANE
@@ -824,6 +909,8 @@ static void msv4l2_detect(MSWebCamManager *obj){
 				}
 			}
 			close(fd);
+		}else if (errno != ENOENT){
+			ms_message("Could not open %s: %s", devname, strerror(errno));
 		}
 	}
 }
