@@ -26,7 +26,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "mediastreamer2/msfilter.h"
 #include "mediastreamer2/msvideo.h"
 #include "mediastreamer2/msticker.h"
-#include "mediastreamer2/videostarter.h"
+#include "mediastreamer2/mscodecutils.h"
 
 #ifdef _WIN32
 #include <ws2tcpip.h>
@@ -171,7 +171,7 @@ static bool_t parse_video_fmtp(const char *fmtp, float *fps, MSVideoSize *vsize)
 		}
 		divider=atoi(equal+1);
 		if (divider!=0){
-			float newfps=29.97/divider;
+			float newfps=29.97f/divider;
 			if (*fps>newfps) *fps=newfps;
 		}else{
 			ms_warning("Could not find video fps");
@@ -225,14 +225,15 @@ static void enc_init(MSFilter *f, enum CodecID codec)
 	ms_ffmpeg_check_init();
 	s->profile=0;/*always default to profile 0*/
 	s->comp_buf=NULL;
-	s->mtu=ms_get_payload_max_size()-2;/*-2 for the H263 payload header*/
+	s->mtu=ms_factory_get_payload_max_size(f->factory)-2;/*-2 for the H263 payload header*/
+//	s->mtu=ms_get_payload_max_size()-2;/*-2 for the H263 payload header*/
 	s->codec=codec;
 	s->qmin=2;
 	s->req_vfu=FALSE;
 	s->framenum=0;
 	s->av_context.codec=NULL;
 	s->vconf_list = get_vconf_list(s);
-	s->vconf = ms_video_find_best_configuration_for_bitrate(s->vconf_list, 500000, ms_get_cpu_count());
+	s->vconf = ms_video_find_best_configuration_for_bitrate(s->vconf_list, 500000,ms_factory_get_cpu_count(f->factory));
 	s->pict = av_frame_alloc();
 }
 
@@ -259,13 +260,13 @@ static void prepare(EncState *s){
 	avcodec_get_context_defaults3(c, NULL);
 	if (s->codec==CODEC_ID_MJPEG)
 	{
-		ms_message("Codec bitrate set to %i",c->bit_rate);
+		ms_message("Codec bitrate set to %i",(int)c->bit_rate);
 		c->width = s->vconf.vsize.width;
 		c->height = s->vconf.vsize.height;
 		c->time_base.num = 1;
 		c->time_base.den = (int)s->vconf.fps;
 		c->gop_size=(int)s->vconf.fps*5; /*emit I frame every 5 seconds*/
-		c->pix_fmt=PIX_FMT_YUVJ420P;
+		c->pix_fmt=AV_PIX_FMT_YUVJ420P;
 		s->comp_buf=allocb(c->bit_rate*2,0);
 		return;
 	}
@@ -275,11 +276,11 @@ static void prepare(EncState *s){
 	 bitrate peaks especially on low bandwidth, we make a correction on the
 	 codec's target bitrate.
 	*/
-	c->bit_rate=(float)s->vconf.required_bitrate*0.92;
+	c->bit_rate=(int)((float)s->vconf.required_bitrate*0.92f);
 	if (c->bit_rate>RATE_CONTROL_MARGIN){
 		 c->bit_rate -= RATE_CONTROL_MARGIN;
 	}
-	c->bit_rate_tolerance=s->vconf.fps>1?(float)c->bit_rate/(s->vconf.fps-1):c->bit_rate;
+	c->bit_rate_tolerance=s->vconf.fps>1.0f?(int)((float)c->bit_rate/(s->vconf.fps-1.0f)):c->bit_rate;
 
 	/* ffmpeg vbv rate control consumes too much cpu above a certain target bitrate.
 	We don't use it above max_br_vbv */
@@ -297,20 +298,19 @@ static void prepare(EncState *s){
 		c->qmin=s->qmin;
 	}
 
-	ms_message("Codec bitrate set to %i",c->bit_rate);
 	c->width = s->vconf.vsize.width;
 	c->height = s->vconf.vsize.height;
 	c->time_base.num = 1;
 	c->time_base.den = (int)s->vconf.fps;
 	c->gop_size=(int)s->vconf.fps*10; /*emit I frame every 10 seconds*/
-	c->pix_fmt=PIX_FMT_YUV420P;
+	c->pix_fmt=AV_PIX_FMT_YUV420P;
 	s->comp_buf=allocb(c->bit_rate*2,0);
 #if HAVE_AVCODEC_SNOW
 	if (s->codec==CODEC_ID_SNOW){
 		c->strict_std_compliance=-2;
 	}
 #endif
-	ms_message("Codec size set to w=%i/h=%i",c->width, c->height);
+	ms_message("Codec size set to w=%i/h=%i, bitrate=%i",c->width, c->height, (int)c->bit_rate);
 
 }
 
@@ -394,14 +394,14 @@ static void enc_postprocess(MSFilter *f){
 	}
 }
 
-static void add_rfc2190_header(mblk_t **packet, AVCodecContext *context){
+static void add_rfc2190_header(mblk_t **packet, AVCodecContext *context, bool_t is_iframe){
 	mblk_t *header;
 	header = allocb(4, 0);
 	memset(header->b_wptr, 0, 4);
 	// assume video size is CIF or QCIF
 	if (context->width == 352 && context->height == 288) header->b_wptr[1] = 0x60;
 	else header->b_wptr[1] = 0x40;
-	if (context->coded_frame->pict_type != FF_I_TYPE) header->b_wptr[1] |= 0x10;
+	if (is_iframe == TRUE) header->b_wptr[1] |= 0x10;
 	header->b_wptr += 4;
 	header->b_cont = *packet;
 	*packet = header;
@@ -449,7 +449,7 @@ static int get_gbsc_bytealigned(uint8_t *begin, uint8_t *end){
 }
 #endif
 
-static void rfc2190_generate_packets(MSFilter *f, EncState *s, mblk_t *frame, uint32_t timestamp){
+static void rfc2190_generate_packets(MSFilter *f, EncState *s, mblk_t *frame, uint32_t timestamp, bool_t is_iframe){
 	mblk_t *packet=NULL;
 
 	while (frame->b_rptr<frame->b_wptr){
@@ -457,7 +457,7 @@ static void rfc2190_generate_packets(MSFilter *f, EncState *s, mblk_t *frame, ui
 		/*frame->b_rptr=packet->b_wptr=packet->b_rptr+get_gbsc(packet->b_rptr, MIN(packet->b_rptr+s->mtu,frame->b_wptr));*/
 		frame->b_rptr = packet->b_wptr =
 			packet->b_rptr + get_gbsc_bytealigned(packet->b_rptr, MIN(packet->b_rptr+s->mtu,frame->b_wptr));
-		add_rfc2190_header(&packet, &s->av_context);
+		add_rfc2190_header(&packet, &s->av_context ,is_iframe);
 		mblk_set_timestamp_info(packet,timestamp);
 		ms_queue_put(f->outputs[0],packet);
 	}
@@ -631,7 +631,7 @@ static void mjpeg_fragment_and_send(MSFilter *f,EncState *s,mblk_t *frame, uint3
 	if (q >= 128) {
 		qtblhdr.mbz = 0;
 		qtblhdr.precision = 0; /* This code uses 8 bit tables only */
-		qtblhdr.length = htons(msgdsize(lqt)+msgdsize(cqt));  /* 2 64-byte tables */
+		qtblhdr.length = htons((uint16_t)(msgdsize(lqt)+msgdsize(cqt)));  /* 2 64-byte tables */
 	}
 
 	while (bytes_left > 0) {
@@ -742,10 +742,10 @@ static mblk_t *skip_jpeg_headers(mblk_t *full_frame, mblk_t **lqt, mblk_t **cqt)
 	return full_frame;
 }
 
-static void split_and_send(MSFilter *f, EncState *s, mblk_t *frame){
+static void split_and_send(MSFilter *f, EncState *s, mblk_t *frame, bool_t is_iframe){
 	uint8_t *lastpsc;
 	uint8_t *psc;
-	uint32_t timestamp=f->ticker->time*90LL;
+	uint32_t timestamp=(uint32_t)(f->ticker->time*90LL);
 
 	if (s->codec==CODEC_ID_MPEG4
 #if HAVE_AVCODEC_SNOW
@@ -784,7 +784,7 @@ static void split_and_send(MSFilter *f, EncState *s, mblk_t *frame){
 		/* send the end of frame */
 		rfc4629_generate_follow_on_packets(f,s,frame, timestamp,lastpsc,frame->b_wptr,TRUE);
 	}else if (f->desc->id==MS_H263_OLD_ENC_ID){
-		rfc2190_generate_packets(f,s,frame,timestamp);
+		rfc2190_generate_packets(f,s,frame,timestamp,is_iframe);
 	}else{
 		ms_fatal("Ca va tres mal.");
 	}
@@ -833,15 +833,21 @@ static void process_frame(MSFilter *f, mblk_t *inm){
 
 	if (error<0) ms_warning("ms_AVencoder_process: error %i.",error);
 	else if (got_packet){
+		bool_t is_iframe = FALSE;
 		s->framenum++;
 		if (s->framenum==1){
 			ms_video_starter_first_frame(&s->starter, f->ticker->time);
 		}
+#ifdef AV_PKT_FLAG_KEY
+		if (packet.flags & AV_PKT_FLAG_KEY) {
+#else
 		if (c->coded_frame->pict_type==FF_I_TYPE){
+#endif
 			ms_message("Emitting I-frame");
+			is_iframe = TRUE;
 		}
 		comp_buf->b_wptr+=packet.size;
-		split_and_send(f,s,comp_buf);
+		split_and_send(f,s,comp_buf,is_iframe);
 	}
 	freemsg(inm);
 }
@@ -905,7 +911,7 @@ static int enc_set_vsize(MSFilter *f, void *arg) {
 	MSVideoConfiguration best_vconf;
 	MSVideoSize *vs = (MSVideoSize *)arg;
 	EncState *s=(EncState*)f->data;
-	best_vconf = ms_video_find_best_configuration_for_size(s->vconf_list, *vs, ms_get_cpu_count());
+	best_vconf = ms_video_find_best_configuration_for_size(s->vconf_list, *vs, ms_factory_get_cpu_count(f->factory));
 	s->vconf.vsize = *vs;
 	s->vconf.fps = best_vconf.fps;
 	s->vconf.bitrate_limit = best_vconf.bitrate_limit;
@@ -939,7 +945,7 @@ static int enc_set_br(MSFilter *f, void *arg) {
 		s->vconf.required_bitrate = br;
 		enc_set_configuration(f, &s->vconf);
 	} else {
-		MSVideoConfiguration best_vconf = ms_video_find_best_configuration_for_bitrate(s->vconf_list, br, ms_get_cpu_count());
+		MSVideoConfiguration best_vconf = ms_video_find_best_configuration_for_bitrate(s->vconf_list, br, ms_factory_get_cpu_count(f->factory));
 		enc_set_configuration(f, &best_vconf);
 	}
 	return 0;
