@@ -48,11 +48,13 @@ void ms_bitrate_driver_unref(MSBitrateDriver *obj){
 
 struct _MSAudioBitrateDriver{
 	MSBitrateDriver parent;
+	RtpSession *session;
 	MSFilter *encoder;
 	int min_ptime;
 	int nom_bitrate;
 	int cur_ptime;
 	int cur_bitrate;
+	int encoder_caps;
 };
 
 typedef struct _MSAudioBitrateDriver MSAudioBitrateDriver;
@@ -60,9 +62,8 @@ typedef struct _MSAudioBitrateDriver MSAudioBitrateDriver;
 static int apply_ptime(MSAudioBitrateDriver *obj,int target_ptime){
 	char tmp[64];
 	int result=-1;
-	
 	if (target_ptime < min_ptime || target_ptime>max_ptime) {
-		ms_error("cannot apply ptime value [%i] on [%p] because out of range [%i..%i]",target_ptime,obj,min_ptime,max_ptime);
+		ms_error("MSAudioBitrateDriver [%p]: cannot apply ptime value [%i] on [%p] because out of range [%i..%i]",obj,target_ptime,obj,min_ptime,max_ptime);
 		return -1;
 	}
 
@@ -83,12 +84,10 @@ static int apply_ptime(MSAudioBitrateDriver *obj,int target_ptime){
 		} /*else ptime remain unchanged*/
 	}
 	if (result == 0) {
-		ms_message("AudioBitrateController [%p]: ptime is now  [%i ms]",obj,obj->cur_ptime);
+		ms_message("MSAudioBitrateDriver [%p]: ptime is now [%i ms]",obj,obj->cur_ptime);
 	} else {
-		ms_message("AudioBitrateController [%p]: cannot move ptime from [%i ms] to [%i ms]",obj,obj->cur_ptime,target_ptime);
+		ms_message("MSAudioBitrateDriver [%p]: cannot move ptime from [%i ms] to [%i ms]",obj,obj->cur_ptime,target_ptime);
 	}
-
-
 	return result;
 }
 
@@ -109,7 +108,7 @@ static int audio_bitrate_driver_execute_action(MSBitrateDriver *objbase, const M
 		if (obj->nom_bitrate==0){
 			ms_warning("MSAudioBitrateDriver: Not doing bitrate control on audio encoder, it does not seem to support that. Controlling ptime only.");
 			obj->nom_bitrate=-1;
-		}else 
+		}else
 			obj->cur_bitrate=obj->nom_bitrate;
 	}
 	if (obj->cur_ptime==0 || ms_filter_has_method(obj->encoder,MS_AUDIO_ENCODER_GET_PTIME)){ /*always sync current ptime if possible*/
@@ -121,25 +120,28 @@ static int audio_bitrate_driver_execute_action(MSBitrateDriver *objbase, const M
 	}
 
 	if (action->type==MSRateControlActionDecreaseBitrate){
+		
 		/*reducing bitrate of the codec isn't sufficient. Increasing ptime is much more efficient*/
-		if (inc_ptime(obj)){
+		if ((obj->encoder_caps & MS_AUDIO_ENCODER_CAP_AUTO_PTIME) || inc_ptime(obj)){
 			if (obj->nom_bitrate>0){
 				int cur_br=0;
 				int new_br;
 
 				/*if max ptime is reached, then try to reduce the codec bitrate if possible */
-				
+
 				if (ms_filter_call_method(obj->encoder,MS_FILTER_GET_BITRATE,&cur_br)!=0){
 					ms_message("MSAudioBitrateDriver: GET_BITRATE failed");
 					return 0;
 				}
 				obj->cur_bitrate=cur_br;
 				new_br=cur_br-((cur_br*action->value)/100);
-		
-				ms_message("MSAudioBitrateDriver: Attempting to reduce audio bitrate to %i",new_br);
+
+				ms_message("MSAudioBitrateDriver: Attempting to reduce audio bitrate from %i to %i",cur_br,new_br);
 				if (ms_filter_call_method(obj->encoder,MS_FILTER_SET_BITRATE,&new_br)!=0){
 					ms_message("MSAudioBitrateDriver: SET_BITRATE failed, incrementing ptime");
 					return inc_ptime(obj);
+				} else {
+					rtp_session_set_target_upload_bandwidth(obj->session, new_br);
 				}
 				new_br=0;
 				ms_filter_call_method(obj->encoder,MS_FILTER_GET_BITRATE,&new_br);
@@ -150,30 +152,38 @@ static int audio_bitrate_driver_execute_action(MSBitrateDriver *objbase, const M
 	}else if (action->type==MSRateControlActionDecreasePacketRate){
 		return inc_ptime(obj);
 	}else if (action->type==MSRateControlActionIncreaseQuality){
-		if (obj->nom_bitrate>0){
-			if (ms_filter_call_method(obj->encoder,MS_FILTER_GET_BITRATE,&obj->cur_bitrate)==0){
-				if (obj->cur_bitrate > 0  && obj->cur_bitrate<obj->nom_bitrate){
-				obj->cur_bitrate=(obj->cur_bitrate*140)/100;
-				if (obj->cur_bitrate> obj->nom_bitrate) obj->cur_bitrate=obj->nom_bitrate;
-				ms_message("MSAudioBitrateDriver: increasing bitrate of codec to %i",obj->cur_bitrate);
-				if (ms_filter_call_method(obj->encoder,MS_FILTER_SET_BITRATE,&obj->cur_bitrate)!=0){
-					ms_message("MSAudioBitrateDriver: could not set codec bitrate to %i",obj->cur_bitrate);
-				}else obj->cur_bitrate=obj->nom_bitrate; /* so that we do not attempt this anymore*/
-				return 0;
+		int ret=0;
+		if (!(obj->encoder_caps & MS_AUDIO_ENCODER_CAP_AUTO_PTIME)){
+			if (obj->cur_ptime>obj->min_ptime){
+				ret=dec_ptime(obj);
 			}
-			}else ms_warning("MSAudioBitrateDriver: MS_FILTER_GET_BITRATE failed.");
-			
 		}
-		if (obj->cur_ptime>obj->min_ptime){
-			return dec_ptime(obj);
-		}else return -1;
+		if (obj->nom_bitrate>0){
+			int cur_bitrate=0;
+			if (ms_filter_call_method(obj->encoder,MS_FILTER_GET_BITRATE,&cur_bitrate)==0){
+				if (cur_bitrate > 0  && cur_bitrate<obj->nom_bitrate){
+					obj->cur_bitrate=(obj->cur_bitrate*140)/100;
+					if (obj->cur_bitrate>= obj->nom_bitrate) {
+						obj->cur_bitrate=obj->nom_bitrate;
+						ret=-1;/*we reached the nominal value*/
+					}
+					ms_message("MSAudioBitrateDriver: increasing bitrate of codec to %i",obj->cur_bitrate);
+					if (ms_filter_call_method(obj->encoder,MS_FILTER_SET_BITRATE,&obj->cur_bitrate)!=0){
+						ms_message("MSAudioBitrateDriver: could not set codec bitrate to %i",obj->cur_bitrate);
+					}else {
+						rtp_session_set_target_upload_bandwidth(obj->session, obj->cur_bitrate);
+					}
+				}
+			}else ms_warning("MSAudioBitrateDriver: MS_FILTER_GET_BITRATE failed.");
+		}
+		return ret;
 	}
 	return 0;
 }
 
 static void audio_bitrate_driver_uninit(MSBitrateDriver *objbase){
 	//MSAudioBitrateDriver *obj=(MSBitrateDriver*)objbase;
-	
+
 }
 
 static MSBitrateDriverDesc audio_bitrate_driver={
@@ -182,22 +192,25 @@ static MSBitrateDriverDesc audio_bitrate_driver={
 };
 
 
-MSBitrateDriver *ms_audio_bitrate_driver_new(MSFilter *encoder){
+MSBitrateDriver *ms_audio_bitrate_driver_new(RtpSession *session, MSFilter *encoder){
 	MSAudioBitrateDriver *obj=ms_new0(MSAudioBitrateDriver,1);
 	obj->parent.desc=&audio_bitrate_driver;
+	obj->session = session;
 	obj->encoder=encoder;
 	obj->min_ptime=20;
 	obj->cur_ptime=0;
 	obj->cur_bitrate=obj->nom_bitrate=0;
+	if (ms_filter_has_method(obj->encoder, MS_AUDIO_ENCODER_GET_CAPABILITIES))
+			ms_filter_call_method(obj->encoder, MS_AUDIO_ENCODER_GET_CAPABILITIES, &obj->encoder_caps);
 	return (MSBitrateDriver*)obj;
 }
 
 static const int min_video_bitrate=64000;
-static const float increase_ramp=1.1;
 
 typedef struct _MSAVBitrateDriver{
 	MSBitrateDriver parent;
 	MSBitrateDriver *audio_driver;
+	RtpSession *vsession;
 	MSFilter *venc;
 	int nom_bitrate;
 	int cur_bitrate;
@@ -205,32 +218,38 @@ typedef struct _MSAVBitrateDriver{
 
 static int dec_video_bitrate(MSAVBitrateDriver *obj, const MSRateControlAction *action){
 	int new_br;
-	
+
 	ms_filter_call_method(obj->venc,MS_FILTER_GET_BITRATE,&obj->cur_bitrate);
-	new_br=((float)obj->cur_bitrate)*(100.0-(float)action->value)/100.0;
+	new_br=(int)(((float)obj->cur_bitrate)*(100.0f-(float)action->value)/100.0f);
 	if (new_br<min_video_bitrate){
 		ms_message("MSAVBitrateDriver: reaching low bound.");
 		new_br=min_video_bitrate;
 	}
-	ms_message("MSAVBitrateDriver: targeting %i bps for video encoder.",new_br);
-	ms_filter_call_method(obj->venc,MS_FILTER_SET_BITRATE,&new_br);
-	obj->cur_bitrate=new_br;
+	if (new_br!=obj->cur_bitrate){
+		ms_message("MSAVBitrateDriver: targeting %i bps for video encoder.",new_br);
+		ms_filter_call_method(obj->venc,MS_FILTER_SET_BITRATE,&new_br);
+		rtp_session_set_target_upload_bandwidth(obj->vsession, new_br);
+		obj->cur_bitrate=new_br;
+	}
 	return new_br==min_video_bitrate ? -1 : 0;
 }
 
 static int inc_video_bitrate(MSAVBitrateDriver *obj, const MSRateControlAction *action){
 	int newbr;
 	int ret=0;
-	
+
 	if (obj->cur_bitrate==0) return -1; /*current  bitrate was not known*/
-	newbr=(float)obj->cur_bitrate*(1.0+((float)action->value/100.0));
+	newbr=(int)((float)obj->cur_bitrate*(1.0f+((float)action->value/100.0f)));
 	if (newbr>obj->nom_bitrate){
 		newbr=obj->nom_bitrate;
 		ret=-1;
 	}
-	obj->cur_bitrate=newbr;
-	ms_message("MSAVBitrateDriver: increasing bitrate to %i bps for video encoder.",obj->cur_bitrate);
-	ms_filter_call_method(obj->venc,MS_FILTER_SET_BITRATE,&obj->cur_bitrate);
+	if (newbr!=obj->cur_bitrate){
+		obj->cur_bitrate=newbr;
+		ms_message("MSAVBitrateDriver: increasing bitrate to %i bps for video encoder.",obj->cur_bitrate);
+		ms_filter_call_method(obj->venc,MS_FILTER_SET_BITRATE,&obj->cur_bitrate);
+		rtp_session_set_target_upload_bandwidth(obj->vsession, obj->cur_bitrate);
+	}
 	return ret;
 }
 
@@ -244,7 +263,7 @@ static int av_driver_execute_action(MSBitrateDriver *objbase, const MSRateContro
 			return -1;
 		}
 	}
-	
+
 	switch(action->type){
 		case MSRateControlActionDecreaseBitrate:
 			ret=dec_video_bitrate(obj,action);
@@ -267,7 +286,7 @@ static void av_bitrate_driver_uninit(MSBitrateDriver *objbase){
 	MSAVBitrateDriver *obj=(MSAVBitrateDriver*)objbase;
 	if (obj->audio_driver)
 		ms_bitrate_driver_unref(obj->audio_driver);
-	
+
 }
 
 static MSBitrateDriverDesc av_bitrate_driver={
@@ -275,13 +294,120 @@ static MSBitrateDriverDesc av_bitrate_driver={
 	av_bitrate_driver_uninit
 };
 
-MSBitrateDriver *ms_av_bitrate_driver_new(MSFilter *aenc, MSFilter *venc){
+MSBitrateDriver *ms_av_bitrate_driver_new(RtpSession *asession, MSFilter *aenc, RtpSession *vsession, MSFilter *venc){
 	MSAVBitrateDriver *obj=ms_new0(MSAVBitrateDriver,1);
 	obj->parent.desc=&av_bitrate_driver;
-	obj->audio_driver=(aenc!=NULL) ? ms_bitrate_driver_ref(ms_audio_bitrate_driver_new(aenc)) : NULL;
+	obj->audio_driver=((asession != NULL) && (aenc!=NULL)) ? ms_bitrate_driver_ref(ms_audio_bitrate_driver_new(asession, aenc)) : NULL;
+	obj->vsession = vsession;
 	obj->venc=venc;
-	
+
 	return (MSBitrateDriver*)obj;
 }
 
 
+
+typedef struct _MSBandwidthBitrateDriver{
+	MSBitrateDriver parent;
+	MSBitrateDriver *audio_driver;
+	RtpSession *vsession;
+	MSFilter *venc;
+	int nom_bitrate;
+	int cur_bitrate;
+}MSBandwidthBitrateDriver;
+
+static int bandwidth_change_ptime(MSAudioBitrateDriver *obj, int percent){
+	return apply_ptime(obj,(int)(obj->min_ptime + (max_ptime - obj->min_ptime) * percent / 100.f));
+}
+
+static int bandwidth_change_video_bitrate(MSBandwidthBitrateDriver *obj, const MSRateControlAction *action){
+	int newbr;
+	int oldbr;
+	int ret=0;
+	bool_t decrease = (action->type == MSRateControlActionDecreaseBitrate);
+	int bound= (decrease) ? min_video_bitrate : obj->nom_bitrate;
+
+	ms_filter_call_method(obj->venc,MS_FILTER_GET_BITRATE,&obj->cur_bitrate);
+	if (obj->cur_bitrate==0){
+		ms_message("MSBandwidthBitrateDriver: current bitrate was not known.");
+		return -1; /*current  bitrate was not known*/
+	}
+	newbr = (int)((float)obj->cur_bitrate*(100.0f+(decrease?-1:1)*(float)action->value)/100.0f);
+	if ((decrease&&newbr<bound) || (!decrease&&newbr>bound)){
+		if (obj->cur_bitrate==bound){
+			ms_message("MSBandwidthBitrateDriver: bitrate already reached %s limit %d b/s.", (decrease)?"min":"max",bound);
+			return -1;
+		}
+		newbr=bound;
+	}
+	oldbr=obj->cur_bitrate;
+	obj->cur_bitrate=newbr;
+	rtp_session_set_target_upload_bandwidth(obj->vsession, obj->cur_bitrate);
+	ms_filter_call_method(obj->venc,MS_FILTER_SET_BITRATE,&obj->cur_bitrate);
+	ms_filter_call_method(obj->venc,MS_FILTER_GET_BITRATE,&newbr);
+
+	ms_message("MSBandwidthBitrateDriver: changing bitrate from %i to %i bps for video encoder.",oldbr,newbr);
+	return ret;
+}
+
+static int bandwidth_driver_execute_action(MSBitrateDriver *objbase, const MSRateControlAction *action){
+	MSBandwidthBitrateDriver *obj=(MSBandwidthBitrateDriver*)objbase;
+	int ret=0;
+	if (obj->nom_bitrate==0&&obj->venc){
+		ms_filter_call_method(obj->venc,MS_FILTER_GET_BITRATE,&obj->nom_bitrate);
+		if (obj->nom_bitrate==0){
+			ms_warning("MSBandwidthBitrateDriver: Not doing adaptive rate control on video encoder, it does not seem to support that.");
+			return -1;
+		}
+	}
+
+	if (!obj->venc){
+		ret=1;
+	}
+	switch(action->type){
+		case MSRateControlActionDecreaseBitrate:
+			if (obj->venc){
+				ret=bandwidth_change_video_bitrate(obj,action);
+			}
+			if (ret!=0 && obj->audio_driver){
+				ret=ms_bitrate_driver_execute_action(obj->audio_driver,action);
+			}
+		break;
+		case MSRateControlActionDecreasePacketRate:
+			if (obj->audio_driver){
+				ret=bandwidth_change_ptime((MSAudioBitrateDriver*) obj->audio_driver, 100);
+			}else{
+				ret=1;
+			}
+		break;
+		case MSRateControlActionIncreaseQuality:
+			if (obj->venc){
+				ret=bandwidth_change_video_bitrate(obj,action);
+			}
+			if (ret!=0 && obj->audio_driver){
+				ret=ms_bitrate_driver_execute_action(obj->audio_driver,action);
+			}
+		break;
+		case MSRateControlActionDoNothing:
+		break;
+	}
+	ms_message("MSBandwidthBitrateDriver: Action %s %s", ms_rate_control_action_type_name(action->type), ret == 0 ? "succeeded" : "failed");
+	return ret;
+}
+
+static void bandwidth_bitrate_driver_uninit(MSBitrateDriver *objbase){
+}
+
+static MSBitrateDriverDesc bandwidth_bitrate_driver={
+	bandwidth_driver_execute_action,
+	bandwidth_bitrate_driver_uninit
+};
+
+MSBitrateDriver *ms_bandwidth_bitrate_driver_new(RtpSession *asession, MSFilter *aenc, RtpSession *vsession, MSFilter *venc){
+	MSBandwidthBitrateDriver *obj=ms_new0(MSBandwidthBitrateDriver,1);
+	obj->parent.desc=&bandwidth_bitrate_driver;
+	obj->vsession = vsession;
+	obj->audio_driver=(aenc!=NULL) ? ms_bitrate_driver_ref(ms_audio_bitrate_driver_new(asession,aenc)) : NULL;
+	obj->venc=venc;
+
+	return (MSBitrateDriver*)obj;
+}
