@@ -40,13 +40,19 @@ extern "C"{
 #endif
 
 extern void __register_ffmpeg_encoders_if_possible(MSFactory *factory);
-extern void ms_ffmpeg_check_init();
+extern void __register_ffmpeg_h264_decoder_if_possible(MSFactory *factory);
+extern void ms_ffmpeg_check_init(void);
 extern bool_t libmsandroiddisplay_init(MSFactory *factory);
 extern void libmsandroiddisplaybad_init(MSFactory *factory);
 extern void libmsandroidopengldisplay_init(MSFactory *factory);
 
+#if defined(__APPLE__) && defined(VIDEO_ENABLED)
+extern void _register_videotoolbox_if_supported(MSFactory *factory);
+#endif
+
 #include "voipdescs.h"
 #include "mediastreamer2/mssndcard.h"
+#include "mediastreamer2/msvideopresets.h"
 #include "mediastreamer2/mswebcam.h"
 
 #ifdef __APPLE__
@@ -76,7 +82,7 @@ extern MSSndCardDesc oss_card_desc;
 extern MSSndCardDesc arts_card_desc;
 #endif
 
-#if defined(WIN32) && WINAPI_FAMILY_PARTITION(WINAPI_PARTITION_DESKTOP)
+#ifdef MS2_WINDOWS_DESKTOP
 extern MSSndCardDesc winsnd_card_desc;
 #endif
 
@@ -135,7 +141,7 @@ static MSSndCardDesc * ms_snd_card_descs[]={
 	&arts_card_desc,
 #endif
 
-#if defined(WIN32) && WINAPI_FAMILY_PARTITION(WINAPI_PARTITION_DESKTOP)
+#ifdef MS2_WINDOWS_DESKTOP
 	&winsnd_card_desc,
 #endif
 
@@ -180,11 +186,11 @@ extern MSWebCamDesc v4l_desc;
 extern MSWebCamDesc v4l2_card_desc;
 #endif
 
-#ifdef WIN32
+#ifdef _WIN32
 extern MSWebCamDesc ms_vfw_cam_desc;
 #endif
 
-#if defined(WIN32) && defined(HAVE_DIRECTSHOW)
+#if defined(_WIN32) && defined(HAVE_DIRECTSHOW)
 extern MSWebCamDesc ms_directx_cam_desc;
 #endif
 
@@ -207,17 +213,24 @@ extern MSWebCamDesc ms_android_video_capture_desc;
 extern MSWebCamDesc ms_v4ios_cam_desc;
 #endif
 
+#ifdef __QNX__
+extern MSWebCamDesc ms_bb10_camera_desc;
+#endif
+
 #endif /* MS2_FILTERS */
 
 static MSWebCamDesc * ms_web_cam_descs[]={
 #ifdef MS2_FILTERS
+#if defined (ANDROID)
+	&ms_android_video_capture_desc,
+#endif
 #ifdef HAVE_LINUX_VIDEODEV2_H
 	&v4l2_card_desc,
 #endif
 #ifdef HAVE_LINUX_VIDEODEV_H
 	&v4l_desc,
 #endif
-#if defined(WIN32) && defined(HAVE_VFW)
+#if defined(_WIN32) && defined(HAVE_VFW)
 	&ms_vfw_cam_desc,
 #endif
 #if defined(__MINGW32__) || defined (HAVE_DIRECTSHOW)
@@ -226,8 +239,8 @@ static MSWebCamDesc * ms_web_cam_descs[]={
 #if TARGET_OS_MAC && !TARGET_OS_IPHONE
 	&ms_v4m_cam_desc,
 #endif
-#if defined (ANDROID)
-	&ms_android_video_capture_desc,
+#ifdef __QNX__
+	&ms_bb10_camera_desc,
 #endif
 #if TARGET_OS_IPHONE &&  !TARGET_IPHONE_SIMULATOR
 	&ms_v4ios_cam_desc,
@@ -239,90 +252,144 @@ static MSWebCamDesc * ms_web_cam_descs[]={
 };
 
 #endif
-static int ms_voip_ref=0;
-void ms_voip_init(){
-	if (ms_voip_ref++ >0 ) {
-		ms_message ("Skiping ms_voip_init, because [%i] ref",ms_voip_ref);
-		return;
-	}
-	ms_srtp_init();
-	ms_factory_init_voip(ms_factory_get_fallback());
-}
 
-static int managers_ref=0;
 
 void ms_factory_init_voip(MSFactory *obj){
 	MSSndCardManager *cm;
 	int i;
 
+	if (obj->voip_initd) return;
+
+	ms_srtp_init();
+
 	/* register builtin VoIP MSFilter's */
 	for (i=0;ms_voip_filter_descs[i]!=NULL;i++){
 		ms_factory_register_filter(obj,ms_voip_filter_descs[i]);
 	}
 
-	if (managers_ref==0){
-		managers_ref++;
-		cm=ms_snd_card_manager_get();
-		if (cm->descs==NULL){
-			ms_message("Registering all soundcard handlers");
-			for (i=0;ms_snd_card_descs[i]!=NULL;i++){
-				ms_snd_card_manager_register_desc(cm,ms_snd_card_descs[i]);
-			}
-		}
+#if defined(VIDEO_ENABLED) && defined(MS2_FILTERS) && !defined(NO_FFMPEG) && defined(HAVE_LIBAVCODEC_AVCODEC_H)
+	ms_ffmpeg_check_init();
+	__register_ffmpeg_encoders_if_possible(obj);
+	__register_ffmpeg_h264_decoder_if_possible(obj);
+#endif
 
+#if defined(__APPLE__) && defined(VIDEO_ENABLED)
+	_register_videotoolbox_if_supported(obj);
+#endif
+
+	cm=ms_snd_card_manager_new();
+	ms_message("Registering all soundcard handlers");
+	cm->factory=obj;
+	obj->sndcardmanager = cm;
+	for (i=0;ms_snd_card_descs[i]!=NULL;i++){
+		ms_snd_card_manager_register_desc(cm,ms_snd_card_descs[i]);
+	}
+
+	{
+		MSWebCamManager *wm;
+		wm=ms_web_cam_manager_new();
+		wm->factory = obj;
+		obj->wbcmanager = wm;
 #ifdef VIDEO_ENABLED
-		{
-			MSWebCamManager *wm;
-			wm=ms_web_cam_manager_get();
-			if (wm->descs==NULL){
-				ms_message("Registering all webcam handlers");
-				for (i=0;ms_web_cam_descs[i]!=NULL;i++){
-					ms_web_cam_manager_register_desc(wm,ms_web_cam_descs[i]);
-				}
-			}
+		ms_message("Registering all webcam handlers");
+		for (i=0;ms_web_cam_descs[i]!=NULL;i++){
+			ms_web_cam_manager_register_desc(wm,ms_web_cam_descs[i]);
 		}
 #endif
 	}
 
-#if defined(VIDEO_ENABLED) && defined(MS2_FILTERS) && !defined(NO_FFMPEG) && defined(HAVE_LIBAVCODEC_AVCODEC_H)
-	ms_ffmpeg_check_init();
-	__register_ffmpeg_encoders_if_possible(obj);
+#ifdef VIDEO_ENABLED
+	{
+		MSVideoPresetsManager *vpm = ms_video_presets_manager_new(obj);
+		register_video_preset_high_fps(vpm);
+	}
 #endif
 
+	obj->devices_info = ms_devices_info_new();
+
 #if defined(ANDROID) && defined (VIDEO_ENABLED)
-	if (1) {
-		libmsandroidopengldisplay_init(obj);
-	} else {
-		if (!libmsandroiddisplay_init(obj)) {
-			libmsandroiddisplaybad_init(obj);
+	{
+		MSDevicesInfo *devices = ms_factory_get_devices_info(obj);
+		SoundDeviceDescription *description = ms_devices_info_get_sound_device_description(devices);
+		if (description && description->flags & DEVICE_HAS_CRAPPY_OPENGL) {
+			if (!libmsandroiddisplay_init(obj)) {
+				libmsandroiddisplaybad_init(obj);
+			}
+		} else {
+			libmsandroidopengldisplay_init(obj);
 		}
 	}
 #endif
 	obj->voip_initd=TRUE;
+	obj->voip_uninit_func = ms_factory_uninit_voip;
 	ms_message("ms_factory_init_voip() done");
 }
 
 void ms_factory_uninit_voip(MSFactory *obj){
 	if (obj->voip_initd){
-		managers_ref--;
-		if (managers_ref==0){
-			ms_snd_card_manager_destroy();
+		ms_snd_card_manager_destroy(obj->sndcardmanager);
+		obj->sndcardmanager = NULL;
 #ifdef VIDEO_ENABLED
-			ms_web_cam_manager_destroy();
+		ms_web_cam_manager_destroy(obj->wbcmanager);
+		obj->wbcmanager = NULL;
+		ms_video_presets_manager_destroy(obj->video_presets_manager);
 #endif
-		}
+		ms_srtp_shutdown();
+		if (obj->devices_info) ms_devices_info_free(obj->devices_info);
+		obj->voip_initd = FALSE;
 	}
 }
 
+MSFactory *ms_factory_new_with_voip(void){
+	MSFactory *f = ms_factory_new();
+	ms_factory_init_voip(f);
+	ms_factory_init_plugins(f);
+	return f;
+}
+
+
+
+PayloadType * ms_offer_answer_context_match_payload(MSOfferAnswerContext *context, const MSList *local_payloads, const PayloadType *remote_payload, const MSList *remote_payloads, bool_t is_reading){
+	return context->match_payload(context, local_payloads, remote_payload, remote_payloads, is_reading);
+}
+
+ MSOfferAnswerContext *ms_offer_answer_create_simple_context(MSPayloadMatcherFunc func){
+	 MSOfferAnswerContext *ctx = ms_new0(MSOfferAnswerContext,1);
+	 ctx->match_payload = func;
+	 ctx->destroy = (void (*)(MSOfferAnswerContext *)) ms_free;
+	 return ctx;
+}
+
+void ms_offer_answer_context_destroy(MSOfferAnswerContext *ctx){
+	if (ctx->destroy)
+		ctx->destroy(ctx);
+}
+
+
+#ifdef _MSC_VER
+#pragma warning(disable : 4996)
+#else
+#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#endif
+
+static int ms_voip_ref=0;
+void ms_voip_init(){
+	if (ms_voip_ref++ >0 ) {
+		ms_message ("Skipping ms_voip_init, because [%i] ref",ms_voip_ref);
+		return;
+	}
+	ms_factory_init_voip(ms_factory_get_fallback());
+}
+
 void ms_voip_exit(){
 	if (--ms_voip_ref >0 ) {
-		ms_message ("Skiping ms_voip_exit, still [%i] ref",ms_voip_ref);
+		ms_message ("Skipping ms_voip_exit, still [%i] ref",ms_voip_ref);
 		return;
 	}
-	ms_srtp_shutdown();
 	ms_factory_uninit_voip(ms_factory_get_fallback());
 }
 
+
 #ifdef __cplusplus
 }
 #endif
