@@ -22,9 +22,10 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #define PRIVATE_H
 
 #include "mediastreamer2/mediastream.h"
+#include "mediastreamer2/msvideopresets.h"
 
 
-#if defined(WIN32) || defined(_WIN32) || defined(__WIN32) || defined(__WIN32__)
+#if defined(_WIN32) || defined(_WIN32) || defined(__WIN32) || defined(__WIN32__)
 #ifdef MEDIASTREAMER2_INTERNAL_EXPORTS
 #define MEDIASTREAMER2_INTERNAL_EXPORT __declspec(dllexport)
 #define MEDIASTREAMER2_INTERNAL_VAR_EXPORT __declspec(dllexport)
@@ -48,8 +49,6 @@ extern "C"
 
 MSTickerPrio __ms_get_default_prio(bool_t is_video);
 
-MEDIASTREAMER2_INTERNAL_EXPORT RtpSession * create_duplex_rtpsession(const char* local_ip,int loc_rtp_port, int loc_rtcp_port);
-
 void media_stream_start_ticker(MediaStream *stream);
 
 const char * media_stream_type_str(MediaStream *stream);
@@ -108,12 +107,18 @@ MS2_PUBLIC void ms_dtls_srtp_set_stream_sessions(MSDtlsSrtpContext *dtls_context
  */
 MS2_PUBLIC void ms_zrtp_set_stream_sessions(MSZrtpContext *zrtp_context, MSMediaStreamSessions *stream_sessions);
 
-bool_t media_stream_session_secured(const struct _MSMediaStreamSessions *sessions,MediaStreamDir dir);
+bool_t ms_media_stream_sessions_secured(const MSMediaStreamSessions *sessions,MediaStreamDir dir);
 
-MSSrtpCtx* ms_srtp_context_new();
+MSSrtpCtx* ms_srtp_context_new(void);
 void ms_srtp_context_delete(MSSrtpCtx *session);
 
 
+void register_video_preset_high_fps(MSVideoPresetsManager *manager);
+
+MSFilter *_ms_create_av_player(const char *filename, MSFactory* factory);
+void video_recorder_handle_event(void *userdata, MSFilter *recorder, unsigned int event, void *event_arg);
+
+
 #ifdef __cplusplus
 }
 #endif
