@@ -26,32 +26,35 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "mediastreamer2_tester.h"
 #include "mediastreamer2_tester_private.h"
 
-#include <stdio.h>
-#include "CUnit/Basic.h"
-
-
 static RtpProfile rtp_profile;
+static MSFactory *_factory= NULL;
 
 #define OPUS_PAYLOAD_TYPE    121
 #define SPEEX16_PAYLOAD_TYPE 122
 #define SILK16_PAYLOAD_TYPE  123
 #define PCMA8_PAYLOAD_TYPE 8
 
-static int tester_init(void) {
-	ms_init();
-	ms_filter_enable_statistics(TRUE);
+static int tester_before_all(void) {
+	//ms_init();
+	_factory = ms_factory_new();
+	ms_factory_init_voip(_factory);
+	ms_factory_init_plugins(_factory);
+
+	//ms_filter_enable_statistics(TRUE);
+	ms_factory_enable_statistics(_factory, TRUE);
 	ortp_init();
 	rtp_profile_set_payload (&rtp_profile,0,&payload_type_pcmu8000);
 	rtp_profile_set_payload (&rtp_profile,OPUS_PAYLOAD_TYPE,&payload_type_opus);
 	rtp_profile_set_payload (&rtp_profile,SPEEX16_PAYLOAD_TYPE,&payload_type_speex_wb);
 	rtp_profile_set_payload (&rtp_profile,SILK16_PAYLOAD_TYPE,&payload_type_silk_wb);
 	rtp_profile_set_payload (&rtp_profile,PCMA8_PAYLOAD_TYPE,&payload_type_pcma8000);
-
 	return 0;
 }
 
-static int tester_cleanup(void) {
-	ms_exit();
+static int tester_after_all(void) {
+	//ms_exit();
+
+	ms_factory_destroy(_factory);
 	rtp_profile_clear_all(&rtp_profile);
 	return 0;
 }
@@ -66,8 +69,8 @@ static int tester_cleanup(void) {
 
 #define HELLO_8K_1S_FILE  "sounds/hello8000-1s.wav"
 #define HELLO_16K_1S_FILE  "sounds/hello16000-1s.wav"
-#define RECORDED_8K_1S_FILE  "sounds/recorded_hello8000-1s.wav"
-#define RECORDED_16K_1S_FILE  "sounds/recorded_hello16000-1s.wav"
+#define RECORDED_8K_1S_FILE  "recorded_hello8000-1s.wav"
+#define RECORDED_16K_1S_FILE  "recorded_hello16000-1s.wav"
 
 #define MULTICAST_IP  "224.1.2.3"
 
@@ -121,85 +124,120 @@ static void event_queue_cb(MediaStream *ms, void *user_pointer) {
 	}
 }
 
-static void basic_audio_stream_base(	const char* marielle_local_ip
-									, 	int marielle_local_rtp_port
-									, 	int marielle_local_rtcp_port
-									, 	const char*  margaux_local_ip
-									, 	int margaux_local_rtp_port
-									, 	int margaux_local_rtcp_port) {
-	AudioStream * 	marielle = audio_stream_new2 (marielle_local_ip, marielle_local_rtp_port, marielle_local_rtcp_port);
+static void basic_audio_stream_base_2(	const char* marielle_local_ip
+									  ,	const char* marielle_remote_ip
+									  , int marielle_local_rtp_port
+									  , int marielle_remote_rtp_port
+									  ,	int marielle_local_rtcp_port
+									  , int marielle_remote_rtcp_port
+									  ,	const char*  margaux_local_ip
+									  , const char*  margaux_remote_ip
+									  ,	int margaux_local_rtp_port
+									  , int margaux_remote_rtp_port
+									  ,	int margaux_local_rtcp_port
+									  , int margaux_remote_rtcp_port
+									  , int lost_percentage) {
+	AudioStream * 	marielle = audio_stream_new2 (_factory, marielle_local_ip, marielle_local_rtp_port, marielle_local_rtcp_port);
 	stats_t marielle_stats;
-	AudioStream * 	margaux = audio_stream_new2 (margaux_local_ip, margaux_local_rtp_port,margaux_local_rtcp_port);
+	AudioStream * 	margaux = audio_stream_new2 (_factory, margaux_local_ip, margaux_local_rtp_port,margaux_local_rtcp_port);
 	stats_t margaux_stats;
 	RtpProfile* profile = rtp_profile_new("default profile");
-	char* hello_file = ms_strdup_printf("%s/%s", bc_tester_read_dir_prefix, HELLO_8K_1S_FILE);
-	char* recorded_file = ms_strdup_printf("%s/%s", bc_tester_writable_dir_prefix, RECORDED_8K_1S_FILE);
-	int dummy=0;
+	char* hello_file = bc_tester_res(HELLO_8K_1S_FILE);
+	char* recorded_file = bc_tester_file(RECORDED_8K_1S_FILE);
+	uint64_t marielle_rtp_sent=0;
+
 	rtp_session_set_multicast_loopback(marielle->ms.sessions.rtp_session,TRUE);
 	rtp_session_set_multicast_loopback(margaux->ms.sessions.rtp_session,TRUE);
+	rtp_session_set_rtcp_report_interval(marielle->ms.sessions.rtp_session, 1000);
+	rtp_session_set_rtcp_report_interval(margaux->ms.sessions.rtp_session, 1000);
 
 	reset_stats(&marielle_stats);
 	reset_stats(&margaux_stats);
 
 	rtp_profile_set_payload (profile,0,&payload_type_pcmu8000);
 
-
-	CU_ASSERT_EQUAL(audio_stream_start_full(margaux
+	BC_ASSERT_EQUAL(audio_stream_start_full(margaux
 											, profile
-											, ms_is_multicast(margaux_local_ip)?margaux_local_ip:marielle_local_ip
-											, ms_is_multicast(margaux_local_ip)?margaux_local_rtp_port:marielle_local_rtp_port
-											, marielle_local_ip
-											, marielle_local_rtcp_port
+											, ms_is_multicast(margaux_local_ip)?margaux_local_ip:margaux_remote_ip
+											, ms_is_multicast(margaux_local_ip)?margaux_local_rtp_port:margaux_remote_rtp_port
+											, margaux_remote_ip
+											, margaux_remote_rtcp_port
 											, 0
 											, 50
 											, NULL
 											, recorded_file
 											, NULL
 											, NULL
-											, 0),0);
+											, 0)
+					,0, int, "%d");
 
-	CU_ASSERT_EQUAL(audio_stream_start_full(marielle
+	BC_ASSERT_EQUAL(audio_stream_start_full(marielle
 											, profile
-											, margaux_local_ip
-											, margaux_local_rtp_port
-											, margaux_local_ip
-											, margaux_local_rtcp_port
+											, marielle_remote_ip
+											, marielle_remote_rtp_port
+											, marielle_remote_ip
+											, marielle_remote_rtcp_port
 											, 0
 											, 50
 											, hello_file
 											, NULL
 											, NULL
 											, NULL
-											, 0),0);
+											, 0)
+					,0, int, "%d");
 
 	ms_filter_add_notify_callback(marielle->soundread, notify_cb, &marielle_stats,TRUE);
 
-	CU_ASSERT_TRUE(wait_for_until(&marielle->ms,&margaux->ms,&marielle_stats.number_of_EndOfFile,1,12000));
-
-	/*make sure packets can cross from sender to receiver*/
-	wait_for_until(&marielle->ms,&margaux->ms,&dummy,1,500);
+	wait_for_until(&marielle->ms,&margaux->ms,&marielle_stats.number_of_EndOfFile,1,12000);
 
 	audio_stream_get_local_rtp_stats(marielle,&marielle_stats.rtp);
 	audio_stream_get_local_rtp_stats(margaux,&margaux_stats.rtp);
+	marielle_rtp_sent = marielle_stats.rtp.sent;
 
-	/* No packet loss is assumed */
-	CU_ASSERT_EQUAL(marielle_stats.rtp.sent,margaux_stats.rtp.recv);
+
+	if (rtp_session_rtcp_enabled(marielle->ms.sessions.rtp_session) && rtp_session_rtcp_enabled(margaux->ms.sessions.rtp_session)) {
+		BC_ASSERT_GREATER_STRICT(rtp_session_get_round_trip_propagation(marielle->ms.sessions.rtp_session),0,float,"%f");
+		BC_ASSERT_GREATER_STRICT(rtp_session_get_stats(marielle->ms.sessions.rtp_session)->recv_rtcp_packets,0,unsigned long long,"%llu");
+	}
 
 	audio_stream_stop(marielle);
+
+	BC_ASSERT_TRUE(wait_for_until(&margaux->ms,NULL,(int*)&margaux_stats.rtp.hw_recv,(int)((marielle_rtp_sent*(100-lost_percentage))/100),2500));
+
 	audio_stream_stop(margaux);
 
 	unlink(recorded_file);
-	ms_free(recorded_file);
-	ms_free(hello_file);
+	free(recorded_file);
+	free(hello_file);
 	rtp_profile_destroy(profile);
 }
+static void basic_audio_stream_base(	const char* marielle_local_ip
+									, 	int marielle_local_rtp_port
+									, 	int marielle_local_rtcp_port
+									, 	const char*  margaux_local_ip
+									, 	int margaux_local_rtp_port
+									, 	int margaux_local_rtcp_port) {
+	basic_audio_stream_base_2( marielle_local_ip
+							  , margaux_local_ip
+							  , marielle_local_rtp_port
+							  , margaux_local_rtp_port
+							  , marielle_local_rtcp_port
+							  , margaux_local_rtcp_port
+							  , margaux_local_ip
+							  , marielle_local_ip
+							  , margaux_local_rtp_port
+							  , marielle_local_rtp_port
+							  , margaux_local_rtcp_port
+							  , marielle_local_rtcp_port
+							  , 0);
 
-static void basic_audio_stream()  {
+}
+static void basic_audio_stream(void)  {
 	basic_audio_stream_base(MARIELLE_IP,MARIELLE_RTP_PORT,MARIELLE_RTCP_PORT
 							,MARGAUX_IP, MARGAUX_RTP_PORT, MARGAUX_RTCP_PORT);
 }
 
-static void multicast_audio_stream()  {
+static void multicast_audio_stream(void)  {
 	basic_audio_stream_base("0.0.0.0",MARIELLE_RTP_PORT, 0
 							,MULTICAST_IP, MARGAUX_RTP_PORT, 0);
 }
@@ -208,98 +246,133 @@ static void encrypted_audio_stream_base( bool_t change_ssrc,
 										 bool_t change_send_key_in_the_middle
 										,bool_t set_both_send_recv_key
 										,bool_t send_key_first
-										,bool_t encryption_mandatory) {
-	AudioStream * 	marielle = audio_stream_new (MARIELLE_RTP_PORT, MARIELLE_RTCP_PORT,FALSE);
-	AudioStream * 	margaux = audio_stream_new (MARGAUX_RTP_PORT,MARGAUX_RTCP_PORT, FALSE);
+										,bool_t encryption_mandatory,
+										MSCryptoSuite suite) {
+	AudioStream * 	marielle = audio_stream_new (_factory, MARIELLE_RTP_PORT, MARIELLE_RTCP_PORT,FALSE);
+	AudioStream * 	margaux = audio_stream_new (_factory, MARGAUX_RTP_PORT,MARGAUX_RTCP_PORT, FALSE);
 	RtpProfile* profile = rtp_profile_new("default profile");
-	char* hello_file = ms_strdup_printf("%s/%s", bc_tester_read_dir_prefix, HELLO_8K_1S_FILE);
-	char* recorded_file = ms_strdup_printf("%s/%s", bc_tester_writable_dir_prefix, RECORDED_8K_1S_FILE);
+	char* hello_file = bc_tester_res(HELLO_8K_1S_FILE);
+	char* recorded_file = bc_tester_file(RECORDED_8K_1S_FILE);
 	stats_t marielle_stats;
 	stats_t margaux_stats;
 	int dummy=0;
-	int number_of_dropped_packets=0;
-	media_stream_session_encryption_mandatory_enable(&marielle->ms.sessions,encryption_mandatory);
+	uint64_t number_of_dropped_packets=0;
 
-	if (ms_srtp_supported()) {
+	const char *aes_128_bits_send_key = "d0RmdmcmVCspeEc3QGZiNWpVLFJhQX1cfHAwJSoj";
+	const char *aes_128_bits_send_key_2 = "eCYF4nYyCvmCpFWjUeDaxI2GWp2BzCRlIPfg52Te";
+	const char *aes_128_bits_recv_key = "6jCLmtRkVW9E/BUuJtYj/R2z6+4iEe06/DWohQ9F";
+
+	const char *aes_256_bits_send_key = "nJNTwiMkyAu8zs0MWUiSQbnBL4M+xkWTYgrVLR2eFwZyO+ca2UqBy2Uh9pVRbA==";
+	const char *aes_256_bits_send_key_2 = "N3vq6TMfvtyYpqGaEi9vAHMCzgWJvaD1PIfwEYtdEgI2ACezZo2vpOdV2YWEcQ==";
+	const char *aes_256_bits_recv_key = "UKg69sFLbrA7d0hEVKMtT83R3GR3sjhE0XMqNBbQ+axoDWMP5dQNfjNuSQQHbw==";
+
+	const char *send_key = NULL;
+	const char *send_key_2 = NULL;
+	const char *recv_key = NULL;
+
+	ms_media_stream_sessions_set_encryption_mandatory(&marielle->ms.sessions,encryption_mandatory);
+
+	switch (suite) {
+		case MS_AES_128_SHA1_32:
+		case MS_AES_128_SHA1_80:
+			send_key = aes_128_bits_send_key;
+			send_key_2 = aes_128_bits_send_key_2;
+			recv_key = aes_128_bits_recv_key;
+			break;
+		case MS_AES_256_SHA1_32:
+		case MS_AES_256_SHA1_80:
+			send_key = aes_256_bits_send_key;
+			send_key_2 = aes_256_bits_send_key_2;
+			recv_key = aes_256_bits_recv_key;
+			break;
+		default:
+			BC_FAIL("Unsupported suite");
+			return;
+	}
+
+
+ 	if (ms_srtp_supported()) {
 		reset_stats(&marielle_stats);
 		reset_stats(&margaux_stats);
 
 		rtp_profile_set_payload (profile,0,&payload_type_pcmu8000);
 
-		CU_ASSERT_EQUAL(audio_stream_start_full(margaux
-				, profile
-				, MARIELLE_IP
-				, MARIELLE_RTP_PORT
-				, MARIELLE_IP
-				, MARIELLE_RTCP_PORT
-				, 0
-				, 50
-				, NULL
-				, recorded_file
-				, NULL
-				, NULL
-				, 0),0);
+		BC_ASSERT_EQUAL(audio_stream_start_full(margaux
+												, profile
+												, MARIELLE_IP
+												, MARIELLE_RTP_PORT
+												, MARIELLE_IP
+												, MARIELLE_RTCP_PORT
+												, 0
+												, 50
+												, NULL
+												, recorded_file
+												, NULL
+												, NULL
+												, 0)
+		,0, int, "%d");
 
-		CU_ASSERT_EQUAL(audio_stream_start_full(marielle
-				, profile
-				, MARGAUX_IP
-				, MARGAUX_RTP_PORT
-				, MARGAUX_IP
-				, MARGAUX_RTCP_PORT
-				, 0
-				, 50
-				, hello_file
-				, NULL
-				, NULL
-				, NULL
-				, 0),0);
+		BC_ASSERT_EQUAL(audio_stream_start_full(marielle
+												, profile
+												, MARGAUX_IP
+												, MARGAUX_RTP_PORT
+												, MARGAUX_IP
+												, MARGAUX_RTCP_PORT
+												, 0
+												, 50
+												, hello_file
+												, NULL
+												, NULL
+												, NULL
+												, 0)
+		,0, int, "%d");
 
 		if (encryption_mandatory) {
 			/*wait a bit to make sure packets are discarded*/
 			wait_for_until(&marielle->ms,&margaux->ms,&dummy,1,1000);
 			audio_stream_get_local_rtp_stats(margaux,&margaux_stats.rtp);
 			audio_stream_get_local_rtp_stats(marielle,&marielle_stats.rtp);
-			CU_ASSERT_EQUAL(margaux_stats.rtp.recv,0);
+			BC_ASSERT_EQUAL(margaux_stats.rtp.recv,0, unsigned long long, "%llu");
 			number_of_dropped_packets=marielle_stats.rtp.packet_sent;
 		}
 
 		if (send_key_first) {
-			CU_ASSERT_TRUE(media_stream_set_srtp_send_key_b64(&(marielle->ms.sessions), MS_AES_128_SHA1_32, "d0RmdmcmVCspeEc3QGZiNWpVLFJhQX1cfHAwJSoj") == 0);
+			BC_ASSERT_TRUE(ms_media_stream_sessions_set_srtp_send_key_b64(&(marielle->ms.sessions), suite, send_key) == 0);
 			if (set_both_send_recv_key)
-				CU_ASSERT_TRUE(media_stream_set_srtp_send_key_b64(&(margaux->ms.sessions), MS_AES_128_SHA1_32, "6jCLmtRkVW9E/BUuJtYj/R2z6+4iEe06/DWohQ9F") == 0);
+				BC_ASSERT_TRUE(ms_media_stream_sessions_set_srtp_send_key_b64(&(margaux->ms.sessions), suite, recv_key) == 0);
 
-			CU_ASSERT_TRUE(media_stream_set_srtp_recv_key_b64(&(margaux->ms.sessions), MS_AES_128_SHA1_32, "d0RmdmcmVCspeEc3QGZiNWpVLFJhQX1cfHAwJSoj") ==0);
+			BC_ASSERT_TRUE(ms_media_stream_sessions_set_srtp_recv_key_b64(&(margaux->ms.sessions), suite, send_key) ==0);
 			if (set_both_send_recv_key)
-				CU_ASSERT_TRUE(media_stream_set_srtp_recv_key_b64(&(marielle->ms.sessions), MS_AES_128_SHA1_32, "6jCLmtRkVW9E/BUuJtYj/R2z6+4iEe06/DWohQ9F") ==0);
+				BC_ASSERT_TRUE(ms_media_stream_sessions_set_srtp_recv_key_b64(&(marielle->ms.sessions), suite, recv_key) ==0);
 
 		} else {
-			CU_ASSERT_TRUE(media_stream_set_srtp_recv_key_b64(&(margaux->ms.sessions), MS_AES_128_SHA1_32, "d0RmdmcmVCspeEc3QGZiNWpVLFJhQX1cfHAwJSoj") ==0);
+			BC_ASSERT_TRUE(ms_media_stream_sessions_set_srtp_recv_key_b64(&(margaux->ms.sessions), suite, send_key) ==0);
 			if (set_both_send_recv_key)
-				CU_ASSERT_TRUE(media_stream_set_srtp_recv_key_b64(&(marielle->ms.sessions), MS_AES_128_SHA1_32, "6jCLmtRkVW9E/BUuJtYj/R2z6+4iEe06/DWohQ9F") ==0);
+				BC_ASSERT_TRUE(ms_media_stream_sessions_set_srtp_recv_key_b64(&(marielle->ms.sessions), suite, recv_key) ==0);
 
-			CU_ASSERT_TRUE(media_stream_set_srtp_send_key_b64(&(marielle->ms.sessions), MS_AES_128_SHA1_32, "d0RmdmcmVCspeEc3QGZiNWpVLFJhQX1cfHAwJSoj") == 0);
+			BC_ASSERT_TRUE(ms_media_stream_sessions_set_srtp_send_key_b64(&(marielle->ms.sessions), suite, send_key) == 0);
 			if (set_both_send_recv_key)
-				CU_ASSERT_TRUE(media_stream_set_srtp_send_key_b64(&(margaux->ms.sessions), MS_AES_128_SHA1_32, "6jCLmtRkVW9E/BUuJtYj/R2z6+4iEe06/DWohQ9F") == 0);
+				BC_ASSERT_TRUE(ms_media_stream_sessions_set_srtp_send_key_b64(&(margaux->ms.sessions), suite, recv_key) == 0);
 
 		}
 
 		if (set_both_send_recv_key) {
 			wait_for_until(&marielle->ms,&margaux->ms,&dummy,1,1000);
-			CU_ASSERT_TRUE(media_stream_secured((MediaStream*)marielle));
-			CU_ASSERT_TRUE(media_stream_secured((MediaStream*)margaux));
+			BC_ASSERT_TRUE(media_stream_secured((MediaStream*)marielle));
+			BC_ASSERT_TRUE(media_stream_secured((MediaStream*)margaux));
 		} else {
 			/*so far, not possible to know audio stream direction*/
-			CU_ASSERT_FALSE(media_stream_secured((MediaStream*)marielle));
-			CU_ASSERT_FALSE(media_stream_secured((MediaStream*)margaux));
+			BC_ASSERT_FALSE(media_stream_secured((MediaStream*)marielle));
+			BC_ASSERT_FALSE(media_stream_secured((MediaStream*)margaux));
 		}
 
 		ms_filter_add_notify_callback(marielle->soundread, notify_cb, &marielle_stats,TRUE);
 		if (change_send_key_in_the_middle) {
 			wait_for_until(&marielle->ms,&margaux->ms,&dummy,1,2000);
-			CU_ASSERT_TRUE(media_stream_set_srtp_send_key_b64(&(marielle->ms.sessions), MS_AES_128_SHA1_32, "eCYF4nYyCvmCpFWjUeDaxI2GWp2BzCRlIPfg52Te") == 0);
-			CU_ASSERT_TRUE(media_stream_set_srtp_recv_key_b64(&(margaux->ms.sessions), MS_AES_128_SHA1_32, "eCYF4nYyCvmCpFWjUeDaxI2GWp2BzCRlIPfg52Te") ==0);
+			BC_ASSERT_TRUE(ms_media_stream_sessions_set_srtp_send_key_b64(&(marielle->ms.sessions), suite, send_key_2) == 0);
+			BC_ASSERT_TRUE(ms_media_stream_sessions_set_srtp_recv_key_b64(&(margaux->ms.sessions), suite, send_key_2) ==0);
 		}
-		CU_ASSERT_TRUE(wait_for_until(&marielle->ms,&margaux->ms,&marielle_stats.number_of_EndOfFile,1,12000));
+		BC_ASSERT_TRUE(wait_for_until(&marielle->ms,&margaux->ms,&marielle_stats.number_of_EndOfFile,1,12000));
 
 		/*make sure packets can cross from sender to receiver*/
 		wait_for_until(&marielle->ms,&margaux->ms,&dummy,1,500);
@@ -310,31 +383,32 @@ static void encrypted_audio_stream_base( bool_t change_ssrc,
 		/* No packet loss is assumed */
 		if (change_send_key_in_the_middle) {
 			/*we can accept one or 2 error in such case*/
-			CU_ASSERT_TRUE((marielle_stats.rtp.packet_sent-margaux_stats.rtp.packet_recv-number_of_dropped_packets)<3);
+			BC_ASSERT_TRUE((marielle_stats.rtp.packet_sent-margaux_stats.rtp.packet_recv-number_of_dropped_packets)<3);
 		} else
-			CU_ASSERT_EQUAL(marielle_stats.rtp.packet_sent,margaux_stats.rtp.packet_recv+number_of_dropped_packets);
+			BC_ASSERT_EQUAL(marielle_stats.rtp.packet_sent,margaux_stats.rtp.packet_recv+number_of_dropped_packets, unsigned long long, "%llu");
 
 		if (change_ssrc) {
 			audio_stream_stop(marielle);
-			marielle = audio_stream_new (MARIELLE_RTP_PORT, MARIELLE_RTCP_PORT,FALSE);
-			CU_ASSERT_EQUAL(audio_stream_start_full(marielle
-							, profile
-							, MARGAUX_IP
-							, MARGAUX_RTP_PORT
-							, MARGAUX_IP
-							, MARGAUX_RTCP_PORT
-							, 0
-							, 50
-							, hello_file
-							, NULL
-							, NULL
-							, NULL
-							, 0),0);
-			CU_ASSERT_FATAL(media_stream_set_srtp_send_key_b64(&(marielle->ms.sessions), MS_AES_128_SHA1_32, "d0RmdmcmVCspeEc3QGZiNWpVLFJhQX1cfHAwJSoj") == 0);
+			marielle = audio_stream_new (_factory, MARIELLE_RTP_PORT, MARIELLE_RTCP_PORT,FALSE);
+			BC_ASSERT_EQUAL(audio_stream_start_full(marielle
+													, profile
+													, MARGAUX_IP
+													, MARGAUX_RTP_PORT
+													, MARGAUX_IP
+													, MARGAUX_RTCP_PORT
+													, 0
+													, 50
+													, hello_file
+													, NULL
+													, NULL
+													, NULL
+													, 0)
+			,0, int, "%d");
+			BC_ASSERT(ms_media_stream_sessions_set_srtp_send_key_b64(&(marielle->ms.sessions), suite, send_key) == 0);
 
 			ms_filter_add_notify_callback(marielle->soundread, notify_cb, &marielle_stats,TRUE);
 
-			CU_ASSERT_TRUE(wait_for_until(&marielle->ms,&margaux->ms,&marielle_stats.number_of_EndOfFile,2,12000));
+			BC_ASSERT_TRUE(wait_for_until(&marielle->ms,&margaux->ms,&marielle_stats.number_of_EndOfFile,2,12000));
 
 			/*make sure packets can cross from sender to receiver*/
 			wait_for_until(&marielle->ms,&margaux->ms,&dummy,1,500);
@@ -343,13 +417,13 @@ static void encrypted_audio_stream_base( bool_t change_ssrc,
 			audio_stream_get_local_rtp_stats(margaux,&margaux_stats.rtp);
 
 			/* No packet loss is assumed */
-			CU_ASSERT_EQUAL(marielle_stats.rtp.sent*2,margaux_stats.rtp.recv);
+			BC_ASSERT_EQUAL(marielle_stats.rtp.sent*2,margaux_stats.rtp.recv, unsigned long long, "%llu");
 
 		}
 
 		unlink(recorded_file);
-		ms_free(recorded_file);
-		ms_free(hello_file);
+		free(recorded_file);
+		free(hello_file);
 	} else {
 		ms_warning("srtp not available, skiping...");
 	}
@@ -359,40 +433,42 @@ static void encrypted_audio_stream_base( bool_t change_ssrc,
 }
 
 static void encrypted_audio_stream(void) {
-	encrypted_audio_stream_base(FALSE, FALSE, FALSE, TRUE,FALSE);
+	encrypted_audio_stream_base(FALSE, FALSE, FALSE, TRUE,FALSE,MS_AES_128_SHA1_32);
+	encrypted_audio_stream_base(FALSE, FALSE, FALSE, TRUE,FALSE,MS_AES_256_SHA1_80);
 }
 
 static void encrypted_audio_stream_with_2_srtp_stream(void) {
-	encrypted_audio_stream_base(FALSE, FALSE, TRUE, TRUE,FALSE);
+	encrypted_audio_stream_base(FALSE, FALSE, TRUE, TRUE,FALSE,MS_AES_128_SHA1_32);
+	encrypted_audio_stream_base(FALSE, FALSE, TRUE, TRUE,FALSE,MS_AES_256_SHA1_80);
 }
 
 static void encrypted_audio_stream_with_2_srtp_stream_recv_first(void) {
-	encrypted_audio_stream_base(FALSE, FALSE, TRUE, FALSE,FALSE);
+	encrypted_audio_stream_base(FALSE, FALSE, TRUE, FALSE,FALSE,MS_AES_128_SHA1_32);
 }
 
 static void encrypted_audio_stream_with_key_change(void) {
-	encrypted_audio_stream_base(FALSE, TRUE, FALSE, TRUE,FALSE);
+	encrypted_audio_stream_base(FALSE, TRUE, FALSE, TRUE,FALSE,MS_AES_128_SHA1_32);
 }
 
 static void encrypted_audio_stream_with_ssrc_change(void) {
-	encrypted_audio_stream_base(TRUE, FALSE, FALSE, TRUE,FALSE);
+	encrypted_audio_stream_base(TRUE, FALSE, FALSE, TRUE,FALSE,MS_AES_128_SHA1_32);
 }
 static void encrypted_audio_stream_encryption_mandatory(void) {
-	encrypted_audio_stream_base(FALSE, FALSE, TRUE, TRUE,TRUE);
+	encrypted_audio_stream_base(FALSE, FALSE, TRUE, TRUE,TRUE,MS_AES_128_SHA1_32);
 }
 
 static void encrypted_audio_stream_with_key_change_encryption_mandatory(void) {
-	encrypted_audio_stream_base(FALSE, TRUE, FALSE, TRUE,TRUE);
+	encrypted_audio_stream_base(FALSE, TRUE, FALSE, TRUE,TRUE,MS_AES_128_SHA1_32);
 }
 
 static void codec_change_for_audio_stream(void) {
-	AudioStream *marielle = audio_stream_new2(MARIELLE_IP, MARIELLE_RTP_PORT, MARIELLE_RTCP_PORT);
+	AudioStream *marielle = audio_stream_new2(_factory, MARIELLE_IP, MARIELLE_RTP_PORT, MARIELLE_RTCP_PORT);
 	stats_t marielle_stats;
-	AudioStream *margaux = audio_stream_new2(MARGAUX_IP, MARGAUX_RTP_PORT, MARGAUX_RTCP_PORT);
+	AudioStream *margaux = audio_stream_new2(_factory, MARGAUX_IP, MARGAUX_RTP_PORT, MARGAUX_RTCP_PORT);
 	stats_t margaux_stats;
 	RtpProfile *profile = rtp_profile_new("default profile");
-	char* hello_file = ms_strdup_printf("%s/%s", bc_tester_read_dir_prefix, HELLO_8K_1S_FILE);
-	char* recorded_file = ms_strdup_printf("%s/%s", bc_tester_writable_dir_prefix, RECORDED_8K_1S_FILE);
+	char* hello_file = bc_tester_res(HELLO_8K_1S_FILE);
+	char* recorded_file = bc_tester_file(RECORDED_8K_1S_FILE);
 	uint64_t marielle_rtp_sent = 0;
 	int dummy=0;
 
@@ -402,15 +478,15 @@ static void codec_change_for_audio_stream(void) {
 	rtp_profile_set_payload(profile, 0, &payload_type_pcmu8000);
 	rtp_profile_set_payload(profile, 8, &payload_type_pcma8000);
 
-	CU_ASSERT_EQUAL(audio_stream_start_full(margaux, profile, MARIELLE_IP, MARIELLE_RTP_PORT, MARIELLE_IP, MARIELLE_RTCP_PORT,
-		0, 50, NULL, recorded_file, NULL, NULL, 0), 0);
+	BC_ASSERT_EQUAL(audio_stream_start_full(margaux, profile, MARIELLE_IP, MARIELLE_RTP_PORT, MARIELLE_IP, MARIELLE_RTCP_PORT,
+		0, 50, NULL, recorded_file, NULL, NULL, 0), 0, int, "%d");
 
-	CU_ASSERT_EQUAL(audio_stream_start_full(marielle, profile, MARGAUX_IP, MARGAUX_RTP_PORT, MARGAUX_IP, MARGAUX_RTCP_PORT,
-		0, 50, hello_file, NULL, NULL, NULL, 0), 0);
+	BC_ASSERT_EQUAL(audio_stream_start_full(marielle, profile, MARGAUX_IP, MARGAUX_RTP_PORT, MARGAUX_IP, MARGAUX_RTCP_PORT,
+		0, 50, hello_file, NULL, NULL, NULL, 0), 0, int, "%d");
 
 	ms_filter_add_notify_callback(marielle->soundread, notify_cb, &marielle_stats, TRUE);
 
-	CU_ASSERT_TRUE(wait_for_until(&marielle->ms, &margaux->ms, &marielle_stats.number_of_EndOfFile, 1, 12000));
+	BC_ASSERT_TRUE(wait_for_until(&marielle->ms, &margaux->ms, &marielle_stats.number_of_EndOfFile, 1, 12000));
 
 	/*make sure packets can cross from sender to receiver*/
 	wait_for_until(&marielle->ms, &margaux->ms, &dummy, 1, 500);
@@ -419,19 +495,19 @@ static void codec_change_for_audio_stream(void) {
 	audio_stream_get_local_rtp_stats(margaux, &margaux_stats.rtp);
 
 	/* No packet loss is assumed */
-	CU_ASSERT_EQUAL(marielle_stats.rtp.sent, margaux_stats.rtp.recv);
+	BC_ASSERT_EQUAL(marielle_stats.rtp.sent, margaux_stats.rtp.recv, unsigned long long, "%llu");
 	marielle_rtp_sent = marielle_stats.rtp.sent;
 
 	audio_stream_stop(marielle);
 	reset_stats(&marielle_stats);
 	reset_stats(&margaux_stats);
-	marielle = audio_stream_new2(MARIELLE_IP, MARIELLE_RTP_PORT, MARIELLE_RTCP_PORT);
-	CU_ASSERT_EQUAL(audio_stream_start_full(marielle, profile, MARGAUX_IP, MARGAUX_RTP_PORT, MARGAUX_IP, MARGAUX_RTCP_PORT,
-		8, 50, hello_file, NULL, NULL, NULL, 0), 0);
+	marielle = audio_stream_new2(_factory, MARIELLE_IP, MARIELLE_RTP_PORT, MARIELLE_RTCP_PORT);
+	BC_ASSERT_EQUAL(audio_stream_start_full(marielle, profile, MARGAUX_IP, MARGAUX_RTP_PORT, MARGAUX_IP, MARGAUX_RTCP_PORT,
+		8, 50, hello_file, NULL, NULL, NULL, 0), 0, int, "%d");
 
 	ms_filter_add_notify_callback(marielle->soundread, notify_cb, &marielle_stats, TRUE);
 
-	CU_ASSERT_TRUE(wait_for_until(&marielle->ms, &margaux->ms, &marielle_stats.number_of_EndOfFile, 1, 12000));
+	BC_ASSERT_TRUE(wait_for_until(&marielle->ms, &margaux->ms, &marielle_stats.number_of_EndOfFile, 1, 12000));
 
 	/*make sure packets can cross from sender to receiver*/
 	wait_for_until(&marielle->ms, &margaux->ms, &dummy, 1, 500);
@@ -440,26 +516,26 @@ static void codec_change_for_audio_stream(void) {
 	audio_stream_get_local_rtp_stats(margaux, &margaux_stats.rtp);
 
 	/* No packet loss is assumed */
-	CU_ASSERT_EQUAL(marielle_stats.rtp.sent + marielle_rtp_sent, margaux_stats.rtp.recv);
-	CU_ASSERT_EQUAL(strcasecmp(margaux->ms.decoder->desc->enc_fmt, "pcma"), 0);
+	BC_ASSERT_EQUAL(marielle_stats.rtp.sent + marielle_rtp_sent, margaux_stats.rtp.recv, unsigned long long, "%llu");
+	BC_ASSERT_EQUAL(strcasecmp(margaux->ms.decoder->desc->enc_fmt, "pcma"), 0, int, "%d");
 	audio_stream_stop(marielle);
 	audio_stream_stop(margaux);
 
 	unlink(recorded_file);
-	ms_free(recorded_file);
-	ms_free(hello_file);
+	free(recorded_file);
+	free(hello_file);
 	rtp_profile_destroy(profile);
 }
 
 static void tmmbr_feedback_for_audio_stream(void) {
-	AudioStream *marielle = audio_stream_new2(MARIELLE_IP, MARIELLE_RTP_PORT, MARIELLE_RTCP_PORT);
+	AudioStream *marielle = audio_stream_new2(_factory, MARIELLE_IP, MARIELLE_RTP_PORT, MARIELLE_RTCP_PORT);
 	stats_t marielle_stats;
-	AudioStream *margaux = audio_stream_new2(MARGAUX_IP, MARGAUX_RTP_PORT, MARGAUX_RTCP_PORT);
+	AudioStream *margaux = audio_stream_new2(_factory, MARGAUX_IP, MARGAUX_RTP_PORT, MARGAUX_RTCP_PORT);
 	stats_t margaux_stats;
 	RtpProfile *profile = rtp_profile_new("default profile");
 	RtpSession *marielle_session;
 	RtpSession *margaux_session;
-	char* hello_file = ms_strdup_printf("%s/%s", bc_tester_read_dir_prefix, HELLO_8K_1S_FILE);
+	char* hello_file = bc_tester_res(HELLO_8K_1S_FILE);
 	int dummy=0;
 
 	reset_stats(&marielle_stats);
@@ -478,11 +554,11 @@ static void tmmbr_feedback_for_audio_stream(void) {
 	margaux_stats.q = ortp_ev_queue_new();
 	rtp_session_register_event_queue(margaux->ms.sessions.rtp_session, margaux_stats.q);
 
-	CU_ASSERT_EQUAL(audio_stream_start_full(margaux, profile, MARIELLE_IP, MARIELLE_RTP_PORT, MARIELLE_IP, MARIELLE_RTCP_PORT,
-		0, 50, hello_file, NULL, NULL, NULL, 0), 0);
+	BC_ASSERT_EQUAL(audio_stream_start_full(margaux, profile, MARIELLE_IP, MARIELLE_RTP_PORT, MARIELLE_IP, MARIELLE_RTCP_PORT,
+		0, 50, hello_file, NULL, NULL, NULL, 0), 0, int, "%d");
 
-	CU_ASSERT_EQUAL(audio_stream_start_full(marielle, profile, MARGAUX_IP, MARGAUX_RTP_PORT, MARGAUX_IP, MARGAUX_RTCP_PORT,
-		0, 50, hello_file, NULL, NULL, NULL, 0), 0);
+	BC_ASSERT_EQUAL(audio_stream_start_full(marielle, profile, MARGAUX_IP, MARGAUX_RTP_PORT, MARGAUX_IP, MARGAUX_RTCP_PORT,
+		0, 50, hello_file, NULL, NULL, NULL, 0), 0, int, "%d");
 
 	ms_filter_add_notify_callback(margaux->soundread, notify_cb, &margaux_stats, TRUE);
 	ms_filter_add_notify_callback(marielle->soundread, notify_cb, &marielle_stats, TRUE);
@@ -493,13 +569,13 @@ static void tmmbr_feedback_for_audio_stream(void) {
 	rtp_session_send_rtcp_fb_tmmbr(margaux_session, 100000);
 	rtp_session_send_rtcp_fb_tmmbr(marielle_session, 200000);
 
-	CU_ASSERT_TRUE(wait_for_until(&margaux->ms, &marielle->ms, &margaux_stats.number_of_EndOfFile, 1, 12000));
-	CU_ASSERT_TRUE(wait_for_until(&marielle->ms, &margaux->ms, &marielle_stats.number_of_EndOfFile, 1, 12000));
+	BC_ASSERT_TRUE(wait_for_until(&margaux->ms, &marielle->ms, &margaux_stats.number_of_EndOfFile, 1, 12000));
+	BC_ASSERT_TRUE(wait_for_until(&marielle->ms, &margaux->ms, &marielle_stats.number_of_EndOfFile, 1, 12000));
 
-	CU_ASSERT_TRUE(wait_for_until_with_parse_events(&marielle->ms, &margaux->ms, &marielle_stats.number_of_TMMBR, 1, 100, event_queue_cb, &marielle_stats, event_queue_cb, &margaux_stats));
-	CU_ASSERT_TRUE(wait_for_until_with_parse_events(&margaux->ms, &marielle->ms, &margaux_stats.number_of_TMMBR, 1, 100, event_queue_cb, &margaux_stats, event_queue_cb, &marielle_stats));
-	CU_ASSERT_EQUAL(marielle_stats.number_of_TMMBR, 1);
-	CU_ASSERT_EQUAL(margaux_stats.number_of_TMMBR, 1);
+	BC_ASSERT_TRUE(wait_for_until_with_parse_events(&marielle->ms, &margaux->ms, &marielle_stats.number_of_TMMBR, 1, 100, event_queue_cb, &marielle_stats, event_queue_cb, &margaux_stats));
+	BC_ASSERT_TRUE(wait_for_until_with_parse_events(&margaux->ms, &marielle->ms, &margaux_stats.number_of_TMMBR, 1, 100, event_queue_cb, &margaux_stats, event_queue_cb, &marielle_stats));
+	BC_ASSERT_EQUAL(marielle_stats.number_of_TMMBR, 1, int, "%d");
+	BC_ASSERT_EQUAL(margaux_stats.number_of_TMMBR, 1, int, "%d");
 
 	/*make sure packets can cross from sender to receiver*/
 	wait_for_until(&marielle->ms, &margaux->ms, &dummy, 1, 500);
@@ -507,7 +583,7 @@ static void tmmbr_feedback_for_audio_stream(void) {
 	audio_stream_stop(marielle);
 	audio_stream_stop(margaux);
 
-	ms_free(hello_file);
+	free(hello_file);
 	ortp_ev_queue_destroy(marielle_stats.q);
 	ortp_ev_queue_destroy(margaux_stats.q);
 	rtp_profile_destroy(profile);
@@ -544,7 +620,7 @@ static void audio_stream_dtmf(int codec_payload, int initial_bitrate,int target_
 	marielle_send_bw=media_stream_get_up_bw(&marielle->stream->ms);
 	recv_send_bw_ratio=params.max_bandwidth/marielle_send_bw;
 	ms_message("marielle sent bw= [%f] , target was [%f] recv/send [%f]",marielle_send_bw,params.max_bandwidth,recv_send_bw_ratio);
-	CU_ASSERT_TRUE(recv_send_bw_ratio>0.9);
+	BC_ASSERT_TRUE(recv_send_bw_ratio>0.9);
 
 	stream_manager_delete(marielle);
 	stream_manager_delete(margaux);
@@ -552,6 +628,40 @@ static void audio_stream_dtmf(int codec_payload, int initial_bitrate,int target_
 }
 #endif
 
+static void symetric_rtp_with_wrong_addr(void)  {
+	basic_audio_stream_base_2(  MARIELLE_IP
+							  , "10.10.10.10" /*dummy ip*/
+							  , MARIELLE_RTP_PORT
+							  , MARGAUX_RTP_PORT
+							  , MARIELLE_RTCP_PORT
+							  , MARGAUX_RTCP_PORT
+
+							  , MARGAUX_IP
+							  , MARIELLE_IP
+							  , MARGAUX_RTP_PORT
+							  , MARIELLE_RTP_PORT
+							  , MARGAUX_RTCP_PORT
+							  , MARIELLE_RTCP_PORT
+							  ,5);
+}
+
+static void symetric_rtp_with_wrong_rtcp_port(void)  {
+	basic_audio_stream_base_2(  MARIELLE_IP
+							  , MARGAUX_IP
+							  , MARIELLE_RTP_PORT
+							  , MARGAUX_RTP_PORT
+							  , MARIELLE_RTCP_PORT
+							  , MARGAUX_RTCP_PORT
+
+							  , MARGAUX_IP
+							  , MARIELLE_IP
+							  , MARGAUX_RTP_PORT
+							  , MARIELLE_RTP_PORT
+							  , MARGAUX_RTCP_PORT
+							  , MARIELLE_RTCP_PORT +10 /*dummy port*/
+							  ,5);
+}
+
 
 static test_t tests[] = {
 	{ "Basic audio stream", basic_audio_stream },
@@ -564,13 +674,17 @@ static test_t tests[] = {
 	{ "Encrypted audio stream, encryption mandatory", encrypted_audio_stream_encryption_mandatory },
 	{ "Encrypted audio stream with key change + encryption mandatory", encrypted_audio_stream_with_key_change_encryption_mandatory},
 	{ "Codec change for audio stream", codec_change_for_audio_stream },
-	{ "TMMBR feedback for audio stream", tmmbr_feedback_for_audio_stream }
+	{ "TMMBR feedback for audio stream", tmmbr_feedback_for_audio_stream },
+	{ "Symetric rtp with wrong address", symetric_rtp_with_wrong_addr },
+	{ "Symetric rtp with wrong rtcp port", symetric_rtp_with_wrong_rtcp_port },
 };
 
 test_suite_t audio_stream_test_suite = {
 	"AudioStream",
-	tester_init,
-	tester_cleanup,
+	tester_before_all,
+	tester_after_all,
+	NULL,
+	NULL,
 	sizeof(tests) / sizeof(tests[0]),
 	tests
 };
