@@ -29,7 +29,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include <ctype.h>
 #include <signal.h>
 #include <sys/types.h>
-#ifndef WIN32
+#ifndef _WIN32
 #include <unistd.h>
 #include <poll.h>
 #else
@@ -39,28 +39,30 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include <stdlib.h>
 #include <string.h>
 
-#ifdef __APPLE__
+#ifdef TARGET_OS_MAC
 #include <CoreFoundation/CFRunLoop.h>
 #endif
-#if  TARGET_OS_IPHONE || defined (ANDROID)
+
 #if TARGET_OS_IPHONE
 #import <UIKit/UIKit.h>
 #include <AudioToolbox/AudioToolbox.h>
 #endif
+
+#if  TARGET_OS_IPHONE || defined (ANDROID)
 extern void ms_set_video_stream(VideoStream* video);
-#ifdef HAVE_X264
+#if TARGET_OS_IPHONE || defined(HAVE_X264)
 extern void libmsx264_init();
 #endif
-#ifdef HAVE_OPENH264
+#if TARGET_OS_IPHONE || defined(HAVE_OPENH264)
 extern void libmsopenh264_init();
 #endif
-#ifdef HAVE_SILK
+#if TARGET_OS_IPHONE || defined(HAVE_SILK)
 extern void libmssilk_init();
 #endif
-#ifdef HAVE_WEBRTC
+#if TARGET_OS_IPHONE || defined(HAVE_WEBRTC)
 extern void libmswebrtc_init();
 #endif
-#endif
+#endif // TARGET_OS_IPHONE || defined (ANDROID)
 
 #ifdef ANDROID
 #include <android/log.h>
@@ -148,12 +150,13 @@ typedef struct _MediastreamDatas {
 	int ice_remote_candidates_nb;
 	char * video_display_filter;
 	FILE * logfile;
+	bool_t enable_speaker;
 } MediastreamDatas;
 
 
 // MAIN METHODS
 /* init default arguments */
-MediastreamDatas* init_default_args();
+MediastreamDatas* init_default_args(void);
 /* parse args */
 bool_t parse_args(int argc, char** argv, MediastreamDatas* out);
 /* setup streams */
@@ -165,8 +168,8 @@ void clear_mediastreams(MediastreamDatas* args);
 
 // HELPER METHODS
 void stop_handler(int signum);
-static bool_t parse_addr(const char *addr, char *ip, int len, int *port);
-static bool_t parse_ice_addr(char* addr, char* type, int type_len, char* ip, int ip_len, int* port);
+static bool_t parse_addr(const char *addr, char *ip, size_t len, int *port);
+static bool_t parse_ice_addr(char* addr, char* type, size_t type_len, char* ip, size_t ip_len, int* port);
 static void display_items(void *user_data, uint32_t csrc, rtcp_sdes_type_t t, const char *content, uint8_t content_len);
 static void parse_rtcp(mblk_t *m);
 static void parse_events(RtpSession *session, OrtpEvQueue *q);
@@ -222,6 +225,9 @@ const char *usage="mediastream --local <port>\n"
 								"[ --width <pixels> ]\n"
 								"[ --zoom zoom factor ]\n"
 								"[ --zrtp <secrets file> (enable zrtp) ]\n"
+								#if TARGET_OS_IPHONE
+								"[ --speaker route audio to speaker ]\n"
+								#endif
 								;
 
 #if TARGET_OS_IPHONE
@@ -249,7 +255,7 @@ int main(int argc, char * argv[]) {
 static int _main(int argc, char * argv[])
 #endif
 
-#if !__APPLE__ && !ANDROID
+#if !TARGET_OS_MAC && !ANDROID
 int main(int argc, char * argv[])
 #endif
 
@@ -279,7 +285,7 @@ int main(int argc, char * argv[])
 #endif
 
 
-MediastreamDatas* init_default_args() {
+MediastreamDatas* init_default_args(void) {
 	MediastreamDatas* args = (MediastreamDatas*)ms_malloc0(sizeof(MediastreamDatas));
 	args->localport=0;
 	args->remoteport=0;
@@ -450,10 +456,10 @@ bool_t parse_args(int argc, char** argv, MediastreamDatas* out) {
 			out->use_rc=1;
 		}else if (strcmp(argv[i],"--ng-threshold")==0){
 			i++;
-			out->ng_threshold=atof(argv[i]);
+			out->ng_threshold=(float)atof(argv[i]);
 		}else if (strcmp(argv[i],"--ng-floorgain")==0){
 			i++;
-			out->ng_floorgain=atof(argv[i]);
+			out->ng_floorgain=(float)atof(argv[i]);
 		}else if (strcmp(argv[i],"--two-windows")==0){
 			out->two_windows=TRUE;
 		}else if (strcmp(argv[i],"--infile")==0){
@@ -469,19 +475,19 @@ bool_t parse_args(int argc, char** argv, MediastreamDatas* out) {
 			out->el=TRUE;
 		}else if (strcmp(argv[i],"--el-speed")==0){
 			i++;
-			out->el_speed=atof(argv[i]);
+			out->el_speed=(float)atof(argv[i]);
 		}else if (strcmp(argv[i],"--el-thres")==0){
 			i++;
-			out->el_thres=atof(argv[i]);
+			out->el_thres=(float)atof(argv[i]);
 		}else if (strcmp(argv[i],"--el-force")==0){
 			i++;
-			out->el_force=atof(argv[i]);
+			out->el_force=(float)atof(argv[i]);
 		}else if (strcmp(argv[i],"--el-sustain")==0){
 			i++;
 			out->el_sustain=atoi(argv[i]);
 		}else if (strcmp(argv[i],"--el-transmit-thres")==0){
 			i++;
-			out->el_transmit_thres=atof(argv[i]);
+			out->el_transmit_thres=(float)atof(argv[i]);
 		} else if (strcmp(argv[i],"--zrtp")==0){
 			out->zrtp_secrets=argv[++i];
 		} else if (strcmp(argv[i],"--verbose")==0){
@@ -510,7 +516,7 @@ bool_t parse_args(int argc, char** argv, MediastreamDatas* out) {
 		} else if (strcmp(argv[i],"--netsim-bandwidth")==0){
 			i++;
 			if (i<argc){
-				out->netsim.max_bandwidth=atoi(argv[i]);
+				out->netsim.max_bandwidth=(float)atoi(argv[i]);
 				out->netsim.enabled=TRUE;
 			}else{
 				ms_error("Missing argument for --netsim-bandwidth");
@@ -519,7 +525,7 @@ bool_t parse_args(int argc, char** argv, MediastreamDatas* out) {
 		}else if (strcmp(argv[i],"--netsim-lossrate")==0){
 			i++;
 			if (i<argc){
-				out->netsim.loss_rate=atoi(argv[i]);
+				out->netsim.loss_rate=(float)atoi(argv[i]);
 				if (out->netsim.loss_rate < 0 || out->netsim.loss_rate>100) {
 					ms_error("Loss rate must be between 0 and 100.");
 					return FALSE;
@@ -622,6 +628,8 @@ bool_t parse_args(int argc, char** argv, MediastreamDatas* out) {
 			out->logfile = fopen(argv[i], "a+");
 		} else if (strcmp(argv[i], "--freeze-on-error") == 0) {
 			out->freeze_on_error=TRUE;
+		} else if (strcmp(argv[i], "--speaker") == 0) {
+			out->enable_speaker=TRUE;
 		} else {
 			ms_error("Unknown option '%s'\n", argv[i]);
 			return FALSE;
@@ -670,29 +678,30 @@ void setup_media_streams(MediastreamDatas* args) {
 #ifdef VIDEO_ENABLED
 	MSWebCam *cam=NULL;
 #endif
+	MSFactory *factory;
 	ortp_init();
 	if (args->logfile)
 		ortp_set_log_file(args->logfile);
 
 	if (args->is_verbose) {
-		ortp_set_log_level_mask(ORTP_DEBUG|ORTP_MESSAGE|ORTP_WARNING|ORTP_ERROR|ORTP_FATAL);
+		ortp_set_log_level_mask(ORTP_LOG_DOMAIN, ORTP_DEBUG|ORTP_MESSAGE|ORTP_WARNING|ORTP_ERROR|ORTP_FATAL);
 	} else {
-		ortp_set_log_level_mask(ORTP_MESSAGE|ORTP_WARNING|ORTP_ERROR|ORTP_FATAL);
+		ortp_set_log_level_mask(ORTP_LOG_DOMAIN, ORTP_MESSAGE|ORTP_WARNING|ORTP_ERROR|ORTP_FATAL);
 	}
 
-	ms_init();
+	factory = ms_factory_new_with_voip();
 
 #if TARGET_OS_IPHONE || defined(ANDROID)
-#if defined (HAVE_X264) && defined (VIDEO_ENABLED)
+#if TARGET_OS_IPHONE || (defined(HAVE_X264) && defined(VIDEO_ENABLED))
 	libmsx264_init(); /*no plugin on IOS/Android */
 #endif
-#if defined (HAVE_OPENH264) && defined (VIDEO_ENABLED)
+#if TARGET_OS_IPHONE || (defined (HAVE_OPENH264) && defined (VIDEO_ENABLED))
 	libmsopenh264_init(); /*no plugin on IOS/Android */
 #endif
-#if defined (HAVE_SILK)
+#if TARGET_OS_IPHONE || defined (HAVE_SILK)
 	libmssilk_init(); /*no plugin on IOS/Android */
 #endif
-#if defined (HAVE_WEBRTC)
+#if TARGET_OS_IPHONE || defined (HAVE_WEBRTC)
 	libmswebrtc_init();
 #endif
 
@@ -706,7 +715,7 @@ void setup_media_streams(MediastreamDatas* args) {
 	rtp_profile_set_payload(&av_profile,115,&payload_type_lpc1015);
 #ifdef VIDEO_ENABLED
 	cam=ms_web_cam_new(ms_mire_webcam_desc_get());
-	if (cam) ms_web_cam_manager_add_cam(ms_web_cam_manager_get(), cam);
+	if (cam) ms_web_cam_manager_add_cam(ms_factory_get_web_cam_manager(factory), cam);
 	cam=NULL;
 
 	rtp_profile_set_payload(&av_profile,26,&payload_type_jpeg);
@@ -722,9 +731,10 @@ void setup_media_streams(MediastreamDatas* args) {
 	args->profile=rtp_profile_clone_full(&av_profile);
 	args->q=ortp_ev_queue_new();
 
-	if (args->mtu) ms_set_mtu(args->mtu);
-	ms_filter_enable_statistics(TRUE);
-	ms_filter_reset_statistics();
+	if (args->mtu) ms_factory_set_mtu(factory, args->mtu);
+	ms_factory_enable_statistics(factory, TRUE);
+	ms_factory_reset_statistics(factory);
+
 	args->ice_session=ice_session_new();
 	ice_session_set_remote_credentials(args->ice_session,"1234","1234567890abcdef123456");
 	// ICE local credentials are assigned when creating the ICE session, but force them here to simplify testing
@@ -778,17 +788,21 @@ void setup_media_streams(MediastreamDatas* args) {
 	}
 
 	if (args->pt->type!=PAYLOAD_VIDEO){
-		MSSndCardManager *manager=ms_snd_card_manager_get();
+		MSSndCardManager *manager=ms_factory_get_snd_card_manager(factory);
 		MSSndCard *capt= args->capture_card==NULL ? ms_snd_card_manager_get_default_capture_card(manager) :
 				get_sound_card(manager,args->capture_card);
 		MSSndCard *play= args->playback_card==NULL ? ms_snd_card_manager_get_default_capture_card(manager) :
 				get_sound_card(manager,args->playback_card);
-		args->audio=audio_stream_new(args->localport,args->localport+1,ms_is_ipv6(args->ip));
+		args->audio=audio_stream_new(factory, args->localport,args->localport+1,ms_is_ipv6(args->ip));
 		audio_stream_enable_automatic_gain_control(args->audio,args->agc);
 		audio_stream_enable_noise_gate(args->audio,args->use_ng);
 		audio_stream_set_echo_canceller_params(args->audio,args->ec_len_ms,args->ec_delay_ms,args->ec_framesize);
 		audio_stream_enable_echo_limiter(args->audio,args->el);
 		audio_stream_enable_adaptive_bitrate_control(args->audio,args->use_rc);
+		if (capt)
+			ms_snd_card_set_preferred_sample_rate(capt,rtp_profile_get_payload(args->profile, args->payload)->clock_rate);
+		if (play)
+			ms_snd_card_set_preferred_sample_rate(play,rtp_profile_get_payload(args->profile, args->payload)->clock_rate);
 		ms_message("Starting audio stream.\n");
 
 		audio_stream_start_full(args->audio,args->profile,args->ip,args->remoteport,args->ip,args->enable_rtcp?args->remoteport+1:-1, args->payload, args->jitter,args->infile,args->outfile,
@@ -804,8 +818,8 @@ void setup_media_streams(MediastreamDatas* args) {
 			int c;
 			for (c=0;c<args->ice_local_candidates_nb;c++){
 				candidate=&args->ice_local_candidates[c];
-				ice_add_local_candidate(args->audio->ms.ice_check_list,candidate->type,candidate->ip,candidate->port,1,NULL);
-				ice_add_local_candidate(args->audio->ms.ice_check_list,candidate->type,candidate->ip,candidate->port+1,2,NULL);
+				ice_add_local_candidate(args->audio->ms.ice_check_list,candidate->type,AF_INET,candidate->ip,candidate->port,1,NULL);
+				ice_add_local_candidate(args->audio->ms.ice_check_list,candidate->type,AF_INET,candidate->ip,candidate->port+1,2,NULL);
 			}
 		}
 		if (args->ice_remote_candidates_nb) {
@@ -816,8 +830,8 @@ void setup_media_streams(MediastreamDatas* args) {
 				candidate=&args->ice_remote_candidates[c];
 				memset(foundation, '\0', sizeof(foundation));
 				snprintf(foundation, sizeof(foundation) - 1, "%u", c + 1);
-				ice_add_remote_candidate(args->audio->ms.ice_check_list,candidate->type,candidate->ip,candidate->port,1,0,foundation,FALSE);
-				ice_add_remote_candidate(args->audio->ms.ice_check_list,candidate->type,candidate->ip,candidate->port+1,2,0,foundation,FALSE);
+				ice_add_remote_candidate(args->audio->ms.ice_check_list,candidate->type,AF_INET,candidate->ip,candidate->port,1,0,foundation,FALSE);
+				ice_add_remote_candidate(args->audio->ms.ice_check_list,candidate->type,AF_INET,candidate->ip,candidate->port+1,2,0,foundation,FALSE);
 			}
 		}
 
@@ -846,13 +860,12 @@ void setup_media_streams(MediastreamDatas* args) {
 				}
 			}
 
-			#ifndef TARGET_OS_IPHONE
 			if (args->zrtp_secrets != NULL) {
 				MSZrtpParams params;
 				params.zid_file=args->zrtp_secrets;
 				audio_stream_enable_zrtp(args->audio,&params);
 			}
-			#endif
+
 
 			args->session=args->audio->ms.sessions.rtp_session;
 		}
@@ -865,18 +878,30 @@ void setup_media_streams(MediastreamDatas* args) {
 					args->srtp_local_master_key,
 					args->srtp_remote_master_key));
 		}
+	#if TARGET_OS_IPHONE
+		if (args->enable_speaker) {
+				ms_message("Setting audio route to spaker");
+				UInt32 audioRouteOverride = kAudioSessionOverrideAudioRoute_Speaker;
+				if (AudioSessionSetProperty(kAudioSessionProperty_OverrideAudioRoute, sizeof(audioRouteOverride),&audioRouteOverride) != kAudioSessionNoError) {
+					ms_error("Cannot set route to speaker");
+				};
+		}
+	#endif
+
+
 	}else{
 #ifdef VIDEO_ENABLED
 		float zoom[] = {
 			args->zoom,
 			args->zoom_cx, args->zoom_cy };
+		MSMediaStreamIO iodef = MS_MEDIA_STREAM_IO_INITIALIZER;
 
 		if (args->eq){
 			ms_fatal("Cannot put an audio equalizer in a video stream !");
 			exit(-1);
 		}
 		ms_message("Starting video stream.\n");
-		args->video=video_stream_new(args->localport, args->localport+1, ms_is_ipv6(args->ip));
+		args->video=video_stream_new(factory, args->localport, args->localport+1, ms_is_ipv6(args->ip));
 		if (args->video_display_filter)
 			video_stream_set_display_filter_name(args->video, args->video_display_filter);
 
@@ -891,20 +916,37 @@ void setup_media_streams(MediastreamDatas* args) {
 		const char*  nowebcam = [[myBundle pathForResource:@"nowebcamCIF"ofType:@"jpg"] cStringUsingEncoding:[NSString defaultCStringEncoding]];
 		ms_static_image_set_default_image(nowebcam);
 		NSUInteger cpucount = [[NSProcessInfo processInfo] processorCount];
-		ms_set_cpu_count(cpucount);
+		ms_factory_set_cpu_count(args->audio->ms.factory, cpucount);
+		//ms_set_cpu_count(cpucount);
 #endif
 		video_stream_set_event_callback(args->video,video_stream_event_cb, args);
 		video_stream_set_freeze_on_error(args->video,args->freeze_on_error);
 		video_stream_enable_adaptive_bitrate_control(args->video,args->use_rc);
 		if (args->camera)
-			cam=ms_web_cam_manager_get_cam(ms_web_cam_manager_get(),args->camera);
+			cam=ms_web_cam_manager_get_cam(ms_factory_get_web_cam_manager(factory),args->camera);
 		if (cam==NULL)
-			cam=ms_web_cam_manager_get_default_cam(ms_web_cam_manager_get());
-		video_stream_start(args->video,args->profile,
+			cam=ms_web_cam_manager_get_default_cam(ms_factory_get_web_cam_manager(factory));
+
+		if (args->infile){
+			iodef.input.type = MSResourceFile;
+			iodef.input.file = args->infile;
+		}else{
+			iodef.input.type = MSResourceCamera;
+			iodef.input.camera = cam;
+		}
+		if (args->outfile){
+			iodef.output.type = MSResourceFile;
+			iodef.output.file = args->outfile;
+		}else{
+			iodef.output.type = MSResourceDefault;
+			iodef.output.resource_arg = NULL;
+		}
+		rtp_session_set_jitter_compensation(args->video->ms.sessions.rtp_session, args->jitter);
+		video_stream_start_from_io(args->video, args->profile,
 					args->ip,args->remoteport,
 					args->ip,args->enable_rtcp?args->remoteport+1:-1,
 					args->payload,
-					args->jitter,cam
+					&iodef
 					);
 		args->session=args->video->ms.sessions.rtp_session;
 
@@ -934,7 +976,7 @@ void setup_media_streams(MediastreamDatas* args) {
 
 
 static void mediastream_tool_iterate(MediastreamDatas* args) {
-#ifndef WIN32
+#ifndef _WIN32
 	struct pollfd pfd;
 	int err;
 
@@ -953,27 +995,34 @@ static void mediastream_tool_iterate(MediastreamDatas* args) {
 			ms_message("\nPlease enter equalizer requests, such as 'eq active 1', 'eq active 0', 'eq 1200 0.1 200'\n");
 
 			if (fgets(commands,sizeof(commands)-1,stdin)!=NULL){
-				int active,freq,freq_width;
-
-				float gain;
+				MSEqualizerGain d = {0};
+				int active;
 				if (sscanf(commands,"eq active %i",&active)==1){
-					audio_stream_enable_equalizer(args->audio,active);
+					audio_stream_enable_equalizer(args->audio, args->audio->eq_loc, active);
 					ms_message("OK\n");
-				}else if (sscanf(commands,"eq %i %f %i",&freq,&gain,&freq_width)==3){
-					audio_stream_equalizer_set_gain(args->audio,freq,gain,freq_width);
+				}else if (sscanf(commands,"eq %f %f %f",&d.frequency,&d.gain,&d.width)==3){
+					audio_stream_equalizer_set_gain(args->audio, args->audio->eq_loc, &d);
 					ms_message("OK\n");
-				}else if (sscanf(commands,"eq %i %f",&freq,&gain)==2){
-					audio_stream_equalizer_set_gain(args->audio,freq,gain,0);
+				}else if (sscanf(commands,"eq %f %f",&d.frequency,&d.gain)==2){
+					audio_stream_equalizer_set_gain(args->audio, args->audio->eq_loc, &d);
 					ms_message("OK\n");
 				}else if (strstr(commands,"dump")){
 					int n=0,i;
 					float *t;
-					ms_filter_call_method(args->audio->equalizer,MS_EQUALIZER_GET_NUM_FREQUENCIES,&n);
-					t=(float*)alloca(sizeof(float)*n);
-					ms_filter_call_method(args->audio->equalizer,MS_EQUALIZER_DUMP_STATE,t);
-					for(i=0;i<n;++i){
-						if (fabs(t[i]-1)>0.01){
-						ms_message("%i:%f:0 ",(i*args->pt->clock_rate)/(2*n),t[i]);
+					MSFilter *equalizer = NULL;
+					if(args->audio->eq_loc == MSEqualizerHP) {
+						equalizer = args->audio->spk_equalizer;
+					} else if(args->audio->eq_loc == MSEqualizerMic) {
+						equalizer = args->audio->mic_equalizer;
+					}
+					if(equalizer) {
+						ms_filter_call_method(equalizer,MS_EQUALIZER_GET_NUM_FREQUENCIES,&n);
+						t=(float*)alloca(sizeof(float)*n);
+						ms_filter_call_method(equalizer,MS_EQUALIZER_DUMP_STATE,t);
+						for(i=0;i<n;++i){
+							if (fabs(t[i]-1)>0.01){
+							ms_message("%i:%f:0 ",(i*args->pt->clock_rate)/(2*n),t[i]);
+							}
 						}
 					}
 					ms_message("\nOK\n");
@@ -1058,7 +1107,7 @@ void clear_mediastreams(MediastreamDatas* args) {
 	if (args->video) {
 		if (args->video->ms.ice_check_list) ice_check_list_destroy(args->video->ms.ice_check_list);
 		video_stream_stop(args->video);
-		ms_filter_log_statistics();
+		ms_factory_log_statistics(args->video->ms.factory);
 	}
 #endif
 	if (args->ice_session) ice_session_destroy(args->ice_session);
@@ -1068,7 +1117,7 @@ void clear_mediastreams(MediastreamDatas* args) {
 	if (args->logfile)
 		fclose(args->logfile);
 
-	ms_exit();
+	ms_factory_destroy(args->video->ms.factory);
 }
 
 // ANDROID JNI WRAPPER
@@ -1086,7 +1135,7 @@ JNIEXPORT void JNICALL Java_org_linphone_mediastream_MediastreamerActivity_setVi
 	MediastreamDatas* args =  (MediastreamDatas*)_args;
 	if (!args->video)
 		return;
-	video_stream_set_native_window_id(args->video,(unsigned long)id);
+	video_stream_set_native_window_id(args->video,id);
 #endif
 }
 
@@ -1096,7 +1145,7 @@ JNIEXPORT void JNICALL Java_org_linphone_mediastream_MediastreamerActivity_setVi
 	MediastreamDatas* args =  (MediastreamDatas*)_args;
 	if (!args->video)
 		return;
-	video_stream_set_native_preview_window_id(args->video,(unsigned long)id);
+	video_stream_set_native_preview_window_id(args->video,id);
 #endif
 }
 
@@ -1119,7 +1168,7 @@ JNIEXPORT void JNICALL Java_org_linphone_mediastream_MediastreamerActivity_chang
 	char* id = (char*)malloc(15);
 	snprintf(id, 15, "Android%d", camId);
 	ms_message("Changing camera, trying to use: '%s'\n", id);
-	video_stream_change_camera(args->video, ms_web_cam_manager_get_cam(ms_web_cam_manager_get(), id));
+	video_stream_change_camera(args->video, ms_web_cam_manager_get_cam(ms_factory_get_web_cam_manager(args->video->ms.factory), id));
 #endif
 }
 
@@ -1189,10 +1238,10 @@ void stop_handler(int signum)
 	}
 }
 
-static bool_t parse_addr(const char *addr, char *ip, int len, int *port)
+static bool_t parse_addr(const char *addr, char *ip, size_t len, int *port)
 {
 	const char *semicolon=NULL;
-	int iplen;
+	size_t iplen;
 	int slen;
 	const char *p;
 
@@ -1223,10 +1272,10 @@ static bool_t parse_addr(const char *addr, char *ip, int len, int *port)
 	return TRUE;
 }
 
-static bool_t parse_ice_addr(char *addr, char *type, int type_len, char *ip, int ip_len, int *port)
+static bool_t parse_ice_addr(char *addr, char *type, size_t type_len, char *ip, size_t ip_len, int *port)
 {
 	char *semicolon=NULL;
-	int slen;
+	size_t slen;
 
 	semicolon=strrchr(addr,':');
 	if (semicolon==NULL) return FALSE;
