@@ -17,9 +17,6 @@ along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
-#ifdef HAVE_CONFIG_H
-#include "mediastreamer-config.h"
-#endif
 
 #include "mediastreamer2/mediastream.h"
 
@@ -28,10 +25,14 @@ int main(int argc, char *argv[]){
 	const char *file;
 	MSSndCard *sc;
 	const char * card_id=NULL;
+	MSFactory *factory;
 
 	ortp_init();
-	ortp_set_log_level_mask(ORTP_MESSAGE|ORTP_WARNING|ORTP_ERROR|ORTP_FATAL);
-	ms_init();
+	ortp_set_log_level_mask(ORTP_LOG_DOMAIN, ORTP_MESSAGE|ORTP_WARNING|ORTP_ERROR|ORTP_FATAL);
+	
+	
+	factory = ms_factory_new_with_voip();
+	
 	if (argc>1){
 		file=argv[1];
 	}else file="/usr/share/sounds/linphone/rings/oldphone.wav";
@@ -39,14 +40,16 @@ int main(int argc, char *argv[]){
 		card_id=argv[2];
 	}
 
-	sc=ms_snd_card_manager_get_card(ms_snd_card_manager_get(),card_id);
+	sc=ms_snd_card_manager_get_card(ms_factory_get_snd_card_manager(factory),card_id);
 #ifdef __linux
 	if (sc==NULL)
 	  sc = ms_alsa_card_new_custom(card_id, card_id);
 #endif
 
-	r=ring_start(file,2000,sc);
+	r=ring_start(factory, file,2000,sc);
 	ms_sleep(10);
 	ring_stop(r);
+	
+	ms_factory_destroy(factory);
 	return 0;
 }
