@@ -71,7 +71,7 @@ typedef struct rtcp_common_header
 #define rtcp_common_header_set_length(ch,l)	(ch)->length=htons(l)
 
 #define rtcp_common_header_get_version(ch) ((ch)->version)
-#define rtcp_common_header_get padbit(ch) ((ch)->padbit)
+#define rtcp_common_header_get_padbit(ch) ((ch)->padbit)
 #define rtcp_common_header_get_rc(ch) ((ch)->rc)
 #define rtcp_common_header_get_packet_type(ch) ((ch)->packet_type)
 #define rtcp_common_header_get_length(ch)	ntohs((ch)->length)
@@ -88,8 +88,11 @@ typedef struct sender_info
 	uint32_t senders_octet_count;
 } sender_info_t;
 
-uint64_t sender_info_get_ntp_timestamp(const sender_info_t *si);
-#define sender_info_get_rtp_timestamp(si)	((si)->rtp_timestamp)
+static ORTP_INLINE uint64_t sender_info_get_ntp_timestamp(const sender_info_t *si) {
+  return ((((uint64_t)ntohl(si->ntp_timestamp_msw)) << 32) +
+          ((uint64_t) ntohl(si->ntp_timestamp_lsw)));
+}
+#define sender_info_get_rtp_timestamp(si)	ntohl((si)->rtp_timestamp)
 #define sender_info_get_packet_count(si) \
 	ntohl((si)->senders_packet_count)
 #define sender_info_get_octet_count(si) \
@@ -310,6 +313,16 @@ typedef struct rtcp_fb_header {
 	uint32_t media_source_ssrc;
 } rtcp_fb_header_t;
 
+typedef struct rtcp_fb_generic_nack_fci {
+	uint16_t pid;
+	uint16_t blp;
+} rtcp_fb_generic_nack_fci_t;
+
+#define rtcp_fb_generic_nack_fci_get_pid(nack) ntohs((nack)->pid)
+#define rtcp_fb_generic_nack_fci_set_pid(nack, value) ((nack)->pid) = htons(value)
+#define rtcp_fb_generic_nack_fci_get_blp(nack) ntohs((nack)->blp)
+#define rtcp_fb_generic_nack_fci_set_blp(nack, value) ((nack)->blp) = htons(value)
+
 typedef struct rtcp_fb_tmmbr_fci {
 	uint32_t ssrc;
 	uint32_t value;
@@ -486,6 +499,7 @@ ORTP_PUBLIC uint16_t rtcp_XR_voip_metrics_get_jb_abs_max(const mblk_t *m);
 /* RTCP RTPFB accessors */
 ORTP_PUBLIC bool_t rtcp_is_RTPFB(const mblk_t *m);
 ORTP_PUBLIC rtcp_rtpfb_type_t rtcp_RTPFB_get_type(const mblk_t *m);
+ORTP_PUBLIC rtcp_fb_generic_nack_fci_t * rtcp_RTPFB_generic_nack_get_fci(const mblk_t *m);
 ORTP_PUBLIC rtcp_fb_tmmbr_fci_t * rtcp_RTPFB_tmmbr_get_fci(const mblk_t *m);
 ORTP_PUBLIC uint32_t rtcp_RTPFB_get_packet_sender_ssrc(const mblk_t *m);
 ORTP_PUBLIC uint32_t rtcp_RTPFB_get_media_source_ssrc(const mblk_t *m);
@@ -510,14 +524,14 @@ typedef struct OrtpLossRateEstimator{
 	float loss_rate;
 	/**
 	* Total number of outgoing duplicate packets on last
-	* #ortp_loss_rate_estimator_process_report_block iteration.
+	* ortp_loss_rate_estimator_process_report_block iteration.
 	**/
-	int32_t last_dup_packet_sent_count;
+	int64_t last_dup_packet_sent_count;
 	/**
 	* Total number of outgoing unique packets on last
-	* #ortp_loss_rate_estimator_process_report_block iteration.
+	* ortp_loss_rate_estimator_process_report_block iteration.
 	**/
-	int32_t last_packet_sent_count;
+	int64_t last_packet_sent_count;
 }OrtpLossRateEstimator;
 
 
@@ -537,12 +551,12 @@ ORTP_PUBLIC void ortp_loss_rate_estimator_init(OrtpLossRateEstimator *obj, int m
  * cumulative loss factor which allows us to take into consideration duplicates
  * packets as well.
  * @param[in] obj #OrtpLossRateEstimator object.
- * @param[in] stream #RtpStream stream in which the report block to consider belongs.
+ * @param[in] session #_RtpSession stream in which the report block to consider belongs.
  * @param[in] rb Report block to analyze.
  * @return TRUE if a new loss rate estimation is ready, FALSE otherwise.
  */
 ORTP_PUBLIC bool_t ortp_loss_rate_estimator_process_report_block(OrtpLossRateEstimator *obj,
-																 const struct _RtpStream *stream,
+																 const struct _RtpSession *session,
 																 const report_block_t *rb);
 /**
  * Get the latest loss rate in percentage estimation computed.
