@@ -19,15 +19,14 @@
 */
 
 
-#if defined(WIN32) || defined(_WIN32_WCE)
-#include "ortp-config-win32.h"
-#elif HAVE_CONFIG_H
+#ifdef HAVE_CONFIG_H
 #include "ortp-config.h"
 #endif
 #include "ortp/logging.h"
 #include "ortp/port.h"
 #include "ortp/str_utils.h"
 #include "utils.h"
+#include <bctoolbox/port.h>
 
 #if	defined(_WIN32) && !defined(_WIN32_WCE)
 #include <process.h>
@@ -103,10 +102,7 @@ char * ortp_strdup(const char *tmp){
  * ioctlsocket on Win32.
  * int retrun the result of the system method
  */
-int set_non_blocking_socket (ortp_socket_t sock)
-{
-
-
+int set_non_blocking_socket (ortp_socket_t sock){
 #if	!defined(_WIN32) && !defined(_WIN32_WCE)
 	return fcntl (sock, F_SETFL, O_NONBLOCK);
 #else
@@ -185,7 +181,7 @@ int __ortp_thread_join(ortp_thread_t thread, void **ptr){
 	return err;
 }
 
-int __ortp_thread_create(pthread_t *thread, pthread_attr_t *attr, void * (*routine)(void*), void *arg){
+int __ortp_thread_create(ortp_thread_t *thread, pthread_attr_t *attr, void * (*routine)(void*), void *arg){
 	pthread_attr_t my_attr;
 	pthread_attr_init(&my_attr);
 	if (attr)
@@ -197,42 +193,46 @@ int __ortp_thread_create(pthread_t *thread, pthread_attr_t *attr, void * (*routi
 	return pthread_create(thread, &my_attr, routine, arg);
 }
 
+unsigned long __ortp_thread_self(void) {
+	return (unsigned long)pthread_self();
+}
+
 #endif
 #if	defined(_WIN32) || defined(_WIN32_WCE)
 
 int WIN_mutex_init(ortp_mutex_t *mutex, void *attr)
 {
-#ifdef WINAPI_FAMILY_PHONE_APP
-	InitializeSRWLock(mutex);
-#else
+#ifdef ORTP_WINDOWS_DESKTOP
 	*mutex=CreateMutex(NULL, FALSE, NULL);
+#else
+	InitializeSRWLock(mutex);
 #endif
 	return 0;
 }
 
 int WIN_mutex_lock(ortp_mutex_t * hMutex)
 {
-#ifdef WINAPI_FAMILY_PHONE_APP
-	AcquireSRWLockExclusive(hMutex);
-#else
+#ifdef ORTP_WINDOWS_DESKTOP
 	WaitForSingleObject(*hMutex, INFINITE); /* == WAIT_TIMEOUT; */
+#else
+	AcquireSRWLockExclusive(hMutex);
 #endif
 	return 0;
 }
 
 int WIN_mutex_unlock(ortp_mutex_t * hMutex)
 {
-#ifdef WINAPI_FAMILY_PHONE_APP
-	ReleaseSRWLockExclusive(hMutex);
-#else
+#ifdef ORTP_WINDOWS_DESKTOP
 	ReleaseMutex(*hMutex);
+#else
+	ReleaseSRWLockExclusive(hMutex);
 #endif
 	return 0;
 }
 
 int WIN_mutex_destroy(ortp_mutex_t * hMutex)
 {
-#ifndef WINAPI_FAMILY_PHONE_APP
+#ifdef ORTP_WINDOWS_DESKTOP
 	CloseHandle(*hMutex);
 #endif
 	return 0;
@@ -245,9 +245,9 @@ typedef struct thread_param{
 
 static unsigned WINAPI thread_starter(void *data){
 	thread_param_t *params=(thread_param_t*)data;
-	void *ret=params->func(params->arg);
+	params->func(params->arg);
 	ortp_free(data);
-	return (DWORD)ret;
+	return 0;
 }
 
 #if defined _WIN32_WCE
@@ -257,9 +257,9 @@ static unsigned WINAPI thread_starter(void *data){
 
 int WIN_thread_create(ortp_thread_t *th, void *attr, void * (*func)(void *), void *data)
 {
-    thread_param_t *params=ortp_new(thread_param_t,1);
-    params->func=func;
-    params->arg=data;
+	thread_param_t *params=ortp_new(thread_param_t,1);
+	params->func=func;
+	params->arg=data;
 	*th=(HANDLE)_beginthreadex( NULL, 0, thread_starter, params, 0, NULL);
 	return 0;
 }
@@ -274,35 +274,39 @@ int WIN_thread_join(ortp_thread_t thread_h, void **unused)
 	return 0;
 }
 
+unsigned long WIN_thread_self(void) {
+	return (unsigned long)GetCurrentThreadId();
+}
+
 int WIN_cond_init(ortp_cond_t *cond, void *attr)
 {
-#ifdef WINAPI_FAMILY_PHONE_APP
-	InitializeConditionVariable(cond);
-#else
+#ifdef ORTP_WINDOWS_DESKTOP
 	*cond=CreateEvent(NULL, FALSE, FALSE, NULL);
+#else
+	InitializeConditionVariable(cond);
 #endif
 	return 0;
 }
 
 int WIN_cond_wait(ortp_cond_t* hCond, ortp_mutex_t * hMutex)
 {
-#ifdef WINAPI_FAMILY_PHONE_APP
-	SleepConditionVariableSRW(hCond, hMutex, INFINITE, 0);
-#else
+#ifdef ORTP_WINDOWS_DESKTOP
 	//gulp: this is not very atomic ! bug here ?
 	WIN_mutex_unlock(hMutex);
 	WaitForSingleObject(*hCond, INFINITE);
 	WIN_mutex_lock(hMutex);
+#else
+	SleepConditionVariableSRW(hCond, hMutex, INFINITE, 0);
 #endif
 	return 0;
 }
 
 int WIN_cond_signal(ortp_cond_t * hCond)
 {
-#ifdef WINAPI_FAMILY_PHONE_APP
-	WakeConditionVariable(hCond);
-#else
+#ifdef ORTP_WINDOWS_DESKTOP
 	SetEvent(*hCond);
+#else
+	WakeConditionVariable(hCond);
 #endif
 	return 0;
 }
@@ -315,13 +319,12 @@ int WIN_cond_broadcast(ortp_cond_t * hCond)
 
 int WIN_cond_destroy(ortp_cond_t * hCond)
 {
-#ifndef WINAPI_FAMILY_PHONE_APP
+#ifdef ORTP_WINDOWS_DESKTOP
 	CloseHandle(*hCond);
 #endif
 	return 0;
 }
 
-
 #if defined(_WIN32_WCE)
 #include <time.h>
 
@@ -351,7 +354,7 @@ gettimeofday (struct timeval *tv, void *tz)
 
 #else
 
-int gettimeofday (struct timeval *tv, void* tz)
+int ortp_gettimeofday (struct timeval *tv, void* tz)
 {
 	union
 	{
@@ -398,7 +401,7 @@ char * WSAAPI gai_strerror(int errnum){
 
 #endif
 
-#ifndef WIN32
+#ifndef _WIN32
 
 #include <sys/socket.h>
 #include <netdb.h>
@@ -452,9 +455,19 @@ int ortp_server_pipe_close(ortp_socket_t spipe){
 }
 
 ortp_socket_t ortp_client_pipe_connect(const char *name){
+	ortp_socket_t sock = -1;
 	struct sockaddr_un sa;
+	struct stat fstats;
 	char *pipename=make_pipe_name(name);
-	ortp_socket_t sock=socket(AF_UNIX,SOCK_STREAM,0);
+	uid_t uid = getuid();
+
+	// check that the creator of the pipe is us
+	if( (stat(name, &fstats) == 0) && (fstats.st_uid != uid) ){
+		ortp_error("UID of file %s (%lu) differs from ours (%lu)", pipename, (unsigned long)fstats.st_uid, (unsigned long)uid);
+		return -1;
+	}
+
+	sock = socket(AF_UNIX,SOCK_STREAM,0);
 	sa.sun_family=AF_UNIX;
 	strncpy(sa.sun_path,pipename,sizeof(sa.sun_path)-1);
 	ortp_free(pipename);
@@ -482,7 +495,8 @@ int ortp_client_pipe_close(ortp_socket_t sock){
 void *ortp_shm_open(unsigned int keyid, int size, int create){
 	key_t key=keyid;
 	void *mem;
-	int fd=shmget(key,size,create ? (IPC_CREAT | 0666) : 0666);
+	int perms=S_IRUSR|S_IWUSR;
+	int fd=shmget(key,size,create ? (IPC_CREAT | perms ) : perms);
 	if (fd==-1){
 		printf("shmget failed: %s\n",strerror(errno));
 		return NULL;
@@ -501,7 +515,7 @@ void ortp_shm_close(void *mem){
 
 #endif
 
-#elif defined(WIN32) && !defined(_WIN32_WCE)
+#elif defined(_WIN32) && !defined(_WIN32_WCE)
 
 static char *make_pipe_name(const char *name){
 	return ortp_strdup_printf("\\\\.\\pipe\\%s",name);
@@ -511,10 +525,7 @@ static HANDLE event=NULL;
 
 /* portable named pipes */
 ortp_pipe_t ortp_server_pipe_create(const char *name){
-#ifdef WINAPI_FAMILY_PHONE_APP
-	ortp_error("%s not supported!", __FUNCTION__);
-	return INVALID_HANDLE_VALUE;
-#else
+#ifdef ORTP_WINDOWS_DESKTOP
 	ortp_pipe_t h;
 	char *pipename=make_pipe_name(name);
 	h=CreateNamedPipe(pipename,PIPE_ACCESS_DUPLEX|FILE_FLAG_OVERLAPPED,PIPE_TYPE_MESSAGE|PIPE_WAIT,1,
@@ -525,6 +536,9 @@ ortp_pipe_t ortp_server_pipe_create(const char *name){
 	}
 	if (event==NULL) event=CreateEvent(NULL,TRUE,FALSE,NULL);
 	return h;
+#else
+	ortp_error("%s not supported!", __FUNCTION__);
+	return INVALID_HANDLE_VALUE;
 #endif
 }
 
@@ -534,10 +548,7 @@ even if nobody connects to the pipe.
 ortp_server_pipe_close() makes this function to exit.
 */
 ortp_pipe_t ortp_server_pipe_accept_client(ortp_pipe_t server){
-#ifdef WINAPI_FAMILY_PHONE_APP
-	ortp_error("%s not supported!", __FUNCTION__);
-	return INVALID_HANDLE_VALUE;
-#else
+#ifdef ORTP_WINDOWS_DESKTOP
 	OVERLAPPED ol;
 	DWORD undef;
 	HANDLE handles[2];
@@ -553,46 +564,49 @@ ortp_pipe_t ortp_server_pipe_accept_client(ortp_pipe_t server){
 	}
 	CloseHandle(ol.hEvent);
 	return INVALID_HANDLE_VALUE;
+#else
+	ortp_error("%s not supported!", __FUNCTION__);
+	return INVALID_HANDLE_VALUE;
 #endif
 }
 
 int ortp_server_pipe_close_client(ortp_pipe_t server){
-#ifdef WINAPI_FAMILY_PHONE_APP
+#ifdef ORTP_WINDOWS_DESKTOP
+	return DisconnectNamedPipe(server)==TRUE ? 0 : -1;
+#else
 	ortp_error("%s not supported!", __FUNCTION__);
 	return -1;
-#else
-	return DisconnectNamedPipe(server)==TRUE ? 0 : -1;
 #endif
 }
 
 int ortp_server_pipe_close(ortp_pipe_t spipe){
-#ifdef WINAPI_FAMILY_PHONE_APP
-	ortp_error("%s not supported!", __FUNCTION__);
-	return -1;
-#else
+#ifdef ORTP_WINDOWS_DESKTOP
 	SetEvent(event);
 	//CancelIoEx(spipe,NULL); /*vista only*/
 	return CloseHandle(spipe);
+#else
+	ortp_error("%s not supported!", __FUNCTION__);
+	return -1;
 #endif
 }
 
 ortp_pipe_t ortp_client_pipe_connect(const char *name){
-#ifdef WINAPI_FAMILY_PHONE_APP
-	ortp_error("%s not supported!", __FUNCTION__);
-	return INVALID_HANDLE_VALUE;
-#else
+#ifdef ORTP_WINDOWS_DESKTOP
 	char *pipename=make_pipe_name(name);
 	ortp_pipe_t hpipe = CreateFile(
-         pipename,   // pipe name
-         GENERIC_READ |  // read and write access
-         GENERIC_WRITE,
-         0,              // no sharing
-         NULL,           // default security attributes
-         OPEN_EXISTING,  // opens existing pipe
-         0,              // default attributes
-         NULL);          // no template file
+		 pipename,   // pipe name
+		 GENERIC_READ |  // read and write access
+		 GENERIC_WRITE,
+		 0,              // no sharing
+		 NULL,           // default security attributes
+		 OPEN_EXISTING,  // opens existing pipe
+		 0,              // default attributes
+		 NULL);          // no template file
 	ortp_free(pipename);
 	return hpipe;
+#else
+	ortp_error("%s not supported!", __FUNCTION__);
+	return INVALID_HANDLE_VALUE;
 #endif
 }
 
@@ -626,10 +640,7 @@ typedef struct MapInfo{
 static OList *maplist=NULL;
 
 void *ortp_shm_open(unsigned int keyid, int size, int create){
-#ifdef WINAPI_FAMILY_PHONE_APP
-	ortp_error("%s not supported!", __FUNCTION__);
-	return NULL;
-#else
+#ifdef ORTP_WINDOWS_DESKTOP
 	HANDLE h;
 	char name[64];
 	void *buf;
@@ -638,16 +649,16 @@ void *ortp_shm_open(unsigned int keyid, int size, int create){
 	if (create){
 		h = CreateFileMapping(
 			INVALID_HANDLE_VALUE,    // use paging file
-			NULL,                    // default security 
+			NULL,                    // default security
 			PAGE_READWRITE,          // read/write access
-			0,                       // maximum object size (high-order DWORD) 
-			size,                // maximum object size (low-order DWORD)  
+			0,                       // maximum object size (high-order DWORD)
+			size,                // maximum object size (low-order DWORD)
 			name);                 // name of mapping object
 	}else{
 		h = OpenFileMapping(
 			FILE_MAP_ALL_ACCESS,   // read/write access
 			FALSE,                 // do not inherit the name
-			name);               // name of mapping object 
+			name);               // name of mapping object
 	}
 	if (h==(HANDLE)-1) {
 		ortp_error("Fail to open file mapping (create=%i)",create);
@@ -655,8 +666,8 @@ void *ortp_shm_open(unsigned int keyid, int size, int create){
 	}
 	buf = (LPTSTR) MapViewOfFile(h, // handle to map object
 		FILE_MAP_ALL_ACCESS,  // read/write permission
-		0,                    
-		0,                    
+		0,
+		0,
 		size);
 	if (buf!=NULL){
 		MapInfo *i=(MapInfo*)ortp_new(MapInfo,1);
@@ -668,13 +679,14 @@ void *ortp_shm_open(unsigned int keyid, int size, int create){
 		ortp_error("MapViewOfFile failed");
 	}
 	return buf;
+#else
+	ortp_error("%s not supported!", __FUNCTION__);
+	return NULL;
 #endif
 }
 
 void ortp_shm_close(void *mem){
-#ifdef WINAPI_FAMILY_PHONE_APP
-	ortp_error("%s not supported!", __FUNCTION__);
-#else
+#ifdef ORTP_WINDOWS_DESKTOP
 	OList *elem;
 	for(elem=maplist;elem!=NULL;elem=elem->next){
 		MapInfo *i=(MapInfo*)elem->data;
@@ -687,6 +699,8 @@ void ortp_shm_close(void *mem){
 		}
 	}
 	ortp_error("No shared memory at %p was found.",mem);
+#else
+	ortp_error("%s not supported!", __FUNCTION__);
 #endif
 }
 
@@ -699,13 +713,9 @@ void ortp_shm_close(void *mem){
 #include <sys/timeb.h>
 #endif
 
-void ortp_get_cur_time(ortpTimeSpec *ret){
-#if defined(_WIN32_WCE) || defined(WIN32)
-#ifdef WINAPI_FAMILY_PHONE_APP
-	ULONGLONG timemillis = GetTickCount64();
-	ret->tv_sec = timemillis / 1000;
-	ret->tv_nsec = (timemillis % 1000) * 1000000LL;
-#else
+void _ortp_get_cur_time(ortpTimeSpec *ret, bool_t realtime){
+#if defined(_WIN32_WCE) || defined(_WIN32)
+#ifdef ORTP_WINDOWS_DESKTOP
 	DWORD timemillis;
 #	if defined(_WIN32_WCE)
 	timemillis=GetTickCount();
@@ -714,6 +724,10 @@ void ortp_get_cur_time(ortpTimeSpec *ret){
 #	endif
 	ret->tv_sec=timemillis/1000;
 	ret->tv_nsec=(timemillis%1000)*1000000LL;
+#else
+	ULONGLONG timemillis = GetTickCount64();
+	ret->tv_sec = timemillis / 1000;
+	ret->tv_nsec = (timemillis % 1000) * 1000000LL;
 #endif
 #elif defined(__MACH__) && defined(__GNUC__) && (__GNUC__ >= 3)
 	struct timeval tv;
@@ -728,7 +742,7 @@ void ortp_get_cur_time(ortpTimeSpec *ret){
 	ret->tv_nsec = time_val.millitm * 1000000LL;
 #else
 	struct timespec ts;
-	if (clock_gettime(CLOCK_MONOTONIC,&ts)<0){
+	if (clock_gettime(realtime ? CLOCK_REALTIME : CLOCK_MONOTONIC,&ts)<0){
 		ortp_fatal("clock_gettime() doesn't work: %s",strerror(errno));
 	}
 	ret->tv_sec=ts.tv_sec;
@@ -736,23 +750,177 @@ void ortp_get_cur_time(ortpTimeSpec *ret){
 #endif
 }
 
+void ortp_get_cur_time(ortpTimeSpec *ret){
+	_ortp_get_cur_time(ret, FALSE);
+}
+
+
+uint64_t ortp_get_cur_time_ms(void) {
+	ortpTimeSpec ts;
+	ortp_get_cur_time(&ts);
+	return (ts.tv_sec * 1000LL) + ((ts.tv_nsec + 500000LL) / 1000000LL);
+}
+
+void ortp_sleep_ms(int ms){
+#ifdef _WIN32
+#ifdef ORTP_WINDOWS_DESKTOP
+	Sleep(ms);
+#else
+	HANDLE sleepEvent = CreateEventEx(NULL, NULL, CREATE_EVENT_MANUAL_RESET, EVENT_ALL_ACCESS);
+	if (!sleepEvent) return;
+	WaitForSingleObjectEx(sleepEvent, ms, FALSE);
+	CloseHandle(sleepEvent);
+#endif
+#else
+	struct timespec ts;
+	ts.tv_sec=ms/1000;
+	ts.tv_nsec=(ms%1000)*1000000LL;
+	nanosleep(&ts,NULL);
+#endif
+}
+
+void ortp_sleep_until(const ortpTimeSpec *ts){
+#ifdef __linux
+	struct timespec rq;
+	rq.tv_sec=ts->tv_sec;
+	rq.tv_nsec=ts->tv_nsec;
+	while (clock_nanosleep(CLOCK_REALTIME, TIMER_ABSTIME, &rq, NULL)==-1 && errno==EINTR){
+	}
+#else
+	ortpTimeSpec current;
+	ortpTimeSpec diff;
+	_ortp_get_cur_time(&current, TRUE);
+	diff.tv_sec=ts->tv_sec-current.tv_sec;
+	diff.tv_nsec=ts->tv_nsec-current.tv_nsec;
+	if (diff.tv_nsec<0){
+		diff.tv_nsec+=1000000000LL;
+		diff.tv_sec-=1;
+	}
+#ifdef _WIN32
+		ortp_sleep_ms((int)((diff.tv_sec * 1000LL) + (diff.tv_nsec/1000000LL)));
+#else
+	{
+		struct timespec dur,rem;
+		dur.tv_sec=diff.tv_sec;
+		dur.tv_nsec=diff.tv_nsec;
+		while (nanosleep(&dur,&rem)==-1 && errno==EINTR){
+			dur=rem;
+		};
+	}
+#endif
+#endif
+}
+
 #if defined(_WIN32) && !defined(_MSC_VER)
 char* strtok_r(char *str, const char *delim, char **nextp){
-    char *ret;
+	char *ret;
 
-    if (str == NULL){
-        str = *nextp;
-    }
-    str += strspn(str, delim);
-    if (*str == '\0'){
-        return NULL;
-    }
-    ret = str;
-    str += strcspn(str, delim);
-    if (*str){
-        *str++ = '\0';
-    }
-    *nextp = str;
-    return ret;
+	if (str == NULL){
+		str = *nextp;
+	}
+	str += strspn(str, delim);
+	if (*str == '\0'){
+		return NULL;
+	}
+	ret = str;
+	str += strcspn(str, delim);
+	if (*str){
+		*str++ = '\0';
+	}
+	*nextp = str;
+	return ret;
 }
 #endif
+
+
+#if defined(_WIN32)
+
+#if !defined(_MSC_VER)
+#include <wincrypt.h>
+static int ortp_wincrypto_random(unsigned int *rand_number){
+	static HCRYPTPROV hProv=(HCRYPTPROV)-1;
+	static int initd=0;
+
+	if (!initd){
+		if (!CryptAcquireContext(&hProv,NULL,NULL,PROV_RSA_FULL, CRYPT_VERIFYCONTEXT)){
+			ortp_error("ortp_wincrypto_random(): Could not acquire a windows crypto context");
+			return -1;
+		}
+		initd=TRUE;
+	}
+	if (hProv==(HCRYPTPROV)-1)
+		return -1;
+
+	if (!CryptGenRandom(hProv,4,(BYTE*)rand_number)){
+		ortp_error("ortp_wincrypto_random(): CryptGenRandom() failed.");
+		return -1;
+	}
+	return 0;
+}
+#endif
+#elif defined(__QNXNTO__) || ((defined(__linux) || defined(__APPLE__)) && !defined(HAVE_ARC4RANDOM))
+
+static unsigned int ortp_urandom(void) {
+	static int fd=-1;
+	if (fd==-1) fd=open("/dev/urandom",O_RDONLY);
+	if (fd!=-1){
+		unsigned int tmp;
+		if (read(fd,&tmp,4)!=4){
+			ortp_error("Reading /dev/urandom failed.");
+		}else return tmp;
+	} else ortp_error("Could not open /dev/urandom");
+	return (unsigned int) random();
+}
+
+#endif
+
+
+
+
+unsigned int ortp_random(void){
+#ifdef HAVE_ARC4RANDOM
+#if defined(__QNXNTO__) // There is a false positive with blackberry build
+	return ortp_urandom();
+#else
+	return arc4random();
+#endif
+#elif defined(__linux) || defined(__APPLE__)
+	return ortp_urandom();
+#elif defined(_WIN32)
+	static int initd=0;
+	unsigned int ret;
+#ifdef _MSC_VER
+	/*rand_s() is pretty nice and simple function but is not wrapped by mingw.*/
+
+	if (rand_s(&ret)==0){
+		return ret;
+	}
+#else
+	if (ortp_wincrypto_random(&ret)==0){
+		return ret;
+	}
+#endif
+	/* Windows's rand() is unsecure but is used as a fallback*/
+	if (!initd) {
+		struct timeval tv;
+		ortp_gettimeofday(&tv,NULL);
+		srand((unsigned int)tv.tv_sec+tv.tv_usec);
+		initd=1;
+		ortp_warning("ortp: Random generator is using rand(), this is unsecure !");
+	}
+	return rand()<<16 | rand();
+#endif
+}
+
+bool_t ortp_is_multicast_addr(const struct sockaddr *addr) {
+
+	switch (addr->sa_family) {
+		case AF_INET:
+			return IN_MULTICAST(ntohl(((struct sockaddr_in *) addr)->sin_addr.s_addr));
+		case AF_INET6:
+			return IN6_IS_ADDR_MULTICAST(&(((struct sockaddr_in6 *) addr)->sin6_addr));
+		default:
+			return FALSE;
+	}
+
+}
