@@ -17,6 +17,10 @@
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
+#ifdef HAVE_CONFIG_H
+#include "ortp-config.h"
+#endif
+
 #include "ortp/str_utils.h"
 
 
@@ -29,26 +33,23 @@ void qinit(queue_t *q){
 
 void mblk_init(mblk_t *mp)
 {
-	mp->b_cont=mp->b_prev=mp->b_next=NULL;
-	mp->b_rptr=mp->b_wptr=NULL;
-	mp->reserved1=0;
-	mp->reserved2=0;
-#if defined(ORTP_TIMESTAMP)
-	memset(&(mp->timestamp), 0, sizeof(struct timeval));
-#endif
+	memset(mp, 0, sizeof(mblk_t));
 }
 
 void mblk_meta_copy(const mblk_t *source, mblk_t *dest) {
 	dest->reserved1 = source->reserved1;
 	dest->reserved2 = source->reserved2;
+	memcpy(&dest->net_addr,&source->net_addr,source->net_addrlen);
+	dest->net_addrlen = source->net_addrlen;
 #if defined(ORTP_TIMESTAMP)
 	dest->timestamp = source->timestamp;
 #endif
+	dest->ttl_or_hl = source->ttl_or_hl;
 }
 
-dblk_t *datab_alloc(int size){
+dblk_t *datab_alloc(size_t size){
 	dblk_t *db;
-	int total_size=sizeof(dblk_t)+size;
+	size_t total_size=sizeof(dblk_t)+size;
 	db=(dblk_t *) ortp_malloc(total_size);
 	db->db_base=(uint8_t*)db+sizeof(dblk_t);
 	db->db_lim=db->db_base+size;
@@ -57,11 +58,11 @@ dblk_t *datab_alloc(int size){
 	return db;
 }
 
-static inline void datab_ref(dblk_t *d){
+static ORTP_INLINE void datab_ref(dblk_t *d){
 	d->db_ref++;
 }
 
-static inline void datab_unref(dblk_t *d){
+static ORTP_INLINE void datab_unref(dblk_t *d){
 	d->db_ref--;
 	if (d->db_ref==0){
 		if (d->db_freefn!=NULL)
@@ -71,48 +72,45 @@ static inline void datab_unref(dblk_t *d){
 }
 
 
-mblk_t *allocb(int size, int pri)
+mblk_t *allocb(size_t size, int pri)
 {
 	mblk_t *mp;
 	dblk_t *datab;
-	
-	mp=(mblk_t *) ortp_malloc(sizeof(mblk_t));
-	mblk_init(mp);
+
+	mp=(mblk_t *) ortp_malloc0(sizeof(mblk_t));
 	datab=datab_alloc(size);
-	
+
 	mp->b_datap=datab;
 	mp->b_rptr=mp->b_wptr=datab->db_base;
 	mp->b_next=mp->b_prev=mp->b_cont=NULL;
 	return mp;
 }
 
-mblk_t *esballoc(uint8_t *buf, int size, int pri, void (*freefn)(void*) )
+mblk_t *esballoc(uint8_t *buf, size_t size, int pri, void (*freefn)(void*) )
 {
 	mblk_t *mp;
 	dblk_t *datab;
-	
-	mp=(mblk_t *) ortp_malloc(sizeof(mblk_t));
-	mblk_init(mp);
+
+	mp=(mblk_t *) ortp_malloc0(sizeof(mblk_t));
 	datab=(dblk_t *) ortp_malloc(sizeof(dblk_t));
-	
 
 	datab->db_base=buf;
 	datab->db_lim=buf+size;
 	datab->db_ref=1;
 	datab->db_freefn=freefn;
-	
+
 	mp->b_datap=datab;
 	mp->b_rptr=mp->b_wptr=buf;
 	mp->b_next=mp->b_prev=mp->b_cont=NULL;
 	return mp;
 }
 
-	
+
 void freeb(mblk_t *mp)
 {
 	return_if_fail(mp->b_datap!=NULL);
 	return_if_fail(mp->b_datap->db_base!=NULL);
-	
+
 	datab_unref(mp->b_datap);
 	ortp_free(mp);
 }
@@ -134,10 +132,9 @@ mblk_t *dupb(mblk_t *mp)
 	mblk_t *newm;
 	return_val_if_fail(mp->b_datap!=NULL,NULL);
 	return_val_if_fail(mp->b_datap->db_base!=NULL,NULL);
-	
+
 	datab_ref(mp->b_datap);
-	newm=(mblk_t *) ortp_malloc(sizeof(mblk_t));
-	mblk_init(newm);
+	newm=(mblk_t *) ortp_malloc0(sizeof(mblk_t));
 	mblk_meta_copy(mp, newm);
 	newm->b_datap=mp->b_datap;
 	newm->b_rptr=mp->b_rptr;
@@ -200,7 +197,7 @@ void insq(queue_t *q,mblk_t *emp, mblk_t *mp)
 	emp->b_prev->b_next=mp;
 	mp->b_prev=emp->b_prev;
 	emp->b_prev=mp;
-	mp->b_next=emp;	
+	mp->b_next=emp;
 }
 
 void remq(queue_t *q, mblk_t *mp){
@@ -215,36 +212,36 @@ void remq(queue_t *q, mblk_t *mp){
 void flushq(queue_t *q, int how)
 {
 	mblk_t *mp;
-	
+
 	while ((mp=getq(q))!=NULL)
 	{
 		freemsg(mp);
 	}
 }
 
-int msgdsize(const mblk_t *mp)
+size_t msgdsize(const mblk_t *mp)
 {
-	int msgsize=0;
+	size_t msgsize=0;
 	while(mp!=NULL){
-		msgsize+=(int) (mp->b_wptr-mp->b_rptr);
+		msgsize+=(size_t) (mp->b_wptr-mp->b_rptr);
 		mp=mp->b_cont;
 	}
 	return msgsize;
 }
 
-void msgpullup(mblk_t *mp,int len)
+void msgpullup(mblk_t *mp,size_t len)
 {
 	mblk_t *firstm=mp;
 	dblk_t *db;
-	int wlen=0;
+	size_t wlen=0;
 
 	if (mp->b_cont==NULL && len==-1) return;	/*nothing to do, message is not fragmented */
 
 	if (len==-1) len=msgdsize(mp);
 	db=datab_alloc(len);
 	while(wlen<len && mp!=NULL){
-		int remain=len-wlen;
-		int mlen=mp->b_wptr-mp->b_rptr;
+		int remain=(int)(len-wlen);
+		int mlen=(int)(mp->b_wptr-mp->b_rptr);
 		if (mlen<=remain){
 			memcpy(&db->db_base[wlen],mp->b_rptr,mlen);
 			wlen+=mlen;
@@ -264,7 +261,7 @@ void msgpullup(mblk_t *mp,int len)
 }
 
 
-mblk_t *copyb(mblk_t *mp)
+mblk_t *copyb(const mblk_t *mp)
 {
 	mblk_t *newm;
 	int len=(int) (mp->b_wptr-mp->b_rptr);
@@ -275,7 +272,7 @@ mblk_t *copyb(mblk_t *mp)
 	return newm;
 }
 
-mblk_t *copymsg(mblk_t *mp)
+mblk_t *copymsg(const mblk_t *mp)
 {
 	mblk_t *newm=0,*m;
 	m=newm=copyb(mp);
@@ -288,15 +285,15 @@ mblk_t *copymsg(mblk_t *mp)
 	return newm;
 }
 
-mblk_t * appendb(mblk_t *mp, const char *data, int size, bool_t pad){
-	int padcnt=0;
-	int i;
+mblk_t * appendb(mblk_t *mp, const char *data, size_t size, bool_t pad){
+	size_t padcnt=0;
+	size_t i;
 	if (pad){
-		padcnt= (int)(4L-( (long)(((long)mp->b_wptr)+size) % 4L)) % 4L;
+		padcnt = (size_t)(4 - ((((intptr_t)mp->b_wptr) + size) % 4)) % 4;
 	}
 	if ((mp->b_wptr + size +padcnt) > mp->b_datap->db_lim){
 		/* buffer is not large enough: append a new block (with the same size ?)*/
-		int plen=(int)((char*)mp->b_datap->db_lim - (char*) mp->b_datap->db_base);
+		size_t plen=(size_t)((char*)mp->b_datap->db_lim - (char*) mp->b_datap->db_base);
 		mp->b_cont=allocb(MAX(plen,size),0);
 		mp=mp->b_cont;
 	}
@@ -309,7 +306,7 @@ mblk_t * appendb(mblk_t *mp, const char *data, int size, bool_t pad){
 	return mp;
 }
 
-void msgappend(mblk_t *mp, const char *data, int size, bool_t pad){
+void msgappend(mblk_t *mp, const char *data, size_t size, bool_t pad){
 	while(mp->b_cont!=NULL) mp=mp->b_cont;
 	appendb(mp,data,size,pad);
 }
@@ -325,13 +322,13 @@ void msgb_allocator_init(msgb_allocator_t *a){
 	qinit(&a->q);
 }
 
-mblk_t *msgb_allocator_alloc(msgb_allocator_t *a, int size){
+mblk_t *msgb_allocator_alloc(msgb_allocator_t *a, size_t size){
 	queue_t *q=&a->q;
 	mblk_t *m,*found=NULL;
 
 	/*lookup for an unused msgb (data block with ref count ==1)*/
 	for(m=qbegin(q);!qend(q,m);m=qnext(q,m)){
-		if (m->b_datap->db_ref==1 && m->b_datap->db_lim-m->b_datap->db_base>=size){
+		if ((m->b_datap->db_ref == 1) && ((size_t)(m->b_datap->db_lim - m->b_datap->db_base) >= size)){
 			found=m;
 			break;
 		}
@@ -346,3 +343,19 @@ mblk_t *msgb_allocator_alloc(msgb_allocator_t *a, int size){
 void msgb_allocator_uninit(msgb_allocator_t *a){
 	flushq(&a->q,-1);
 }
+
+void ortp_recvaddr_to_sockaddr(ortp_recv_addr_t *recvaddr, struct sockaddr *addr, socklen_t *socklen) {
+	if (recvaddr->family == AF_INET) {
+		struct sockaddr_in *addr_in = (struct sockaddr_in *)addr;
+		addr_in->sin_family = AF_INET;
+		addr_in->sin_addr = recvaddr->addr.ipi_addr;
+		addr_in->sin_port = recvaddr->port;
+		*socklen = sizeof(struct sockaddr_in);
+	} else if (recvaddr->family == AF_INET6) {
+		struct sockaddr_in6 *addr_in6 = (struct sockaddr_in6 *)addr;
+		addr_in6->sin6_family = AF_INET6;
+		addr_in6->sin6_addr = recvaddr->addr.ipi6_addr;
+		addr_in6->sin6_port = recvaddr->port;
+		*socklen = sizeof(struct sockaddr_in6);
+	}
+}
