@@ -20,20 +20,32 @@
 #include "linphonecore.h"
 #include "private.h"
 #include "liblinphone_tester.h"
-#if HAVE_CU_CURSES
-#include "CUnit/CUCurses.h"
+
+#if __clang__ || ((__GNUC__ == 4 && __GNUC_MINOR__ >= 6) || __GNUC__ > 4)
+#pragma GCC diagnostic push
 #endif
+#ifdef _MSC_VER
+#pragma warning(disable : 4996)
+#else
+#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#pragma GCC diagnostic ignored "-Wstrict-prototypes"
+#endif
+
 #ifdef HAVE_GTK
 #include <gtk/gtk.h>
 #endif
 
+#if __clang__ || ((__GNUC__ == 4 && __GNUC_MINOR__ >= 6) || __GNUC__ > 4)
+#pragma GCC diagnostic pop
+#endif
+
+
 static FILE * log_file = NULL;
 
 #ifdef ANDROID
 
 #include <android/log.h>
 #include <jni.h>
-#include <CUnit/Util.h>
 #define CALLBACK_BUFFER_SIZE  1024
 
 static JNIEnv *current_env = NULL;
@@ -62,7 +74,7 @@ void liblinphone_android_log_handler(int prio, const char *fmt, va_list args) {
 	}
 }
 
-static void liblinphone_android_ortp_log_handler(OrtpLogLevel lev, const char *fmt, va_list args) {
+static void liblinphone_android_ortp_log_handler(const char *domain, OrtpLogLevel lev, const char *fmt, va_list args) {
 	int prio;
 	switch(lev){
 		case ORTP_DEBUG:	prio = ANDROID_LOG_DEBUG;	break;
@@ -75,15 +87,31 @@ static void liblinphone_android_ortp_log_handler(OrtpLogLevel lev, const char *f
 	liblinphone_android_log_handler(prio, fmt, args);
 }
 
+static void liblinphone_android_bctbx_log_handler(const char *domain, BctbxLogLevel lev, const char *fmt, va_list args) {
+	int prio;
+	switch(lev){
+		case BCTBX_LOG_DEBUG:	prio = ANDROID_LOG_DEBUG;	break;
+		case BCTBX_LOG_MESSAGE:	prio = ANDROID_LOG_INFO;	break;
+		case BCTBX_LOG_WARNING:	prio = ANDROID_LOG_WARN;	break;
+		case BCTBX_LOG_ERROR:	prio = ANDROID_LOG_ERROR;	break;
+		case BCTBX_LOG_FATAL:	prio = ANDROID_LOG_FATAL;	break;
+		default:			prio = ANDROID_LOG_DEFAULT;	break;
+	}
+	liblinphone_android_log_handler(prio, fmt, args);
+}
+
 void cunit_android_trace_handler(int level, const char *fmt, va_list args) {
 	char buffer[CALLBACK_BUFFER_SIZE];
+	jstring javaString;
+	jclass cls;
+	jmethodID method;
+	jint javaLevel = level;
 	JNIEnv *env = current_env;
 	if(env == NULL) return;
 	vsnprintf(buffer, CALLBACK_BUFFER_SIZE, fmt, args);
-	jstring javaString = (*env)->NewStringUTF(env, buffer);
-	jint javaLevel = level;
-	jclass cls = (*env)->GetObjectClass(env, current_obj);
-	jmethodID method = (*env)->GetMethodID(env, cls, "printLog", "(ILjava/lang/String;)V");
+	javaString = (*env)->NewStringUTF(env, buffer);
+	cls = (*env)->GetObjectClass(env, current_obj);
+	method = (*env)->GetMethodID(env, cls, "printLog", "(ILjava/lang/String;)V");
 	(*env)->CallVoidMethod(env, current_obj, method, javaLevel, javaString);
 	(*env)->DeleteLocalRef(env,javaString);
 	(*env)->DeleteLocalRef(env,cls);
@@ -102,10 +130,10 @@ JNIEXPORT jint JNICALL Java_org_linphone_tester_Tester_run(JNIEnv *env, jobject
 	}
 	current_env = env;
 	current_obj = obj;
-	CU_set_trace_handler(cunit_android_trace_handler);
+	bc_set_trace_handler(cunit_android_trace_handler);
 	ret = main(argc, argv);
 	current_env = NULL;
-	CU_set_trace_handler(NULL);
+	bc_set_trace_handler(NULL);
 	for (i=0; i<argc; i++) {
 		free(argv[i]);
 	}
@@ -140,7 +168,7 @@ static void log_handler(int lev, const char *fmt, va_list args) {
 	va_end(cap);
 #endif
 	if (log_file){
-		ortp_logv_out(lev, fmt, args);
+		ortp_logv_out(ORTP_LOG_DOMAIN, lev, fmt, args);
 	}
 }
 
@@ -148,18 +176,15 @@ void liblinphone_tester_init(void(*ftester_printf)(int level, const char *fmt, v
 	if (! log_file) {
 #if defined(ANDROID)
 		linphone_core_set_log_handler(liblinphone_android_ortp_log_handler);
+		bctbx_set_log_handler(liblinphone_android_bctbx_log_handler);
 #endif
 	}
 
 	if (ftester_printf == NULL) ftester_printf = log_handler;
-	bc_tester_init(ftester_printf, ORTP_MESSAGE, ORTP_ERROR);
+	bc_tester_init(ftester_printf, ORTP_MESSAGE, ORTP_ERROR, "rcfiles");
 	liblinphone_tester_add_suites();
 }
 
-void liblinphone_tester_uninit(void) {
-	bc_tester_uninit();
-}
-
 int liblinphone_tester_set_log_file(const char *filename) {
 	if (log_file) {
 		fclose(log_file);
@@ -170,12 +195,13 @@ int liblinphone_tester_set_log_file(const char *filename) {
 		return -1;
 	}
 	ms_message("Redirecting traces to file [%s]", filename);
+	bctbx_set_log_file(log_file);
 	ortp_set_log_file(log_file);
 	return 0;
 }
 
 
-#if !__ios && !(defined(LINPHONE_WINDOWS_PHONE) || defined(LINPHONE_WINDOWS_UNIVERSAL))
+#if !TARGET_OS_IPHONE && !(defined(LINPHONE_WINDOWS_PHONE) || defined(LINPHONE_WINDOWS_UNIVERSAL))
 
 static const char* liblinphone_helper =
 		"\t\t\t--verbose\n"
@@ -185,7 +211,9 @@ static const char* liblinphone_helper =
 		"\t\t\t--auth-domain <test auth domain>\n"
 		"\t\t\t--dns-hosts </etc/hosts -like file to used to override DNS names (default: tester_hosts)>\n"
 		"\t\t\t--keep-recorded-files\n"
-		"\t\t\t--disable-leak-detector\n";
+		"\t\t\t--disable-leak-detector\n"
+		"\t\t\t--6\n"
+		;
 
 int main (int argc, char *argv[])
 {
@@ -223,6 +251,8 @@ int main (int argc, char *argv[])
 			liblinphone_tester_keep_recorded_files(TRUE);
 		} else if (strcmp(argv[i],"--disable-leak-detector")==0){
 			liblinphone_tester_disable_leak_detector(TRUE);
+		} else if (strcmp(argv[i],"--6")==0){
+			liblinphonetester_ipv6=TRUE;
 		} else {
 			int bret = bc_tester_parse_args(argc, argv, i);
 			if (bret>0) {
@@ -239,4 +269,6 @@ int main (int argc, char *argv[])
 	liblinphone_tester_uninit();
 	return ret;
 }
+
+
 #endif
