@@ -89,7 +89,7 @@ static size_t getline(char **lineptr, size_t *n, FILE *stream) {
 #endif
 
 static LinphoneLogCollectionState old_collection_state;
-static void collect_init()  {
+static void collect_init(void)  {
 	old_collection_state = linphone_core_log_collection_enabled();
 	linphone_core_set_log_collection_path(bc_tester_get_writable_dir_prefix());
 }
@@ -101,14 +101,14 @@ static void collect_cleanup(LinphoneCoreManager *marie)  {
 	linphone_core_reset_log_collection();
 }
 
-static LinphoneCoreManager* setup(bool_t enable_logs)  {
+static LinphoneCoreManager* setup(LinphoneLogCollectionState log_collection_state)  {
 	LinphoneCoreManager *marie;
 	int timeout = 300;
 
 	collect_init();
-	linphone_core_enable_log_collection(enable_logs);
+	linphone_core_enable_log_collection(log_collection_state);
 
-	marie = linphone_core_manager_new2( "marie_rc", 0);
+	marie = linphone_core_manager_new2("marie_rc", 0);
 	// wait a few seconds to generate some traffic
 	while (--timeout){
 		// Generate some logs - error logs because we must ensure that
@@ -140,7 +140,7 @@ static FILE* gzuncompress(const char* filepath) {
 }
 #endif
 
-static time_t get_current_time() {
+static time_t get_current_time(void) {
 	struct timeval tp;
 	struct tm *lt;
 #ifndef _WIN32
@@ -161,7 +161,7 @@ static time_t get_current_time() {
 static time_t check_file(LinphoneCoreManager* mgr)  {
 
 	time_t cur_time = get_current_time();
-	char*    filepath = linphone_core_compress_log_collection(mgr->lc);
+	char*    filepath = linphone_core_compress_log_collection();
 	time_t  log_time = -1;
 	uint32_t timediff = 0;
 	FILE *file = NULL;
@@ -215,7 +215,6 @@ static time_t check_file(LinphoneCoreManager* mgr)  {
 
 
 		timediff = labs((long int)log_time - (long int)cur_time);
-		(void)timediff;
 #ifndef _WIN32
 		BC_ASSERT_LOWER(timediff, 1, unsigned, "%u");
 		if( !(timediff <= 1) ){
@@ -230,6 +229,7 @@ static time_t check_file(LinphoneCoreManager* mgr)  {
 			);
 		}
 #else
+		(void)timediff;
 		ms_warning("strptime() not available for this platform, test is incomplete.");
 #endif
 	}
@@ -238,26 +238,26 @@ static time_t check_file(LinphoneCoreManager* mgr)  {
 }
 
 static void collect_files_disabled(void)  {
-	LinphoneCoreManager* marie = setup(FALSE);
-	BC_ASSERT_PTR_NULL(linphone_core_compress_log_collection(marie->lc));
+	LinphoneCoreManager* marie = setup(LinphoneLogCollectionDisabled);
+	BC_ASSERT_PTR_NULL(linphone_core_compress_log_collection());
 	collect_cleanup(marie);
 }
 
 static void collect_files_filled(void) {
-	LinphoneCoreManager* marie = setup(TRUE);
+	LinphoneCoreManager* marie = setup(LinphoneLogCollectionEnabled);
 	check_file(marie);
 	collect_cleanup(marie);
 }
 
 static void collect_files_small_size(void)  {
-	LinphoneCoreManager* marie = setup(TRUE);
+	LinphoneCoreManager* marie = setup(LinphoneLogCollectionEnabled);
 	linphone_core_set_log_collection_max_file_size(5000);
 	check_file(marie);
 	collect_cleanup(marie);
 }
 
 static void collect_files_changing_size(void)  {
-	LinphoneCoreManager* marie = setup(TRUE);
+	LinphoneCoreManager* marie = setup(LinphoneLogCollectionEnabled);
 	int waiting = 100;
 
 	check_file(marie);
@@ -291,7 +291,7 @@ static void logCollectionUploadStateChangedCb(LinphoneCore *lc, LinphoneCoreLogC
 }
 static void upload_collected_traces(void)  {
 	if (transport_supported(LinphoneTransportTls)) {
-		LinphoneCoreManager* marie = setup(TRUE);
+		LinphoneCoreManager* marie = setup(LinphoneLogCollectionEnabled);
 		int waiting = 100;
 		LinphoneCoreVTable *v_table = linphone_core_v_table_new();
 		v_table->log_collection_upload_state_changed = logCollectionUploadStateChangedCb;
@@ -301,27 +301,27 @@ static void upload_collected_traces(void)  {
 		linphone_core_set_log_collection_upload_server_url(marie->lc,"https://www.linphone.org:444/lft.php");
 		// Generate some logs
 		while (--waiting) ms_error("(test error)Waiting %d...", waiting);
-		linphone_core_compress_log_collection(marie->lc);
+		ms_free(linphone_core_compress_log_collection());
 		linphone_core_upload_log_collection(marie->lc);
-		BC_ASSERT_TRUE(wait_for(marie->lc,marie->lc,&marie->stat.number_of_LinphoneCoreLogCollectionUploadStateDelivered,1));
+		BC_ASSERT_TRUE(wait_for_until(marie->lc,marie->lc,&marie->stat.number_of_LinphoneCoreLogCollectionUploadStateDelivered,1, 10000));
 
 		/*try 2 times*/
 		waiting=100;
-		linphone_core_reset_log_collection(marie->lc);
+		linphone_core_reset_log_collection();
 		while (--waiting) ms_error("(test error)Waiting %d...", waiting);
-		linphone_core_compress_log_collection(marie->lc);
+		ms_free(linphone_core_compress_log_collection());
 		linphone_core_upload_log_collection(marie->lc);
-		BC_ASSERT_TRUE(wait_for(marie->lc,marie->lc,&marie->stat.number_of_LinphoneCoreLogCollectionUploadStateDelivered,2));
+		BC_ASSERT_TRUE(wait_for_until(marie->lc,marie->lc,&marie->stat.number_of_LinphoneCoreLogCollectionUploadStateDelivered,2, 10000));
 		collect_cleanup(marie);
 	}
 }
 
 test_t log_collection_tests[] = {
-	{ "No file when disabled", collect_files_disabled},
-	{ "Collect files filled when enabled", collect_files_filled},
-	{ "Logs collected into small file", collect_files_small_size},
-	{ "Logs collected when decreasing max size", collect_files_changing_size},
-	{ "Upload collected traces", upload_collected_traces}
+	TEST_NO_TAG("No file when disabled", collect_files_disabled),
+	TEST_NO_TAG("Collect files filled when enabled", collect_files_filled),
+	TEST_NO_TAG("Logs collected into small file", collect_files_small_size),
+	TEST_NO_TAG("Logs collected when decreasing max size", collect_files_changing_size),
+	TEST_NO_TAG("Upload collected traces", upload_collected_traces)
 };
 
 test_suite_t log_collection_test_suite = {"LogCollection", NULL, NULL, liblinphone_tester_before_each, liblinphone_tester_after_each,
