@@ -21,8 +21,8 @@
 #include "private.h"
 #include "liblinphone_tester.h"
 
-static LinphoneCoreManager* presence_linphone_core_manager_new(char* username) {
-	LinphoneCoreManager* mgr= linphone_core_manager_new2( "empty_rc", FALSE);
+static LinphoneCoreManager* presence_linphone_core_manager_new_with_rc_name(char* username, char * rc_name) {
+	LinphoneCoreManager* mgr= linphone_core_manager_new2( rc_name, FALSE);
 	char* identity_char;
 	mgr->identity= linphone_core_get_primary_contact_parsed(mgr->lc);
 	linphone_address_set_username(mgr->identity,username);
@@ -31,6 +31,10 @@ static LinphoneCoreManager* presence_linphone_core_manager_new(char* username) {
 	ms_free(identity_char);
 	return mgr;
 }
+static LinphoneCoreManager* presence_linphone_core_manager_new(char* username) {
+	return presence_linphone_core_manager_new_with_rc_name(username, "empty_rc");
+}
+
 
 void new_subscription_requested(LinphoneCore *lc, LinphoneFriend *lf, const char *url){
 	char* from=linphone_address_as_string(linphone_friend_get_address(lf));
@@ -44,83 +48,83 @@ void new_subscription_requested(LinphoneCore *lc, LinphoneFriend *lf, const char
 
 void notify_presence_received(LinphoneCore *lc, LinphoneFriend * lf) {
 	stats* counters;
-	LinphonePresenceActivity *activity = NULL;
+	
+	unsigned int i;
 	char* from=linphone_address_as_string(linphone_friend_get_address(lf));
-	ms_message("New Notify request  from [%s] ",from);
+	ms_message("New Notify request from [%s] ",from);
 	ms_free(from);
 	counters = get_stats(lc);
-	counters->number_of_NotifyReceived++;
-
+	counters->number_of_NotifyPresenceReceived++;
 	counters->last_received_presence = linphone_friend_get_presence_model(lf);
-	activity = linphone_presence_model_get_activity(counters->last_received_presence);
-	switch (linphone_presence_activity_get_type(activity)) {
-		case LinphonePresenceActivityOffline:
-			counters->number_of_LinphonePresenceActivityOffline++; break;
-		case LinphonePresenceActivityOnline:
-			counters->number_of_LinphonePresenceActivityOnline++; break;
-		case LinphonePresenceActivityAppointment:
-			counters->number_of_LinphonePresenceActivityAppointment++; break;
-		case LinphonePresenceActivityAway:
-			counters->number_of_LinphonePresenceActivityAway++; break;
-		case LinphonePresenceActivityBreakfast:
-			counters->number_of_LinphonePresenceActivityBreakfast++; break;
-		case LinphonePresenceActivityBusy:
-			counters->number_of_LinphonePresenceActivityBusy++; break;
-		case LinphonePresenceActivityDinner:
-			counters->number_of_LinphonePresenceActivityDinner++; break;
-		case LinphonePresenceActivityHoliday:
-			counters->number_of_LinphonePresenceActivityHoliday++; break;
-		case LinphonePresenceActivityInTransit:
-			counters->number_of_LinphonePresenceActivityInTransit++; break;
-		case LinphonePresenceActivityLookingForWork:
-			counters->number_of_LinphonePresenceActivityLookingForWork++; break;
-		case LinphonePresenceActivityLunch:
-			counters->number_of_LinphonePresenceActivityLunch++; break;
-		case LinphonePresenceActivityMeal:
-			counters->number_of_LinphonePresenceActivityMeal++; break;
-		case LinphonePresenceActivityMeeting:
-			counters->number_of_LinphonePresenceActivityMeeting++; break;
-		case LinphonePresenceActivityOnThePhone:
-			counters->number_of_LinphonePresenceActivityOnThePhone++; break;
-		case LinphonePresenceActivityOther:
-			counters->number_of_LinphonePresenceActivityOther++; break;
-		case LinphonePresenceActivityPerformance:
-			counters->number_of_LinphonePresenceActivityPerformance++; break;
-		case LinphonePresenceActivityPermanentAbsence:
-			counters->number_of_LinphonePresenceActivityPermanentAbsence++; break;
-		case LinphonePresenceActivityPlaying:
-			counters->number_of_LinphonePresenceActivityPlaying++; break;
-		case LinphonePresenceActivityPresentation:
-			counters->number_of_LinphonePresenceActivityPresentation++; break;
-		case LinphonePresenceActivityShopping:
-			counters->number_of_LinphonePresenceActivityShopping++; break;
-		case LinphonePresenceActivitySleeping:
-			counters->number_of_LinphonePresenceActivitySleeping++; break;
-		case LinphonePresenceActivitySpectator:
-			counters->number_of_LinphonePresenceActivitySpectator++; break;
-		case LinphonePresenceActivitySteering:
-			counters->number_of_LinphonePresenceActivitySteering++; break;
-		case LinphonePresenceActivityTravel:
-			counters->number_of_LinphonePresenceActivityTravel++; break;
-		case LinphonePresenceActivityTV:
-			counters->number_of_LinphonePresenceActivityTV++; break;
-		case LinphonePresenceActivityUnknown:
-			counters->number_of_LinphonePresenceActivityUnknown++; break;
-		case LinphonePresenceActivityVacation:
-			counters->number_of_LinphonePresenceActivityVacation++; break;
-		case LinphonePresenceActivityWorking:
-			counters->number_of_LinphonePresenceActivityWorking++; break;
-		case LinphonePresenceActivityWorship:
-			counters->number_of_LinphonePresenceActivityWorship++; break;
+	if (linphone_presence_model_get_basic_status(counters->last_received_presence) == LinphonePresenceBasicStatusOpen) {
+		counters->number_of_LinphonePresenceBasicStatusOpen++;
+	} else if (linphone_presence_model_get_basic_status(counters->last_received_presence) == LinphonePresenceBasicStatusClosed) {
+		counters->number_of_LinphonePresenceBasicStatusClosed++;
+	} else {
+		ms_error("Unexpected basic status [%i]",linphone_presence_model_get_basic_status(counters->last_received_presence));
 	}
-}
-
-static void wait_core(LinphoneCore *core) {
-	int i;
-
-	for (i = 0; i < 10; i++) {
-		linphone_core_iterate(core);
-		ms_usleep(100000);
+	for (i=0;i<linphone_presence_model_get_nb_activities(counters->last_received_presence); i++) {
+		LinphonePresenceActivity *activity = linphone_presence_model_get_nth_activity(counters->last_received_presence, i);
+		switch (linphone_presence_activity_get_type(activity)) {
+			case LinphonePresenceActivityOffline:
+				counters->number_of_LinphonePresenceActivityOffline++; break;
+			case LinphonePresenceActivityOnline:
+				counters->number_of_LinphonePresenceActivityOnline++; break;
+			case LinphonePresenceActivityAppointment:
+				counters->number_of_LinphonePresenceActivityAppointment++; break;
+			case LinphonePresenceActivityAway:
+				counters->number_of_LinphonePresenceActivityAway++; break;
+			case LinphonePresenceActivityBreakfast:
+				counters->number_of_LinphonePresenceActivityBreakfast++; break;
+			case LinphonePresenceActivityBusy:
+				counters->number_of_LinphonePresenceActivityBusy++; break;
+			case LinphonePresenceActivityDinner:
+				counters->number_of_LinphonePresenceActivityDinner++; break;
+			case LinphonePresenceActivityHoliday:
+				counters->number_of_LinphonePresenceActivityHoliday++; break;
+			case LinphonePresenceActivityInTransit:
+				counters->number_of_LinphonePresenceActivityInTransit++; break;
+			case LinphonePresenceActivityLookingForWork:
+				counters->number_of_LinphonePresenceActivityLookingForWork++; break;
+			case LinphonePresenceActivityLunch:
+				counters->number_of_LinphonePresenceActivityLunch++; break;
+			case LinphonePresenceActivityMeal:
+				counters->number_of_LinphonePresenceActivityMeal++; break;
+			case LinphonePresenceActivityMeeting:
+				counters->number_of_LinphonePresenceActivityMeeting++; break;
+			case LinphonePresenceActivityOnThePhone:
+				counters->number_of_LinphonePresenceActivityOnThePhone++; break;
+			case LinphonePresenceActivityOther:
+				counters->number_of_LinphonePresenceActivityOther++; break;
+			case LinphonePresenceActivityPerformance:
+				counters->number_of_LinphonePresenceActivityPerformance++; break;
+			case LinphonePresenceActivityPermanentAbsence:
+				counters->number_of_LinphonePresenceActivityPermanentAbsence++; break;
+			case LinphonePresenceActivityPlaying:
+				counters->number_of_LinphonePresenceActivityPlaying++; break;
+			case LinphonePresenceActivityPresentation:
+				counters->number_of_LinphonePresenceActivityPresentation++; break;
+			case LinphonePresenceActivityShopping:
+				counters->number_of_LinphonePresenceActivityShopping++; break;
+			case LinphonePresenceActivitySleeping:
+				counters->number_of_LinphonePresenceActivitySleeping++; break;
+			case LinphonePresenceActivitySpectator:
+				counters->number_of_LinphonePresenceActivitySpectator++; break;
+			case LinphonePresenceActivitySteering:
+				counters->number_of_LinphonePresenceActivitySteering++; break;
+			case LinphonePresenceActivityTravel:
+				counters->number_of_LinphonePresenceActivityTravel++; break;
+			case LinphonePresenceActivityTV:
+				counters->number_of_LinphonePresenceActivityTV++; break;
+			case LinphonePresenceActivityUnknown:
+				counters->number_of_LinphonePresenceActivityUnknown++; break;
+			case LinphonePresenceActivityVacation:
+				counters->number_of_LinphonePresenceActivityVacation++; break;
+			case LinphonePresenceActivityWorking:
+				counters->number_of_LinphonePresenceActivityWorking++; break;
+			case LinphonePresenceActivityWorship:
+				counters->number_of_LinphonePresenceActivityWorship++; break;
+		}
 	}
 }
 
@@ -128,19 +132,58 @@ static void simple_publish_with_expire(int expires) {
 	LinphoneCoreManager* marie = linphone_core_manager_new( "marie_rc");
 	LinphoneProxyConfig* proxy;
 	LinphonePresenceModel* presence;
+	LinphoneCoreVTable *vtable = linphone_core_v_table_new();
+	vtable->publish_state_changed = linphone_publish_state_changed;
+	_linphone_core_add_listener(marie->lc, vtable, TRUE, TRUE );
 
-	linphone_core_get_default_proxy(marie->lc,&proxy);
+	proxy = linphone_core_get_default_proxy_config(marie->lc);
 	linphone_proxy_config_edit(proxy);
 	if (expires >0) {
 		linphone_proxy_config_set_publish_expires(proxy,expires);
 	}
 	linphone_proxy_config_enable_publish(proxy,TRUE);
 	linphone_proxy_config_done(proxy);
-	wait_core(marie->lc);
+
+	BC_ASSERT_TRUE(wait_for(marie->lc,marie->lc,&marie->stat.number_of_LinphonePublishProgress,1));
+	BC_ASSERT_TRUE(wait_for(marie->lc,marie->lc,&marie->stat.number_of_LinphonePublishOk,1));
+
 	presence =linphone_presence_model_new_with_activity(LinphonePresenceActivityOffline,NULL);
 	linphone_core_set_presence_model(marie->lc,presence);
-	wait_core(marie->lc);
+
+	BC_ASSERT_TRUE(wait_for(marie->lc,marie->lc,&marie->stat.number_of_LinphonePublishProgress,2));
+	BC_ASSERT_TRUE(wait_for(marie->lc,marie->lc,&marie->stat.number_of_LinphonePublishOk,2));
+
+	linphone_proxy_config_edit(proxy);
+	linphone_proxy_config_done(proxy);
+	/*make sure no publish is sent*/
+	BC_ASSERT_FALSE(wait_for_until(marie->lc,marie->lc,&marie->stat.number_of_LinphonePublishProgress,3,2000));
+
+	linphone_proxy_config_edit(proxy);
+	linphone_proxy_config_enable_publish(proxy,FALSE);
+	linphone_proxy_config_done(proxy);
+
+
+	/*fixme PUBLISH state machine is too simple, clear state should only be propagated at API level  when 200ok is received*/
+	/*BC_ASSERT_TRUE(wait_for(marie->lc,marie->lc,&marie->stat.number_of_LinphonePublishProgress,3));*/
+	wait_for_until(marie->lc,marie->lc,NULL,0,2000);
+	BC_ASSERT_TRUE(wait_for(marie->lc,marie->lc,&marie->stat.number_of_LinphonePublishCleared,1));
+
+	linphone_proxy_config_edit(proxy);
+	linphone_proxy_config_enable_publish(proxy,TRUE);
+	linphone_proxy_config_done(proxy);
+	BC_ASSERT_TRUE(wait_for(marie->lc,marie->lc,&marie->stat.number_of_LinphonePublishProgress,3));
+	BC_ASSERT_TRUE(wait_for(marie->lc,marie->lc,&marie->stat.number_of_LinphonePublishOk,3));
+
+	linphone_proxy_config_edit(proxy);
+	linphone_proxy_config_set_publish_expires(proxy, linphone_proxy_config_get_publish_expires(proxy)+1);
+	linphone_proxy_config_done(proxy);
+	BC_ASSERT_TRUE(wait_for(marie->lc,marie->lc,&marie->stat.number_of_LinphonePublishProgress,4));
+	BC_ASSERT_TRUE(wait_for(marie->lc,marie->lc,&marie->stat.number_of_LinphonePublishOk,4));
+
 	linphone_core_manager_destroy(marie);
+	/*fixme we should wait untill 200okBC_ASSERT_EQUAL(marie->stat.number_of_LinphonePublishCleared,2,int,"%i");*/
+	BC_ASSERT_EQUAL(marie->stat.number_of_LinphonePublishOk,4,int,"%i");
+
 }
 
 static void simple_publish(void) {
@@ -148,7 +191,7 @@ static void simple_publish(void) {
 }
 
 static void publish_with_expires(void) {
-	simple_publish_with_expire(1);
+	simple_publish_with_expire(2);
 }
 
 static bool_t subscribe_to_callee_presence(LinphoneCoreManager* caller_mgr,LinphoneCoreManager* callee_mgr) {
@@ -158,7 +201,7 @@ static bool_t subscribe_to_callee_presence(LinphoneCoreManager* caller_mgr,Linph
 	char* identity=linphone_address_as_string_uri_only(callee_mgr->identity);
 
 
-	LinphoneFriend* friend=linphone_friend_new_with_address(identity);
+	LinphoneFriend* friend=linphone_core_create_friend_with_address(caller_mgr->lc,identity);
 	linphone_friend_edit(friend);
 	linphone_friend_enable_subscribes(friend,TRUE);
 	linphone_friend_done(friend);
@@ -173,9 +216,9 @@ static bool_t subscribe_to_callee_presence(LinphoneCoreManager* caller_mgr,Linph
 
 	BC_ASSERT_EQUAL(callee_mgr->stat.number_of_NewSubscriptionRequest,initial_callee.number_of_NewSubscriptionRequest+1, int, "%d");
 	/*without proxy, callee cannot subscribe to caller
-	BC_ASSERT_EQUAL(callee_mgr->stat.number_of_NotifyReceived,initial_callee.number_of_NotifyReceived+1, int, "%d");
+	BC_ASSERT_EQUAL(callee_mgr->stat.number_of_NotifyPresenceReceived,initial_callee.number_of_NotifyPresenceReceived+1, int, "%d");
 	*/
-	BC_ASSERT_EQUAL(caller_mgr->stat.number_of_NotifyReceived,initial_caller.number_of_NotifyReceived+1, int, "%d");
+	BC_ASSERT_EQUAL(caller_mgr->stat.number_of_NotifyPresenceReceived,initial_caller.number_of_NotifyPresenceReceived+1, int, "%d");
 
 	ms_free(identity);
 	return result;
@@ -187,8 +230,8 @@ static void subscribe_failure_handle_by_app(void) {
 	LinphoneProxyConfig* config;
 	LinphoneFriend* lf;
 	char* lf_identity=linphone_address_as_string_uri_only(pauline->identity);
-	
-	linphone_core_get_default_proxy(marie->lc,&config);
+
+	config = linphone_core_get_default_proxy_config(marie->lc);
 
 	BC_ASSERT_TRUE(subscribe_to_callee_presence(marie,pauline));
 	wait_for(marie->lc,pauline->lc,&pauline->stat.number_of_NewSubscriptionRequest,1); /*just to wait for unsubscription even if not notified*/
@@ -200,6 +243,7 @@ static void subscribe_failure_handle_by_app(void) {
 	sal_set_recv_error(marie->lc->sal, 1);
 
 	lf = linphone_core_get_friend_by_address(marie->lc,lf_identity);
+	BC_ASSERT_PTR_NOT_NULL(lf);
 	linphone_friend_edit(lf);
 	linphone_friend_enable_subscribes(lf,FALSE); /*disable subscription*/
 	linphone_friend_done(lf);
@@ -228,10 +272,57 @@ static void simple_subscribe(void) {
 
 	linphone_core_manager_destroy(pauline);
 }
+static void simple_subscribe_with_early_notify(void) {
+	
+	LinphoneCoreManager* marie = presence_linphone_core_manager_new("marie");
+	LinphoneCoreManager* pauline = presence_linphone_core_manager_new("pauline");
+	LinphoneAddress *marie_identity_addr = linphone_address_clone(marie->identity);
+	LpConfig *pauline_lp;
+	
+	char* pauline_identity=linphone_address_as_string_uri_only(pauline->identity);
+	char* marie_identity;
+	
+	LinphoneFriend* pauline_s_friend;
+	LinphoneFriend* marie_s_friend=linphone_core_create_friend_with_address(marie->lc,pauline_identity);
+	
+	pauline_lp = linphone_core_get_config(pauline->lc);
+	lp_config_set_int(pauline_lp,"sip","notify_pending_state",1);
+	
+	linphone_friend_edit(marie_s_friend);
+	linphone_friend_enable_subscribes(marie_s_friend,TRUE);
+	linphone_friend_done(marie_s_friend);
+	linphone_core_add_friend(marie->lc,marie_s_friend);
+	ms_free(pauline_identity);
+	
+	
+	/*to simulate pending state.*/
+
+	linphone_address_set_port(marie_identity_addr,0);
+	marie_identity=linphone_address_as_string_uri_only(marie_identity_addr);
+	pauline_s_friend=linphone_core_create_friend_with_address(pauline->lc,marie_identity);
+	linphone_core_add_friend(pauline->lc,pauline_s_friend);
+	
+	ms_free(marie_identity);
+
+	BC_ASSERT_TRUE(wait_for(marie->lc,pauline->lc,&marie->stat.number_of_NotifyPresenceReceived,1));
+	BC_ASSERT_EQUAL(linphone_friend_get_subscription_state(marie_s_friend), LinphoneSubscriptionPending,int, "%d");
+	
+	wait_for(marie->lc,pauline->lc,&marie->stat.number_of_LinphonePresenceActivityOnline,marie->stat.number_of_LinphonePresenceActivityOnline+1);
+	
+	BC_ASSERT_EQUAL(marie->stat.number_of_NotifyPresenceReceived,2, int, "%d");
+	
+	linphone_friend_unref(marie_s_friend);
+	linphone_friend_unref(pauline_s_friend);
+	linphone_address_unref(marie_identity_addr);
+	linphone_core_manager_destroy(marie);
+	
+	linphone_core_manager_destroy(pauline);
+}
+
 
 static void unsubscribe_while_subscribing(void) {
 	LinphoneCoreManager* marie = linphone_core_manager_new( "marie_rc");
-	LinphoneFriend* friend = linphone_friend_new_with_address("sip:toto@git.linphone.org"); /*any unexisting address*/
+	LinphoneFriend* friend = linphone_core_create_friend_with_address(marie->lc, "sip:toto@git.linphone.org"); /*any unexisting address*/
 	linphone_friend_edit(friend);
 	linphone_friend_enable_subscribes(friend,TRUE);
 	linphone_friend_done(friend);
@@ -353,218 +444,122 @@ static void presence_information(void) {
 }
 
 
-static void subscribe_presence_forked(){
+static void subscribe_presence_forked(void){
 	LinphoneCoreManager* marie = linphone_core_manager_new("marie_rc");
-	LinphoneCoreManager* pauline1 = linphone_core_manager_new(transport_supported(LinphoneTransportTls) ? "pauline_rc" : "pauline_tcp_rc");
-	LinphoneCoreManager* pauline2 = linphone_core_manager_new(transport_supported(LinphoneTransportTls) ? "pauline_rc" : "pauline_tcp_rc");
+	LinphoneCoreManager* pauline1 = linphone_core_manager_new(transport_supported(LinphoneTransportTls) ? "pauline_tcp_rc" : "pauline_tcp_rc");
+	LinphoneCoreManager* pauline2 = linphone_core_manager_new(transport_supported(LinphoneTransportTls) ? "pauline_tcp_rc" : "pauline_tcp_rc");
 	LinphoneFriend *lf;
-	MSList *lcs = NULL;
-	
-	lcs = ms_list_append(lcs, marie->lc);
-	lcs = ms_list_append(lcs, pauline1->lc);
-	lcs = ms_list_append(lcs, pauline2->lc);
-	
+	bctbx_list_t *lcs = NULL;
+
+	lcs = bctbx_list_append(lcs, marie->lc);
+	lcs = bctbx_list_append(lcs, pauline1->lc);
+	lcs = bctbx_list_append(lcs, pauline2->lc);
+
 	lf = linphone_core_create_friend(marie->lc);
 	linphone_friend_set_address(lf, pauline1->identity);
 	linphone_friend_enable_subscribes(lf, TRUE);
-	
+
 	linphone_core_add_friend(marie->lc, lf);
 	linphone_friend_unref(lf);
-	
+
 	BC_ASSERT_TRUE(wait_for_list(lcs,&pauline1->stat.number_of_NewSubscriptionRequest,1, 10000));
 	BC_ASSERT_TRUE(wait_for_list(lcs,&pauline2->stat.number_of_NewSubscriptionRequest,1, 2000));
-	/*we should get two notifies*/
-	BC_ASSERT_TRUE(wait_for_list(lcs,&marie->stat.number_of_LinphonePresenceActivityOnline,2, 10000));
 	
-	/*marie also shall receive two SUBSCRIBEs from the two paulines, but won't be notified to the app since 
+	/*we should get only one notify*/
+	BC_ASSERT_TRUE(wait_for_list(lcs,&marie->stat.number_of_LinphonePresenceActivityOnline,1, 10000));
+	BC_ASSERT_FALSE(wait_for_list(lcs,&marie->stat.number_of_LinphonePresenceActivityOnline,2, 2000));
+
+	/*marie also shall receive two SUBSCRIBEs from the two paulines, but won't be notified to the app since
 	 Marie set Pauline as a friend.*/
 	BC_ASSERT_EQUAL(marie->stat.number_of_NewSubscriptionRequest, 0, int, "%d");
 	/*and the two paulines shall be notified of marie's presence*/
 	BC_ASSERT_TRUE(wait_for_list(lcs,&pauline1->stat.number_of_LinphonePresenceActivityOnline,1, 3000));
 	BC_ASSERT_TRUE(wait_for_list(lcs,&pauline2->stat.number_of_LinphonePresenceActivityOnline,1, 2000));
-	
+
 	linphone_core_manager_destroy(marie);
 	linphone_core_manager_destroy(pauline1);
 	linphone_core_manager_destroy(pauline2);
-	
-	ms_list_free(lcs);
+
+	bctbx_list_free(lcs);
 }
 
-static void subscribe_presence_expired(){
+static void subscribe_presence_expired(void){
 	LinphoneCoreManager* marie = linphone_core_manager_new("marie_rc");
 	LinphoneCoreManager* pauline1 = linphone_core_manager_new(transport_supported(LinphoneTransportTls) ? "pauline_rc" : "pauline_tcp_rc");
 	LinphoneFriend *lf;
-	MSList *lcs = NULL;
-	
-	lcs = ms_list_append(lcs, marie->lc);
-	lcs = ms_list_append(lcs, pauline1->lc);
-	
+	bctbx_list_t *lcs = NULL;
+
+	lcs = bctbx_list_append(lcs, marie->lc);
+	lcs = bctbx_list_append(lcs, pauline1->lc);
+
 	lp_config_set_int(marie->lc->config, "sip", "subscribe_expires", 10);
-	
+
 	lf = linphone_core_create_friend(marie->lc);
 	linphone_friend_set_address(lf, pauline1->identity);
 	linphone_friend_enable_subscribes(lf, TRUE);
-	
+
 	linphone_core_add_friend(marie->lc, lf);
 	linphone_friend_unref(lf);
-	
 	BC_ASSERT_TRUE(wait_for_list(lcs,&pauline1->stat.number_of_NewSubscriptionRequest,1, 5000));
 	BC_ASSERT_TRUE(wait_for_list(lcs,&marie->stat.number_of_LinphonePresenceActivityOnline,1, 2000));
-	
+
 	lf = linphone_core_find_friend(pauline1->lc, marie->identity);
-	BC_ASSERT_PTR_NOT_NULL(lf->insubs);
-	/*marie comes offline suddenly*/
-	linphone_core_set_network_reachable(marie->lc, FALSE);
-	/*after a certain time, pauline shall see the incoming SUBSCRIBE expired*/
-	wait_for_list(lcs,NULL, 0, 11000);
-	
-	BC_ASSERT_PTR_NULL(lf->insubs);
-	/*just make network reachable so that marie can unregister properly*/
-	linphone_core_set_network_reachable(marie->lc, TRUE);
-	BC_ASSERT_TRUE(wait_for_list(lcs,&marie->stat.number_of_LinphoneRegistrationOk,2, 10000));
+	BC_ASSERT_PTR_NOT_NULL(lf);
+	if (lf) {
+		BC_ASSERT_PTR_NOT_NULL(lf->insubs);
+
+		/*marie comes offline suddenly*/
+		linphone_core_set_network_reachable(marie->lc, FALSE);
+		/*after a certain time, pauline shall see the incoming SUBSCRIBE expired*/
+		wait_for_list(lcs,NULL, 0, 11000);
+		BC_ASSERT_PTR_NULL(lf->insubs);
+
+		/*just make network reachable so that marie can unregister properly*/
+		linphone_core_set_network_reachable(marie->lc, TRUE);
+		BC_ASSERT_TRUE(wait_for_list(lcs,&marie->stat.number_of_LinphoneRegistrationOk,2, 10000));
+	}
 	linphone_core_manager_destroy(marie);
 	linphone_core_manager_destroy(pauline1);
-	
-	ms_list_free(lcs);
+
+	bctbx_list_free(lcs);
 }
 
+static void simple_subscribe_with_friend_from_rc(void) {
+	LinphoneCoreManager* pauline = presence_linphone_core_manager_new("pauline");
+	LinphoneCoreManager *marie = presence_linphone_core_manager_new_with_rc_name("marie", "pauline_as_friend_rc");
+	LinphoneFriend *pauline_as_friend;
 
-#define USE_PRESENCE_SERVER 0
+	BC_ASSERT_EQUAL(bctbx_list_size(linphone_core_get_friend_list(marie->lc)), 1, int , "%i");
 
-#if USE_PRESENCE_SERVER
-static void test_subscribe_notify_publish(void) {
+	if (bctbx_list_size(linphone_core_get_friend_list(marie->lc))>0) {
+		pauline_as_friend = (LinphoneFriend*)linphone_core_get_friend_list(marie->lc)->data;
+		linphone_friend_set_address(pauline_as_friend, pauline->identity); /*hack to update addr with port number*/
+	}
 
-	LinphoneCoreManager* marie = linphone_core_manager_new( "marie_rc");
-	LinphoneCoreManager* pauline = linphone_core_manager_new( "pauline_rc");
-	LinphoneProxyConfig* proxy;
-	LinphonePresenceModel* presence;
+	BC_ASSERT_TRUE (wait_for(marie->lc,pauline->lc,&marie->stat.number_of_LinphonePresenceActivityOnline,1));
 
-	LpConfig *pauline_lp = linphone_core_get_config(pauline->lc);
-	char* lf_identity=linphone_address_as_string_uri_only(marie->identity);
-	LinphoneFriend *lf = linphone_core_create_friend_with_address(pauline->lc,lf_identity);
-
-	lp_config_set_int(pauline_lp,"sip","subscribe_expires",5);
-
-	linphone_core_add_friend(pauline->lc,lf);
-
-	/*wait for subscribe acknowledgment*/
-	wait_for_until(pauline->lc,marie->lc,&pauline->stat.number_of_NotifyReceived,1,2000);
-	BC_ASSERT_EQUAL(LinphoneStatusOffline,linphone_friend_get_status(lf), int, "%d");
-
-	/*enable publish*/
-
-	linphone_core_get_default_proxy(marie->lc,&proxy);
-	linphone_proxy_config_edit(proxy);
-
-	linphone_proxy_config_enable_publish(proxy,TRUE);
-	linphone_proxy_config_set_publish_expires(proxy,3);
-	linphone_proxy_config_done(proxy);
-
-	/*wait for marie status*/
-	wait_for_until(pauline->lc,marie->lc,&pauline->stat.number_of_NotifyReceived,2,2000);
-	BC_ASSERT_EQUAL(LinphoneStatusOnline,linphone_friend_get_status(lf), int, "%d");
-
-	presence =linphone_presence_model_new_with_activity(LinphonePresenceActivityBusy,NULL);
-	linphone_core_set_presence_model(marie->lc,presence);
-
-	/*wait for new status*/
-	wait_for_until(pauline->lc,marie->lc,&pauline->stat.number_of_NotifyReceived,3,2000);
-	BC_ASSERT_EQUAL(LinphoneStatusBusy,linphone_friend_get_status(lf), int, "%d");
-
-	/*wait for refresh*/
-	wait_for_until(pauline->lc,marie->lc,&pauline->stat.number_of_NotifyReceived,4,5000);
-	BC_ASSERT_EQUAL(LinphoneStatusBusy,linphone_friend_get_status(lf), int, "%d");
-
-	/*linphone_core_remove_friend(pauline->lc,lf);*/
-	/*wait for final notify*/
-	/*wait_for_until(pauline->lc,marie->lc,&pauline->stat.number_of_NotifyReceived,4,5000);
-	BC_ASSERT_EQUAL(LinphonePresenceActivityOffline,linphone_friend_get_status(lf), int, "%d");
-	 */
-	linphone_core_manager_destroy(marie);
-	linphone_core_manager_destroy(pauline);
-}
-static void test_forked_subscribe_notify_publish(void) {
-
-	LinphoneCoreManager* marie = linphone_core_manager_new( "marie_rc");
-	LinphoneCoreManager* marie2 = linphone_core_manager_new( "marie_rc");
-	LinphoneCoreManager* pauline = linphone_core_manager_new( "pauline_rc");
-	LinphoneProxyConfig* proxy;
-	LinphonePresenceModel* presence;
-	LpConfig *pauline_lp;
-	char* lf_identity;
-	LinphoneFriend *lf;
-	MSList* lcs=ms_list_append(NULL,pauline->lc);
-	lcs=ms_list_append(lcs,marie->lc);
-	lcs=ms_list_append(lcs,marie->lc);
-	lcs=ms_list_append(lcs,marie2->lc);
-
-	pauline_lp = linphone_core_get_config(pauline->lc);
-	lf_identity=linphone_address_as_string_uri_only(marie->identity);
-	lf = linphone_core_create_friend_with_address(pauline->lc,lf_identity);
-
-	lp_config_set_int(pauline_lp,"sip","subscribe_expires",5);
-
-	linphone_core_add_friend(pauline->lc,lf);
-
-	/*wait for subscribe acknowledgment*/
-	wait_for_list(lcs,&pauline->stat.number_of_NotifyReceived,1,2000);
-	BC_ASSERT_EQUAL(LinphoneStatusOffline,linphone_friend_get_status(lf), int, "%d");
-
-	/*enable publish*/
-
-	linphone_core_get_default_proxy(marie->lc,&proxy);
-	linphone_proxy_config_edit(proxy);
-	linphone_proxy_config_enable_publish(proxy,TRUE);
-	linphone_proxy_config_set_publish_expires(proxy,3);
-	linphone_proxy_config_done(proxy);
-
-	linphone_core_get_default_proxy(marie2->lc,&proxy);
-	linphone_proxy_config_edit(proxy);
-	linphone_proxy_config_enable_publish(proxy,TRUE);
-	linphone_proxy_config_set_publish_expires(proxy,3);
-	linphone_proxy_config_done(proxy);
-
-
-	/*wait for marie status*/
-	wait_for_list(lcs,&pauline->stat.number_of_NotifyReceived,3,2000);
-	BC_ASSERT_EQUAL(LinphoneStatusOnline,linphone_friend_get_status(lf), int, "%d");
-
-	presence =linphone_presence_model_new_with_activity(LinphonePresenceActivityBusy,NULL);
-	linphone_core_set_presence_model(marie->lc,presence);
-
-	/*wait for new status*/
-	wait_for_list(lcs,&pauline->stat.number_of_NotifyReceived,4,2000);
-	BC_ASSERT_EQUAL(LinphoneStatusBusy,linphone_friend_get_status(lf), int, "%d");
-
-
-	presence =linphone_presence_model_new_with_activity(  LinphonePresenceActivityMeeting,NULL);
-	linphone_core_set_presence_model(marie2->lc,presence);
-	/*wait for new status*/
-	wait_for_list(lcs,&pauline->stat.number_of_NotifyReceived,5,2000);
-	BC_ASSERT_EQUAL(LinphoneStatusBusy,linphone_friend_get_status(lf), int, "%d"); /*because liblinphone compositor is very simple for now (I.E only take first occurence)*/
+	BC_ASSERT_EQUAL(pauline->stat.number_of_NewSubscriptionRequest,1, int, "%d");
+	BC_ASSERT_EQUAL(marie->stat.number_of_NotifyPresenceReceived,1, int, "%d");
 
 	linphone_core_manager_destroy(marie);
-	linphone_core_manager_destroy(marie2);
+	/*unsubscribe is not reported ?*/
+	BC_ASSERT_FALSE(wait_for(NULL,pauline->lc,&pauline->stat.number_of_NewSubscriptionRequest,2)); /*just to wait for unsubscription even if not notified*/
+
 	linphone_core_manager_destroy(pauline);
 }
 
-
-#endif
-
 test_t presence_tests[] = {
-	{ "Simple Subscribe", simple_subscribe },
-	{ "Simple Publish", simple_publish },
-	{ "Simple Publish with expires", publish_with_expires },
-	/*{ "Call with presence", call_with_presence },*/
-	{ "Unsubscribe while subscribing", unsubscribe_while_subscribing },
-	{ "Presence information", presence_information },
-	{ "App managed presence failure", subscribe_failure_handle_by_app },
-	{ "Presence SUBSCRIBE forked", subscribe_presence_forked },
-	{ "Presence SUBSCRIBE expired", subscribe_presence_expired },
-#if USE_PRESENCE_SERVER
-	{ "Subscribe with late publish", test_subscribe_notify_publish },
-	{ "Forked subscribe with late publish", test_forked_subscribe_notify_publish },
-#endif
+	TEST_ONE_TAG("Simple Subscribe", simple_subscribe,"presence"),
+	TEST_ONE_TAG("Simple Subscribe with early NOTIFY", simple_subscribe_with_early_notify,"presence"),
+	TEST_NO_TAG("Simple Subscribe with friend from rc", simple_subscribe_with_friend_from_rc),
+	TEST_ONE_TAG("Simple Publish", simple_publish, "LeaksMemory"),
+	TEST_ONE_TAG("Simple Publish with expires", publish_with_expires, "LeaksMemory"),
+	/*TEST_ONE_TAG("Call with presence", call_with_presence, "LeaksMemory"),*/
+	TEST_NO_TAG("Unsubscribe while subscribing", unsubscribe_while_subscribing),
+	TEST_NO_TAG("Presence information", presence_information),
+	TEST_NO_TAG("App managed presence failure", subscribe_failure_handle_by_app),
+	TEST_NO_TAG("Presence SUBSCRIBE forked", subscribe_presence_forked),
+	TEST_NO_TAG("Presence SUBSCRIBE expired", subscribe_presence_expired),
 };
 
 test_suite_t presence_test_suite = {"Presence", NULL, NULL, liblinphone_tester_before_each, liblinphone_tester_after_each,
