@@ -165,7 +165,7 @@ static void quality_reporting_not_sent_if_call_not_started(void) {
 	BC_ASSERT_TRUE(wait_for_until(marie->lc,pauline->lc,&marie->stat.number_of_LinphoneCallError,1, 10000));
 	BC_ASSERT_EQUAL(marie->stat.number_of_LinphoneCallError,1, int, "%d");
 
-	if (ms_list_size(linphone_core_get_call_logs(marie->lc))>0) {
+	if (bctbx_list_size(linphone_core_get_call_logs(marie->lc))>0) {
 		out_call_log=(LinphoneCallLog*)(linphone_core_get_call_logs(marie->lc)->data);
 		BC_ASSERT_PTR_NOT_NULL(out_call_log);
 		BC_ASSERT_EQUAL(linphone_call_log_get_status(out_call_log),LinphoneCallAborted, int, "%d");
@@ -188,7 +188,7 @@ static void quality_reporting_not_sent_if_low_bandwidth(void) {
 	LinphoneCoreManager* pauline = linphone_core_manager_new( "pauline_tcp_rc");
 	LinphoneCallParams* marie_params;
 
-	marie_params=linphone_core_create_default_call_parameters(marie->lc);
+	marie_params=linphone_core_create_call_params(marie->lc, NULL);
 	linphone_call_params_enable_low_bandwidth(marie_params,TRUE);
 
 	if (create_call_for_quality_reporting_tests(marie, pauline, NULL, NULL, marie_params, NULL)) {
@@ -279,6 +279,7 @@ static void quality_reporting_interval_report(void) {
 	linphone_core_manager_destroy(pauline);
 }
 
+#ifdef VIDEO_ENABLED
 static void quality_reporting_session_report_if_video_stopped(void) {
 	LinphoneCoreManager* marie = linphone_core_manager_new( "marie_rc_rtcp_xr");
 	LinphoneCoreManager* pauline = linphone_core_manager_new( "pauline_tcp_rc");
@@ -291,9 +292,9 @@ static void quality_reporting_session_report_if_video_stopped(void) {
 	linphone_core_enable_video_display(marie->lc, FALSE);
 	linphone_core_enable_video_capture(pauline->lc, TRUE);
 	linphone_core_enable_video_display(pauline->lc, FALSE);
-	marie_params=linphone_core_create_default_call_parameters(marie->lc);
+	marie_params=linphone_core_create_call_params(marie->lc, NULL);
 	linphone_call_params_enable_video(marie_params,TRUE);
-	pauline_params=linphone_core_create_default_call_parameters(pauline->lc);
+	pauline_params=linphone_core_create_call_params(pauline->lc, NULL);
 	linphone_call_params_enable_video(pauline_params,TRUE);
 
 	if (create_call_for_quality_reporting_tests(marie, pauline, &call_marie, &call_pauline, marie_params, pauline_params)) {
@@ -325,6 +326,7 @@ static void quality_reporting_session_report_if_video_stopped(void) {
 	linphone_core_manager_destroy(marie);
 	linphone_core_manager_destroy(pauline);
 }
+#endif
 
 void publish_report_with_route_state_changed(LinphoneCore *lc, LinphoneEvent *ev, LinphonePublishState state){
 	if (state == LinphonePublishProgress) {
@@ -358,6 +360,7 @@ static void quality_reporting_sent_using_custom_route(void) {
 	linphone_core_manager_destroy(pauline);
 }
 
+#ifdef VIDEO_ENABLED
 static void quality_reporting_interval_report_video_and_rtt(void) {
 	LinphoneCoreManager* marie = linphone_core_manager_new( "marie_rc_rtcp_xr");
 	LinphoneCoreManager* pauline = linphone_core_manager_new( "pauline_rc_rtcp_xr");
@@ -371,10 +374,10 @@ static void quality_reporting_interval_report_video_and_rtt(void) {
 	linphone_core_enable_video_display(marie->lc, FALSE);
 	linphone_core_enable_video_capture(pauline->lc, TRUE);
 	linphone_core_enable_video_display(pauline->lc, FALSE);
-	marie_params=linphone_core_create_default_call_parameters(marie->lc);
+	marie_params=linphone_core_create_call_params(marie->lc, NULL);
 	linphone_call_params_enable_video(marie_params,TRUE);
 	linphone_call_params_enable_realtime_text(marie_params,TRUE);
-	pauline_params=linphone_core_create_default_call_parameters(pauline->lc);
+	pauline_params=linphone_core_create_call_params(pauline->lc, NULL);
 	linphone_call_params_enable_video(pauline_params,TRUE);
 	linphone_call_params_enable_realtime_text(pauline_params,TRUE);
 
@@ -397,7 +400,7 @@ static void quality_reporting_interval_report_video_and_rtt(void) {
 		BC_ASSERT_PTR_NOT_NULL(pauline_chat_room);
 		if (pauline_chat_room) {
 			const char* message = "Lorem Ipsum Belledonnum Communicatum";
-			int i;
+			size_t i;
 			LinphoneChatMessage* rtt_message = linphone_chat_room_create_message(pauline_chat_room,NULL);
 			LinphoneChatRoom *marie_chat_room = linphone_call_get_chat_room(call_marie);
 
@@ -418,17 +421,20 @@ static void quality_reporting_interval_report_video_and_rtt(void) {
 	linphone_core_manager_destroy(marie);
 	linphone_core_manager_destroy(pauline);
 }
+#endif
 
 test_t quality_reporting_tests[] = {
-	{ "Not used if no config", quality_reporting_not_used_without_config},
-	{ "Call term session report not sent if call did not start", quality_reporting_not_sent_if_call_not_started},
-	{ "Call term session report not sent if low bandwidth", quality_reporting_not_sent_if_low_bandwidth},
-	{ "Call term session report invalid if missing mandatory fields", quality_reporting_invalid_report},
-	{ "Call term session report sent if call ended normally", quality_reporting_at_call_termination},
-	{ "Interval report if interval is configured", quality_reporting_interval_report},
-	{ "Interval report if interval is configured with video and realtime text", quality_reporting_interval_report_video_and_rtt},
-	{ "Session report sent if video stopped during call", quality_reporting_session_report_if_video_stopped},
-	{ "Sent using custom route", quality_reporting_sent_using_custom_route},
+	TEST_NO_TAG("Not used if no config", quality_reporting_not_used_without_config),
+	TEST_NO_TAG("Call term session report not sent if call did not start", quality_reporting_not_sent_if_call_not_started),
+	TEST_NO_TAG("Call term session report not sent if low bandwidth", quality_reporting_not_sent_if_low_bandwidth),
+	TEST_NO_TAG("Call term session report invalid if missing mandatory fields", quality_reporting_invalid_report),
+	TEST_NO_TAG("Call term session report sent if call ended normally", quality_reporting_at_call_termination),
+	TEST_NO_TAG("Interval report if interval is configured", quality_reporting_interval_report),
+	#ifdef VIDEO_ENABLED
+	TEST_NO_TAG("Interval report if interval is configured with video and realtime text", quality_reporting_interval_report_video_and_rtt),
+	TEST_NO_TAG("Session report sent if video stopped during call", quality_reporting_session_report_if_video_stopped),
+	#endif
+	TEST_NO_TAG("Sent using custom route", quality_reporting_sent_using_custom_route)
 };
 
 test_suite_t quality_reporting_test_suite = {"QualityReporting", NULL, NULL, liblinphone_tester_before_each, liblinphone_tester_after_each,
