@@ -45,7 +45,7 @@ static LinphoneCoreManager* create_lcm_with_auth(unsigned int with_auth) {
 	return lcm;
 }
 
-static LinphoneCoreManager* create_lcm() {
+static LinphoneCoreManager* create_lcm(void) {
 	return create_lcm_with_auth(0);
 }
 
@@ -216,7 +216,7 @@ static void simple_unregister(void){
 	LinphoneProxyConfig* proxy_config;
 	register_with_refresh_base(lcm->lc,FALSE,NULL,NULL);
 
-	linphone_core_get_default_proxy(lcm->lc,&proxy_config);
+	proxy_config = linphone_core_get_default_proxy_config(lcm->lc);
 
 	linphone_proxy_config_edit(proxy_config);
 	reset_counters(counters); /*clear stats*/
@@ -235,7 +235,7 @@ static void change_expires(void){
 	LinphoneProxyConfig* proxy_config;
 	register_with_refresh_base(lcm->lc,FALSE,NULL,NULL);
 
-	linphone_core_get_default_proxy(lcm->lc,&proxy_config);
+	proxy_config = linphone_core_get_default_proxy_config(lcm->lc);
 
 	linphone_proxy_config_edit(proxy_config);
 	reset_counters(counters); /*clear stats*/
@@ -459,7 +459,7 @@ static void authenticated_register_with_wrong_credentials_with_params_base(const
 	/*check the detailed error info */
 	if (!user_agent || strcmp(user_agent,"tester-no-403")!=0){
 		LinphoneProxyConfig *cfg=NULL;
-		linphone_core_get_default_proxy(lcm->lc,&cfg);
+		cfg = linphone_core_get_default_proxy_config(lcm->lc);
 		BC_ASSERT_PTR_NOT_NULL(cfg);
 		if (cfg){
 			const LinphoneErrorInfo *ei=linphone_proxy_config_get_error_info(cfg);
@@ -488,7 +488,7 @@ static void authenticated_register_with_wrong_credentials_2(void) {
 
 	authenticated_register_with_wrong_credentials_with_params_base(NULL,lcm);
 
-	linphone_core_get_default_proxy(lcm->lc,&proxy);
+	proxy = linphone_core_get_default_proxy_config(lcm->lc);
 	/*Make sure registration attempts are stopped*/
 	linphone_proxy_config_edit(proxy);
 	linphone_proxy_config_enable_register(proxy,FALSE);
@@ -505,7 +505,7 @@ static LinphoneCoreManager* configure_lcm(void) {
 	if (transport_supported(LinphoneTransportTls)) {
 		LinphoneCoreManager *lcm=linphone_core_manager_new2( "multi_account_rc", FALSE);
 		stats *counters=&lcm->stat;
-		BC_ASSERT_TRUE(wait_for(lcm->lc,lcm->lc,&counters->number_of_LinphoneRegistrationOk,ms_list_size(linphone_core_get_proxy_config_list(lcm->lc))));
+		BC_ASSERT_TRUE(wait_for(lcm->lc,lcm->lc,&counters->number_of_LinphoneRegistrationOk,bctbx_list_size(linphone_core_get_proxy_config_list(lcm->lc))));
 		BC_ASSERT_EQUAL(counters->number_of_LinphoneRegistrationFailed,0, int, "%d");
 		return lcm;
 	}
@@ -541,7 +541,7 @@ static void network_state_change(void){
 static int get_number_of_udp_proxy(const LinphoneCore* lc) {
 	int number_of_udp_proxy=0;
 	LinphoneProxyConfig* proxy_cfg;
-	const MSList* proxys;
+	const bctbx_list_t* proxys;
 	for (proxys=linphone_core_get_proxy_config_list(lc);proxys!=NULL;proxys=proxys->next) {
 			proxy_cfg=(LinphoneProxyConfig*)proxys->data;
 			if (strcmp("udp",linphone_proxy_config_get_transport(proxy_cfg))==0)
@@ -567,7 +567,7 @@ static void transport_change(void){
 		register_ok=counters->number_of_LinphoneRegistrationOk;
 
 		number_of_udp_proxy=get_number_of_udp_proxy(lc);
-		total_number_of_proxies=ms_list_size(linphone_core_get_proxy_config_list(lc));
+		total_number_of_proxies=bctbx_list_size(linphone_core_get_proxy_config_list(lc));
 		linphone_core_get_sip_transports(lc,&sip_tr_orig);
 
 		sip_tr.udp_port=sip_tr_orig.udp_port;
@@ -593,7 +593,7 @@ static void proxy_transport_change(void){
 
 	register_with_refresh_base(lcm->lc,FALSE,auth_domain,NULL);
 
-	linphone_core_get_default_proxy(lcm->lc,&proxy_config);
+	proxy_config = linphone_core_get_default_proxy_config(lcm->lc);
 	reset_counters(counters); /*clear stats*/
 	linphone_proxy_config_edit(proxy_config);
 
@@ -630,7 +630,7 @@ static void proxy_transport_change_with_wrong_port(void) {
 
 	register_with_refresh_base_3(lcm->lc, FALSE, auth_domain, "sip2.linphone.org:5987", 0,transport,LinphoneRegistrationProgress);
 
-	linphone_core_get_default_proxy(lcm->lc,&proxy_config);
+	proxy_config = linphone_core_get_default_proxy_config(lcm->lc);
 	linphone_proxy_config_edit(proxy_config);
 
 	BC_ASSERT_FALSE(wait_for_until(lcm->lc,lcm->lc,&counters->number_of_LinphoneRegistrationCleared,1,3000));
@@ -661,7 +661,7 @@ static void proxy_transport_change_with_wrong_port_givin_up(void) {
 
 	register_with_refresh_base_3(lcm->lc, FALSE, auth_domain, "sip2.linphone.org:5987", 0,transport,LinphoneRegistrationProgress);
 
-	linphone_core_get_default_proxy(lcm->lc,&proxy_config);
+	proxy_config = linphone_core_get_default_proxy_config(lcm->lc);
 	linphone_proxy_config_edit(proxy_config);
 
 	BC_ASSERT_FALSE(wait_for_until(lcm->lc,lcm->lc,&counters->number_of_LinphoneRegistrationCleared,1,3000));
@@ -734,12 +734,12 @@ static void io_recv_error_late_recovery(void){
 		int register_ok;
 		stats* counters ;
 		int number_of_udp_proxy=0;
-		MSList* lcs;
+		bctbx_list_t* lcs;
 		lcm=linphone_core_manager_new2( "multi_account_rc",FALSE); /*to make sure iterates are not call yet*/
 		lc=lcm->lc;
 		sal_set_refresher_retry_after(lc->sal,1000);
 		counters=&lcm->stat;
-		BC_ASSERT_TRUE(wait_for(lcm->lc,lcm->lc,&counters->number_of_LinphoneRegistrationOk,ms_list_size(linphone_core_get_proxy_config_list(lcm->lc))));
+		BC_ASSERT_TRUE(wait_for(lcm->lc,lcm->lc,&counters->number_of_LinphoneRegistrationOk,bctbx_list_size(linphone_core_get_proxy_config_list(lcm->lc))));
 
 
 		counters = get_stats(lc);
@@ -752,12 +752,12 @@ static void io_recv_error_late_recovery(void){
 		BC_ASSERT_TRUE(wait_for(lc,NULL,&counters->number_of_LinphoneRegistrationProgress,(register_ok-number_of_udp_proxy)+register_ok /*because 1 udp*/));
 		BC_ASSERT_EQUAL(counters->number_of_LinphoneRegistrationFailed,0,int,"%d");
 
-		BC_ASSERT_TRUE(wait_for_list(lcs=ms_list_append(NULL,lc),&counters->number_of_LinphoneRegistrationFailed,(register_ok-number_of_udp_proxy),sal_get_refresher_retry_after(lc->sal)+3000));
+		BC_ASSERT_TRUE(wait_for_list(lcs=bctbx_list_append(NULL,lc),&counters->number_of_LinphoneRegistrationFailed,(register_ok-number_of_udp_proxy),sal_get_refresher_retry_after(lc->sal)+3000));
 
 		sal_set_recv_error(lc->sal, 1); /*reset*/
 		sal_set_send_error(lc->sal, 0);
 
-		BC_ASSERT_TRUE(wait_for_list(lcs=ms_list_append(NULL,lc),&counters->number_of_LinphoneRegistrationOk,register_ok-number_of_udp_proxy +register_ok,sal_get_refresher_retry_after(lc->sal)+3000));
+		BC_ASSERT_TRUE(wait_for_list(lcs=bctbx_list_append(NULL,lc),&counters->number_of_LinphoneRegistrationOk,register_ok-number_of_udp_proxy +register_ok,sal_get_refresher_retry_after(lc->sal)+3000));
 		linphone_core_manager_destroy(lcm);
 	}
 }
@@ -767,7 +767,7 @@ static void io_recv_error_without_active_register(void){
 	LinphoneCore* lc;
 	int register_ok;
 	stats* counters ;
-	MSList* proxys;
+	bctbx_list_t* proxys;
 	int dummy=0;
 
 	lcm=configure_lcm();
@@ -777,13 +777,13 @@ static void io_recv_error_without_active_register(void){
 
 		register_ok=counters->number_of_LinphoneRegistrationOk;
 
-		for (proxys=ms_list_copy(linphone_core_get_proxy_config_list(lc));proxys!=NULL;proxys=proxys->next) {
+		for (proxys=bctbx_list_copy(linphone_core_get_proxy_config_list(lc));proxys!=NULL;proxys=proxys->next) {
 			LinphoneProxyConfig* proxy_cfg=(LinphoneProxyConfig*)proxys->data;
 			linphone_proxy_config_edit(proxy_cfg);
 			linphone_proxy_config_enableregister(proxy_cfg,FALSE);
 			linphone_proxy_config_done(proxy_cfg);
 		}
-		ms_list_free(proxys);
+		bctbx_list_free(proxys);
 		/*wait for unregistrations*/
 		BC_ASSERT_TRUE(wait_for(lc,lc,&counters->number_of_LinphoneRegistrationCleared,register_ok /*because 1 udp*/));
 
@@ -791,7 +791,7 @@ static void io_recv_error_without_active_register(void){
 
 		/*nothing should happen because no active registration*/
 		wait_for_until(lc,lc, &dummy, 1, 3000);
-		BC_ASSERT_EQUAL(counters->number_of_LinphoneRegistrationProgress, ms_list_size(linphone_core_get_proxy_config_list(lc)), int, "%d");
+		BC_ASSERT_EQUAL(counters->number_of_LinphoneRegistrationProgress, bctbx_list_size(linphone_core_get_proxy_config_list(lc)), int, "%d");
 
 		BC_ASSERT_EQUAL(counters->number_of_LinphoneRegistrationFailed,0,int,"%d");
 
@@ -839,7 +839,7 @@ static void tls_with_non_tls_server(void){
 		lcm=linphone_core_manager_new2( "marie_rc", 0);
 		lc=lcm->lc;
 		sal_set_transport_timeout(lc->sal,3000);
-		linphone_core_get_default_proxy(lc,&proxy_cfg);
+		proxy_cfg = linphone_core_get_default_proxy_config(lc);
 		linphone_proxy_config_edit(proxy_cfg);
 		addr=linphone_address_new(linphone_proxy_config_get_addr(proxy_cfg));
 		snprintf(tmp,sizeof(tmp),"sip:%s:%i;transport=tls"	,linphone_address_get_domain(addr)
@@ -900,40 +900,40 @@ static void redirect(void){
 }
 
 test_t register_tests[] = {
-	{ "Simple register", simple_register },
-	{ "Simple register unregister", simple_unregister },
-	{ "TCP register", simple_tcp_register },
-	{ "Register with custom headers", register_with_custom_headers },
-	{ "TCP register compatibility mode", simple_tcp_register_compatibility_mode },
-	{ "TLS register", simple_tls_register },
-	{ "TLS register with alt. name certificate", tls_alt_name_register },
-	{ "TLS register with wildcard certificate", tls_wildcard_register },
-	{ "TLS certificate not verified",tls_certificate_failure},
-	{ "TLS with non tls server",tls_with_non_tls_server},
-	{ "Simple authenticated register", simple_authenticated_register },
-	{ "Ha1 authenticated register", ha1_authenticated_register },
-	{ "Digest auth without initial credentials", authenticated_register_with_no_initial_credentials },
-	{ "Digest auth with wrong credentials", authenticated_register_with_wrong_credentials },
-	{ "Digest auth with wrong credentials, check if registration attempts are stopped", authenticated_register_with_wrong_credentials_2 },
-	{ "Digest auth with wrong credentials without 403", authenticated_register_with_wrong_credentials_without_403},
-	{ "Authenticated register with wrong late credentials", authenticated_register_with_wrong_late_credentials},
-	{ "Authenticated register with late credentials", authenticated_register_with_late_credentials },
-	{ "Authenticated register with provided credentials", authenticated_register_with_provided_credentials },
-	{ "Register with refresh", simple_register_with_refresh },
-	{ "Authenticated register with refresh", simple_auth_register_with_refresh },
-	{ "Register with refresh and send error", register_with_refresh_with_send_error },
-	{ "Multi account", multiple_proxy },
-	{ "Transport changes", transport_change },
-	{ "Proxy transport changes", proxy_transport_change},
-	{ "Proxy transport changes with wrong address at first", proxy_transport_change_with_wrong_port},
-	{ "Proxy transport changes with wrong address, giving up",proxy_transport_change_with_wrong_port_givin_up},
-	{ "Change expires", change_expires},
-	{ "Network state change", network_state_change },
-	{ "Io recv error", io_recv_error },
-	{ "Io recv error with recovery", io_recv_error_retry_immediatly},
-	{ "Io recv error with late recovery", io_recv_error_late_recovery},
-	{ "Io recv error without active registration", io_recv_error_without_active_register},
-	{ "Simple redirect", redirect}
+	TEST_NO_TAG("Simple register", simple_register),
+	TEST_NO_TAG("Simple register unregister", simple_unregister),
+	TEST_NO_TAG("TCP register", simple_tcp_register),
+	TEST_NO_TAG("Register with custom headers", register_with_custom_headers),
+	TEST_NO_TAG("TCP register compatibility mode", simple_tcp_register_compatibility_mode),
+	TEST_NO_TAG("TLS register", simple_tls_register),
+	TEST_NO_TAG("TLS register with alt. name certificate", tls_alt_name_register),
+	TEST_NO_TAG("TLS register with wildcard certificate", tls_wildcard_register),
+	TEST_NO_TAG("TLS certificate not verified",tls_certificate_failure),
+	TEST_NO_TAG("TLS with non tls server",tls_with_non_tls_server),
+	TEST_NO_TAG("Simple authenticated register", simple_authenticated_register),
+	TEST_NO_TAG("Ha1 authenticated register", ha1_authenticated_register),
+	TEST_NO_TAG("Digest auth without initial credentials", authenticated_register_with_no_initial_credentials),
+	TEST_NO_TAG("Digest auth with wrong credentials", authenticated_register_with_wrong_credentials),
+	TEST_NO_TAG("Digest auth with wrong credentials, check if registration attempts are stopped", authenticated_register_with_wrong_credentials_2),
+	TEST_NO_TAG("Digest auth with wrong credentials without 403", authenticated_register_with_wrong_credentials_without_403),
+	TEST_NO_TAG("Authenticated register with wrong late credentials", authenticated_register_with_wrong_late_credentials),
+	TEST_NO_TAG("Authenticated register with late credentials", authenticated_register_with_late_credentials),
+	TEST_NO_TAG("Authenticated register with provided credentials", authenticated_register_with_provided_credentials),
+	TEST_NO_TAG("Register with refresh", simple_register_with_refresh),
+	TEST_NO_TAG("Authenticated register with refresh", simple_auth_register_with_refresh),
+	TEST_NO_TAG("Register with refresh and send error", register_with_refresh_with_send_error),
+	TEST_NO_TAG("Multi account", multiple_proxy),
+	TEST_NO_TAG("Transport changes", transport_change),
+	TEST_NO_TAG("Proxy transport changes", proxy_transport_change),
+	TEST_NO_TAG("Proxy transport changes with wrong address at first", proxy_transport_change_with_wrong_port),
+	TEST_NO_TAG("Proxy transport changes with wrong address, giving up",proxy_transport_change_with_wrong_port_givin_up),
+	TEST_NO_TAG("Change expires", change_expires),
+	TEST_NO_TAG("Network state change", network_state_change),
+	TEST_NO_TAG("Io recv error", io_recv_error),
+	TEST_NO_TAG("Io recv error with recovery", io_recv_error_retry_immediatly),
+	TEST_NO_TAG("Io recv error with late recovery", io_recv_error_late_recovery),
+	TEST_NO_TAG("Io recv error without active registration", io_recv_error_without_active_register),
+	TEST_NO_TAG("Simple redirect", redirect)
 };
 
 test_suite_t register_test_suite = {"Register", NULL, NULL, liblinphone_tester_before_each, liblinphone_tester_after_each,
