@@ -35,12 +35,15 @@ static void core_init_test(void) {
 	lc = linphone_core_new(&v_table,NULL,NULL,NULL);
 	/* until we have good certificates on our test server... */
 	linphone_core_verify_server_certificates(lc,FALSE);
-	BC_ASSERT_PTR_NOT_NULL_FATAL(lc);
-	linphone_core_destroy(lc);
+	if (BC_ASSERT_PTR_NOT_NULL(lc)) {
+		linphone_core_destroy(lc);
+	}
 }
 
 static void linphone_address_test(void) {
 	linphone_address_destroy(create_linphone_address(NULL));
+	BC_ASSERT_PTR_NULL(linphone_address_new("sip:@sip.linphone.org"));
+
 }
 
 static void core_sip_transport_test(void) {
@@ -49,7 +52,7 @@ static void core_sip_transport_test(void) {
 	LCSipTransports tr;
 	memset (&v_table,0,sizeof(v_table));
 	lc = linphone_core_new(&v_table,NULL,NULL,NULL);
-	BC_ASSERT_PTR_NOT_NULL_FATAL(lc);
+	if (!BC_ASSERT_PTR_NOT_NULL(lc)) return;
 	linphone_core_get_sip_transports(lc,&tr);
 	BC_ASSERT_EQUAL(tr.udp_port,5060, int, "%d"); /*default config*/
 	BC_ASSERT_EQUAL(tr.tcp_port,5060, int, "%d"); /*default config*/
@@ -77,21 +80,51 @@ static void linphone_interpret_url_test(void)
 	LinphoneCore* lc;
 	const char* sips_address = "sips:margaux@sip.linphone.org";
 	LinphoneAddress* address;
-
+	LinphoneProxyConfig *proxy_config;
+	char *tmp;
 	memset ( &v_table,0,sizeof ( v_table ) );
 	lc = linphone_core_new ( &v_table,NULL,NULL,NULL );
-	BC_ASSERT_PTR_NOT_NULL_FATAL ( lc );
+	if (!BC_ASSERT_PTR_NOT_NULL( lc )) return;
+
+	proxy_config =linphone_core_create_proxy_config(lc);
+	linphone_proxy_config_set_identity(proxy_config, "sip:moi@sip.linphone.org");
+	linphone_proxy_config_enable_register(proxy_config, FALSE);
+	linphone_proxy_config_set_server_addr(proxy_config,"sip:sip.linphone.org");
+	linphone_core_add_proxy_config(lc, proxy_config);
+	linphone_core_set_default_proxy_config(lc,proxy_config);
+	linphone_proxy_config_unref(proxy_config);
 
 	address = linphone_core_interpret_url(lc, sips_address);
-
-	BC_ASSERT_PTR_NOT_NULL_FATAL(address);
-	BC_ASSERT_STRING_EQUAL_FATAL(linphone_address_get_scheme(address), "sips");
-	BC_ASSERT_STRING_EQUAL_FATAL(linphone_address_get_username(address), "margaux");
-	BC_ASSERT_STRING_EQUAL_FATAL(linphone_address_get_domain(address), "sip.linphone.org");
-
+	BC_ASSERT_PTR_NOT_NULL(address);
+	BC_ASSERT_STRING_EQUAL(linphone_address_get_scheme(address), "sips");
+	BC_ASSERT_STRING_EQUAL(linphone_address_get_username(address), "margaux");
+	BC_ASSERT_STRING_EQUAL(linphone_address_get_domain(address), "sip.linphone.org");
 	linphone_address_destroy(address);
 
-	linphone_core_destroy ( lc );
+	address = linphone_core_interpret_url(lc,"23");
+	BC_ASSERT_PTR_NOT_NULL(address);
+	BC_ASSERT_STRING_EQUAL(linphone_address_get_scheme(address), "sip");
+	BC_ASSERT_STRING_EQUAL(linphone_address_get_username(address), "23");
+	BC_ASSERT_STRING_EQUAL(linphone_address_get_domain(address), "sip.linphone.org");
+	linphone_address_destroy(address);
+
+	address = linphone_core_interpret_url(lc,"#24");
+	BC_ASSERT_PTR_NOT_NULL(address);
+	BC_ASSERT_STRING_EQUAL(linphone_address_get_scheme(address), "sip");
+	BC_ASSERT_STRING_EQUAL(linphone_address_get_username(address), "#24");
+	BC_ASSERT_STRING_EQUAL(linphone_address_get_domain(address), "sip.linphone.org");
+	tmp = linphone_address_as_string(address);
+	BC_ASSERT_TRUE(strcmp (tmp,"sip:%2324@sip.linphone.org") == 0);
+	linphone_address_destroy(address);
+
+	address = linphone_core_interpret_url(lc,tmp);
+	BC_ASSERT_STRING_EQUAL(linphone_address_get_scheme(address), "sip");
+	BC_ASSERT_STRING_EQUAL(linphone_address_get_username(address), "#24");
+	BC_ASSERT_STRING_EQUAL(linphone_address_get_domain(address), "sip.linphone.org");
+	linphone_address_destroy(address);
+	ms_free(tmp);
+
+	linphone_core_destroy (lc);
 }
 
 static void linphone_lpconfig_from_buffer(void){
@@ -240,7 +273,7 @@ static void chat_room_test(void) {
 	LinphoneCore* lc;
 	memset (&v_table,0,sizeof(v_table));
 	lc = linphone_core_new(&v_table,NULL,NULL,NULL);
-	BC_ASSERT_PTR_NOT_NULL_FATAL(lc);
+	if (!BC_ASSERT_PTR_NOT_NULL(lc)) return;
 	BC_ASSERT_PTR_NOT_NULL(linphone_core_get_chat_room_from_uri(lc,"sip:toto@titi.com"));
 	linphone_core_destroy(lc);
 }
@@ -260,7 +293,13 @@ static void devices_reload_test(void) {
 	devid1 = ms_strdup(linphone_core_get_video_device(mgr->lc));
 	linphone_core_reload_video_devices(mgr->lc);
 	devid2 = ms_strdup(linphone_core_get_video_device(mgr->lc));
-	BC_ASSERT_STRING_EQUAL(devid1, devid2);
+
+	if (devid1 && devid2) {
+		BC_ASSERT_STRING_EQUAL(devid1, devid2);
+	} else {
+		BC_ASSERT_PTR_NULL(devid1);
+		BC_ASSERT_PTR_NULL(devid2);
+	}
 	ms_free(devid1);
 	ms_free(devid2);
 
@@ -291,21 +330,68 @@ end:
 	linphone_core_manager_destroy(mgr);
 }
 
+/*this test checks default codec list, assuming VP8 and H264 are both supported.
+ * - with an empty config, the order must be as expected: VP8 first, H264 second.
+ * - with a config that references only H264, VP8 must be added automatically as first codec.
+ * - with a config that references only VP8, H264 must be added in second position.
+**/
+static void codec_setup(void){
+	LinphoneCoreManager *mgr = linphone_core_manager_new2("empty_rc", FALSE);
+	PayloadType *vp8, *h264;
+	const bctbx_list_t *codecs;
+	if ((vp8 = linphone_core_find_payload_type(mgr->lc, "VP8", 90000, -1)) == NULL ||
+		(h264 = linphone_core_find_payload_type(mgr->lc, "H264", 90000, -1)) == NULL){
+		linphone_core_manager_destroy(mgr);
+		ms_error("H264 or VP8 not available, test skipped.");
+		BC_PASS("H264 or VP8 not available, test skipped.");
+		return;
+	}
+	codecs = linphone_core_get_video_codecs(mgr->lc);
+	BC_ASSERT_TRUE(bctbx_list_size(codecs)>=2);
+	BC_ASSERT_TRUE(codecs->data == vp8);
+	BC_ASSERT_TRUE(codecs->next->data == h264);
+	linphone_core_manager_destroy(mgr);
+	
+	mgr = linphone_core_manager_new2("marie_h264_rc", FALSE);
+	vp8 = linphone_core_find_payload_type(mgr->lc, "VP8", 90000, -1);
+	h264 = linphone_core_find_payload_type(mgr->lc, "H264", 90000, -1);
+	codecs = linphone_core_get_video_codecs(mgr->lc);
+	BC_ASSERT_TRUE(bctbx_list_size(codecs)>=2);
+	BC_ASSERT_PTR_NOT_NULL(vp8);
+	BC_ASSERT_PTR_NOT_NULL(h264);
+	BC_ASSERT_TRUE(codecs->data == vp8);
+	BC_ASSERT_TRUE(codecs->next->data == h264);
+	linphone_core_manager_destroy(mgr);
+	
+	mgr = linphone_core_manager_new2("marie_rc", FALSE);
+	vp8 = linphone_core_find_payload_type(mgr->lc, "VP8", 90000, -1);
+	h264 = linphone_core_find_payload_type(mgr->lc, "H264", 90000, -1);
+	codecs = linphone_core_get_video_codecs(mgr->lc);
+	BC_ASSERT_TRUE(bctbx_list_size(codecs)>=2);
+	BC_ASSERT_PTR_NOT_NULL(vp8);
+	BC_ASSERT_PTR_NOT_NULL(h264);
+	BC_ASSERT_TRUE(codecs->data == vp8);
+	BC_ASSERT_TRUE(codecs->next->data == h264);
+	linphone_core_manager_destroy(mgr);
+	
+}
+
 test_t setup_tests[] = {
-	{ "Version check", linphone_version_test },
-	{ "Linphone Address", linphone_address_test },
-	{ "Linphone proxy config address equal (internal api)", linphone_proxy_config_address_equal_test},
-	{ "Linphone proxy config server address change (internal api)", linphone_proxy_config_is_server_config_changed_test},
-	{ "Linphone core init/uninit", core_init_test },
-	{ "Linphone random transport port",core_sip_transport_test},
-	{ "Linphone interpret url", linphone_interpret_url_test },
-	{ "LPConfig from buffer", linphone_lpconfig_from_buffer },
-	{ "LPConfig zero_len value from buffer", linphone_lpconfig_from_buffer_zerolen_value },
-	{ "LPConfig zero_len value from file", linphone_lpconfig_from_file_zerolen_value },
-	{ "LPConfig zero_len value from XML", linphone_lpconfig_from_xml_zerolen_value },
-	{ "Chat room", chat_room_test },
-	{ "Devices reload", devices_reload_test },
-	{ "Codec usability", codec_usability_test }
+	TEST_NO_TAG("Version check", linphone_version_test),
+	TEST_NO_TAG("Linphone Address", linphone_address_test),
+	TEST_NO_TAG("Linphone proxy config address equal (internal api)", linphone_proxy_config_address_equal_test),
+	TEST_NO_TAG("Linphone proxy config server address change (internal api)", linphone_proxy_config_is_server_config_changed_test),
+	TEST_NO_TAG("Linphone core init/uninit", core_init_test),
+	TEST_NO_TAG("Linphone random transport port",core_sip_transport_test),
+	TEST_NO_TAG("Linphone interpret url", linphone_interpret_url_test),
+	TEST_NO_TAG("LPConfig from buffer", linphone_lpconfig_from_buffer),
+	TEST_NO_TAG("LPConfig zero_len value from buffer", linphone_lpconfig_from_buffer_zerolen_value),
+	TEST_NO_TAG("LPConfig zero_len value from file", linphone_lpconfig_from_file_zerolen_value),
+	TEST_NO_TAG("LPConfig zero_len value from XML", linphone_lpconfig_from_xml_zerolen_value),
+	TEST_NO_TAG("Chat room", chat_room_test),
+	TEST_NO_TAG("Devices reload", devices_reload_test),
+	TEST_NO_TAG("Codec usability", codec_usability_test),
+	TEST_NO_TAG("Codec setup", codec_setup)
 };
 
 test_suite_t setup_test_suite = {"Setup", NULL, NULL, liblinphone_tester_before_each, liblinphone_tester_after_each,
