@@ -17,22 +17,31 @@
  */
 
 #include <stdio.h>
-#include "CUnit/TestRun.h"
-#include "CUnit/Automated.h"
 #include "linphonecore.h"
 #include "private.h"
 #include "liblinphone_tester.h"
-#if HAVE_CU_CURSES
-#include "CUnit/CUCurses.h"
+#include <bctoolbox/tester.h>
+
+#if __clang__ || ((__GNUC__ == 4 && __GNUC_MINOR__ >= 6) || __GNUC__ > 4)
+#pragma GCC diagnostic push
 #endif
+#ifndef _MSC_VER
+#pragma GCC diagnostic ignored "-Wstrict-prototypes"
+#endif
+
 #ifdef HAVE_GTK
 #include <gtk/gtk.h>
 #endif
+
+#if __clang__ || ((__GNUC__ == 4 && __GNUC_MINOR__ >= 6) || __GNUC__ > 4)
+#pragma GCC diagnostic pop
+#endif
+
 #if _WIN32
 #define unlink _unlink
 #endif
 
-static bool_t liblinphone_tester_ipv6_enabled=FALSE;
+
 static int liblinphone_tester_keep_accounts_flag = 0;
 static int liblinphone_tester_keep_record_files = FALSE;
 static int liblinphone_tester_leak_detector_disabled = FALSE;
@@ -46,6 +55,7 @@ const char* test_username="liblinphone_tester";
 const char* test_password="secret";
 const char* test_route="sip2.linphone.org";
 const char *userhostsfile = "tester_hosts";
+bool_t liblinphonetester_ipv6 = FALSE;
 
 const char *liblinphone_tester_mire_id="Mire: Mire (synthetic moving picture)";
 
@@ -70,13 +80,10 @@ bool_t liblinphone_tester_clock_elapsed(const MSTimeSpec *start, int value_ms){
 	return FALSE;
 }
 
-void liblinphone_tester_enable_ipv6(bool_t enabled){
-	liblinphone_tester_ipv6_enabled=enabled;
-}
 
 LinphoneAddress * create_linphone_address(const char * domain) {
 	LinphoneAddress *addr = linphone_address_new(NULL);
-	BC_ASSERT_PTR_NOT_NULL_FATAL(addr);
+	if (!BC_ASSERT_PTR_NOT_NULL(addr)) return NULL;
 	linphone_address_set_username(addr,test_username);
 	BC_ASSERT_STRING_EQUAL(test_username,linphone_address_get_username(addr));
 	if (!domain) domain= test_route;
@@ -112,6 +119,7 @@ LinphoneCore* configure_lc_from(LinphoneCoreVTable* v_table, const char* path, c
 	char *rootcapath       = NULL;
 	char *dnsuserhostspath = NULL;
 	char *nowebcampath     = NULL;
+	char *chatdb     = NULL;
 
 	if (path==NULL) path=".";
 
@@ -129,7 +137,7 @@ LinphoneCore* configure_lc_from(LinphoneCoreVTable* v_table, const char* path, c
 	ringbackpath     = ms_strdup_printf("%s/sounds/ringback.wav", path);
 	nowebcampath     = ms_strdup_printf("%s/images/nowebcamCIF.jpg", path);
 	rootcapath       = ms_strdup_printf("%s/certificates/cn/cafile.pem", path);
-	dnsuserhostspath = ms_strdup_printf( "%s/%s", path, userhostsfile);
+	dnsuserhostspath = ms_strdup_printf("%s/%s", path, userhostsfile);
 
 
 	if( config != NULL ) {
@@ -144,18 +152,24 @@ LinphoneCore* configure_lc_from(LinphoneCoreVTable* v_table, const char* path, c
 		linphone_core_set_ringback(lc, ringbackpath);
 		linphone_core_set_root_ca(lc,rootcapath);
 	}
+	chatdb = ms_strdup_printf("%s/messages-%p.db",bc_tester_get_writable_dir_prefix(),lc);
+
+	linphone_core_enable_ipv6(lc, liblinphonetester_ipv6);
 
 	sal_enable_test_features(lc->sal,TRUE);
 	sal_set_dns_user_hosts_file(lc->sal, dnsuserhostspath);
+#ifdef VIDEO_ENABLED
 	linphone_core_set_static_picture(lc,nowebcampath);
+#endif
 
-	linphone_core_enable_ipv6(lc, liblinphone_tester_ipv6_enabled);
+	linphone_core_set_chat_database_path(lc, chatdb);
 
 	ms_free(ringpath);
 	ms_free(ringbackpath);
 	ms_free(nowebcampath);
 	ms_free(rootcapath);
 	ms_free(dnsuserhostspath);
+	ms_free(chatdb);
 
 	if( filepath ) ms_free(filepath);
 
@@ -166,14 +180,14 @@ LinphoneCore* configure_lc_from(LinphoneCoreVTable* v_table, const char* path, c
 
 
 bool_t wait_for_until(LinphoneCore* lc_1, LinphoneCore* lc_2,int* counter,int value,int timout) {
-	MSList* lcs=NULL;
+	bctbx_list_t* lcs=NULL;
 	bool_t result;
 	if (lc_1)
-		lcs=ms_list_append(lcs,lc_1);
+		lcs=bctbx_list_append(lcs,lc_1);
 	if (lc_2)
-		lcs=ms_list_append(lcs,lc_2);
+		lcs=bctbx_list_append(lcs,lc_2);
 	result=wait_for_list(lcs,counter,value,timout);
-	ms_list_free(lcs);
+	bctbx_list_free(lcs);
 	return result;
 }
 
@@ -181,8 +195,8 @@ bool_t wait_for(LinphoneCore* lc_1, LinphoneCore* lc_2,int* counter,int value) {
 	return wait_for_until(lc_1, lc_2,counter,value,10000);
 }
 
-bool_t wait_for_list(MSList* lcs,int* counter,int value,int timeout_ms) {
-	MSList* iterator;
+bool_t wait_for_list(bctbx_list_t* lcs,int* counter,int value,int timeout_ms) {
+	bctbx_list_t* iterator;
 	MSTimeSpec start;
 
 	liblinphone_tester_clock_start(&start);
@@ -210,17 +224,28 @@ bool_t wait_for_list(MSList* lcs,int* counter,int value,int timeout_ms) {
 	else return TRUE;
 }
 
+bool_t wait_for_stun_resolution(LinphoneCoreManager *m) {
+	MSTimeSpec start;
+	int timeout_ms = 10000;
+	liblinphone_tester_clock_start(&start);
+	while (linphone_core_get_stun_server_addrinfo(m->lc) == NULL && !liblinphone_tester_clock_elapsed(&start,timeout_ms)) {
+		linphone_core_iterate(m->lc);
+		ms_usleep(20000);
+	}
+	return linphone_core_get_stun_server_addrinfo(m->lc) != NULL;
+}
+
 static void set_codec_enable(LinphoneCore* lc,const char* type,int rate,bool_t enable) {
-	MSList* codecs=ms_list_copy(linphone_core_get_audio_codecs(lc));
-	MSList* codecs_it;
+	bctbx_list_t* codecs=bctbx_list_copy(linphone_core_get_audio_codecs(lc));
+	bctbx_list_t* codecs_it;
 	PayloadType* pt;
 	for (codecs_it=codecs;codecs_it!=NULL;codecs_it=codecs_it->next) {
 		linphone_core_enable_payload_type(lc,(PayloadType*)codecs_it->data,0);
 	}
-	if((pt = linphone_core_find_payload_type(lc,type,rate,1))) {
+	if ((pt = linphone_core_find_payload_type(lc,type,rate,1))) {
 		linphone_core_enable_payload_type(lc,pt, enable);
 	}
-	ms_list_free(codecs);
+	bctbx_list_free(codecs);
 }
 
 static void enable_codec(LinphoneCore* lc,const char* type,int rate) {
@@ -237,23 +262,25 @@ LinphoneCoreManager *get_manager(LinphoneCore *lc){
 }
 
 bool_t transport_supported(LinphoneTransportType transport) {
-	Sal *sal = sal_init();
+	Sal *sal = sal_init(NULL);
 	bool_t supported = sal_transport_available(sal,(SalTransport)transport);
 	if (!supported) ms_message("TLS transport not supported, falling back to TCP if possible otherwise skipping test.");
 	sal_uninit(sal);
 	return supported;
 }
 
-
-static void display_status(LinphoneCore *lc, const char *status){
-	ms_message("display_status(): %s",status);
-}
-
-LinphoneCoreManager* linphone_core_manager_init(const char* rc_file) {
-	LinphoneCoreManager* mgr= ms_new0(LinphoneCoreManager,1);
+#if __clang__ || ((__GNUC__ == 4 && __GNUC_MINOR__ >= 6) || __GNUC__ > 4)
+#pragma GCC diagnostic push
+#endif
+#ifdef _MSC_VER
+#pragma warning(disable : 4996)
+#else
+#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#endif
+void linphone_core_manager_init(LinphoneCoreManager *mgr, const char* rc_file) {
 	char *rc_path = NULL;
 	char *hellopath = bc_tester_res("sounds/hello8000.wav");
-	mgr->number_of_cunit_error_at_creation = CU_get_number_of_failures();
+	mgr->number_of_cunit_error_at_creation =  bc_get_number_of_failures();
 	mgr->v_table.registration_state_changed=registration_state_changed;
 	mgr->v_table.auth_info_requested=auth_info_requested;
 	mgr->v_table.call_state_changed=call_state_changed;
@@ -272,7 +299,6 @@ LinphoneCoreManager* linphone_core_manager_init(const char* rc_file) {
 	mgr->v_table.network_reachable=network_reachable;
 	mgr->v_table.dtmf_received=dtmf_received;
 	mgr->v_table.call_stats_updated=call_stats_updated;
-	mgr->v_table.display_status=display_status;
 
 	reset_counters(&mgr->stat);
 	if (rc_file) rc_path = ms_strdup_printf("rcfiles/%s", rc_file);
@@ -292,13 +318,13 @@ LinphoneCoreManager* linphone_core_manager_init(const char* rc_file) {
 	{
 		MSWebCam *cam;
 
-		cam = ms_web_cam_manager_get_cam(ms_web_cam_manager_get(), "Mire: Mire (synthetic moving picture)");
+		cam = ms_web_cam_manager_get_cam(ms_factory_get_web_cam_manager(mgr->lc->factory), "Mire: Mire (synthetic moving picture)");
 
 		if (cam == NULL) {
 			MSWebCamDesc *desc = ms_mire_webcam_desc_get();
 			if (desc){
 				cam=ms_web_cam_new(desc);
-				ms_web_cam_manager_add_cam(ms_web_cam_manager_get(), cam);
+				ms_web_cam_manager_add_cam(ms_factory_get_web_cam_manager(mgr->lc->factory), cam);
 			}
 		}
 	}
@@ -317,21 +343,27 @@ LinphoneCoreManager* linphone_core_manager_init(const char* rc_file) {
 	}
 
 	linphone_core_set_user_certificates_path(mgr->lc,bc_tester_get_writable_dir_prefix());
+	/*for now, we need the periodical updates facility to compute bandwidth measurements correctly during tests*/
+	lp_config_set_int(linphone_core_get_config(mgr->lc), "misc", "send_call_stats_periodical_updates", 1);
 
 	if (rc_path) ms_free(rc_path);
-
-	return mgr;
 }
+#if __clang__ || ((__GNUC__ == 4 && __GNUC_MINOR__ >= 6) || __GNUC__ > 4)
+#pragma GCC diagnostic pop
+#endif
 
-void linphone_core_manager_start(LinphoneCoreManager *mgr, const char* rc_file, int check_for_proxies) {
+void linphone_core_manager_start(LinphoneCoreManager *mgr, int check_for_proxies) {
 	LinphoneProxyConfig* proxy;
 	int proxy_count;
 
-	/*BC_ASSERT_EQUAL(ms_list_size(linphone_core_get_proxy_config_list(lc)),proxy_count, int, "%d");*/
-	if (check_for_proxies && rc_file) /**/
-		proxy_count=ms_list_size(linphone_core_get_proxy_config_list(mgr->lc));
-	else
+	/*BC_ASSERT_EQUAL(bctbx_list_size(linphone_core_get_proxy_config_list(lc)),proxy_count, int, "%d");*/
+	if (check_for_proxies){ /**/
+		proxy_count=bctbx_list_size(linphone_core_get_proxy_config_list(mgr->lc));
+	}else{
 		proxy_count=0;
+		/*this is to prevent registration to go on*/
+		linphone_core_set_network_reachable(mgr->lc, FALSE);
+	}
 
 	if (proxy_count){
 #define REGISTER_TIMEOUT 20 /* seconds */
@@ -344,22 +376,33 @@ void linphone_core_manager_start(LinphoneCoreManager *mgr, const char* rc_file,
 	BC_ASSERT_EQUAL(mgr->stat.number_of_LinphoneRegistrationOk,proxy_count, int, "%d");
 	enable_codec(mgr->lc,"PCMU",8000);
 
-	linphone_core_get_default_proxy(mgr->lc,&proxy);
+	proxy = linphone_core_get_default_proxy_config(mgr->lc);
 	if (proxy) {
+		if (mgr->identity){
+			linphone_address_destroy(mgr->identity);
+		}
 		mgr->identity = linphone_address_clone(linphone_proxy_config_get_identity_address(proxy));
 		linphone_address_clean(mgr->identity);
 	}
+
+	linphone_core_manager_wait_for_stun_resolution(mgr);
+	if (!check_for_proxies){
+		/*now that stun server resolution is done, we can start registering*/
+		linphone_core_set_network_reachable(mgr->lc, TRUE);
+	}
 }
 
 LinphoneCoreManager* linphone_core_manager_new( const char* rc_file) {
-	LinphoneCoreManager *manager = linphone_core_manager_init(rc_file);
-	linphone_core_manager_start(manager, rc_file, TRUE);
+	LinphoneCoreManager *manager = ms_new0(LinphoneCoreManager, 1);
+	linphone_core_manager_init(manager, rc_file);
+	linphone_core_manager_start(manager, TRUE);
 	return manager;
 }
 
-LinphoneCoreManager* linphone_core_manager_new2( const char* rc_file, int check_for_proxies) {
-	LinphoneCoreManager *manager = linphone_core_manager_init(rc_file);
-	linphone_core_manager_start(manager, rc_file, check_for_proxies);
+LinphoneCoreManager* linphone_core_manager_new2(const char* rc_file, int check_for_proxies) {
+	LinphoneCoreManager *manager = ms_new0(LinphoneCoreManager, 1);
+	linphone_core_manager_init(manager, rc_file);
+	linphone_core_manager_start(manager, check_for_proxies);
 	return manager;
 }
 
@@ -370,38 +413,51 @@ void linphone_core_manager_stop(LinphoneCoreManager *mgr){
 	}
 }
 
-void linphone_core_manager_destroy(LinphoneCoreManager* mgr) {
+void linphone_core_manager_uninit(LinphoneCoreManager *mgr) {
 	if (mgr->stat.last_received_chat_message) {
 		linphone_chat_message_unref(mgr->stat.last_received_chat_message);
 	}
 	if (mgr->stat.last_received_info_message) linphone_info_message_destroy(mgr->stat.last_received_info_message);
 	if (mgr->lc){
 		const char *record_file=linphone_core_get_record_file(mgr->lc);
-		int unterminated_calls;
-
+		char *chatdb = ms_strdup(linphone_core_get_chat_database_path(mgr->lc));
 		if (!liblinphone_tester_keep_record_files && record_file){
-			if ((CU_get_number_of_failures()-mgr->number_of_cunit_error_at_creation)>0) {
+			if ((bc_get_number_of_failures()-mgr->number_of_cunit_error_at_creation)>0) {
 				ms_message ("Test has failed, keeping recorded file [%s]",record_file);
 			} else {
 				unlink(record_file);
 			}
 		}
-		BC_ASSERT_EQUAL((unterminated_calls=ms_list_size(mgr->lc->calls)), 0, int, "%i");
-		if (unterminated_calls != 0) {
-			ms_error("There are still %d calls pending, please terminates them before invoking linphone_core_manager_destroy().", unterminated_calls);
-		}
 		linphone_core_destroy(mgr->lc);
+		if (chatdb) {
+			unlink(chatdb);
+			ms_free(chatdb);
+		}
 	}
 	if (mgr->identity) {
 		linphone_address_destroy(mgr->identity);
 	}
-	manager_count--;
 
+	manager_count--;
+}
+
+void linphone_core_manager_wait_for_stun_resolution(LinphoneCoreManager *mgr) {
+	LinphoneNatPolicy *nat_policy = linphone_core_get_nat_policy(mgr->lc);
+	if ((nat_policy != NULL) && (linphone_nat_policy_get_stun_server(nat_policy) != NULL) &&
+		(linphone_nat_policy_stun_enabled(nat_policy) || linphone_nat_policy_turn_enabled(nat_policy)) &&
+		(linphone_nat_policy_ice_enabled(nat_policy))) {
+		/*before we go, ensure that the stun server is resolved, otherwise all ice related test will fail*/
+		BC_ASSERT_TRUE(wait_for_stun_resolution(mgr));
+	}
+}
+
+void linphone_core_manager_destroy(LinphoneCoreManager* mgr) {
+	linphone_core_manager_uninit(mgr);
 	ms_free(mgr);
 }
 
 int liblinphone_tester_ipv6_available(void){
-	struct addrinfo *ai=belle_sip_ip_address_to_addrinfo(AF_INET6,"2a01:e00::2",53);
+	struct addrinfo *ai=bctbx_ip_address_to_addrinfo(AF_INET6,SOCK_STREAM,"2a01:e00::2",53);
 	if (ai){
 		struct sockaddr_storage ss;
 		struct addrinfo src;
@@ -411,13 +467,32 @@ int liblinphone_tester_ipv6_available(void){
 		belle_sip_get_src_addr_for(ai->ai_addr,(socklen_t)ai->ai_addrlen,(struct sockaddr*) &ss,&slen,4444);
 		src.ai_addr=(struct sockaddr*) &ss;
 		src.ai_addrlen=slen;
-		belle_sip_addrinfo_to_ip(&src,localip, sizeof(localip),&port);
+		bctbx_addrinfo_to_ip_address(&src,localip, sizeof(localip),&port);
 		freeaddrinfo(ai);
 		return strcmp(localip,"::1")!=0;
 	}
 	return FALSE;
 }
 
+int liblinphone_tester_ipv4_available(void){
+	struct addrinfo *ai=bctbx_ip_address_to_addrinfo(AF_INET,SOCK_STREAM,"212.27.40.240",53);
+	if (ai){
+		struct sockaddr_storage ss;
+		struct addrinfo src;
+		socklen_t slen=sizeof(ss);
+		char localip[128];
+		int port=0;
+		belle_sip_get_src_addr_for(ai->ai_addr,(socklen_t)ai->ai_addrlen,(struct sockaddr*) &ss,&slen,4444);
+		src.ai_addr=(struct sockaddr*) &ss;
+		src.ai_addrlen=slen;
+		bctbx_addrinfo_to_ip_address(&src,localip, sizeof(localip),&port);
+		freeaddrinfo(ai);
+		return strcmp(localip,"127.0.0.1")!=0;
+	}
+	return FALSE;
+}
+
+
 void liblinphone_tester_keep_accounts( int keep ){
 	liblinphone_tester_keep_accounts_flag = keep;
 }
@@ -437,11 +512,15 @@ void liblinphone_tester_clear_accounts(void){
 void liblinphone_tester_add_suites() {
 	bc_tester_add_suite(&setup_test_suite);
 	bc_tester_add_suite(&register_test_suite);
+	bc_tester_add_suite(&tunnel_test_suite);
 	bc_tester_add_suite(&offeranswer_test_suite);
 	bc_tester_add_suite(&call_test_suite);
+	bc_tester_add_suite(&call_video_test_suite);
+	bc_tester_add_suite(&audio_bypass_suite);
 	bc_tester_add_suite(&multi_call_test_suite);
 	bc_tester_add_suite(&message_test_suite);
 	bc_tester_add_suite(&presence_test_suite);
+	bc_tester_add_suite(&presence_server_test_suite);
 #ifdef UPNP
 	bc_tester_add_suite(&upnp_test_suite);
 #endif
@@ -451,7 +530,6 @@ void liblinphone_tester_add_suites() {
 	bc_tester_add_suite(&remote_provisioning_test_suite);
 	bc_tester_add_suite(&quality_reporting_test_suite);
 	bc_tester_add_suite(&log_collection_test_suite);
-	bc_tester_add_suite(&tunnel_test_suite);
 	bc_tester_add_suite(&player_test_suite);
 	bc_tester_add_suite(&dtmf_test_suite);
 #if defined(VIDEO_ENABLED) && defined(HAVE_GTK)
@@ -462,6 +540,9 @@ void liblinphone_tester_add_suites() {
 #if HAVE_SIPP
 	bc_tester_add_suite(&complex_sip_call_test_suite);
 #endif
+#ifdef VCARD_ENABLED
+	bc_tester_add_suite(&vcard_test_suite);
+#endif
 }
 
 static int linphone_core_manager_get_max_audio_bw_base(const int array[],int array_size) {
@@ -505,20 +586,254 @@ void liblinphone_tester_before_each(void) {
 	}
 }
 
-void liblinphone_tester_after_each(void) {
+static char* all_leaks_buffer = NULL;
+
+int liblinphone_tester_after_each(void) {
+	int err = 0;
 	if (!liblinphone_tester_leak_detector_disabled){
 		int leaked_objects = belle_sip_object_get_object_count() - leaked_objects_count;
-		// this will NOT be counted in tests fail but at least it will be shown
-		BC_ASSERT_EQUAL(leaked_objects, 0, int, "%d");
 		if (leaked_objects > 0) {
+			char* format = ms_strdup_printf("%d object%s leaked in suite [%s] test [%s], please fix that!",
+											leaked_objects, leaked_objects>1?"s were":" was",
+											bc_tester_current_suite_name(), bc_tester_current_test_name());
 			belle_sip_object_dump_active_objects();
 			belle_sip_object_flush_active_objects();
-			ms_error("%d object%s leaked in latest test, please fix that!", leaked_objects, leaked_objects>1?"s were":"was");
+			bc_tester_printf(ORTP_MESSAGE, format);
+			ms_error("%s", format);
+
+			all_leaks_buffer = ms_strcat_printf(all_leaks_buffer, "\n%s", format);
+			ms_free(format);
+		}
+
+		// prevent any future leaks
+		{
+			const char **tags = bc_tester_current_test_tags();
+			int leaks_expected =
+				(tags && ((tags[0] && !strcmp(tags[0], "LeaksMemory")) || (tags[1] && !strcmp(tags[1], "LeaksMemory"))));
+			// if the test is NOT marked as leaking memory and it actually is, we should make it fail
+			if (!leaks_expected && leaked_objects > 0) {
+				BC_FAIL("This test is leaking memory!");
+				err = 1;
+				// and reciprocally
+			} else if (leaks_expected && leaked_objects == 0) {
+				BC_FAIL("This test is not leaking anymore, please remove LeaksMemory tag!");
+				// err = 1; // do not force fail actually, because it can be some false positive warning
+			}
 		}
 	}
 
 	if (manager_count != 0) {
 		ms_fatal("%d Linphone core managers are still alive!", manager_count);
 	}
+	return err;
 }
 
+void liblinphone_tester_uninit(void) {
+	// show all leaks that happened during the test
+	if (all_leaks_buffer) {
+		bc_tester_printf(ORTP_MESSAGE, all_leaks_buffer);
+		ms_free(all_leaks_buffer);
+		all_leaks_buffer = NULL;
+	}
+	bc_tester_uninit();
+}
+
+static void check_ice_from_rtp(LinphoneCall *c1, LinphoneCall *c2, LinphoneStreamType stream_type) {
+	MediaStream *ms;
+	switch (stream_type) {
+	case LinphoneStreamTypeAudio:
+		ms=&c1->audiostream->ms;
+		break;
+	case LinphoneStreamTypeVideo:
+		ms=&c1->videostream->ms;
+		break;
+	case LinphoneStreamTypeText:
+		ms=&c1->textstream->ms;
+		break;
+	default:
+		ms_error("Unknown stream type [%s]",  linphone_stream_type_to_string(stream_type));
+		BC_ASSERT_FALSE(stream_type >= LinphoneStreamTypeUnknown);
+		return;
+	}
+
+
+	if (linphone_call_get_audio_stats(c1)->ice_state == LinphoneIceStateHostConnection && media_stream_started(ms)) {
+		char ip[16];
+		char port[8];
+		getnameinfo((const struct sockaddr *)&c1->audiostream->ms.sessions.rtp_session->rtp.gs.rem_addr
+					, c1->audiostream->ms.sessions.rtp_session->rtp.gs.rem_addrlen
+					, ip
+					, sizeof(ip)
+					, port
+					, sizeof(port)
+					, NI_NUMERICHOST|NI_NUMERICSERV);
+		BC_ASSERT_STRING_EQUAL(ip, c2->media_localip);
+	}
+}
+bool_t check_ice(LinphoneCoreManager* caller, LinphoneCoreManager* callee, LinphoneIceState state) {
+	LinphoneCall *c1,*c2;
+	bool_t audio_success=FALSE;
+	bool_t video_success=FALSE;
+	bool_t text_success=FALSE;
+	bool_t video_enabled, realtime_text_enabled;
+	MSTimeSpec ts;
+
+	c1=linphone_core_get_current_call(caller->lc);
+	c2=linphone_core_get_current_call(callee->lc);
+
+	BC_ASSERT_PTR_NOT_NULL(c1);
+	BC_ASSERT_PTR_NOT_NULL(c2);
+	if (!c1 || !c2) return FALSE;
+	linphone_call_ref(c1);
+	linphone_call_ref(c2);
+
+	BC_ASSERT_EQUAL(linphone_call_params_video_enabled(linphone_call_get_current_params(c1)),linphone_call_params_video_enabled(linphone_call_get_current_params(c2)), int, "%d");
+	BC_ASSERT_EQUAL(linphone_call_params_realtime_text_enabled(linphone_call_get_current_params(c1)),linphone_call_params_realtime_text_enabled(linphone_call_get_current_params(c2)), int, "%d");
+	video_enabled=linphone_call_params_video_enabled(linphone_call_get_current_params(c1));
+	realtime_text_enabled=linphone_call_params_realtime_text_enabled(linphone_call_get_current_params(c1));
+	liblinphone_tester_clock_start(&ts);
+	do{
+		if ((c1 != NULL) && (c2 != NULL)) {
+			if (linphone_call_get_audio_stats(c1)->ice_state==state &&
+				linphone_call_get_audio_stats(c2)->ice_state==state ){
+				audio_success=TRUE;
+				check_ice_from_rtp(c1,c2,LinphoneStreamTypeAudio);
+				check_ice_from_rtp(c2,c1,LinphoneStreamTypeAudio);
+				break;
+			}
+			linphone_core_iterate(caller->lc);
+			linphone_core_iterate(callee->lc);
+		}
+		ms_usleep(20000);
+	}while(!liblinphone_tester_clock_elapsed(&ts,10000));
+
+	if (video_enabled){
+		liblinphone_tester_clock_start(&ts);
+		do{
+			if ((c1 != NULL) && (c2 != NULL)) {
+				if (linphone_call_get_video_stats(c1)->ice_state==state &&
+					linphone_call_get_video_stats(c2)->ice_state==state ){
+					video_success=TRUE;
+					check_ice_from_rtp(c1,c2,LinphoneStreamTypeVideo);
+					check_ice_from_rtp(c2,c1,LinphoneStreamTypeVideo);
+					break;
+				}
+				linphone_core_iterate(caller->lc);
+				linphone_core_iterate(callee->lc);
+			}
+			ms_usleep(20000);
+		}while(!liblinphone_tester_clock_elapsed(&ts,10000));
+	}
+
+	if (realtime_text_enabled){
+		liblinphone_tester_clock_start(&ts);
+		do{
+			if ((c1 != NULL) && (c2 != NULL)) {
+				if (linphone_call_get_text_stats(c1)->ice_state==state &&
+					linphone_call_get_text_stats(c2)->ice_state==state ){
+					text_success=TRUE;
+					check_ice_from_rtp(c1,c2,LinphoneStreamTypeText);
+					check_ice_from_rtp(c2,c1,LinphoneStreamTypeText);
+					break;
+				}
+				linphone_core_iterate(caller->lc);
+				linphone_core_iterate(callee->lc);
+			}
+			ms_usleep(20000);
+		}while(!liblinphone_tester_clock_elapsed(&ts,10000));
+	}
+
+	/*make sure encryption mode are preserved*/
+	if (c1) {
+		const LinphoneCallParams* call_param = linphone_call_get_current_params(c1);
+		BC_ASSERT_EQUAL(linphone_call_params_get_media_encryption(call_param),linphone_core_get_media_encryption(caller->lc), int, "%d");
+	}
+	if (c2) {
+		const LinphoneCallParams* call_param = linphone_call_get_current_params(c2);
+		BC_ASSERT_EQUAL(linphone_call_params_get_media_encryption(call_param),linphone_core_get_media_encryption(callee->lc), int, "%d");
+	}
+	linphone_call_unref(c1);
+	linphone_call_unref(c2);
+	return video_enabled ? (realtime_text_enabled ? text_success && audio_success && video_success : audio_success && video_success) : realtime_text_enabled ? text_success && audio_success : audio_success;
+}
+
+static void linphone_conference_server_call_state_changed(LinphoneCore *lc, LinphoneCall *call, LinphoneCallState cstate, const char *msg) {
+	LinphoneCoreVTable *vtable = linphone_core_get_current_vtable(lc);
+	LinphoneConferenceServer *conf_srv = (LinphoneConferenceServer *)vtable->user_data;
+
+	switch(cstate) {
+		case LinphoneCallIncomingReceived:
+			linphone_core_accept_call(lc, call);
+			break;
+
+		case LinphoneCallStreamsRunning:
+			if(linphone_call_get_conference(call) == NULL) {
+				linphone_core_add_to_conference(lc, call);
+				linphone_core_leave_conference(lc);
+				if(conf_srv->first_call == NULL) conf_srv->first_call = call;
+			}
+			break;
+
+		case LinphoneCallEnd:
+			if(call == conf_srv->first_call) {
+				if(linphone_core_get_conference(lc)) {
+					linphone_core_terminate_conference(lc);
+				}
+				conf_srv->first_call = NULL;
+			}
+			break;
+
+		default: break;
+	}
+}
+
+static void linphone_conference_server_refer_received(LinphoneCore *core, const char *refer_to) {
+	char method[20];
+	LinphoneAddress *refer_to_addr = linphone_address_new(refer_to);
+	char *uri;
+	LinphoneCall *call;
+
+	if(refer_to_addr == NULL) return;
+	strncpy(method, linphone_address_get_method_param(refer_to_addr), sizeof(method));
+	if(strcmp(method, "BYE") == 0) {
+		linphone_address_clean(refer_to_addr);
+		uri = linphone_address_as_string_uri_only(refer_to_addr);
+		call = linphone_core_find_call_from_uri(core, uri);
+		if(call) linphone_core_terminate_call(core, call);
+		ms_free(uri);
+	}
+	linphone_address_destroy(refer_to_addr);
+}
+
+static void linphone_conference_server_registration_state_changed(LinphoneCore *core,
+																  LinphoneProxyConfig *cfg,
+																  LinphoneRegistrationState cstate,
+																  const char *message) {
+	LinphoneCoreVTable *vtable = linphone_core_get_current_vtable(core);
+	LinphoneConferenceServer *m = (LinphoneConferenceServer *)linphone_core_v_table_get_user_data(vtable);
+	if(cfg == linphone_core_get_default_proxy_config(core)) {
+		m->reg_state = cstate;
+	}
+}
+
+LinphoneConferenceServer* linphone_conference_server_new(const char *rc_file, bool_t do_registration) {
+	LinphoneConferenceServer *conf_srv = (LinphoneConferenceServer *)ms_new0(LinphoneConferenceServer, 1);
+	LinphoneCoreManager *lm = (LinphoneCoreManager *)conf_srv;
+
+	conf_srv->vtable = linphone_core_v_table_new();
+	conf_srv->vtable->call_state_changed = linphone_conference_server_call_state_changed;
+	conf_srv->vtable->refer_received = linphone_conference_server_refer_received;
+	conf_srv->vtable->registration_state_changed = linphone_conference_server_registration_state_changed;
+	conf_srv->vtable->user_data = conf_srv;
+	conf_srv->reg_state = LinphoneRegistrationNone;
+	linphone_core_manager_init(lm, rc_file);
+	linphone_core_add_listener(lm->lc, conf_srv->vtable);
+	linphone_core_manager_start(lm, do_registration);
+	return conf_srv;
+}
+
+void linphone_conference_server_destroy(LinphoneConferenceServer *conf_srv) {
+	linphone_core_manager_uninit((LinphoneCoreManager *)conf_srv);
+	linphone_core_v_table_destroy(conf_srv->vtable);
+	ms_free(conf_srv);
+}
