@@ -36,7 +36,7 @@ static bool_t dump_stats=FALSE;
 static void stop(int signum){
 	running=FALSE;
 }
-#ifndef WIN32
+#ifndef _WIN32
 static void stats(int signum){
 	print_stats=TRUE;
 }
@@ -55,26 +55,26 @@ static void call_state_changed(LinphoneCore *lc, LinphoneCall *call, LinphoneCal
 	LinphoneCallParams * call_params;
 	switch(cstate){
 		case LinphoneCallIncomingReceived:
-			ms_message("Incoming call arrive  !\n");
+			ms_message("Incoming call arriving !\n");
 			/* accept the incoming call*/
-			call_params = linphone_core_create_default_call_parameters(lc);
+			call_params = linphone_core_create_call_params(lc, call);
 			linphone_call_params_enable_video(call_params,TRUE);
 			linphone_call_params_set_audio_direction(call_params,LinphoneMediaDirectionSendOnly);
 			linphone_call_params_set_video_direction(call_params,LinphoneMediaDirectionSendOnly);
 			linphone_core_accept_call_with_params(lc,call,call_params);
 			linphone_call_params_destroy(call_params);
-
 		break;
 		default:
 			break;
 	}
 }
 extern MSWebCamDesc mire_desc;
-static void helper() {
-	printf("auto_answer --help\n"
+static void helper(const char *progname) {
+	printf("%s --help\n"
 			"\t\t\t--listening-uri <uri> uri to listen on, default [sip:localhost:5060]\n"
 			"\t\t\t--max-call-duration max duration of a call in seconds, default [3600]\n"
-			"\t\t\t--verbose\n");
+			"\t\t\t--media-file <wav or mkv file to play to caller>\n"
+			"\t\t\t--verbose\n", progname);
 	exit(0);
 }
 
@@ -88,10 +88,11 @@ int main(int argc, char *argv[]){
 	char * tmp = NULL;
 	LpConfig * lp_config = lp_config_new(NULL);
 	int max_call_duration=3600;
+	static const char *media_file = NULL;
 
 	policy.automatically_accept=TRUE;
 	signal(SIGINT,stop);
-#ifndef WIN32
+#ifndef _WIN32
 	signal(SIGUSR1,stats);
 	signal(SIGUSR2,dump_call_logs);
 #endif
@@ -104,7 +105,7 @@ int main(int argc, char *argv[]){
 			addr = linphone_address_new(argv[++i]);
 			if (!addr) {
 				printf("Error, bad sip uri");
-				helper();
+				helper(argv[0]);
 			}
 /*			switch(linphone_address_get_transport(addr)) {
 			case LinphoneTransportUdp:
@@ -115,8 +116,13 @@ int main(int argc, char *argv[]){
 				helper();
 				break;
 			}*/
+		} else if (strcmp(argv[i], "--media-file") == 0){
+			i++;
+			if (i<argc){
+				media_file = argv[i];
+			}else helper(argv[0]);
 		} else {
-			helper();
+			helper(argv[0]);
 		}
 	}
 
@@ -138,13 +144,27 @@ int main(int argc, char *argv[]){
 
 
 	/*instead of using sound capture card, a file is played to the calling party*/
-	linphone_core_set_play_file(lc,PACKAGE_DATA_DIR "/sounds/linphone/hello16000.wav");
 	linphone_core_set_use_files(lc,TRUE);
+	linphone_core_enable_echo_cancellation(lc, FALSE); /*no need for local echo cancellation when playing files*/
+	if (!media_file){
+		linphone_core_set_play_file(lc,PACKAGE_DATA_DIR "/sounds/linphone/hello16000.wav");
+		linphone_core_set_preferred_framerate(lc,5);
+	}else{
+		PayloadType *pt = linphone_core_find_payload_type(lc, "opus", 48000, -1);
+		/*if opus is present, give it a bitrate for good quality with music, and stereo enabled*/
+		if (pt){
+			linphone_core_set_payload_type_bitrate(lc, pt, 150);
+			payload_type_set_send_fmtp(pt, "stereo=1");
+			payload_type_set_recv_fmtp(pt, "stereo=1");
+		}
+		linphone_core_set_play_file(lc, media_file);
+		linphone_core_set_preferred_video_size_by_name(lc, "720p");
+	}
 
 	{
 		MSWebCamDesc *desc = ms_mire_webcam_desc_get();
 		if (desc){
-			ms_web_cam_manager_add_cam(ms_web_cam_manager_get(),ms_web_cam_new(desc));
+			ms_web_cam_manager_add_cam(ms_factory_get_web_cam_manager(linphone_core_get_ms_factory(lc)),ms_web_cam_new(desc));
 			linphone_core_set_video_device(lc,"Mire: Mire (synthetic moving picture)");
 		}
 	}
@@ -158,19 +178,19 @@ int main(int argc, char *argv[]){
 
 	linphone_core_set_sip_transports(lc,&tp);
 	linphone_core_set_audio_port_range(lc,1024,65000);
-	linphone_core_set_preferred_framerate(lc,5);
+	linphone_core_set_video_port_range(lc,1024,65000);
 	linphone_core_set_primary_contact(lc,tmp=linphone_address_as_string(addr));
 	ms_free(tmp);
 
 	/* main loop for receiving notifications and doing background linphonecore work: */
 	while(running){
-		const MSList * iterator;
+		const bctbx_list_t * iterator;
 		linphone_core_iterate(lc);
 		ms_usleep(50000);
 		if (print_stats) {
 			ms_message("*********************************");
-			ms_message("*Current number of call   [%10i]  *",ms_list_size(linphone_core_get_calls(lc)));
-			ms_message("*Number of call until now [%10i]  *",ms_list_size(linphone_core_get_call_logs(lc)));
+			ms_message("*Current number of calls   [%10u]  *", (unsigned int)bctbx_list_size(linphone_core_get_calls(lc)));
+			ms_message("*Number of calls until now [%10u]  *", (unsigned int)bctbx_list_size(linphone_core_get_call_logs(lc)));
 			ms_message("*********************************");
 			print_stats=FALSE;
 		}
